/*  descartes
 *  Copyright (C) 2001 Luc Brun
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


#include "color_axis.h"

#define NB       3
#define UN_TIER  1.0/3.0

static double M0;
static double M1[NB],M2[NB][NB];

static void color_axis_init_priv    ();
static void compute_color_axis_priv (any_ptr,point,int);

void color_axis_calc(contour_set contours,dart f,vecteur val,mat_passage vect)
{ 
  int i,j;
  double cov_rg,cov_rb,cov_gb;
  double mean[NB],var[NB];
  
  color_axis_init_priv();
  
  contour_set_apply_function(contours,f,compute_color_axis_priv);
  
  for(i=0;i<NB;i++)
    {
      mean[i] = M1[i]/M0;
      var[i] = (M2[i][i]/M0)-(mean[i]*mean[i]);
    }
  
  cov_rg = (M2[0][1]/M0)-(M1[0]*M1[1])/(M0*M0);
  cov_rb = (M2[0][2]/M0)-(M1[0]*M1[2])/(M0*M0);
  cov_gb = (M2[1][2]/M0)-(M1[1]*M1[2])/(M0*M0);
  
  mat_covariance cov =
    {
      {var[0],cov_rg,cov_rb},
      {cov_rg,var[1],cov_gb},
      {cov_rb,cov_gb,var[2]},
    };
  
  matrice  val_initiale = 
    {
      {UN_TIER,UN_TIER,UN_TIER},
      {1.0,0,-1.0},
      {-0.5,1.0,-0.5},
    };

  if((cov[0][0]==0) && (cov[1][1]==0) && (cov[2][2]==0))
    {
      for(i=0;i<NB;i++)
	{
	  val[i]=0;
	  for(j=0;j<NB;j++)
	    {
	      vect[i][j]=0;
	    }
	}
    }
  else
    {
      matrice_val_vect_propre(cov,val,vect,val_initiale);
    }
}

static void color_axis_init_priv()
{
  int i,j;

  M0=0;

  for(i=0;i<NB;i++)
    {
      M1[i]=0;
      for(j=0;j<NB;j++)
	{
	  M2[i][j]=0;
	}
    }
}

static void compute_color_axis_priv(any_ptr obj,point p,int flag)
{
  int *pixel=(int*)obj;
  int i,j;
 
  M0++;
  for(i=0;i<NB;i++)
    {
      M1[i] += pixel[i];
      for(j=0;j<NB;j++)
	{
	  M2[i][j] += pixel[i]*pixel[j];
	}
    }
}

void color_axis_plane_calc(contour_set contours,dart f,double *axe,double *plan)
{
  vecteur val;
  mat_passage vect;
  
  color_axis_calc(contours,f,val,vect);

  double sum_val = val[0]+val[1]+val[2];
  
  if(sum_val==0)
    {
      *axe = -1;
      *plan = -1;
    }
  else
    {
      *axe = val[0]/sum_val;
      *plan = (val[0]+val[1])/sum_val;
    }
}
