/*  descartes
 *  Copyright (C) 2001 Luc Brun
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include "geometrie.h"
#include <math.h>


int surface_calc(contour_set contours,dart f)
{
  int res=contour_set_give_face_area(contours,f);
  
  return res;
}

int perimetre_calc(contour_set contours,dart f)
{
  dart_list b_list;
  dart b;
  int perimetre=0;
  
  b_list=FAIRE_dart_list();
  contour_set_give_segments_of_face(contours,f,b_list);
  
  dart_list_first(b_list);
  do
    {
      b=dart_list_give_current(b_list);
      perimetre += contour_set_give_segment_lenght(contours,b);
    }
  while(dart_list_next(b_list));
  
  DEFAIRE_dart_list(b_list);
  
  return perimetre;
}

void mean_calc(contour_set contours,dart f,int *xmean,int *ymean)
{ 
  dart_list b_list;
  dart b;
  
  b_list=FAIRE_dart_list();
  contour_set_give_segments_of_face(contours,f,b_list);
  
  output_segment p_list;
  p_list=FAIRE_output_segment();
  
  int sum_x=0;
  int sum_y=0;
  int i=0;

  Point p;
  
  dart_list_first(b_list);

  do
    {
      b=dart_list_give_current(b_list);
       
      contour_set_give_segment(contours,b,p_list);
      
      output_segment_first(p_list);
      do
	{
	  output_segment_give_current_point(p_list,&p);
	  
          sum_x += COORDX(p);
          sum_y += COORDY(p);
          i++;
	}
      while(output_segment_next(p_list));

      output_segment_reinitialize(p_list);
    }
  while(dart_list_next(b_list));

  DEFAIRE_output_segment(p_list);
  DEFAIRE_dart_list(b_list);
  
  *xmean=(int)((sum_x/i)+0.5);
  *ymean=(int)((sum_y/i)+0.5);
}

void points_calc(contour_set contours,dart f,
		 int *x1,int *y1,int *x2,int *y2)
{ 
  Point p;

  *x1 = INT_MAX;
  *y1 = INT_MAX;
  *x2 = INT_MIN;
  *y2 = INT_MIN;

  dart_list b_list;
  dart b;
  
  b_list=FAIRE_dart_list();
  contour_set_give_segments_of_face(contours,f,b_list);
  
  output_segment p_list;
  p_list=FAIRE_output_segment();  
  
  dart_list_first(b_list);

  do
    {
      b=dart_list_give_current(b_list);
      
      output_segment_reinitialize(p_list);
      contour_set_give_segment(contours,b,p_list);
      
      output_segment_first(p_list);
      do
	{
	  output_segment_give_current_point(p_list,&p);
	  
	  if(*x1 > COORDX(p)) *x1 = COORDX(p);
	  if(*y1 > COORDY(p)) *y1 = COORDY(p);
	  if(*x2 < COORDX(p)) *x2 = COORDX(p);
	  if(*y2 < COORDY(p)) *y2 = COORDY(p);
	  
	}
      while(output_segment_next(p_list));
      
      output_segment_reinitialize(p_list); 
    }
  while(dart_list_next(b_list));
  
  DEFAIRE_output_segment(p_list);
  DEFAIRE_dart_list(b_list);
}


int height_calc(contour_set contours,dart f)
{
  int res;
  int x1,y1,x2,y2;
  
  points_calc(contours,f,&x1,&y1,&x2,&y2);
  
  res=(y2-y1);
  
  return res;
}

int width_calc(contour_set contours,dart f)
{
  int res;
  int x1,y1,x2,y2;
  
  points_calc(contours,f,&x1,&y1,&x2,&y2);
  
  res=(x2-x1);
  
  return res;
}

void center_calc(contour_set contours,dart f,int *x,int *y)
{
  int x1,y1,x2,y2;

  points_calc(contours,f,&x1,&y1,&x2,&y2);

  *x = (int)((((x2-x1)/2)+x1)+0.5);
  *y = (int)((((y2-y1)/2)+y1)+0.5);
}

double ratio_surface_calc(contour_set contours,dart f)
{
  double res,s,h,w;
  
  s=(double)surface_calc(contours,f);
  h=(double)height_calc(contours,f);
  w=(double)width_calc(contours,f);

  res=s/(h*w);
  
  return res;
}

double ratio_perimetre_calc(contour_set contours,dart f)
{
  double res,p,h,w;

  p=(double)perimetre_calc(contours,f);
  h=(double)height_calc(contours,f);
  w=(double)width_calc(contours,f);

  res=p/(2*h+2*w);

  return res;
}

double ratio_pixel_calc(contour_set contours,dart f)
{
  double res,s,p;

  s=(double)surface_calc(contours,f);
  p=(double)perimetre_calc(contours,f);

  res=(16*s)/(p*p);

  return res;
}

double log_calc(contour_set contours,dart f)
{
  double res,h,w;
  
  h=(double)height_calc(contours,f);
  w=(double)width_calc(contours,f);

  res=log10(h/w);

  return res;
}

void direction_calc(contour_set contours,dart f,double *v)
{
  double a1,a2;
  double alpha=angle_calc(contours,f,&a1,&a2);
  
  v[0]=cos(alpha);v[1]=sin(alpha);
}

double angle_calc(contour_set contours,dart f,double *a1,double *a2)
{
  int nb_coord=2,i,j;
  double res;
  double alpha_tan,delta,Cov;
  int M0=0;
  double M1[nb_coord],M2[nb_coord][nb_coord];
  double Mean[nb_coord],Var[nb_coord];
  
  for(i=0;i<nb_coord;i++)
    {
      M1[i]=0;
      for(j=0;j<nb_coord;j++)
	{
	  M2[i][j]=0;
	}
    }
  
  dart_list b_list;
  dart b;
  
  b_list=FAIRE_dart_list();
  contour_set_give_segments_of_face(contours,f,b_list);
  
  output_segment p_list;
  p_list=FAIRE_output_segment();  

  Point p;
  
  dart_list_first(b_list);
  do
    {
      b=dart_list_give_current(b_list);
      
      output_segment_reinitialize(p_list);
      contour_set_give_segment(contours,b,p_list);
      
      output_segment_first(p_list);
      do
	{
	  output_segment_give_current_point(p_list,&p);

	  M0++;
	  M1[0]+=COORDX(p);
	  M1[1]+=COORDY(p);
	  M2[0][0]+=COORDX(p)*COORDX(p);
	  M2[1][1]+=COORDY(p)*COORDY(p);
	  M2[0][1]+=COORDX(p)*COORDY(p);
	  M2[1][0]+=COORDX(p)*COORDY(p);
	}
      while(output_segment_next(p_list));
    }
  while(dart_list_next(b_list));

  Mean[0] = (M1[0]/M0);Mean[1]=(M1[1]/M0);
  
  Var[0] = ((M2[0][0]/M0)-(Mean[0]*Mean[0]));
  Var[1] = ((M2[1][1]/M0)-(Mean[1]*Mean[1]));

  Cov = (M2[0][1]/M0)-((M1[0]*M1[1])/(M0*M0));

  delta = (Var[0]-Var[1])*(Var[0]-Var[1])+4*Cov*Cov;

  *a1 = (Var[0]+Var[1]+sqrt(delta))/2;

  *a2 = (Var[0]+Var[1]-sqrt(delta))/2;

  if(*a1-Var[0]==0) res=0;
  else
    {
      if(Cov == 0) res=1.5708;
      else
	{
	  alpha_tan = (*a1-Var[0])/Cov;
	  res = atan(alpha_tan);
	}
    }
  
  DEFAIRE_output_segment(p_list);
  DEFAIRE_dart_list(b_list);
  
  return res;
}

double elongation_calc(contour_set contours,dart f)
{
  double val1,val2,min1,max1,min2,max2,res;
  double v1[2],v2[2];
  dart_list b_list;
  dart b;
  output_segment p_list;
  Point p;
  
  direction_calc(contours,f,v1);

  v2[0] = 0-v1[1];
  v2[1] = v1[0];

  //Initialisation des variables min et max
  contour_set_assoc_face_point(contours,f,&p);
  min1=COORDX(p)*v1[0]+COORDY(p)*v1[1]; 
  max1=min1;
  min2=COORDX(p)*v2[0]+COORDY(p)*v2[1]; 
  max2=min2;

  b_list=FAIRE_dart_list();
  contour_set_give_segments_of_face(contours,f,b_list);
  
  p_list=FAIRE_output_segment();  
  
  dart_list_first(b_list);
  do
    {
      b=dart_list_give_current(b_list);
      
      output_segment_reinitialize(p_list);
      contour_set_give_segment(contours,b,p_list);
      
      output_segment_first(p_list);
      do
	{
	  output_segment_give_current_point(p_list,&p);
	  
	  val1=COORDX(p)*v1[0]+COORDY(p)*v1[1];
	  val2=COORDX(p)*v2[0]+COORDY(p)*v2[1];

	  if(val1>max1) max1=val1;
	  else
	    {
	      if(val1<min1) min1=val1;
	    }
	  if(val2>max2) max2=val2;
	  else
	    {
	      if(val2<min2) min2=val2;
	    }
	}
      while(output_segment_next(p_list));
    }
  while(dart_list_next(b_list));

  DEFAIRE_output_segment(p_list);
  DEFAIRE_dart_list(b_list);

  if((max1-min1)>(max2-min2)) res=(max1-min1)/(max2-min2);
  else res=(max2-min2)/(max1-min1);
  
  return res;
}

double elongation_bis_calc(contour_set contours,dart f)
{
  double a1,a2,angle,res;

  angle = angle_calc(contours,f,&a1,&a2);

  res = a1/(a1+a2);

  return res;
}

double give_distance_calc(Point p1,Point p2)
{
  int x1,y1,x2,y2;
  double dist;

  x1=COORDX(p1);
  y1=COORDY(p1);
  x2=COORDX(p2);
  y2=COORDY(p2);
  
  dist = sqrt(abs((y2-y1)*(y2-y1))+abs((x2-x1)*(x2-x1)));

  return dist;
}

double max_dist_calc(contour_set contours,dart f)
{
  double dist_max=0;
  Point p1,p2;

  dart_list b_list1,b_list2;
  dart b1,b2;
  
  b_list1=FAIRE_dart_list();
  b_list2=FAIRE_dart_list();

  contour_set_give_segments_of_face(contours,f,b_list1);
  contour_set_give_segments_of_face(contours,f,b_list2);
  
  output_segment p_list1,p_list2;

  p_list1=FAIRE_output_segment();
  p_list2=FAIRE_output_segment();
  
  dart_list_first(b_list1);
  do
    {
      b1=dart_list_give_current(b_list1);
      dart_list_remove_elt(b_list2,b1);
      
      contour_set_give_segment(contours,b1,p_list1);
      
      output_segment_first(p_list1);
      do
	{
	  output_segment_give_current_point(p_list1,&p1);
	  
	  dart_list_first(b_list2);
	  do
	    {
	      b2=dart_list_give_current(b_list2);
	      
	      contour_set_give_segment(contours,b2,p_list2);
	      
	      output_segment_first(p_list2);
	      do
		{
		  output_segment_give_current_point(p_list2,&p2);
		  
		  if(dist_max<give_distance_calc(p1,p2))
		    dist_max=give_distance_calc(p1,p2);
		}
	      while(output_segment_next(p_list2));
	      
	      output_segment_reinitialize(p_list2);
	      
	    }
	  while(dart_list_next(b_list2));
	}
      while(output_segment_next(p_list1));
      
      output_segment_reinitialize(p_list1); 
    }
  while(dart_list_next(b_list1));
  
  DEFAIRE_output_segment(p_list2);
  DEFAIRE_output_segment(p_list1);
  DEFAIRE_dart_list(b_list2);
  DEFAIRE_dart_list(b_list1);

  return dist_max;
}
