/*
 * ATTENTION!! Les fonctions suivantes dépendent de l'ordre des
 * conversions dans les fichier conversion.h et conversion.c .
 * Si l'ordre a été modifié, alors, la sélection des dimensions de
 * l'histogramme 2d peut ne pas être associé au bonnes conversions.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>

#include "interface.h"
#include "support.h"
#include "histogramme.h"

static contour_set contours_priv;

static dart selected_face_priv;

static int DIM1_SELECTED = 0;
static int DIM2_SELECTED = 0;

static gchar* t1;
static gchar* t2;

static int p1;
static int p2;

#define nb_proj 24

static gchar* list_proj[]=
{ 
  "RGB->R","RGB->G","RGB->B",
  "I1I2I3->I1","I1I2I3->I2","I1I2I3->I3",
  "Luv->L","Luv->u","Luv->v",
  "Lab->L","Lab->a","Lab->b",
  "YIQ->Y","YIQ->I","YIQ->Q",
  "HSL(triangle)->H","HSL(triangle)->S","HSL(triangle)->L",
  "HSV(hexcone)->H","HSV(hexcone)->S","HSV(hexcone)->V",
  "HLS(double hexcone)->H","HLS(double hexcone)->L","HLS(double hexcone)->S",
};


static void on_annuler_clicked (GtkButton*,
				gpointer);

static void on_afficher_clicked(GtkButton*,
				gpointer);


static void on_select1_activate(GtkEntry*,
				gpointer);

static void on_select2_activate(GtkEntry*,
				gpointer);


static void on_annuler_clicked (GtkButton *button,
				gpointer user_data)
{
  DIM1_SELECTED = 0;
  DIM2_SELECTED = 0;

  gtk_widget_destroy(GTK_WIDGET(user_data));
}

static void on_afficher_clicked(GtkButton *button,
				gpointer user_data)
{ 
  if(GTK_WIDGET_IS_SENSITIVE((GtkWidget*)button))
    {
      DIM1_SELECTED = 0;
      DIM2_SELECTED = 0;

      gchar title[50] = "Histogram ";
      gchar* t0 = " ";
      strcat(title,t1);
      strcat(title,t0);
      strcat(title,t2);
      
      gtk_widget_destroy(GTK_WIDGET(user_data));
      
      create_histo2d(contours_priv,selected_face_priv,p1,p2,title);
    }
}


static void on_select1_activate(GtkEntry* combo_entry,
				gpointer user_data)
{
  int i;
  
  t1 = gtk_entry_get_text(combo_entry);
  
  for(i=0; i<nb_proj; i++)
    {
      //comparaison des caractère caractéristiques des gchar*
      //afin de récupérer le numéro correspondant a la fonction 
      //de conversion sélectionnée
      if((i<3 && *(t1+5) == *(list_proj[i]+5))||
	 ((i>=3 && i<6) && *(t1+9) == *(list_proj[i]+9))||
	 ((i>=6 && i<9) && *(t1+5) == *(list_proj[i]+5))||
	 ((i>=9 && i<12) && *(t1+5) == *(list_proj[i]+5))||
	 ((i>=12 && i<15) && *(t1+5) == *(list_proj[i]+5))||
	 ((i>=15 && i<18) && ((*(t1+4) == *(list_proj[i]+4)) &&
			      (*(t1+15) == *(list_proj[i]+15))))||
	 ((i>=18 && i<21) && ((*(t1+4) == *(list_proj[i]+4)) &&
			      (*(t1+14) == *(list_proj[i]+14))))||
	 ((i>=21 && i<24) && ((*(t1+4) == *(list_proj[i]+4)) && 
			      (*(t1+21) == *(list_proj[i]+21)))))
	{
	  p1 = i;
	  
	  DIM1_SELECTED = 1;
	  
	  if(DIM2_SELECTED == 1)
	    gtk_widget_set_sensitive(GTK_WIDGET(user_data),TRUE);
	}
    }
}

static void on_select2_activate(GtkEntry* combo_entry,
				gpointer user_data)
{
  int i;

  t2 = gtk_entry_get_text(combo_entry);
  
  for(i=0; i<nb_proj; i++)
    {
      //comparaison des caractère caractéristiques des gchar*
      //afin de récupérer le numéro correspondant a la fonction 
      //de conversion sélectionnée
      if((i<3 && *(t2+5) == *(list_proj[i]+5))||
	 ((i>=3 && i<6) && *(t2+9) == *(list_proj[i]+9))||
	 ((i>=6 && i<9) && *(t2+5) == *(list_proj[i]+5))||
	 ((i>=9 && i<12) && *(t2+5) == *(list_proj[i]+5))||
	 ((i>=12 && i<15) && *(t2+5) == *(list_proj[i]+5))||
	 ((i>=15 && i<18) && ((*(t2+4) == *(list_proj[i]+4)) && 
			      (*(t2+15) == *(list_proj[i]+15))))||
	 ((i>=18 && i<21) && ((*(t2+4) == *(list_proj[i]+4)) &&
			      (*(t2+14) == *(list_proj[i]+14))))||
	 ((i>=21 && i<24) && ((*(t2+4) == *(list_proj[i]+4)) &&
			      (*(t2+21) == *(list_proj[i]+21)))))
	{
	  p2 = i;

	  DIM2_SELECTED = 1;
	  
	  if(DIM1_SELECTED == 1)
	    gtk_widget_set_sensitive(GTK_WIDGET(user_data),TRUE);
	}
    }
}

void
create_select_histo2d (contour_set contours, dart f)
{
  GtkWidget *window1;
  GtkWidget *vbox8;
  GtkWidget *vbox9;
  GtkWidget *accellabel5;
  GtkWidget *hbox4;
  GtkWidget *combo10;
  GList *combo10_items = NULL;
  GtkWidget *combo_entry10;
  GtkWidget *combo11;
  GList *combo11_items = NULL;
  GtkWidget *combo_entry11;
  GtkWidget *hbuttonbox1;
  GtkWidget *button11;
  GtkWidget *button12;
  int i;

  contours_priv = contours;
  selected_face_priv = f;

  window1 = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_object_set_data (GTK_OBJECT (window1), "window1", window1);
  gtk_window_set_title (GTK_WINDOW (window1), "");
  gtk_window_set_default_size(GTK_WINDOW (window1), 400, 50);

  vbox8 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox8);
  gtk_object_set_data_full (GTK_OBJECT (window1), "vbox8", vbox8,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox8);
  gtk_container_add (GTK_CONTAINER (window1), vbox8);

  vbox9 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox9);
  gtk_object_set_data_full (GTK_OBJECT (window1), "vbox9", vbox9,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox9);
  gtk_box_pack_start (GTK_BOX (vbox8), vbox9, TRUE, TRUE, 0);

  accellabel5 = gtk_accel_label_new ("Select the 2 components of the histogram");
  gtk_widget_ref (accellabel5);
  gtk_object_set_data_full (GTK_OBJECT (window1), "accellabel5", accellabel5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (accellabel5);
  gtk_box_pack_start (GTK_BOX (vbox9), accellabel5, FALSE, FALSE, 0);

  hbox4 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox4);
  gtk_object_set_data_full (GTK_OBJECT (window1), "hbox4", hbox4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox4);
  gtk_box_pack_start (GTK_BOX (vbox9), hbox4, TRUE, TRUE, 0);

  combo10 = gtk_combo_new ();
  gtk_widget_ref (combo10);
  gtk_object_set_data_full (GTK_OBJECT (window1), "combo10", combo10,
                            (GtkDestroyNotify) gtk_widget_unref);
  //gtk_combo_set_value_in_list(GTK_COMBO(combo10),TRUE,FALSE);
  gtk_widget_show (combo10);
  gtk_box_pack_start (GTK_BOX (hbox4), combo10, TRUE, TRUE, 0);

  for(i=0; i<nb_proj; i++)
    {
      combo10_items = g_list_append (combo10_items, (gpointer) list_proj[i]);
    }

  gtk_combo_set_popdown_strings (GTK_COMBO (combo10), combo10_items);
  g_list_free (combo10_items);

  combo_entry10 = GTK_COMBO (combo10)->entry;
  gtk_widget_ref (combo_entry10);
  gtk_object_set_data_full (GTK_OBJECT (window1), "combo_entry10",
			    combo_entry10,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (combo_entry10);
  gtk_editable_set_editable(GTK_EDITABLE(combo_entry10),FALSE);
  gtk_entry_set_text (GTK_ENTRY (combo_entry10), "Component 1");

  combo11 = gtk_combo_new ();
  gtk_widget_ref (combo11);
  gtk_object_set_data_full (GTK_OBJECT (window1), "combo11", combo11,
                            (GtkDestroyNotify) gtk_widget_unref);
  //gtk_combo_set_value_in_list(GTK_COMBO(combo11),TRUE,FALSE);
  gtk_widget_show (combo11);
  gtk_box_pack_start (GTK_BOX (hbox4), combo11, TRUE, TRUE, 0);

  for(i=0; i<nb_proj; i++)
    {
      combo11_items = g_list_append (combo11_items, (gpointer) list_proj[i]);
    }

  gtk_combo_set_popdown_strings (GTK_COMBO (combo11), combo11_items);
  g_list_free (combo11_items);

  combo_entry11 = GTK_COMBO (combo11)->entry;
  gtk_widget_ref (combo_entry11);
  gtk_object_set_data_full (GTK_OBJECT (window1), "combo_entry11", combo_entry11,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (combo_entry11);
  gtk_editable_set_editable(GTK_EDITABLE(combo_entry11),FALSE);
  gtk_entry_set_text (GTK_ENTRY (combo_entry11), "Component 2");


  hbuttonbox1 = gtk_hbutton_box_new ();
  gtk_widget_ref (hbuttonbox1);
  gtk_object_set_data_full (GTK_OBJECT (window1), "hbuttonbox1", hbuttonbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbuttonbox1);
  gtk_box_pack_start (GTK_BOX (vbox8), hbuttonbox1, TRUE, TRUE, 0);


  button11 = gtk_button_new_with_label ("Valider");
  gtk_widget_ref (button11);
  gtk_object_set_data_full (GTK_OBJECT (window1), "button11", button11,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button11);
  gtk_container_add (GTK_CONTAINER (hbuttonbox1), button11);
  gtk_widget_set_sensitive(GTK_WIDGET(button11),FALSE);
  //GTK_WIDGET_SET_FLAGS (button11, GTK_CAN_DEFAULT);

  gtk_signal_connect (GTK_OBJECT (button11), "clicked",
                      GTK_SIGNAL_FUNC (on_afficher_clicked),
                      window1);


  gtk_signal_connect (GTK_OBJECT (combo_entry10), "changed",
		      GTK_SIGNAL_FUNC (on_select1_activate),
		      button11);
  gtk_signal_connect (GTK_OBJECT (combo_entry11), "changed",
		      GTK_SIGNAL_FUNC (on_select2_activate),
		      button11);

  
  button12 = gtk_button_new_with_label ("Annuler");
  gtk_widget_ref (button12);
  gtk_object_set_data_full (GTK_OBJECT (window1), "button12", button12,
                            (GtkDestroyNotify) gtk_widget_unref);
  
  gtk_widget_show (button12);
  gtk_container_add (GTK_CONTAINER (hbuttonbox1), button12);
  //GTK_WIDGET_SET_FLAGS (button12, GTK_CAN_DEFAULT);

  gtk_signal_connect (GTK_OBJECT (button12), "clicked",
                      GTK_SIGNAL_FUNC (on_annuler_clicked),
                      window1);

  gtk_widget_show(window1);
}

