/*  descartes
 *  Copyright (C) 2001 Luc Brun
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <assert.h>
#include "histo1d.h"
#include "tab_int.h"
#include "tab_gen.h"

static histo1d tmp_hist;

struct moments
{
  int m1[3];
};

#define Histo_pos       SELF(histo_pos)
#define Histo_neg       SELF(histo_neg)
#define Projection      SELF(projection)
#define MinX            SELF(minX)
#define MaxX            SELF(maxX)
#define MaxY            SELF(maxY)
#define Histo_color_pos SELF(histo_color_pos)
#define Histo_color_neg SELF(histo_color_neg)

static void compute_histo_priv(any_ptr,point,int);

struct histo1d
{
  tab_int histo_pos;
  tab_int histo_neg;
  PFprojection projection;
  int minX;
  int maxX;
  int maxY;
  tab_gen histo_color_pos;
  tab_gen histo_color_neg;
};

histo1d FAIRE_histo1d(contour_set contours,
		      dart f,
		      PFprojection proj)
{
  INSTANCIER(histo1d);
  assert(!contour_set_is_infinite(contours,f));
  
  Histo_pos = FAIRE_tab_int();
  Histo_neg = FAIRE_tab_int();
  Projection = proj;
  MinX = INT_MAX;
  MaxX = INT_MIN;
  MaxY = INT_MIN;
  Histo_color_pos = FAIRE_tab_gen();
  Histo_color_neg = FAIRE_tab_gen();
  tmp_hist = self;
  
  contour_set_apply_function(contours,f,compute_histo_priv);
  
  return self;
}

void DEFAIRE_histo1d(histo1d self)
{
  int i;
  
  assert(self);
  
  DEFAIRE_tab_int(Histo_pos);
  DEFAIRE_tab_int(Histo_neg);
  
  for(i=MinX; i<=MaxX; i++)
    {
      if(i<0)
	free(tab_gen_give_object(Histo_color_neg,-i));
      else 
	free(tab_gen_give_object(Histo_color_pos,i));
    }
  DEFAIRE_tab_gen(Histo_color_pos);
  DEFAIRE_tab_gen(Histo_color_neg);
  free(self);
}

static void compute_histo_priv(any_ptr obj,point p,int flag)
{
  int *pixel = (int*)obj;
  
  histo1d self = tmp_hist;
  
  int val,freq;
  struct moments *mom_ptr,*mom_ptr2;
  struct moments mom;

  mom_ptr2 = (struct moments*)malloc(sizeof(struct moments));

  int r,g,b;
  
  val = Projection(pixel[0],pixel[1],pixel[2]);
  if(val<0)
    {
      freq = tab_int_give_value(Histo_neg,-val);
      tab_int_put_value(Histo_neg,-val,freq+1);
      
      mom_ptr = tab_gen_give_object(Histo_color_neg,-val);
      tab_gen_remove_object(Histo_color_neg,-val);
       
      if(mom_ptr == NULL)
	{
	  r = 0;
	  g = 0;
	  b = 0;
	}
      else
	{  
	  r = (int)mom_ptr->m1[0];
	  g = (int)mom_ptr->m1[1];
	  b = (int)mom_ptr->m1[2];
	}
      
      mom.m1[0] = r+pixel[0];
      mom.m1[1] = g+pixel[1];
      mom.m1[2] = b+pixel[2];

      *mom_ptr2 = mom;
      
      tab_gen_insert_object(Histo_color_neg,mom_ptr2,-val);
    }
  else
    {
      freq = tab_int_give_value(Histo_pos,val);
      tab_int_put_value(Histo_pos,val,freq+1);
      
      mom_ptr = tab_gen_give_object(Histo_color_pos,val);
      tab_gen_remove_object(Histo_color_pos,val);
      
      if(mom_ptr == NULL)
	{
	  r = 0;
	  g = 0;
	  b = 0;
	} 
      else
	{
	  r = (int)mom_ptr->m1[0];
	  g = (int)mom_ptr->m1[1];
	  b = (int)mom_ptr->m1[2];
	}
      
      mom.m1[0] = r+pixel[0];
      mom.m1[1] = g+pixel[1];
      mom.m1[2] = b+pixel[2];
      
      *mom_ptr2 = mom;     
      
      tab_gen_insert_object(Histo_color_pos,mom_ptr2,val);
    }

  free(mom_ptr);
  
  if(val < MinX)
    MinX = val;
  else
    if(val > MaxX) MaxX = val;
  
  if(freq+1 > MaxY) MaxY = freq+1;
}

int histo1d_get_value(histo1d self, int i)
{
  assert(self);
  if(i < 0)
    return tab_int_give_value(Histo_neg,-i);
  else
    return tab_int_give_value(Histo_pos,i);
}

int histo1d_get_maxX(histo1d self)
{
  assert(self);
  
  return MaxX;
}

int histo1d_get_minX(histo1d self)
{
  assert(self);

  return MinX;
}

int histo1d_get_maxY(histo1d self)
{
  assert(self);

  return MaxY;
}

int histo1d_get_M1_red(histo1d self, int i)
{
  assert(self);
  
  struct moments* mom_ptr;

  if(i>=0)
    {
      mom_ptr = tab_gen_give_object(Histo_color_pos,i);
      if(mom_ptr == NULL)
	return 0;
      else
	return mom_ptr->m1[0];
    }
  else
    {
      mom_ptr = tab_gen_give_object(Histo_color_neg,-i);
      if(mom_ptr == NULL)
	return 0;
      else
	return mom_ptr->m1[0];
    }
}

int histo1d_get_M1_green(histo1d self, int i)
{
  assert(self);

  struct moments* mom_ptr;

  if(i>=0)
    {
      mom_ptr = tab_gen_give_object(Histo_color_pos,i);
      if(mom_ptr == NULL)
	return 0;
      else
	return mom_ptr->m1[1];
    }
  else
    {
      mom_ptr = tab_gen_give_object(Histo_color_neg,-i);
      if(mom_ptr == NULL)
	return 0;
      else
	return mom_ptr->m1[1];
    }
}

int histo1d_get_M1_blue(histo1d self, int i)
{
  assert(self);
  
  struct moments* mom_ptr;

  if(i>=0)
    {
      mom_ptr = tab_gen_give_object(Histo_color_pos,i);
      if(mom_ptr == NULL)
	return 0;
      else
	return mom_ptr->m1[2];
    }
  else
    {
      mom_ptr = tab_gen_give_object(Histo_color_neg,-i);
      if(mom_ptr == NULL)
	return 0;
      else
	return mom_ptr->m1[2];
    }
}

void histo1d_affiche_histo(histo1d self)
{
  assert(self);

  int i;
  printf("Affichage de l'histogramme : \n");
  printf("| ");
  for(i=MinX;i<=MaxX;i++)
    {
      if(i<0)
	printf("%d | ",tab_int_give_value(Histo_neg,-i));
      else
	printf("%d | ",tab_int_give_value(Histo_pos,i));
    }
  printf("\n");
}

void histo1d_affiche_histo_color(histo1d self, int color)
{
  assert(self);
  
  int i;
  struct moments* mom_ptr;
  printf("Affichage de l'histogramme des couleurs : \n");
  printf("| ");
  for(i=MinX;i<=MaxX;i++)
    {
      if(i<0){
	mom_ptr = tab_gen_give_object(Histo_color_neg,-i);
	if(mom_ptr == NULL)
	  printf("%d | ",0);
	else
	  printf("%d | ",mom_ptr->m1[color]);
      }
      else
	{
	  mom_ptr = tab_gen_give_object(Histo_color_pos,i);
	  if(mom_ptr == NULL)
	    printf("%d | ",0);
	  else
	    printf("%d | ",mom_ptr->m1[color]);
	}
    }
  printf("\n");
}
