/*  descartes
 *  Copyright (C) 2001 Luc Brun
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/* Le membre "pixmap" de la structure n'est pas utilisé dans l'état
 * actuel du logiciel. Il pourra être utilisé en vue d'une
 * optimisation de l'affichage de l'histo2d dans le fichier
 * gtk_histo2d.c
*/

#include <assert.h>
#include "histo2d.h"

static histo2d tmp_hist;

#define Histo          SELF(histo)
#define Projection1    SELF(projection1)
#define Projection2    SELF(projection2)
#define Taille_i       SELF(taille_i)
#define Taille_j       SELF(taille_j)
#define Min1           SELF(min1)
#define Max1           SELF(max1)
#define Min2           SELF(min2)
#define Max2           SELF(max2)
#define Freq_max       SELF(freq_max)
//#define Pixmap         SELF(pixmap)

static void compute_minmax_priv (any_ptr,point,int);
static void init_histo_priv     (void);
static void compute_histo_priv  (any_ptr,point,int);

struct histo2d
{
  short int** histo;
  PFprojection projection1;
  PFprojection projection2;
  int taille_i;
  int taille_j;
  int min1, max1, min2, max2;
  short int freq_max;
  //GdkPixmap *pixmap;
};

histo2d FAIRE_histo2d(contour_set contours,
		      dart f,
		      PFprojection proj1,
		      PFprojection proj2)
{
  INSTANCIER(histo2d);
  assert(!contour_set_is_infinite(contours,f));

  Histo = NULL;
  Projection1 = proj1;
  Projection2 = proj2;
  Taille_i = 0;
  Taille_j = 0;
  Max1 = INT_MIN;
  Min1 = INT_MAX;
  Max2 = INT_MIN;
  Min2 = INT_MAX;
  Freq_max = 0;
  //Pixmap = NULL;
  tmp_hist = self;
  
  contour_set_apply_function(contours,f,compute_minmax_priv);
  
  Taille_i = (Max1-Min1+1);
  Taille_j = (Max2-Min2+1);
  short int** tab;
  tab = (short int**)malloc(Taille_i*sizeof(short int*));
  int i;
  for(i=0; i<Taille_i; i++)
    {
      tab[i] = (short int*)malloc(Taille_j*sizeof(short int));
    }
  Histo = tab;
  tmp_hist = self;
  
  init_histo_priv();
  contour_set_apply_function(contours,f,compute_histo_priv);
  
  return self;
}

void DEFAIRE_histo2d(histo2d self)
{
  assert(self);
  
  int i;
  for(i=0; i<Taille_i; i++)
    {
      free(Histo[i]);
    }
  free(Histo);
  //free(Pixmap);
  free(self);
}


static void compute_minmax_priv(any_ptr obj,point p,int flag)
{
  int *pixel = (int*)obj;

  histo2d self = tmp_hist;

  int val1,val2;
  val1 = Projection1(pixel[0],pixel[1],pixel[2]);
  val2 = Projection2(pixel[0],pixel[1],pixel[2]);

  if(val1>Max1) 
    Max1 = val1;
  else
    if(val1<Min1) Min1 = val1;
  if(val2>Max2)
    Max2 = val2;
  else
    if(val2<Min2) Min2 = val2;
}

static void init_histo_priv()
{
  int i,j;

  histo2d self = tmp_hist;

  for(i=0; i<Taille_i; i++)
    {
      for(j=0; j<Taille_j; j++)
	{
	Histo[i][j] = 0;
	}
    }
}

static void compute_histo_priv(any_ptr obj,point p,int flag)
{
  int *pixel = (int*)obj;
  
  histo2d self = tmp_hist;
  
  int val1,val2;
  short int freq;

  val1 = Projection1(pixel[0],pixel[1],pixel[2]);
  val2 = Projection2(pixel[0],pixel[1],pixel[2]);
  freq = Histo[val1-Min1][val2-Min2];
  Histo[val1-Min1][val2-Min2] = freq+1;

  if((freq+1)>Freq_max) Freq_max = freq;
}

void histo2d_get_minmax(histo2d self, 
			int *mi1, int *ma1, 
			int *mi2, int *ma2)
{
  assert(self);

  *mi1 = Min1;
  *mi2 = Min2;
  *ma1 = Max1;
  *ma2 = Max2;
}

/*
void histo2d_set_pixmap(histo2d self, GdkPixmap *p)
{
  assert(self);

  Pixmap = p;
}

GdkPixmap* histo2d_get_pixmap(histo2d self)
{
  assert(self);

  return Pixmap;
}
*/

short int histo2d_get_value(histo2d self, int i, int j)
{
  assert(self);

  return Histo[i][j];
}

void histo2d_get_taille(histo2d self, int* i, int *j)
{
  assert(self);
  
  *i = Taille_i;
  *j = Taille_j;
}

short int histo2d_get_freq_max(histo2d self)
{
  assert(self);

  return Freq_max;
}

void histo2d_affiche_histo(histo2d self)
{
  assert(self);
  
  int i,j;
  printf("Affichage de l'histogramme : \n");
  printf("| ");
  for(i=0; i<Taille_i; i++)
    {
      for(j=0; j<Taille_j; j++)
	{
	  printf("%d | ",Histo[i][j]);
	}
      printf("\n");
    }
  printf("\n");
}

