/*  descartes
 *  Copyright (C) 2001 Luc Brun
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#define __NO_MOVES__
#include <stdio.h>

#include <gtk/gtk.h>
#include <interface_zoom.h>
#include <gdk_imlib.h>
#include <display.h> 
#include <ximage.h>
#include <segmentation_link.h>
#include <chargement_seg.h>
#include <rules_link.h>
#include <image.h>
#include <tab_label.h>
#include <dart_list.h>
#include "utility.h"
#include "my_gtk.h"
#include "gladesrc.h"
#include "interface.h"
#include "export_regions.h"
#include "info_region.h"
#include "histogramme.h"
#include <conversion.h>
#include "gtk_select_histo2d.h"

static export_region_kind export_mode=export_one_region_as_bitmap;

static unsigned char normal_color[3] = {0,0xff,0};
static unsigned char selection_color[3] = {0xff,0,0};

static ximage ximg;
static GdkImlibImage *gdk_img;
static bouleen select_face   = vrai;
static bouleen select_segment    = faux;
static bouleen existe        = vrai;

static contour_set contours  = NULL_CONTOUR_SET;

static bouleen image_chargee = faux;

static dart selected_face    = 1;
static output_segment selected_segment;
static segmentation seg_algorithms;
static rule regle;

static double ecart_max      = 30.0;
static guint minimal_region_size= 5;

static int nb_ppt[3]={16,15,4}; 

//--- new mon module -------------


bouleen is_image_chargee()
{
  return image_chargee;
}

gint interface_save_tablabel( GtkObject *FileSelection )
{
  gchar *filename;
  
  filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(FileSelection));

  if( tab_label_save(contours,filename) == -1 )
     {
      gtk_widget_destroy(GTK_WIDGET(FileSelection));
      create_dialog("Impossible de sauvegarder le tableau de labels !", gtk_widget_destroy);
      return FALSE;
    }

  gtk_widget_destroy(GTK_WIDGET(FileSelection));  

  return TRUE; 
}

gint interface_load_tablabel( GtkObject *FileSelection )
{
  gchar *filename;
  GtkWidget *load;

  filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(FileSelection));

  if(  tab_label_load(contours,filename) != 0 )
    {
      gtk_widget_destroy(GTK_WIDGET(FileSelection));
      create_dialog("Impossible de charger le tableau de labels !", gtk_widget_destroy);
      return FALSE;
    }

  load = get_widget(Descartes,"Segmentation");
  gtk_widget_set_sensitive(load,0);
   
  load = get_widget(Descartes,"Tableau_de_labels");
  gtk_widget_set_sensitive(load,0);
 
  gtk_widget_destroy(GTK_WIDGET(FileSelection ));

  ximage_draw_set_of_faces(ximg,contours,contour_set_give_image_faces(contours),normal_color);
  ximage_redraw_all(ximg,contours,normal_color,selection_color);
  
  return TRUE;
}

//--- delete mon module ----------

gint interface_load_image(GtkObject *FileSelection)
{
  gchar* filename;
  gint val;
  GtkWidget *load;

  filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(FileSelection));
  
  fprintf(stderr,"NAme of the file : %s \n",filename);

  val=interface_load_image_from_name(filename);

  if(val)
    {
      load = get_widget(Descartes, "ouvrir_button");
      gtk_widget_set_sensitive(load,0);
      
      load = get_widget(Descartes, "Image");
      gtk_widget_set_sensitive(load,0);
    }

  gtk_widget_destroy(GTK_WIDGET(FileSelection)); 
  return val;
}

gboolean ximage_exposure(GtkWidget *widget,GdkEventExpose *event,gpointer user_data)
{
  ximage_display(ximg);
  return TRUE;
}

gint interface_load_image_from_name(gchar* filename)
{
  GtkWidget *darea;
  GtkScrolledWindow *scrolled_window;
  GtkWidget *align;
  int i,j,k;
  unsigned char* data;
  image img;
  display disp;
  int pixel[3];
  int largeur,hauteur;

  gdk_img=gdk_imlib_load_image(filename);

  if(!gdk_img)
    {
      create_dialog(" Impossible de charger l'image ",
		    gtk_widget_destroy);
      return FALSE;
    };
  img=FAIRE_image();
  image_initialize(img,
		   3,
		   gdk_img->rgb_width,
		   gdk_img->rgb_height
		   );
  
  scrolled_window = GTK_SCROLLED_WINDOW(get_widget(Descartes,"Scrolled Window"));
  largeur=MIN(gdk_img->rgb_width,gdk_screen_width());
  hauteur=MIN(gdk_img->rgb_height+50,gdk_screen_height());
  gtk_widget_set_usize(GTK_WIDGET(scrolled_window),largeur,hauteur);

  align = get_widget(Descartes,"align");
//  gtk_alignment_set(GTK_ALIGNMENT(align),.5,.5,1.,1.);

  darea=get_widget(Descartes,"ximage");  
  gtk_drawing_area_size (GTK_DRAWING_AREA (darea), gdk_img->rgb_width+1, gdk_img->rgb_height+1);
  


// gtk_container_add(GTK_CONTAINER(scrolled_window),darea);


  /* Recopie de la gdkImlibImage dans une image */
  image_debut(img);
  data=gdk_img->rgb_data;
  for(i=0;i<gdk_img->rgb_height;i++)
    for(j=0;j<gdk_img->rgb_width;j++)
      {
	for(k=0;k<3;k++)
	  {
	    pixel[k]=*data;
	    data++;
	  }  
	image_ecrire_pixel(img,pixel);
	image_pixel_suivant(img);
      }

  contours=FAIRE_contour_set(gdk_img->rgb_data,
			     3,
			     gdk_img->rgb_width,
			     gdk_img->rgb_height);


  gdk_imlib_destroy_image(gdk_img); 
  disp=FAIRE_display(img);
  ximg=FAIRE_ximage(disp,
		    darea,
		    normal_color);

  gtk_signal_connect (GTK_OBJECT (darea), "expose-event", GTK_SIGNAL_FUNC (ximage_exposure), NULL);


  /* Initialisation des variables statiques utilises tout au long du
     programme*/
  image_chargee=vrai;
  selected_segment = FAIRE_output_segment();
  seg_algorithms   = FAIRE_segmentation();
  regle            = FAIRE_rule();

  ximage_display(ximg); 
  gtk_widget_show(darea);
  gtk_widget_show(align);

  return TRUE;
}
gint interface_load_segmentation(GtkObject *FileSelection)
{
  char *filename;
  int val;
  GtkWidget *load;

  if(!image_chargee)
    {
      create_dialog(" Vous devez d'abord charger une image",
		    gtk_widget_destroy);

      return FALSE;
    }
  filename =
    gtk_file_selection_get_filename(GTK_FILE_SELECTION(FileSelection));
  
  if (filename == NULL)
    {
      create_dialog(" Nom d'image non valide",
		    gtk_widget_destroy);
      
      return FALSE;
    }
  
  printf ("Nom du fichier a charger=%s\n", filename);
  
  val = interface_load_segmentation_from_name(filename);
  if(val)
    {
      load = get_widget(Descartes, "Segmentation");
      gtk_widget_set_sensitive(load,0);

      load = get_widget(Descartes,"Tableau_de_labels");
      gtk_widget_set_sensitive(load,0);
    }

  gtk_widget_destroy(GTK_WIDGET(FileSelection));  

  return val;
}

gint interface_load_segmentation_from_name(gchar* filename)
{

  if(open_load_seg(contours,filename) != 0)    
    {
      create_dialog(" Impossible de charger la segmentation",
		    gtk_widget_destroy);
      
      return FALSE;
    }
  do_load_seg();
  close_load_seg();
  ximage_redraw_all(ximg,contours,normal_color,selection_color);
  ximage_display(ximg);

  return TRUE;
}

gint interface_save_segmentation(GtkObject *FileSelection)
{
  gchar* filename =
    gtk_file_selection_get_filename(GTK_FILE_SELECTION(FileSelection));

  if(!image_chargee)
    {
      create_dialog(" Vous devez d'abord charger une image\net dfinir une segmentation",
		    gtk_widget_destroy);
      gtk_widget_destroy(GTK_WIDGET(FileSelection));  
      return FALSE;
    }
  
  printf ("Nom du fichier a sauvegarder= %s\n",filename);
  contour_set_save(contours,filename);

  gtk_widget_destroy(GTK_WIDGET(FileSelection));  
  return TRUE;
}

void interface_update_selection(int x,int y)
{
  dart b;
  Point p;

  if(contours==NULL_CONTOUR_SET)
    {
      create_dialog("Erreur:\n aucune selection",
		    gtk_widget_destroy);
	  
      return;
    }

  COORDX(p) = x;
  COORDY(p) = y;

  if (select_face)
    {
      if(existe)
	ximage_draw_face(ximg,contours,selected_face,normal_color);
      selected_face = contour_set_assoc_point_face(contours,&p);
      ximage_draw_face(ximg,contours,selected_face,selection_color);
      printf("numero de la face : %d\n", selected_face);
    }
  if (select_segment)
    {     
      if (existe)
	ximage_draw_segment(ximg,selected_segment,normal_color);
      
      b = contour_set_assoc_point_arc(contours,&p);
      printf("b=%d\n",b);
      contour_set_extend_seg(contours,b);
      contour_set_give_segment(contours,b,selected_segment);
      ximage_draw_segment(ximg,selected_segment,selection_color);
    }

  ximage_display(ximg);
  existe = vrai;
}

void interface_set_selection(selection selected)
{
  if(contours==NULL_CONTOUR_SET)
    {
      create_dialog("Erreur:\n aucune face selectionne",
		    gtk_widget_destroy);
	  
      return;
    }

  /* First, the case of including region */
  if(selected == region_englobante_selection)
    {
        if(!select_face)
	  {
	          create_dialog(" Vous devez selectionner une face",
				gtk_widget_destroy);
		  
		  return;
	  }
	if(!contour_set_is_infinite(contours,selected_face))
	  {
	    create_dialog("Erreur: La face selectionne est infinie",
			  gtk_widget_destroy);

	    return;
	  }
	if(existe)
	  ximage_draw_face(ximg,contours,selected_face,normal_color);
  
	selected_face =  contour_set_give_including_face(contours,
							 selected_face);
  
	ximage_draw_face(ximg,contours,selected_face,selection_color);
	
	ximage_display(ximg);
	existe = vrai;
	return;
    }
  /* Now we switch from face selection to segment selection or vice
     versa*/
  

  /*  We delete the previous selection*/
  if(existe)
    {
      if (select_face)
	ximage_draw_face(ximg,contours,selected_face,normal_color);
      
      if (select_segment)
	ximage_draw_segment(ximg,selected_segment,normal_color);
    }

  select_segment = selected == segment_selection;
  select_face    = !select_segment;
  existe = faux;
  ximage_display(ximg);
}
void interface_delete_selection()
{
  list_gen seg_list;
  dart b;

  if(contours==NULL_CONTOUR_SET)
    {
      create_dialog("Erreur:\n aucune face selectionne",
		    gtk_widget_destroy);
	  
      return;
    }
  
  if(!existe)
    {
      create_dialog(" Selectionnez un segment ou une region",
		    gtk_widget_destroy);
      
      return;
    }
  contour_set_memorize_modified_segments(contours);

  if(select_face)
    contour_set_remove_face(contours,selected_face);

  if(select_segment)
    {
      b = output_segment_give_dart_nb(selected_segment);
      contour_set_remove_arc(contours,b);
    }
  seg_list = FAIRE_list_gen();
  contour_set_give_modified_segments(contours,seg_list);

  ximage_erase_set_of_segments(ximg,seg_list);

  ximage_display(ximg);

  DEFAIRE_list_gen(seg_list,free_output_segment);
  existe = faux;
}
void interface_apply_segmentation(segmentation_alg alg)
{
  dart_list face_list;
  int nb_face_cree;

  if(contours==NULL_CONTOUR_SET)
    {
      create_dialog("Erreur:\n aucune face selectionne",
		    gtk_widget_destroy);
	  
      return;
    }

  if(!select_face)
    {
      create_dialog("Erreur:\n aucune face selectionne",
		    gtk_widget_destroy);
      
      return;
    }
  if(contour_set_is_infinite(contours,selected_face))
    {
      create_dialog("Erreur:\n la face selectionne est infinie",
		    gtk_widget_destroy);
      
      return;
    }
  if(existe)
    ximage_draw_face(ximg,contours,selected_face,normal_color );

  contour_set_memorize_inserted_faces(contours);


  segmentation_set_algorithm(seg_algorithms,algo_seg[alg]);
  segmentation_make_segments(seg_algorithms,
			     contours,
			     selected_face,
			     ecart_max);

  face_list = FAIRE_dart_list();
  contour_set_give_inserted_faces(contours,face_list);
  ximage_draw_set_of_faces(ximg,contours,face_list,normal_color );

  ximage_display(ximg);
  nb_face_cree = dart_list_give_cardinal(face_list);
  printf("%d faces ont etes crees \n",nb_face_cree);
  DEFAIRE_dart_list(face_list);
  existe = vrai;
}
void interface_apply_rule(rule_alg selected_rule)
{
  if(contours==NULL_CONTOUR_SET)
    {
      create_dialog("Erreur:\n aucune face selectionne",
		    gtk_widget_destroy);
	  
      return;
    }
  /* We do nothing if the rule requires a face which is not defined*/
  if(selected_rule == smooth_rule ||
     selected_rule == remove_details_rule||
     selected_rule == remove_included_faces_rule ||
     selected_rule == meta_seg_uniform_rule)
    {
      if(!select_face)
	{
	  create_dialog("Erreur:\n aucune face selectionne",
			gtk_widget_destroy);
	  
	  return;
	}
      if (contour_set_is_infinite(contours,selected_face) )
	{
	  create_dialog("Erreur:\n la face selectionne est infinie",
			gtk_widget_destroy);
	  
	  return ;
	}   
    }
  /* We specify the rule */
  rule_specify_rule(regle,algo_rule[selected_rule]);

  /* We call the right method, with the adapted parameter and pre/post
     processings */
  switch(selected_rule)
    {
    case smooth_rule:
      {	
	ximage_draw_face(ximg,contours,selected_face,normal_color);
	rule_apply_rule(regle,contours,selected_face);
	existe=faux;
	break;
      }
    case remove_details_rule:
      {
	rule_apply_rule(regle,contours,selected_face,minimal_region_size);
	break;
      }
    case remove_all_details_rule:
      {
	rule_apply_rule(regle,contours,selected_face,minimal_region_size);
	existe=faux;
	break;
      }

    case remove_included_faces_rule:
      {
	rule_apply_rule(regle,contours,selected_face);
	break;
      }
    case meta_seg_uniform_rule:
    case meta_seg_uniform_rec_rule:
      {
	ximage_draw_face(ximg,contours,selected_face,normal_color);
	rule_apply_rule(regle,
			contours,
			selected_face,
			ecart_max,ximg,normal_color);
	existe=faux;

	break;
      }
      
    default:
      return;
    };
  ximage_redraw_all(ximg,contours,normal_color,selection_color);
  ximage_display(ximg);
  return;
}

void on_zoom_activate_interface( int zfactor )
{
  GtkWidget *Zoom;
  
  if(contours==NULL_CONTOUR_SET || !select_face)
    {
      create_dialog("Erreur:\n aucune face selectionne",
		    gtk_widget_destroy);
      
      return;
    }

  Zoom = create_Zoom (ximage_get_display(ximg),zfactor,contours,selected_face,normal_color);
  gtk_widget_show(Zoom);  
}

void interface_set_export_region_mode(export_region_kind kind)
{
  export_mode =kind;
}
gboolean interface_finite_face_selected()
{
  if(!image_chargee)
    return FALSE;

  if (!existe || !select_face)
    return FALSE;
  if (contour_set_is_infinite(contours,selected_face))
    return FALSE;
  
  return TRUE;
}
gint interface_export_region(GtkObject *FileSelection)
{
  gchar* filename;
  export_region region;


  filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(FileSelection));
  region = FAIRE_export_region(contours,export_mode);
  export_region_do_export(region,contours,selected_face);
  export_region_save(region,filename);
  DEFAIRE_export_region(region);

  gtk_widget_destroy(GTK_WIDGET(FileSelection));  
  return TRUE;
}
void interface_info_region()
{
  create_info_region(contours,selected_face);
}

void interface_histogramme_R(gpointer user_data)
{
  PFconversion* conv = changer_espace(RGB_to_RGB);

  create_histogramme(contours,selected_face,conv[0],(gchar*)user_data);
}

void interface_histogramme_G(gpointer user_data)
{
  PFconversion* conv = changer_espace(RGB_to_RGB);

  create_histogramme(contours,selected_face,conv[1],(gchar*)user_data);
}

void interface_histogramme_B(gpointer user_data)
{
  PFconversion* conv = changer_espace(RGB_to_RGB);

  create_histogramme(contours,selected_face,conv[2],(gchar*)user_data);
}

void interface_histogramme_I1(gpointer user_data)
{
  PFconversion* conv = changer_espace(RGB_to_I1I2I3);

  create_histogramme(contours,selected_face,conv[0],(gchar*)user_data);
}

void interface_histogramme_I2(gpointer user_data)
{
  PFconversion* conv = changer_espace(RGB_to_I1I2I3);

  create_histogramme(contours,selected_face,conv[1],(gchar*)user_data);
}

void interface_histogramme_I3(gpointer user_data)
{
  PFconversion* conv = changer_espace(RGB_to_I1I2I3);

  create_histogramme(contours,selected_face,conv[2],(gchar*)user_data);
}

void interface_histogramme_L(gpointer user_data)
{
  PFconversion* conv = changer_espace(RGB_to_Luv);

  create_histogramme(contours,selected_face,conv[0],(gchar*)user_data);
}

void interface_histogramme_u(gpointer user_data)
{
  PFconversion* conv = changer_espace(RGB_to_Luv);

  create_histogramme(contours,selected_face,conv[1],(gchar*)user_data);
}

void interface_histogramme_v(gpointer user_data)
{
  PFconversion* conv = changer_espace(RGB_to_Luv);

  create_histogramme(contours,selected_face,conv[2],(gchar*)user_data);
}

void interface_histogramme_a(gpointer user_data)
{
  PFconversion* conv = changer_espace(RGB_to_Lab);

  create_histogramme(contours,selected_face,conv[1],(gchar*)user_data);
}

void interface_histogramme_b(gpointer user_data)
{
  PFconversion* conv = changer_espace(RGB_to_Lab);

  create_histogramme(contours,selected_face,conv[2],(gchar*)user_data);
}

void interface_histogramme_Y(gpointer user_data)
{
  PFconversion* conv = changer_espace(RGB_to_YIQ);

  create_histogramme(contours,selected_face,conv[0],(gchar*)user_data);
}

void interface_histogramme_I(gpointer user_data)
{
  PFconversion* conv = changer_espace(RGB_to_YIQ);

  create_histogramme(contours,selected_face,conv[1],(gchar*)user_data);
}

void interface_histogramme_Q(gpointer user_data)
{
  PFconversion* conv = changer_espace(RGB_to_YIQ);

  create_histogramme(contours,selected_face,conv[2],(gchar*)user_data);
}

void interface_histogramme_Ht(gpointer user_data)
{
  PFconversion* conv = changer_espace(RGB_to_HSL);

  create_histogramme(contours,selected_face,conv[0],(gchar*)user_data);
}

void interface_histogramme_St(gpointer user_data)
{
  PFconversion* conv = changer_espace(RGB_to_HSL);

  create_histogramme(contours,selected_face,conv[1],(gchar*)user_data);
}

void interface_histogramme_Lt(gpointer user_data)
{
  PFconversion* conv = changer_espace(RGB_to_HSL);

  create_histogramme(contours,selected_face,conv[2],(gchar*)user_data);
}

void interface_histogramme_Hh(gpointer user_data)
{
  PFconversion* conv = changer_espace(RGB_to_HSV);

  create_histogramme(contours,selected_face,conv[0],(gchar*)user_data);
}

void interface_histogramme_Sh(gpointer user_data)
{
  PFconversion* conv = changer_espace(RGB_to_HSV);

  create_histogramme(contours,selected_face,conv[1],(gchar*)user_data);
}

void interface_histogramme_Vh(gpointer user_data)
{
  PFconversion* conv = changer_espace(RGB_to_HSV);

  create_histogramme(contours,selected_face,conv[2],(gchar*)user_data);
}

void interface_histogramme_Hd(gpointer user_data)
{
  PFconversion* conv = changer_espace(RGB_to_HLS);

  create_histogramme(contours,selected_face,conv[0],(gchar*)user_data);
}

void interface_histogramme_Ld(gpointer user_data)
{
  PFconversion* conv = changer_espace(RGB_to_HLS);

  create_histogramme(contours,selected_face,conv[1],(gchar*)user_data);
}

void interface_histogramme_Sd(gpointer user_data)
{
  PFconversion* conv = changer_espace(RGB_to_HLS);

  create_histogramme(contours,selected_face,conv[2],(gchar*)user_data);
}

void get_store_widget_ppt(GtkWidget** ptr,int flag)
{
  static GtkWidget* store;
  if (flag==READ_WIDGET_PPT)
    *ptr=store;
  else
    store=*ptr;
}


gint interface_export_geo_info (GtkObject *FileSelection)
{
  int i,j=0;
  gchar* filename;
  GtkWidget* clist;
  gchar* text1;
  gchar* text2;

  get_store_widget_ppt(&clist, READ_WIDGET_PPT);

  filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(FileSelection));
  //creation fichier csv
  FILE *fp;
  if((fp = fopen(filename,"a")) != NULL)
    {
      for(i=0; i<nb_ppt[j]; i++)
	{
	  gtk_clist_get_text(GTK_CLIST(clist),i,0,&text1);
	  gtk_clist_get_text(GTK_CLIST(clist),i,1,&text2);

	  fprintf(fp,"%s;%s\n",text1,text2);
	}
    }

  fclose(fp);
  
  gtk_widget_destroy(GTK_WIDGET(FileSelection));
  
  return TRUE;
}

gint interface_export_photo_info (GtkObject *FileSelection)
{
  int i,j=1;
  gchar* filename;
  GtkWidget* clist;
  gchar* text1;
  gchar* text2;
  
  get_store_widget_ppt(&clist, READ_WIDGET_PPT);
  
  filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(FileSelection));
  //creation fichier csv
  FILE *fp;
  if((fp = fopen(filename,"a")) != NULL)
    {
      for(i=0; i<nb_ppt[j]; i++)
	{
	  gtk_clist_get_text(GTK_CLIST(clist),i,0,&text1);
	  gtk_clist_get_text(GTK_CLIST(clist),i,1,&text2);

	  fprintf(fp,"%s;%s\n",text1,text2);	  
	}
    }
  
  fclose(fp);
  
  gtk_widget_destroy(GTK_WIDGET(FileSelection));
  
  return TRUE;
}

gint interface_export_topo_info (GtkObject *FileSelection)
{
  int i,j=2;
  gchar* filename;
  GtkWidget* clist;
  gchar* text1;
  gchar* text2;
  
  get_store_widget_ppt(&clist, READ_WIDGET_PPT);
  
  //creation fichier csv
  filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(FileSelection));
  FILE *fp;
  if((fp = fopen(filename,"a")) != NULL)
    {
      for(i=0; i<nb_ppt[j]; i++)
	{
	  gtk_clist_get_text(GTK_CLIST(clist),i,0,&text1);
	  gtk_clist_get_text(GTK_CLIST(clist),i,1,&text2);

	  fprintf(fp,"%s;%s\n",text1,text2);
	}
    }
  
  fclose(fp);
  
  gtk_widget_destroy(GTK_WIDGET(FileSelection));
  
  return TRUE;
}

void get_store_histo1d(histo1d* ptr,int flag)
{
  static histo1d store;
  if (flag==READ_HISTO)
    *ptr=store;
  else
    store=*ptr;
}

gint interface_export_histo1d(GtkObject *FileSelection)
{
  gchar* filename;
  FILE *fp;
  histo1d histo_tmp;
  int i;
  
  get_store_histo1d(&histo_tmp,READ_HISTO);
  
  filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(FileSelection));
  if((fp = fopen(filename,"w")) != NULL)
    {
      for(i=histo1d_get_minX(histo_tmp); i<=histo1d_get_maxX(histo_tmp); i++)
	{
	  fprintf(fp,"%d %d\n",i,histo1d_get_value(histo_tmp,i));
	}
    }
  
  fclose(fp);  
  
  gtk_widget_destroy(GTK_WIDGET(FileSelection));
  
  return TRUE;
}


void interface_histo2d()
{
  create_select_histo2d(contours,selected_face);
}

void get_store_histo2d(histo2d* ptr,int flag)
{
  static histo2d store;
  if (flag==READ_HISTO)
    *ptr=store;
  else
    store=*ptr;
}

gint interface_export_histo2d(GtkObject *FileSelection)
{
  gchar* filename;
  FILE *fp;
  histo2d histo_tmp;
  int width, height;
  int vmax = 255;
  int i, j;
  int r,g,b;
  short int freq_max,freq_cour;
  float t;
  char * buf,*buf2;
  get_store_histo2d(&histo_tmp,READ_HISTO);

  histo2d_get_taille(histo_tmp,&width,&height);
  buf=(char*)malloc(3*sizeof(char)*width*height);
  buf2=buf;
  filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(FileSelection));
  if((fp = fopen(filename,"w")) != NULL)
    {
      //ecriture du fichier ppm a partir de l'histogramme 2D
      fprintf(fp,"P6\n");
      fprintf(fp,"#%s\n",filename);
      fprintf(fp,"%d %d\n",width,height);
      fprintf(fp,"%d\n",vmax);

      freq_max = histo2d_get_freq_max(histo_tmp);

      for(j=0; j<height; j++)
	{
	  for(i=0; i<width; i++)
	    {
	      //calcul de r,g,b
	      
	      freq_cour = histo2d_get_value(histo_tmp,i,j);
	      
	      t = (float)freq_cour/(float)freq_max;
	      
	      if(freq_cour<(freq_max/2))
		{
		  r = 0;
		  g = (int)(255*2*t);
		  b = (int)(255*(1-2*t));
		}
	      else
		{
		  r = (int)(255*(2*t-1));
		  g = (int)(2*255*(1-t));
		  b = 0;
		}	      
	      *buf2=r;buf2++;
	      *buf2=g;buf2++;
	      *buf2=b;buf2++;
	    }
	}
    }
  fwrite(buf,sizeof(char),3*width*height,fp);
  fclose(fp);  
  
  gtk_widget_destroy(GTK_WIDGET(FileSelection));
  free(buf);
  return vrai;
}
