/*  descartes
 *  Copyright (C) 2001 Luc Brun
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


#include "photometrie.h"
#include <math.h>

void mean_color_calc(contour_set contours,dart f,vector V)
{
  contour_set_give_face_mean(contours,f,V);
}

double red_variance_calc(contour_set contours,dart f)
{
  double res,M0;
  vector M1,M2;

  contour_set_give_face_moments(contours,f,&M0,M1,M2);
  res = (M2[0]-M1[0]*M1[0]/M0)/M0;

  return res;
}

double green_variance_calc(contour_set contours,dart f)
{
  double res,M0;
  vector M1,M2;

  contour_set_give_face_moments(contours,f,&M0,M1,M2);
  res = (M2[1]-M1[1]*M1[1]/M0)/M0;

  return res;
}

double blue_variance_calc(contour_set contours,dart f)
{
  double res,M0;
  vector M1,M2;

  contour_set_give_face_moments(contours,f,&M0,M1,M2);
  res = (M2[2]-M1[2]*M1[2]/M0)/M0;

  return res;
}

double squared_error_calc(contour_set contours,dart f)
{
  double res;

  res=contour_set_give_face_squared_error(contours,f);

  return res;
}

void excolor_calc(contour_set contours,dart f,vector V)
{
  vector Vmean;

  mean_color_calc(contours,f,Vmean);

  V[0]=(2*Vmean[0])-Vmean[1]-Vmean[2];
  V[1]=(2*Vmean[1])-Vmean[0]-Vmean[2];
  V[2]=(2*Vmean[2])-Vmean[0]-Vmean[1];
}

/* retourne l'entier correspondant a la couleur ayant le plus grand "mean"
   (0 pour rouge, 1 pour vert et 2 pour bleu).
   Si deux "mean" ont la valeur maximale,
   alors retourne -1. */

int dominant_color_calc(contour_set contours,dart f)
{
  vector V;
  int i,color;
  double mean_max;

  mean_color_calc(contours,f,V);

  mean_max = (double)V[0];
  color = 0;

  for(i=1;i<DIM_MAX;i++)
    {
      if(V[i]>mean_max)
	{
	  mean_max = (double)V[i];
	  color = i;
	}
      else
	{
	  if(V[i]==mean_max) color = -1;
	}
    }
  
  return color;
}

double intensity_calc(contour_set contours,dart f)
{
  double res;
  vector V;

  mean_color_calc(contours,f,V);
  res=(double)(V[0]+V[1]+V[2])/3;

  return res;
}

double hue_calc(contour_set contours,dart f)
{
  int taille=3;
  double h,r,g,b,min,max,rgb[taille];
  vector RGB;
  
  mean_color_calc(contours,f,RGB);
  
  if((RGB[0]+RGB[1]+RGB[2])==0) h=-1;
  else
    {
      r=(double)(RGB[0])/(RGB[0]+RGB[1]+RGB[2]);
      g=(double)(RGB[1])/(RGB[0]+RGB[1]+RGB[2]);
      b=(double)(RGB[2])/(RGB[0]+RGB[1]+RGB[2]);
      
      rgb[0]=r;
      rgb[1]=g;
      rgb[2]=b;
      
      give_min_max(rgb,taille,&min,&max);
      
      if(max==min) h=-1;
      else
	{
	  if(r==max)
	    {
	      if(g>b) h=(60*(g-b)/(max-min));
	      else h=360+(60*(g-b)/(max-min));
	    }
	  else
	    {
	      if(g==max) h=120+(60*(b-r)/(max-min));
	      else
		{
		  h=240+(60*(r-g)/(max-min));
		}
	    }
	}
    }
  
  return h;
}

double saturation_calc(contour_set contours,dart f)
{
  int taille=3;
  double s,r,g,b,min,max,rgb[taille];
  vector RGB;
  
  mean_color_calc(contours,f,RGB);
  
  if((RGB[0]+RGB[1]+RGB[2])==0) s=0;
  else
    {
      r=(double)(RGB[0])/(RGB[0]+RGB[1]+RGB[2]);
      g=(double)(RGB[1])/(RGB[0]+RGB[1]+RGB[2]);
      b=(double)(RGB[2])/(RGB[0]+RGB[1]+RGB[2]);
      
      rgb[0]=r;
      rgb[1]=g;
      rgb[2]=b;
      
      give_min_max(rgb,taille,&min,&max);
      
      if(max==0) s=0;
      else s=(max-min)/max;
    }
  
  return s;
}

void give_min_max(double *tab_val,int taille,double *min,double *max)
{
  *min=tab_val[0];
  *max=tab_val[0];
  int i;
  
  for(i=1;i<taille;i++)
    {
      if(tab_val[i]>*max) *max=tab_val[i];
      if(tab_val[i]<*min) *min=tab_val[i];
    }
}
