/*  descartes
 *  Copyright (C) 2001 Luc Brun
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gdk_imlib.h>
#include "types.h"
#include "point.h"
#include "output_segment.h"
#include "contour_set.h"
#include "utility.h"
#include "display.h"
#include "type_obj.h"
#include "euclid_path_set.h"
#include "chemin_euclidien.h"
#include "xdrawing.h"
#include "objet.h"

static int current;

#define Parent SELF(parent)
#define Darea  SELF(darea)
#define Img SELF(img)
#define Segments SELF(segments)
#define Width SELF(width)
#define Height SELF(height)
#define Pmin SELF(pmin)
#define Pmax SELF(pmax)
#define Zfactor SELF(zfactor)
#define Contours SELF(contours)
#define Dl SELF(dl)
#define Dis SELF(dis)

#define Sface SELF(selected_face)
#define Discret SELF(discret)
#define Euclidien SELF(euclidien)
#define Nbpoints SELF(nbpoints)
#define Mode SELF(mode)

#define Gc SELF(gc)
#define Cmap SELF(cmap)

GdkColor white, black;

struct xdrawing
{
  GtkWidget *parent;
  GtkWidget *darea;
  GdkPixmap *img;
  GdkPixmap *segments;
  
  display dis;
  int width;
  int height;
  Point pmin, pmax;
  int zfactor;

  contour_set contours;
  dart_list dl;

  // tableaux des points de la face slectione
  GdkGC *gc;
  GdkColormap *cmap;

  dart selected_face;
  GdkPoint *discret;
  GdkPoint *euclidien;
  int nbpoints;
  draw_mode mode;
};

/*--- listing des fonctions priv ---*/

void xdrawing_draw_segment(xdrawing,
			   output_segment);

static void fill_euclidean_tab_priv_child( any_ptr,
					   float,float,
					   float,float);

static void fill_discret_tab_priv( xdrawing,dart_list);
static void fill_euclidean_tab_priv( xdrawing);

/*--- debut du code ---*/

xdrawing FAIRE_xdrawing( GtkWidget *_parent, 
			 GtkWidget *_darea, 
			 display dis, 
			 int zfactor, 
			 contour_set _contours, 
			 unsigned char *normal_color)
{
  INSTANCIER(xdrawing);

  Parent = _parent; Darea = _darea;
  Zfactor = zfactor;
  Sface = -1;
  Contours = _contours;
  Dis = dis;

  // par defaut on est en mode euclidien
  Mode = smooth;

  // on alloue le gc, et on slectionne la couleur  
  Gc = gdk_gc_new(Darea->window);
  Cmap = gdk_window_get_colormap(Darea->window);
  gdk_color_white(Cmap,&white);
  gdk_color_black(Cmap,&black);
  gdk_gc_set_foreground(Gc,&white);
  
  return self;
}

void DEFAIRE_xdrawing(xdrawing self)
{
  gdk_gc_unref(Gc);
  gdk_colormap_unref(Cmap);
  gdk_pixmap_unref(Segments);
  gdk_pixmap_unref(Img);
  DEFAIRE_dart_list(Dl);
  free(Discret);
  free(Euclidien);
  free(self);
}


void xdrawing_set_face( xdrawing self, dart selected_face )
{
  dart_list segments_to_draw, selected_face_list;
  dart d;
  output_segment seg;
  Point p;
  
  segments_to_draw=FAIRE_dart_list();
  selected_face_list=FAIRE_dart_list();
  contour_set_give_segments_of_face(Contours,selected_face,selected_face_list);
  
  Sface = selected_face;
  compute_bounding_box(Contours,Sface,&Pmin,&Pmax);
  xdrawing_set_bounding_box(self,&Pmin,&Pmax);

  dart_list_first(Dl);
  do
    {
      d = dart_list_give_current(Dl);
      if(!(contour_set_assoc_seg_face(Contours,d)==Sface || contour_set_assoc_seg_face(Contours,-d)==Sface))
	dart_list_insert(segments_to_draw,d);
    }
  while(dart_list_next(Dl));
  
  fill_discret_tab_priv(self,selected_face_list);
  fill_euclidean_tab_priv(self);

  DEFAIRE_dart_list(selected_face_list);
  DEFAIRE_dart_list(Dl);
  Dl=segments_to_draw;
}
 
void xdrawing_set_bounding_box( xdrawing self, point sp, point dp )
{
  int lwidth, lheight;
  GdkImlibImage *tmp, *limg;
  GdkPixmap *tmp2;
  int x,y;
  
  Dl = intersect_box(Contours,sp,dp);

  lwidth = COORDX(*dp)-COORDX(*sp);
  lheight = COORDY(*dp)-COORDY(*sp);

  Width = lwidth;
  Height = lheight;

  tmp = display_give_gdk_imlib_image(Dis);
  gdk_imlib_crop_image(tmp,COORDX(*sp)+1,COORDY(*sp)+1,lwidth,lheight);
  limg = gdk_imlib_clone_scaled_image(tmp,lwidth*Zfactor,lheight*Zfactor);

  tmp2 = gdk_pixmap_new(Darea->window,
			lwidth*Zfactor+1,
			lheight*Zfactor+1,
			gdk_window_get_visual(Darea->window)->depth);
  assert(limg);

  Width = limg->rgb_width;
  Height = limg->rgb_height;

  gdk_imlib_render(limg,Width,Height);
  Img = gdk_imlib_move_image(limg);

  Width=lwidth*Zfactor+1;
  Height=lheight*Zfactor+1;

  gdk_draw_pixmap(tmp2,Gc,Img,
		  0,0,0,0,
		  lwidth*Zfactor+1,lheight*Zfactor+1);

  gdk_pixmap_unref(Img);
  Img = tmp2;

  assert(Img);

  Segments = gdk_pixmap_new(Darea->window,
			    Width,
			    Height,
			    gdk_window_get_visual(Darea->window)->depth);
  assert(Segments);

  gdk_imlib_destroy_image(limg);
  gdk_imlib_destroy_image(tmp);  
}

void xdrawing_draw_selected_face( xdrawing self )
{
  switch(Mode)
    {
    case sharp:
	gdk_draw_lines(Darea->window,Gc,Discret,Nbpoints);
	break;

    case smooth:
	gdk_draw_lines(Darea->window,Gc,Euclidien,Nbpoints);
	break;

    case sharp_and_smooth:
	gdk_draw_lines(Darea->window,Gc,Discret,Nbpoints);
	gdk_draw_lines(Darea->window,Gc,Euclidien,Nbpoints);
    };
}

void xdrawing_set_mode( xdrawing self, draw_mode mode )
{
  Mode = mode;
  gdk_window_clear(Darea->window);
}

void xdrawing_draw_face(xdrawing self)
{
  output_segment current_seg;
  dart current;

  current_seg = FAIRE_output_segment();

  dart_list_first(Dl);
  do
    {
      current = dart_list_give_current(Dl);

      contour_set_give_segment(Contours,current,current_seg);
      xdrawing_draw_segment(self,current_seg);
    }
  while(dart_list_next(Dl));

  Cmap = gdk_window_get_colormap(Darea->window);  
  gdk_color_white(Cmap,&white);
  gdk_gc_set_foreground(Gc,&white);

  DEFAIRE_output_segment(current_seg);
}

void xdrawing_draw_segment(xdrawing self, output_segment seg)
{
  // point tmp + point de controle pour le clip
  Point p,cp;
  int i=0;
  int n=0;

  // on prend la couleur vert
  GdkColor vert;
  GdkPoint *tabp=NULL;

  vert.green = 0xffff;
  vert.red = vert.blue = 0;
  gdk_color_alloc(Cmap,&vert);
  gdk_gc_set_foreground(Gc,&vert);

  // n = nombre de mouvement + 1 point (pour le nb de points) + 1 pour fermet les contours
  n =  output_segment_give_lenght(seg)+2; 
  tabp = (GdkPoint*)calloc(n,sizeof(GdkPoint));
  assert(tabp);
  
  output_segment_first(seg);
  do
    {
      output_segment_give_current_point(seg,&p);
      COORDX(cp) = COORDX(p)-COORDX(Pmin);
      COORDY(cp) = COORDY(p)-COORDY(Pmin);
   
      if( (COORDX(cp) < 0 || COORDX(cp) >= Width || COORDY(cp) < 0 || COORDY(cp) >= Height) )
	{
	  if( i > 0)
	    {
	      gdk_draw_lines(Img,Gc,tabp,i);
	      gdk_draw_lines(Segments,Gc,tabp,i);
	    }
	  i=0;
	  continue;
	}
      tabp[i].x = COORDX(cp)*Zfactor;
      tabp[i].y = COORDY(cp)*Zfactor;
      i++;
    } while(output_segment_next(seg)); 

  output_segment_give_last_point(seg,&p);
  COORDX(cp) = COORDX(p)-COORDX(Pmin);
  COORDY(cp) = COORDY(p)-COORDY(Pmin);

  if( (COORDX(cp) >= 0 && COORDX(cp) < Width && COORDY(cp) >= 0 && COORDY(cp) < Height) )
    {
      tabp[i].x = COORDX(cp)*Zfactor;
      tabp[i].y = COORDY(cp)*Zfactor;
      i++;	  
    }
  
  if(i>0)
    {
      gdk_draw_lines(Img,Gc,tabp,i);
      gdk_draw_lines(Segments,Gc,tabp,i);
    }

  gdk_window_set_back_pixmap(Darea->window,Img,0);
  gdk_window_clear(Darea->window);

  free(tabp);
}

void xdrawing_set_background(xdrawing self)
{
  gdk_window_set_back_pixmap(Darea->window,Img,0);
  gdk_window_clear(Darea->window);
  xdrawing_draw_selected_face(self);
}

void xdrawing_unset_background(xdrawing self)
{
  gdk_window_set_back_pixmap(Darea->window,Segments,0);
  gdk_window_clear(Darea->window);
  xdrawing_draw_selected_face(self);
}

GtkWidget* xdrawing_get_parent( xdrawing self )
{
  return Parent;
}

GdkPixmap* xdrawing_get_pixmap( xdrawing self )
{
  return Img;
}

int xdrawing_get_width( xdrawing self )
{
  return Width;
}

int xdrawing_get_height( xdrawing self )
{
  return Height;
}


static void fill_euclidean_tab_priv_child( any_ptr obj,
					   float x1,float y1,
					   float x2,float y2)
{
  xdrawing self = (xdrawing)obj;

  Euclidien[current].x = (int)Zfactor*(x1 - COORDX(Pmin));
  Euclidien[current].y = (int)Zfactor*(y1 - COORDY(Pmin));
  current++;
}


static void fill_euclidean_tab_priv( xdrawing self )
{
  Point pmini;
  euclid_path euclids;
  unsigned char p[3] = {0,0,0};

  COORDX(pmini)=0;
  COORDY(pmini)=0;
  
  display_set_child(Dis,self);
  display_set_draw_line(Dis,fill_euclidean_tab_priv_child);

  Euclidien = (GdkPoint*)malloc((Nbpoints)*sizeof(GdkPoint));
  current = 0;

  euclids = FAIRE_euclid_path(Contours);
  euclid_path_init_from_face(euclids,Contours,Sface);
  euclid_path_write_face(euclids,Contours,Sface,Dis,Zfactor,&Pmin,p);
		/* Boucle sur la face */
  Euclidien[current].x= Euclidien[0].x;
  Euclidien[current].y= Euclidien[0].y;

  DEFAIRE_euclid_path(euclids);
}
			

static void fill_discret_tab_priv( xdrawing self, dart_list dlist )
{
  output_segment seg;
  dart current;
  point_set tab_point;
  point plot_tmp;
  Point plot,res;
  int i;
  bouleen  (*next)(output_segment);
  void  (*first)(output_segment);

  seg = FAIRE_output_segment();
  tab_point = FAIRE_point_set(20);

  dart_list_first(dlist);
  do
    {
      current = dart_list_give_current(dlist);
      contour_set_give_segment(Contours,current,seg);
      if( current > 0)
	{
	  first = output_segment_first;
	  next = output_segment_next;
	}
      else
	{
	  first = output_segment_last;
	  next = output_segment_previous;
	}
      first(seg);
      do
	{
	  output_segment_give_current_point(seg,&plot);
      
	  /* on remet le point  l'chelle */
	  COORDX(res) = (COORDX(plot)-COORDX(Pmin))*Zfactor;
	  COORDY(res) = (COORDY(plot)-COORDY(Pmin))*Zfactor;
	  
	  point_set_insert_point(tab_point,&res);
	  
	} 
      while(next(seg)); 
    }
  while(dart_list_next(dlist));  
  
  Nbpoints = point_set_give_cardinal(tab_point);

  Discret = (GdkPoint*)malloc((Nbpoints+1)*sizeof(GdkPoint));
  
  for( i = 0; i < Nbpoints; i++ )
    {
      plot_tmp = point_set_give_point(tab_point,i);
      Discret[i].x = COORDX(*plot_tmp);
      Discret[i].y = COORDY(*plot_tmp);
    }
  Discret[Nbpoints].x =   Discret[0].x;
  Discret[Nbpoints].y  =  Discret[0].y;
  Nbpoints++;
  DEFAIRE_output_segment(seg);
  DEFAIRE_point_set(tab_point);
  
}
