/*  descartes
 *  Copyright (C) 2001 Luc Brun
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <assert.h>
#include <glib.h>

#define __NO_MOVES__
#include "ximage.h"
#include "conversion.h"
#include "contour_set.h"
#include "output_segment.h"
#include "point.h"
#include "display.h"
#include "interface.h"

#include "objet.h"

static void draw_point_priv           (ximage,int,int,unsigned char*);
static void init_table_of_indices_priv(ximage);
static void make_border_priv          (ximage);

#define Indices       SELF(indices)
#define Father        SELF(father)
#define Largeur       SELF(largeur)
#define Hauteur       SELF(hauteur)
#define Window        SELF(window)
#define Border_color  SELF(border_color)

struct ximage
{
  unsigned char*      indices;
  display        father;
  int            largeur;
  int            hauteur;
  unsigned char*  border_color;
  GtkWidget* window;
};

ximage FAIRE_ximage(display father,
		    GtkWidget* win,
		    unsigned char* border_s_color)
{
  INSTANCIER(ximage); 

  Father       = father;
  Border_color = border_s_color;
  Largeur      = display_give_width(Father)+1; 
  Hauteur      = display_give_lenght(Father)+1;
  Window = win;
  Indices = (unsigned char*)malloc(Largeur*Hauteur*3);
  init_table_of_indices_priv(self);
  make_border_priv(self);
  
  return self; 
} 

void  DEFAIRE_ximage(ximage self) 
{ 
  assert(self);
   free(Indices);
  free(self);
}
void ximage_display(ximage self)
{
  assert(self);

  gdk_draw_rgb_image(Window->window,
		     Window->style->fg_gc[GTK_STATE_NORMAL],
		     0,0,
		     Largeur,Hauteur,
		     GDK_RGB_DITHER_MAX,
		     Indices,
		     Largeur*3);
}

void ximage_draw_point(ximage self,point p,unsigned char* couleur)
{
  int i,j;
  
  j = COORDX(*p)+1;
  i = COORDY(*p)+1;
  
  draw_point_priv(self,i,j,couleur);
}
void ximage_draw_point_gen(any_ptr obj,point p,unsigned char* couleur)
{
  ximage self = (ximage)obj;
  int i,j;
  
  j = COORDX(*p)+1;
  i = COORDY(*p)+1;

  if((i<0)||(i>=Hauteur))
    return;
  
  if((j<0)||(j>=Largeur))
    return;

  draw_point_priv(self,i,j,couleur);
}
void ximage_draw_segment(ximage self,output_segment seg,unsigned char* couleur)
{
  
  Point p;

  assert(self);
  assert(seg);
  
  output_segment_first(seg);
  do
    {
      output_segment_give_current_point(seg,&p);
      ximage_draw_point(self,&p,couleur);
    }
  while( output_segment_next(seg) );
  
  output_segment_give_last_point(seg,&p);
  ximage_draw_point(self,&p,couleur);
}
void ximage_draw_set_of_segments(ximage self,
				 list_gen set_of_segments,
				 unsigned char* couleur_segments)
{
  output_segment seg;
  
  assert(self);
  assert(set_of_segments);
 
  if (list_gen_give_cardinal(set_of_segments) == 0)
    return;
  
  list_gen_first(set_of_segments);

  do
    {
      seg = (output_segment)list_gen_give_current(set_of_segments);
      ximage_draw_segment(self,seg,couleur_segments);
    }
  while( list_gen_next(set_of_segments) );  

}

void ximage_draw_face(ximage self,
		      contour_set contours,
		      dart f,
		      unsigned char* couleur)
{
  dart_list b_list;
  output_segment seg;
  dart b;

  assert(self);
  assert(contours);
  assert(f>0);
  
  seg = FAIRE_output_segment();
  b_list = FAIRE_dart_list();
  contour_set_give_segments_of_face(contours,f,b_list);
  dart_list_first(b_list);
  do
    {
      b = dart_list_give_current(b_list);
      contour_set_give_segment(contours,b,seg);
      ximage_draw_segment(self,seg,couleur);
    }
  while(dart_list_next(b_list));
  
  DEFAIRE_output_segment(seg);
  DEFAIRE_dart_list(b_list);
}

void ximage_draw_set_of_faces(ximage self,
			      contour_set contours,
			      dart_list face_list,			      
			      unsigned char* couleur)
{ 
  dart f;

  assert(self);
  assert(contours);
  assert(face_list);
  
  if(dart_list_is_empty(face_list))
    return;

  dart_list_first(face_list);
  do
    {
      f = dart_list_give_current(face_list);
      ximage_draw_face(self,contours,f,couleur);
    }
  while(dart_list_next(face_list));
}
void ximage_erase_point(ximage self,point p)
{
  Point p_int;
  unsigned char* ind;
  int i,j;

  assert(self);
  assert(p);

  

  j = COORDX(p_int) = COORDX(*p)+1;
  i = COORDY(p_int) = COORDY(*p)+1;
  if(
     (i==0)||(j==0)||
     (i==Hauteur-1)||(j==Largeur-1)
     )
    ind = Border_color;
  else
    ind = display_give_point_value(Father,p);//_int);
  draw_point_priv(self,i,j,ind);
}

void ximage_erase_segment(ximage self,
			  output_segment seg)
{
  Point p;
  
  output_segment_first(seg);
  do
    {
      output_segment_give_current_point(seg,&p);
      ximage_erase_point(self,&p);
    }
  while (output_segment_next(seg));
  
  output_segment_give_last_point(seg,&p);
  ximage_erase_point(self,&p);
}
void ximage_erase_set_of_segments(ximage self,
				  list_gen seg_list)
{
  output_segment seg;

  assert(self);
  assert(seg_list);
  
  if (list_gen_is_empty(seg_list))
    return;
  
  list_gen_first(seg_list);
  do
    {
      seg = (output_segment)list_gen_give_current(seg_list);
      ximage_erase_segment(self,seg);
    }
  while (list_gen_next(seg_list));
}
void ximage_erase_face(ximage self,
		       contour_set contours,
		       dart f)

{
  dart_list b_list;
  output_segment seg;
  dart b;

  assert(self);
  assert(f>0);
  
  seg    = FAIRE_output_segment();
  b_list = FAIRE_dart_list();

  contour_set_give_segments_of_face(contours,f,b_list);

  assert(b_list);
  
  dart_list_first(b_list);
  do
    {
      b = dart_list_give_current(b_list);
      contour_set_give_segment(contours,b,seg);
      ximage_erase_segment(self,seg);
    }
  while (dart_list_next(b_list));
  
  DEFAIRE_output_segment(seg);
  DEFAIRE_dart_list(b_list);
}
void ximage_erase_set_of_faces(ximage self,
			       contour_set contours,
			       dart_list list_of_faces)
{
  dart f;

  assert(self);
  assert(list_of_faces);

  if (list_of_faces == NULL_DART_LIST)
    return;
  
  if (dart_list_is_empty(list_of_faces))
    return;

  dart_list_first(list_of_faces);
  do
    {
      f = dart_list_give_current(list_of_faces);
      ximage_erase_face(self,contours,f);
    }
  while (dart_list_next(list_of_faces));
}
void ximage_redraw_all(ximage self,
		       contour_set contours,
		       unsigned char* couleur_normal,
		       unsigned char* couleur_selection)
{
  int i,j;
  unsigned char* ind;
  Point p;
  
  assert(self);
  assert(contours);

  ind = Indices;

  COORDX(p) = -1;
  COORDY(p) = -1;
  
  for(i=0;i<Hauteur;i++)
    {
      for(j=0;j<Largeur;j++)
	{
	  if(contour_set_is_in_segment(contours,&p))
	    {
	      if( memcmp(ind,couleur_normal,3) != 0)
		draw_point_priv(self,i,j,couleur_normal);
	    }
	  else
	    {
	      if( memcmp(ind,couleur_normal,3) == 0 || memcmp(ind,couleur_selection,3) == 0 )
		ximage_erase_point(self,&p);
	    }
	  ind+=3;
	  COORDX(p)++;
	}
      COORDX(p) = -1;
      COORDY(p)++;
    }
}
  /*************************************/
 /* Effacement les chemins de l'image */
/*************************************/
void   ximage_erase_all(ximage self)
{
 init_table_of_indices_priv(self);
}
  /**********************/
 /*  Fonctions privees */
/**********************/
static void draw_point_priv(ximage self,
			    int i,
			    int j, 
			    unsigned char* couleur)
{
  int dep;
  int k;
  
  if( !( (i<Hauteur) && (i>=0) ) ||
     !( (j<Largeur) && (j>=0) ) )
    printf("Largeur, Hauteur = %d,%d ; i,j = %d,%d",Largeur,Hauteur,i,j);
  assert( (i<Hauteur) && (i>=0) );
  assert( (j<Largeur) && (j>=0) );
  
  dep = (i*Largeur+j)*3;
  memcpy((Indices+dep),couleur,3);
}


static void make_border_priv(ximage self)
{
  int i,j;
  
  for(j=0;j<Largeur;j++)
    {
      draw_point_priv(self,0,j,Border_color);
      draw_point_priv(self,Hauteur-1,j,Border_color);
    }
  for(i=0;i<Hauteur;i++)
    {
      draw_point_priv(self,i,0,Border_color);
      draw_point_priv(self,i,Largeur-1,Border_color);
    }
}
static void init_table_of_indices_priv(ximage self)
{
  Point p;
  int i,j;
  unsigned char * tab;
  unsigned char *tmp;

  tab = Indices+Largeur*3;

  for(i=1;i<Hauteur;i++)
    {
      COORDY(p) = i-1;
      tab+=3;

      for(j=1;j<Largeur;j++)
	{
	  COORDX(p) = j-1;
	  tmp = display_give_point_value(Father,&p);
	  memcpy(tab,tmp,3);
	  tab+=3;
	}
    }
}

display ximage_get_display( ximage self )
{
  return Father;
}



