/*  descartes
 *  Copyright (C) 2001 Luc Brun
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifndef CONTOUR_SET_H
#define CONTOUR_SET_H

#include <glib.h>

#include "type_obj.h"
#include "point.h"
#include "dart_list.h"
#include "move_type.h"
#include "seg_tmp.h"
#include "output_segment.h"
#include "point_set.h"
#include "list_gen.h"
#include "vector.h"
#include "tab_mark_dart.h"

#include "classe.h"

CLASSE(contour_set);

#define STORE 0
#define TEST  0 
#define NULL_CONTOUR_SET (contour_set)NULL

typedef void    (*PFapply_to_pixels)(any_ptr,point,int);
typedef bouleen (*PFsame_cluster)(point,int*,point,int*);



extern contour_set FAIRE_contour_set(guchar*,	/* data */
				     int,	/* dim */
				     int,	/* width */
				     int);	/* height */
extern void        DEFAIRE_contour_set(contour_set);


extern bouleen     contour_set_add_new_contour          (contour_set, seg_tmp);

extern void        contour_set_remove_arc               (contour_set,dart);
extern void        contour_set_remove_set_of_arc        (contour_set,
							 dart_list);
extern void        contour_set_remove_face              (contour_set,dart);
extern void        contour_set_remove_double_arc_of_face(contour_set,dart);

extern bouleen     contour_set_is_face_defined          (contour_set,dart);
/**************************************************/
/*** Fonctions typiquement de segmentation ***/
/**************************************************/

extern void        contour_set_set_same_cluster_criterion(contour_set,
							 PFsame_cluster);
/* Fonctions utilisees pour lire des renseignements sur une face */
extern void        contour_set_compute_face_params       (contour_set,dart);
extern void        contour_set_apply_function            (contour_set,
							  dart,
							  PFapply_to_pixels);

/* Fonctions utilisees pour segmenter la face donc, pour la modifier */
extern void        contour_set_give_included_faces(contour_set,
						   dart,
						   dart_list);
extern dart        contour_set_give_including_face(contour_set,
						   dart);
extern void        contour_set_mark_set_of_face   (contour_set,
						   dart_list,
						   tab_mark,
						   void
						   (*mark_point)(tab_mark,
								 point)
						   );
extern void        contour_set_unmark_set_of_face (contour_set,
						   dart_list,
						   tab_mark);


extern void        contour_set_give_begin_points  (contour_set,
						   dart,
						   point_set);
/* Indique si le point et une fin de ligne et le demarque */
extern bouleen     contour_set_end_of_line             (contour_set,point);

extern void        contour_set_mark_end_points_of_face (contour_set,dart);
extern bouleen     contour_set_direction_belong_segment(contour_set,
							point,
							move_type);
extern bouleen     contour_set_direction_is_allowed    (contour_set,
							point,
							move_type);
extern bouleen    contour_set_is_in_segment            (contour_set,point);
extern bouleen    contour_set_must_be_separated        (contour_set,
							point,
							move_type);
extern void       contour_set_give_finite_faces        (contour_set,
							dart,
							dart_list);
extern dart_list  contour_set_give_all_finite_faces    (contour_set,
							dart);
extern void       contour_set_complete_face_adjacence  (contour_set,
							dart_list,
							dart_list);
extern void       contour_set_mark_adjacent_faces      (contour_set,
							dart,
							tab_mark_dart);
extern seg_tmp    contour_set_copy_closed_curve        (contour_set,seg_tmp);
/* Fonctions servant uniquement a proteger contour_set */

extern double     contour_set_distance_in_image  (contour_set,point,move_type);
extern int        contour_set_give_image_hauteur (contour_set);
extern int        contour_set_give_image_largeur (contour_set);
extern int        contour_set_give_image_dim     (contour_set);
extern double     contour_set_give_image_variance(contour_set);
   /******************************************************************/
  /********* Fonctions utilisees par l'interface ********************/
 /*          Ou des modules de Meta segmentation                   */
/******************************************************************/
extern void        contour_set_save             (contour_set,char*);
extern dart        contour_set_assoc_point_face (contour_set, point);
extern void        contour_set_assoc_face_point (contour_set,dart,point);
extern dart        contour_set_assoc_point_arc  (contour_set, point);
extern dart        contour_set_assoc_face_seg   (contour_set,dart);
extern dart        contour_set_assoc_seg_face   (contour_set,dart);
extern void        contour_set_give_segment     (contour_set,
						 dart,
						 output_segment);
extern void        contour_set_extend_seg       (contour_set,dart);
extern bouleen     contour_set_is_infinite      (contour_set, dart);

extern void     contour_set_memorize_modified_segments(contour_set);
extern void     contour_set_give_modified_segments    (contour_set,list_gen);

extern void     contour_set_memorize_inserted_faces   (contour_set);
extern void     contour_set_give_inserted_faces       (contour_set,dart_list);
extern void     contour_set_give_segments_of_face     (contour_set,
						       dart,
						       dart_list);

  /*****************************************************/
 /* Fonctions de segmentation ou de meta segmentation */
/*****************************************************/
extern double    contour_set_give_face_area         (contour_set,dart);
extern void      contour_set_give_face_mean         (contour_set,dart,vector);
extern double    contour_set_give_face_variance     (contour_set,dart);
extern double    contour_set_give_face_squared_error(contour_set,dart);
extern void      contour_set_give_face_moment1      (contour_set,dart,vector);
extern void      contour_set_give_face_moments      (contour_set,
						     dart,
						     double*,
						     vector,
						     vector);

extern bouleen   contour_set_face_has_neighbour (contour_set,dart,dart);
extern dart      contour_set_give_face_neighbour(contour_set,dart,dart);
extern dart_list contour_set_give_image_faces   (contour_set);

extern int       contour_set_give_segment_lenght  (contour_set,dart);
extern double    contour_set_give_segment_gradiant(contour_set,dart);      
#endif /* CONTOUR_SET_H */






