/*  descartes
 *  Copyright (C) 2001 Luc Brun
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <assert.h>
#include <values.h>
#include "objet.h"
#include "cluster_set.h"
#include "cag.h"

#define Nb_clusters SELF(nb_clusters)
#define Size        SELF(size)
#define Exist       SELF(exist)
#define Links       SELF(links)

static void make_links_priv(cag,cluster_set);

struct cag
{
  int nb_clusters;
  int size;
  bouleen *exist;
  double* links;
};

cag FAIRE_cag(cluster_set clusters)
{

  INSTANCIER(cag);
  
  Nb_clusters = cluster_set_give_cardinal(clusters);
  Size  = Nb_clusters*(Nb_clusters-1)/2;
  Exist = (bouleen*)malloc(Nb_clusters*sizeof(bouleen));  
  Links = (double*) malloc(Size*sizeof(double));
  
  make_links_priv(self,clusters);
  return self;
}
void DEFAIRE_cag(cag self)
{
  free(Links);
  free(Exist);
  free(self);
}

void cag_find_closest_node(cag self,int* clus1,int* clus2)
{
  double *ptr;
  double min_se;
  int i;
  int j;

  min_se  = DBL_MAX;
  ptr = Links;
  for(i=1;i<Nb_clusters;i++)
    {
      if(!Exist[i])
	{
	  ptr += i;
	  continue;
	}
      for(j=0;j<i;j++)
	{
	  if((*ptr<min_se)&&Exist[j])
	    {
	      min_se = *ptr;
	      /* Rem On a i >= j => clus1 >= clus2*/
	      *clus1 = i;
	      *clus2 = j;
	    }
	  ptr++;
	}
    }
  assert(min_se != DBL_MAX);
/*  printf("min = %lf",min_se);*/
}
void cag_give_min_color_dist(cag self,
			     cluster_set clusters,
			     int* clus1,
			     int* clus2)
{
  double dist,min_dist;
  int i;
  int j;

  min_dist  = DBL_MAX;
  for(i=1;i<Nb_clusters;i++)
    {
      if(!Exist[i])
	continue;

      for(j=0;j<i;j++)
	{
	  if(!Exist[j])
	    continue;

	  dist = cluster_set_get_color_dist(clusters,i,j);
	  if(dist < min_dist)
	    {	 
	      min_dist  = dist;
	      /* Rem On a i >= j => clus1 >= clus2*/
	      *clus1 = i;
	      *clus2 = j;
	    }
	}
    }
  assert(min_dist != DBL_MAX);
}

/* On suppose que clus2 < clus1 */
void cag_merge_node(cag self,cluster_set clusters,int clus1,int clus2)
{
  double *ptr1,*ptr2;
  int ptr_incr;
  int i;
  /* On invalide la ligne clus2 */
  Exist[clus2] = faux;
  
  ptr1 = Links+ clus1*(clus1-1)/2;
  ptr2 = Links+ clus2*(clus2-1)/2;
  *(ptr1+clus2) = DBL_MAX;

/* On corrige la ligne clus1 */

  for(i=0;i<clus2;i++)
    {
      if((*ptr1 != DBL_MAX)||(*ptr2 != DBL_MAX))
	*ptr1 = cluster_set_get_dist_mean(clusters,clus1,i);
      ptr1++;
      ptr2++;      
    }
  /*On corrige le reste de la ligne clus1 de clus2+1 a clus1-1 en */
  /*scannant en parallele la ligne clus1 et la colonne clus2 de */
  /*clus2+1 a clus1-1*/
  ptr2 = Links +clus2*(clus2+1)/2+clus2;	/* (clus2+1,clus2) */
  ptr1++;					/* (clus1,clus2+1) */
  ptr_incr = clus2+1;
  for(i=clus2+1;i<clus1;i++)
    {
      if((*ptr1 != DBL_MAX)||(*ptr2 != DBL_MAX))
	*ptr1 = cluster_set_get_dist_mean(clusters,clus1,i);
      ptr1++;
      ptr2 += ptr_incr;
      ptr_incr++;
    }
  /* On corrige la collone clus1*/
  ptr1 = Links+clus1*(clus1+1)/2+clus1; /* (clus1+1,clus1) */
  ptr_incr = clus1+1;
  ptr2 = Links+clus1*(clus1+1)/2+clus2; /* (clus1+1,clus2)*/
  for(i=clus1+1;i<Nb_clusters;i++)
    {
      if((*ptr1 != DBL_MAX)||(*ptr2 != DBL_MAX))
	*ptr1 = cluster_set_get_dist_mean(clusters,clus1,i);
      ptr1 += ptr_incr;
      ptr2 += ptr_incr;
      ptr_incr++;
    }
}
  /***************************************************/
 /* Fonctions prives */
/**************************************************/
static void make_links_priv(cag self,cluster_set clusters)
{
  double* ptr;
  int i,j;

  Exist[0] = vrai;
  ptr = Links;
  for(i=1;i<Nb_clusters;i++)
    {
      Exist[i] = vrai;
      for(j=0;j<i;j++)
	{
	  *ptr = cluster_set_get_dist_mean(clusters,i,j);
	  ptr++;
	}
    }
}





