/*  descartes
 *  Copyright (C) 2001 Luc Brun
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include "type_obj.h"
#include "contour_set.h"
#include "tab_move.h"
#include "output_segment.h"
#include "dart_list.h"
#include "move_type.h"
#include "objet.h"
#include "chemin_discret.h"


/* TYPE CHEMIN DISCRET : code de Freeman*/
#define Code     SELF(code)
#define Nb_codes SELF(nb_codes)

struct Chemin_Discret{
  tab_move code;
  int nb_codes;
};

static void add_segment_priv (Chemin_Discret, output_segment, dart);
  

/* CREATION -  LIBERATION */

Chemin_Discret FAIRE_Chemin_Discret(){
  INSTANCIER(Chemin_Discret);
  Code = FAIRE_tab_move();
  Nb_codes = 0;
  return self;
}

void DEFAIRE_Chemin_Discret(Chemin_Discret self){
  DEFAIRE_tab_move(Code);
  free(self);
}

/* CONSTRUCTION D'UN CHEMIN DISCRET A PARTIR D'UNE FACE */

void Chemin_Discret_compute_from_face(Chemin_Discret self, 
				      contour_set contours, 
				      dart f){
  dart_list seg_list;
  dart brin;
  output_segment seg;

  seg_list = FAIRE_dart_list();
  seg = FAIRE_output_segment();

  contour_set_give_segments_of_face(contours, f, seg_list);
  dart_list_first(seg_list);
  do{ 
    brin = dart_list_give_current(seg_list);
    contour_set_give_segment(contours, brin, seg);
    add_segment_priv(self, seg, brin);
    output_segment_reinitialize(seg);
  }
  while(dart_list_next(seg_list));
  DEFAIRE_output_segment(seg);
  DEFAIRE_dart_list(seg_list);
}


/* CONSTRUCTION D'UN CHEMIN DISCRET A PARTIR D'UN SEGMENT */

void Chemin_Discret_compute_from_segment(Chemin_Discret self, 
					 contour_set contours, 
					 dart brin){
  output_segment seg;

  seg = FAIRE_output_segment();

  contour_set_give_segment(contours, brin, seg);
  add_segment_priv(self, seg, brin);
  
  DEFAIRE_output_segment(seg);
}


/* NOMBRE DE DEPLACEMENTS LE LONG DU CHEMIN DISCRET */

int Chemin_Discret_nb_codes(Chemin_Discret self){
  return Nb_codes;
}


/* ELT DU CODE DISCRET*/
move_type Chemin_Discret_code_decalage_circulaire(Chemin_Discret self, 
						  int ind_code, 
						  int decal)
{
  
  if(decal > 0)
    return tab_move_give_value(Code, (ind_code+decal- 1)%Nb_codes);
  
  if(ind_code+decal >=0)
    return tab_move_give_value(Code, ind_code+decal);

  return tab_move_give_value(Code, (ind_code + decal)%Nb_codes + Nb_codes);
}

/* Donne la valeur du code passe en second argument */
move_type Chemin_Discret_code(Chemin_Discret self, int ind_code)
{
  return tab_move_give_value(Code, ind_code);
}


/* VARIATIONS EN X ET Y LE LONG DU CHEMIN DISCRET */

int Chemin_Discret_delta_x(Chemin_Discret self, int ind_code)
{
  return dep_move_x[tab_move_give_value(Code, ind_code)];
}

int Chemin_Discret_delta_y(Chemin_Discret self, int ind_code)
{
  return dep_move_y[tab_move_give_value(Code, ind_code)];
}



static void add_segment_priv(Chemin_Discret self, 
				  output_segment seg,
				  dart brin){
  int i;
  move_type move;
  void (*first)(output_segment);
  bouleen (*next)(output_segment);

  if (brin < 0){
    first = output_segment_last;
    next = output_segment_previous;
  }
  else{
    first = output_segment_first;
    next = output_segment_next;
  }

  first(seg);
  i = tab_move_give_cardinal(Code);
  do{
    /* remplissage du code */
    move = output_segment_give_current_move(seg);
    if(brin < 0)
      move = REVERSE_SENSE(move);
    tab_move_put_value(Code, i, move);
    Nb_codes++;
    i++;
  }
  while(next(seg));
}













