/*  descartes
 *  Copyright (C) 2001 Luc Brun
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <math.h>
#include <glib.h>
#include "types.h"
#include "type_obj.h"
#include "conversion.h"
#include "definitions.h"
#include "macros.h"
#include "image.h"
#include "matrice.h"

/****************************************************************************/
/* fichier de conversion (R,G,B)--> Espace C1C2C3 */
/* Pour rajouter un espace Il faut : */
/* 1) Rajouter 3 fonctions de type PFconversion RGB_to_Ci */
/* 2) Rajouter 3 fonctions de type PFconversion C1C2C3_to_R, C1C2C3_to_G et C1C2C3_to_B */
/* 3) Toute fonction suplementaire est a ecrire en fin de fichier */
/* 4) Rajouter ces fonctions dans changer_espace et revenir espace*/
/* 5) Aller dans choix.h rajouter le nouvel espace dans les menus sans */
/*    oublier d'incrementer le nombre d'espaces disponibles dans le */
/*    tableau contenant le cardinal de chaque menus*/
/* 6) Ne pas oublier de declarer les fonctions rajoutees en tete du */
/*    fichier */
/****************************************************************************/


#define UN_TIER 1.0/3.0
#define ILLUMINANT_C

#ifdef ILLUMINANT_C
/*--------------------------------*/
/* Illuminant C */
/*--------------------------------*/
#define Xr 0.166
#define Yr 0.06
#define Zr 0.0

#define Xg 0.174
#define Yg 0.587
#define Zg 0.066

#define Xb 0.093
#define Yb 0.005
#define Zb 0.46

#define Xw 98.04
#define Yw 100.0
#define Zw 118.10
#define Uw 0.2009
#define Vw 0.4609

#define Rx 6.686
#define Ry -1.832
#define Rz -1.332

#define Gx -0.684
#define Gy 1.893
#define Gz 0.118

#define Bx 0.098
#define By -0.272
#define Bz 2.157
#endif

#ifdef ILLUMINANT_E
/*--------------------------------*/
/* Illuminant E                   */
/*--------------------------------*/
#define Xr 2.769
#define Yr 1.0
#define Zr 0.0

#define Xg 1.7518
#define Yg 4.5907
#define Zg 0.0565

#define Xb 1.133
#define Yb 0.0601
#define Zb 5.5943

#define Xw 100.0
#define Yw 100.0
#define Zw 100.0
#define Uw 0.2105
#define Vw 0.4737
#endif

#ifdef ILLUMINANT_D65
/*--------------------------------*/
/* Illuminant D65                 */
/*--------------------------------*/
#define Xr 0.478
#define Yr 0.263
#define Zr 0.02

#define Xg 0.299 
#define Yg 0.655 
#define Zg 0.16  

#define Xb 0.175
#define Yb 0.081 
#define Zb 0.908 

#define Xw 95.02
#define Yw 100.0
#define Zw 108.81
#define Uw 0.1978
#define Vw 0.4683


#endif

#ifdef ILLUMINANT_A
/*--------------------------------*/
/* Illuminant A                  */
/*--------------------------------*/
#define Xr 0.892
#define Yr 0.322
#define Zr 0.00

#define Xg 0.33  
#define Yg 0.863 
#define Zg 0.011

#define Xb 0.083
#define Yb 0.004  
#define Zb 0.409 

#define Xw 109.83
#define Yw 100.0
#define Zw 35.55 
#define Uw 0.2560
#define Vw 0.5243
#endif

/*--------------------------------*/
/* variables et outils pour GLHS  */
/*--------------------------------*/
#define Wmin_t 1.0/3.0
#define Wmid_t 1.0/3.0
#define Wmax_t 1.0/3.0

#define Wmin_h 0.0
#define Wmid_h 0.0
#define Wmax_h 1.0

#define Wmin_d 0.5
#define Wmid_d 0.0
#define Wmax_d 0.5

#define M_GLHS 255

#define Max(a,b) ((a)>=(b)?(a):(b))
#define Min(a,b) ((a)<=(b)?(a):(b))


static int private_RGB_to_I1(int,int,int);
static int private_RGB_to_I2(int,int,int);
static int private_RGB_to_I3(int,int,int);

static int private_I1I2I3_to_R(int,int,int);
static int private_I1I2I3_to_G(int,int,int);
static int private_I1I2I3_to_B(int,int,int);

static int private_RGB_to_H2(int,int,int);
static int private_RGB_to_H3(int,int,int);

static int private_H1H2H3_to_R(int,int,int);
static int private_H1H2H3_to_G(int,int,int);
static int private_H1H2H3_to_B(int,int,int);

static int private_RGB_to_L(int,int,int);
static int private_RGB_to_u(int,int,int);
static int private_RGB_to_v(int,int,int);

static int private_Luv_to_R(int,int,int);
static int private_Luv_to_G(int,int,int);
static int private_Luv_to_B(int,int,int);

static int private_RGB_to_Y(int,int,int);
static int private_RGB_to_I(int,int,int);
static int private_RGB_to_Q(int,int,int);

static int private_YIQ_to_R(int,int,int);
static int private_YIQ_to_G(int,int,int);
static int private_YIQ_to_B(int,int,int);

static int private_RGB_to_R(int,int,int);
static int private_RGB_to_G(int,int,int);
static int private_RGB_to_B(int,int,int);

static int private_RGB_to_K1(int,int,int);
static int private_RGB_to_K2(int,int,int);
static int private_RGB_to_K3(int,int,int);

static int private_K1K2K3_to_R(int,int,int);
static int private_K1K2K3_to_G(int,int,int);
static int private_K1K2K3_to_B(int,int,int);

static int private_RGB_to_V1(int,int,int);
static int private_RGB_to_V2(int,int,int);
static int private_RGB_to_V3(int,int,int);

static int private_V1V2V3_to_R(int,int,int);
static int private_V1V2V3_to_G(int,int,int);
static int private_V1V2V3_to_B(int,int,int);

static int private_RGB_to_T1(int,int,int);
static int private_RGB_to_T2(int,int,int);
static int private_RGB_to_T3(int,int,int);

static int private_T1T2T3_to_R(int,int,int);
static int private_T1T2T3_to_G(int,int,int);
static int private_T1T2T3_to_B(int,int,int);

static int private_RGB_to_a(int,int,int);
static int private_RGB_to_b(int,int,int);

static int private_Lab_to_R(int,int,int);
static int private_Lab_to_G(int,int,int);
static int private_Lab_to_B(int,int,int);

static int private_RGB_to_Ht(int,int,int);
static int private_RGB_to_St(int,int,int);
static int private_RGB_to_Lt(int,int,int);

static int private_HSL_to_R(int,int,int);
static int private_HSL_to_G(int,int,int);
static int private_HSL_to_B(int,int,int);

static int private_RGB_to_Hh(int,int,int);
static int private_RGB_to_Sh(int,int,int);
static int private_RGB_to_Vh(int,int,int);

static int private_HSV_to_R(int,int,int);
static int private_HSV_to_G(int,int,int);
static int private_HSV_to_B(int,int,int);

static int private_RGB_to_Hd(int,int,int);
static int private_RGB_to_Ld(int,int,int);
static int private_RGB_to_Sd(int,int,int);

static int private_HLS_to_R(int,int,int);
static int private_HLS_to_G(int,int,int);
static int private_HLS_to_B(int,int,int);


/* Fonctions auxilieres des fonctions de changement d'espace */
static void private_RGB_to_XYZ(int,int,int,float*,float*,float*);
static void private_RGB_to_LUV(int,int,int,double*,double*,double*);
static float private_RGB_to_Xyz(int,int,int);
static float private_RGB_to_xYz(int,int,int);
static float private_RGB_to_xyZ(int,int,int);
static void private_Luv_to_XYZ(int,int,int,float*,float*,float*);
static void private_Lab_to_XYZ(int,int,int,float*,float*,float*);
static int private_XYZ_to_R(float,float,float);
static int private_XYZ_to_G(float,float,float);
static int private_XYZ_to_B(float,float,float);
static float private_RGB_to_Luv(int,int,int);
static void private_val_vect_propre(mat_covariance,vecteur,mat_passage,matrice);
static void private_calcul_mat_cov(image,mat_covariance);
static void private_RGB_to_vecteur_propre(mat_passage,vecteur,vecteur,int,int);
static void private_vecteur_propre_to_RGB(mat_passage,vecteur,vecteur,int,int);
static void private_Calcul_vecteur_propre(image);


static void private_RGB_to_GLHS(int,int,int,float,float,float,int*,int*,int*);
static void private_GLHS_to_RGB(int,int,int,float,float,float,int*,int*,int*);

  /********************************************************/
 /*      Debut du code (ouf !)                           */
/********************************************************/


/****************************************************************************/
/* Fonctions publiques */
/****************************************************************************/
PFconversion* changer_espace(int choix)
{
    static PFconversion conversion[][3]= 
	    {
		{private_RGB_to_R,  private_RGB_to_G,  private_RGB_to_B},
		{private_RGB_to_I1, private_RGB_to_I2, private_RGB_to_I3},
		{private_RGB_to_L,  private_RGB_to_u,  private_RGB_to_v},
		{private_RGB_to_L,  private_RGB_to_a,  private_RGB_to_b},
		{private_RGB_to_Y,  private_RGB_to_I,  private_RGB_to_Q},
		{private_RGB_to_Ht, private_RGB_to_St, private_RGB_to_Lt},
		{private_RGB_to_Hh, private_RGB_to_Sh, private_RGB_to_Vh},
		{private_RGB_to_Hd, private_RGB_to_Ld, private_RGB_to_Sd},
		{private_RGB_to_I1, private_RGB_to_H2, private_RGB_to_H3},
		{private_RGB_to_V1, private_RGB_to_V2, private_RGB_to_V3},
		{private_RGB_to_K1, private_RGB_to_K2, private_RGB_to_K3},
		{private_RGB_to_T1, private_RGB_to_T2, private_RGB_to_T3},
	    };
    return(conversion[choix]);
}
PFconversion* revenir_espace(int choix)
{
    static PFconversion conversion_inverse[][3]= 
	    {
		{private_RGB_to_R,   private_RGB_to_G,   private_RGB_to_B},
		{private_I1I2I3_to_R,private_I1I2I3_to_G,private_I1I2I3_to_B},
		{private_Luv_to_R,   private_Luv_to_G,   private_Luv_to_B},
		{private_Lab_to_R,   private_Lab_to_G,   private_Lab_to_B},
		{private_YIQ_to_R,   private_YIQ_to_G,   private_YIQ_to_B},
		{private_HSL_to_R,   private_HSL_to_G,   private_HSL_to_B},
		{private_HSV_to_R,   private_HSV_to_G,   private_HSV_to_B},
		{private_HLS_to_R,   private_HLS_to_G,   private_HLS_to_B},
		{private_H1H2H3_to_R,private_H1H2H3_to_G,private_H1H2H3_to_B},
		{private_V1V2V3_to_R,private_V1V2V3_to_G,private_V1V2V3_to_B},
		{private_K1K2K3_to_R,private_K1K2K3_to_G,private_K1K2K3_to_B},
		{private_T1T2T3_to_R,private_T1T2T3_to_G,private_T1T2T3_to_B},
	    };
    return(conversion_inverse[choix]);
}
/***************************************************************************/
             /* Fonctions prives, de changement de base */
/***************************************************************************/

/****************Espace I1 I2 I3 *******************************************/
static int private_RGB_to_I1(int R,int G,int B)
{
    static float inv_quotient = 1.0/3.0;
    int retour  = (int)((R+G+B)*inv_quotient+0.5);
    
    return (retour);
}
static int private_RGB_to_I2(int R,int G,int B)
{
    return(R-B);
}
static int private_RGB_to_I3(int R,int G,int B)
{
  float I3 = G-(R+B)*0.5;
  int retour;

  retour = FLOAT_TO_INT(I3);

  return (retour);
}
static int private_I1I2I3_to_R(int I1 ,int I2 ,int I3 )
{
    static float un_d_3 = 1.0/3.0;
    float R;
    int retour;

    R = I1+0.5*I2-un_d_3*I3+0.5;
    retour = (int)MAX(R,0);
    retour  = MIN(retour,255);
    
    return (retour);
}
static int private_I1I2I3_to_G(int I1 ,int I2 ,int I3 )
{
    static float deux_d_3 = 2.0/3.0;
    float G;
    int retour;

    G = I1+deux_d_3*I3+0.5;
    retour = (int)MAX(G,0);
    retour  = MIN(retour,255);
    
    return (retour);
}
static int private_I1I2I3_to_B(int I1 ,int I2 ,int I3 )
{
    static float un_d_3 = 1.0/3.0;
    float B;
    int retour;
    
    B = I1-0.5*I2-un_d_3*I3+0.5;
    retour = (int)MAX (B,0);
    retour  = MIN(retour,255);
    
    return (retour);
}
/****************************************************************************/

/****************Espace H1 H2 H3 *******************************************/
static int private_RGB_to_H2(int R,int G,int B)
{
  return(R-G);
}
static int private_RGB_to_H3(int R,int G,int B)
{
  float H3;
  int retour;
  
  H3 = (0.5*(R+G)-B);
  retour = FLOAT_TO_INT(H3);

  return(retour);
}
static int private_H1H2H3_to_R(int H1,int H2,int H3)
{
	static float un_tier = 1.0/3.0;
	float R;
	int retour;
	
	R = H1+0.5*H2+un_tier*H3+0.5;
	retour = (int)MAX(R,0);
	retour  = MIN(retour,255);
    
	return (retour);
}	
static int private_H1H2H3_to_G(int H1,int H2,int H3)
{
	static float un_tier = 1.0/3.0;
	float G;
	int retour;

	G = H1-0.5*H2+un_tier*H3+0.5;
	retour = (int)MAX(G,0);
	retour  = MIN(retour,255);
    
	return (retour);
}	
static int private_H1H2H3_to_B(int H1,int H2,int H3)
{
	static float deux_tier = 2.0/3.0;
	float B;
	int retour;
	
	B =  H1-deux_tier*H3+0.5;
	retour = (int)MAX(B,0);
	retour  = MIN(retour,255);
    
	return (retour);
}	
/****************************************************************************/

/****************Espace L u v  *******************************************/

static int private_RGB_to_L(int R,int G,int B)
{
    int retour;

    retour = (int)(private_RGB_to_Luv(R,G,B)+0.5);

    return(retour);
}
static int private_RGB_to_u(int R,int G,int B)
{
    float X,Y,Z,up,u;
    int retour;
    float L = private_RGB_to_Luv(R,G,B);
    private_RGB_to_XYZ(R,G,B,&X,&Y,&Z);

    if((X==0.0)&&(Y==0.0)&&(Z==0.0))
	    retour = 0;
    else
	    {
		up = 4*X/(X+15*Y+3*Z);
		u = 13*L*(up-Uw);
		retour = FLOAT_TO_INT(u);
	    }
    return(retour);
}
static int private_RGB_to_v(int R,int G,int B)
{
    float X,Y,Z,vp,v;
    int retour;
    float L = private_RGB_to_L(R,G,B);

    private_RGB_to_XYZ(R,G,B,&X,&Y,&Z);

    if((X==0.0)&&(Y==0.0)&&(Z==0.0))
	    retour = 0;
    else
	    {
		vp = 9*Y/(X+15*Y+3*Z);
		v = 13*L*(vp-Vw);
		retour = FLOAT_TO_INT(v);
	    }
    return(retour);
}   
static int private_Luv_to_R(int L,int u,int v)
{
    int retour;
    float X,Y,Z;

    private_Luv_to_XYZ(L,u,v,&X,&Y,&Z);
    retour = private_XYZ_to_R(X,Y,Z);

    return(retour);
}
static int private_Luv_to_G(int L,int u,int v)
{
    int retour;
    float X,Y,Z;

    private_Luv_to_XYZ(L,u,v,&X,&Y,&Z);
    retour = private_XYZ_to_G(X,Y,Z);

    return(retour);
}
static int private_Luv_to_B(int L,int u,int v)
{
    int retour;
    float X,Y,Z;

    private_Luv_to_XYZ(L,u,v,&X,&Y,&Z);
    retour = private_XYZ_to_B(X,Y,Z);

    return(retour);
}
/****************************************************************************/

/****************Espace Y I Q  *******************************************/
static int private_RGB_to_Y(int R,int G,int B)
{
    static float tab_Y[3] = {0.299,0.587,0.133};
    float Y;

    Y = tab_Y[0]*R+tab_Y[1]*G+tab_Y[2]*B+0.5;

    return(Y);
}
static int private_RGB_to_I(int R,int G,int B)
{
    static float tab_I[3] = {0.596,-0.274,-0.322};
    float I;

    I = tab_I[0]*R+tab_I[1]*G+tab_I[2]*B;

    I= (I<0?I-0.5:I+0.5);
    
    return(I);
}
static int private_RGB_to_Q(int R,int G,int B)
{
    static float tab_Q[3] = {0.211,-0.253,0.312};
    float Q;

    Q = tab_Q[0]*R+tab_Q[1]*G+tab_Q[2]*B;

    Q= (Q<0?Q-0.5:Q+0.5);
    
    return(Q);
}
static int private_YIQ_to_R(int Y,int I,int Q)
{
    static float tab_R[3]= {0.797,1.012,0.753};
    float R;

    R = tab_R[0]*Y+tab_R[1]*I+tab_R[2]*Q+0.5;

    
    return(R);
}
static int private_YIQ_to_G(int Y,int I,int Q)
{
    static float tab_G[3]= {1.212,-0.330,-0.784};
    float G;

    G = tab_G[0]*Y+tab_G[1]*I+tab_G[2]*Q+0.5;

    return(G);
}
static int private_YIQ_to_B(int Y,int I,int Q)
{
    static float tab_B[3]= {0.444,-0.952,2.060};
    float B;

    B = tab_B[0]*Y+tab_B[1]*I+tab_B[2]*Q+0.5;

    return(B);
}
  /**********************************************************************/
 /*************************** Espace R G B *****************************/
/**********************************************************************/
static int private_RGB_to_R(int R,int G,int B)
{
    return(R);
}
static int private_RGB_to_G(int R,int G,int B)
{
    return(G);
}
static int private_RGB_to_B(int R,int G,int B)
{
    return(B);
}

  /***************************************************************************/
 /****************************Espace  K1K2K3 ********************************/
/***************************************************************************/

static int private_RGB_to_K1(int R,int G,int B)
{
  int K1;
  
  K1 = (R+G)<<1;
  
  return(K1);

}
static int private_RGB_to_K2(int R,int G,int B)
{
  int K2;
  
  K2 = (R<<1)-(G<<1);
  
  return(K2);
}
static int private_RGB_to_K3(int R,int G,int B)
{
  int K3;
    
  K3 = (B<<1)-(R+G);
  
  return(K3);
}
static int private_K1K2K3_to_R(int K1,int K2,int K3)
{
  float R;
  int retour;
  
  R = (K1+K2)*0.25+0.5;
  
  retour = (int)MAX(R,0);
  retour  = MIN(retour,255);    
  
  return(retour);
}
static int private_K1K2K3_to_G(int K1,int K2,int K3)
{
    float G;
    int retour;

    G = 0.25*(K1-K2)+0.5;

    retour = (int)MAX(G,0);
    retour  = MIN(retour,255);    
    
    return(retour);
}
static int private_K1K2K3_to_B(int K1,int K2,int K3)
{
    float B;
    int retour;

    B = 0.25*K1 + K3*0.5 + 0.5;

    retour = (int)MAX(B,0);
    retour  = MIN(retour,255);    
    
    return(retour);
}

/****************************************************************************/
/**************** Espace  des vecteurs propres  *****************************/
/****************************************************************************/
static int private_RGB_to_V1(int R,int G,int B)
{
    static bouleen retenu =faux;
    static vecteur V1_to;
    static double val_propre1;
    vecteur val;
    int ligne =0;
    matrice P;
    float V1;
    int retour;

    if(!retenu)
	    {
		private_RGB_to_vecteur_propre(P,V1_to,val,ligne,RETOURNER);
		retenu = vrai;
		val_propre1 = 4.0*val[ligne];
	    }

    V1 = (V1_to[0]*R+V1_to[1]*G+V1_to[2]*B);

    retour = FLOAT_TO_INT(V1);
    return (retour);
}
static int private_RGB_to_V2(int R,int G,int B)
{
    static bouleen retenu =faux;
    static vecteur V2_to;
    static double val_propre2;
    vecteur val;
    int ligne =1;
    matrice P;
    float V2;
    int retour;

    if(!retenu)
	    {
		private_RGB_to_vecteur_propre(P,V2_to,val,ligne,RETOURNER);
		retenu = vrai;
		val_propre2 = 4.0*val[ligne];
	    }

    V2 = (V2_to[0]*R + V2_to[1]*G + V2_to[2]*B);

    retour = FLOAT_TO_INT(V2);
    return (retour);
}
static int private_RGB_to_V3(int R,int G,int B)
{
    static bouleen retenu =faux;
    static vecteur V3_to;
    static double val_propre3;
    vecteur val;
    int ligne =2;
    matrice P;
    float V3;
    int retour;

    if(!retenu)
	    {
		private_RGB_to_vecteur_propre(P,V3_to,val,ligne,RETOURNER);
		retenu = vrai;
		val_propre3 = 4.0*val[ligne];
	    }

    V3 = (V3_to[0]*R + V3_to[1]*G + V3_to[2]*B);

    retour = FLOAT_TO_INT(V3);

    return (retour);
}

static int private_V1V2V3_to_R(int V1,int V2,int V3)
{
    static bouleen retenu =faux;
    static vecteur R_to;
    static vecteur val_propre;
    int ligne =0;
    matrice P;
    float R;
    int retour;

    
    if(!retenu)
	    {
		private_vecteur_propre_to_RGB(P,R_to,val_propre,ligne,RETOURNER);
		retenu = vrai;
	    }

    R= (R_to[0]*V1 + R_to[1]*V2 + R_to[2]*V3) + 0.5;

    retour = (int)MAX(R,0);
    retour  = MIN(retour,255);    

    return (retour);
}
static int private_V1V2V3_to_G(int V1,int V2,int V3)
{
    static bouleen retenu =faux;
    static vecteur G_to;
    static vecteur val_propre;
    int ligne =1;
    matrice P;
    float G;
    int retour;

    if(!retenu)
	    {
		private_vecteur_propre_to_RGB(P,G_to,val_propre,ligne,RETOURNER);
		retenu = vrai;
	    }

    G= (G_to[0]*V1 + G_to[1]*V2 + G_to[2]*V3) + 0.5;

    retour = (int)MAX(G,0);
    retour  = MIN(retour,255);    

    return (retour);
}
static int private_V1V2V3_to_B(int V1,int V2,int V3)
{
    static bouleen retenu =faux;
    static vecteur B_to;
    static vecteur val_propre;
    int ligne =2;
    matrice P;
    float B;
    int retour;

    if(!retenu)
	    {
		private_vecteur_propre_to_RGB(P,B_to,val_propre,ligne,RETOURNER);
		retenu = vrai;
	    }

    B= (B_to[0]*V1 +B_to[1]*V2 + B_to[2]*V3) + 0.5;

    retour = (int)MAX(B,0);
    retour  = MIN(retour,255);    

    return (retour);
}    
/****************************************************************************/
/***********************Espace Test ******************************************/
/****************************************************************************/

static int private_RGB_to_T1(int R,int G,int B)
{
    int T1;
    double L,u,v;
    double  T1_double;
    static double tab_T1[] = {-0.51,0.65,-0.45};
    private_RGB_to_LUV(R,G,B,&L,&u,&v);


    
    T1_double = tab_T1[0]*L+tab_T1[1]*u+tab_T1[2]*v;

    T1 = FLOAT_TO_INT(T1_double);

    return(T1);

}
static int private_RGB_to_T2(int R,int G,int B)
{
    int T2;
    double L,u,v;
    double T2_double;
    static double tab_T2[] = {0.15816,0.58235,0.65837};

    private_RGB_to_LUV(R,G,B,&L,&u,&v);

    
    T2_double = tab_T2[0]*L+tab_T2[1]*u+tab_T2[2]*v;

    T2 = FLOAT_TO_INT(T2_double);
    
    return(T2);
}
static int private_RGB_to_T3(int R,int G,int B)
{
    int T3;
    double L,u,v;
    double T3_double;
    static double tab_T3[] = {0.77244,0.29612,-0.44757};
    
    private_RGB_to_LUV(R,G,B,&L,&u,&v);
    

    
    T3_double = tab_T3[0]*L+tab_T3[1]*u+tab_T3[2]*v; 

    T3 = FLOAT_TO_INT(T3_double);
    

    return(T3);
}
static int private_T1T2T3_to_R(int T1,int T2,int T3)
{
    int retour;
    int L,u,v;
    static double tab_L[3] = {-0.57651,0.19951,0.87311};
    static double tab_u[3] = {0.73309,0.72868,0.33482};
    static double tab_v[3] = {-0.50995,0.82643,-0.50590};
    
    L = tab_L[0]*T1+tab_L[1]*T2+tab_L[2]*T3;
    u = tab_u[0]*T1+tab_u[1]*T2+tab_u[2]*T3;
    v = tab_v[0]*T1+tab_v[1]*T2+tab_v[2]*T3;

    retour = private_Luv_to_R(L,u,v);

    return(retour);
}
static int private_T1T2T3_to_G(int T1,int T2,int T3)
{
  int retour;
  int L,u,v;
  static double tab_L[3] = {-0.57651,0.19951,0.87311};
  static double tab_u[3] = {0.73309,0.72868,0.33482};
  static double tab_v[3] = {-0.50995,0.82643,-0.50590};
    
  L = tab_L[0]*T1+tab_L[1]*T2+tab_L[2]*T3;
  u = tab_u[0]*T1+tab_u[1]*T2+tab_u[2]*T3;
  v = tab_v[0]*T1+tab_v[1]*T2+tab_v[2]*T3;

  retour = private_Luv_to_G(L,u,v);

  return(retour);
}
static int private_T1T2T3_to_B(int T1,int T2,int T3)
{
  int retour;
  int L,u,v;
  static double tab_L[3] = {-0.57651,0.19951,0.87311};
  static double tab_u[3] = {0.73309,0.72868,0.33482};
  static double tab_v[3] = {-0.50995,0.82643,-0.50590};
    
  L = tab_L[0]*T1+tab_L[1]*T2+tab_L[2]*T3;
  u = tab_u[0]*T1+tab_u[1]*T2+tab_u[2]*T3;
  v = tab_v[0]*T1+tab_v[1]*T2+tab_v[2]*T3;

  retour = private_Luv_to_B(L,u,v);

  return(retour);
}
/***************************************************************************/
             /* Fonctions prives, apellees par les fonctions de */
	     /* changement de base  */
/***************************************************************************/
static void private_RGB_to_XYZ(int R,int G,int B,float *X,float*Y,float*Z)
{
  
  *X = Xr*R + Xg*G + Xb*B;
   *Y = Yr*R + Yg*G + Yb*B;
  *Z = Zr*R + Zg*G + Zb*B;

}
static void private_RGB_to_LUV(int R,
			       int G,
			       int B,
			       double * L,
			       double* u,
			       double* v)
{
  float X,Y,Z;
  double YparYw;
  double Lp,up,vp;
  
  private_RGB_to_XYZ(R,G,B,&X,&Y,&Z);
  
  if((X==0.0)&&(Y==0.0)&&(Z==0.0))
    return;
  
  YparYw = Y/Yw;
  if(YparYw < 0.01)
    Lp = 903.3*YparYw;
  else
    Lp = 116.0*cbrt(YparYw) -16.0;
  
  
  *L = Lp;
  
  up = 4*X/(X+15*Y+3*Z);
  *u = 13*Lp*(up-Uw);
  
  vp = 9*Y/(X+15*Y+3*Z);
  *v = 13*Lp*(vp-Vw);
}
static float private_RGB_to_Xyz(int R,int G,int B)
{
  float retour;
  
  retour = Xr*R + Xg*G + Xb*B;
  
  return(retour);
}
static float private_RGB_to_xYz(int R,int G,int B)
{
  float retour;
  
  retour = Yr*R + Yg*G + Yb*B;
  
  return(retour);
}
static float private_RGB_to_xyZ(int R,int G,int B)
{
  float retour;
  
  retour = Zr*R + Zg*G + Zb*B;
  
  return(retour);
}
static void private_Luv_to_XYZ(int L,int u,int v,float*X,float*Y,float*Z)
{
  float Q,R,A;
  static float seuil = 8.0;
  static float pente_moins_un = 1.0/903.327;
  static float cent_seize_p_3 = 116*116*116;
  
  
  Q = u/(13.0*L)+Uw;
  R = v/(13.0*L)+Vw;
  
  if(L > seuil)
    *Y = Yw*(L+16)*(L+16)*(L+16)/cent_seize_p_3;
  else
    *Y = Yw*L*pente_moins_un;
  
  A = 3.0*(*Y)*(5.0*R-3.0);
  *Z = ( (Q-4.0)*A - 15.0*Q*R*(*Y) )/(12.0*R);
  
  *X = -( A/R + 3.0*(*Z) );
}
static int private_XYZ_to_R(float X,float Y,float Z)
{
  int retour;
  float R = Rx*X+Ry*Y+Rz*Z+0.5;
  
  retour = (int)MAX(R,0);
  retour = MIN(retour,255);
  
  return(retour);
}
static int private_XYZ_to_G(float X,float Y,float Z)
{
  
  int retour;
  float G = Gx*X+Gy*Y+Gz*Z+0.5;
  
  retour = (int)MAX(G,0);
  retour = MIN(retour,255);
  
  return(retour);
}
static int private_XYZ_to_B(float X,float Y,float Z)
{
  int retour;
  float B = Bx*X+By*Y+Bz*Z+0.5;
  
  retour = (int)MAX(B,0);
  retour = MIN(retour,255);
  
  return(retour);
}

static float private_RGB_to_Luv(int R,int G,int B)
{
  static double pente = 903.32701;
  static double seuil = 0.008856;
  float Y = private_RGB_to_xYz(R,G,B);
  float YparYw = Y/Yw;
  float retour;
  
  if(YparYw < seuil)
    retour = pente*YparYw;
  else
    retour = 116.0*cbrt(YparYw) -16.0;
  
  return(retour);
}


static void private_Lab_to_XYZ(int L,int a,int b,float*X,float*Y,float*Z)
{
  float fX,fY,fZ;
  static float seuil = 0.206893;
  
  fY = ((float)(L+16.0))/116.0;
  fX = ((float)a)/500.0 + fY;
  fZ = fY - ((float)b)/200.0;

  if(fX < seuil)
    *X = Xw*(fX-16.0/116.0)/7.787;
  else
    *X = Xw*fX*fX*fX;

  if(fY < seuil)
    *Y = Yw*(fY-16.0/116.0)/7.787;
  else
    *Y = Yw*fY*fY*fY;

  if(fZ < seuil)
    *Z = Zw*(fZ-16.0/116.0)/7.787;
  else
    *Z = Zw*fZ*fZ*fZ;
}

   /**********************************************************************/
  /********** Fonctions auxilieres utilisees pour le calcul de **********/
 /******************* l'espace issu des vecteurs propres****************/
/**********************************************************************/
static void private_calcul_mat_cov(image img,
				   mat_covariance M)
{
  double mom0;
  vecteur mom1;
  mat_covariance mom2;
  int i,j,k,l;
  int *x;
  int dim = image_give_dim(img);
  
  /* initialisation des differents moments */
  mom0=0.0;
  for(i=0;i<dim;i++)
    {
      mom1[i] =0.0;
      for(j=0;j<DIM;j++)
	mom2[i][j] = 0.0;
    }
  /*Calcul proprement dit */
  image_debut(img);
  do
    {
      mom0++;
      /* x : coordonees du pixel */
      x = image_lire_pixel(img);
      
      for(k=0;k<dim;k++)
	{
	  mom1[k] +=x[k];
	  for(l=0;l<=k;l++)
	    mom2[k][l] += x[k]*x[l];
	}
    }
  while(image_pixel_suivant(img));
  
  for(i=0;i<dim;i++)
    for(j=0;j<=i;j++)
      M[i][j] =(mom2[i][j]-mom1[i]*(mom1[j]/mom0) )/mom0;
  for(i=0;i<dim;i++)
    for(j=i+1;j<dim;j++)
      M[i][j] = M[j][i];
}

static void private_RGB_to_vecteur_propre(mat_passage P,
					  vecteur V,
					  vecteur lambda,
					  int ligne,
					  int flag)
{
  static mat_passage Pass;
  static vecteur lambda_par_somme;
  int i,j;
  double somme;
  
  if(flag == RETENIR)
    {
      for(i=0;i<DIM;i++)
	for(j=0;j<DIM;j++)
	  Pass[i][j] = P[j][i];
      matrice_affiche_matrice(Pass);
      somme =0.0;
      for(i=0;i<DIM;i++)
	somme += lambda[i];
      for(i=0;i<DIM;i++)
	lambda_par_somme[i] = lambda[i]/somme;
    }
  else
    {
      if(flag == RETOURNER)
	{
	  for(i=0;i<DIM;i++)
	    {
	      V[i] = Pass[ligne][i];
	      lambda[i] = lambda_par_somme[i];
	      
	    }
	}
      else
	{
	  printf("Erreur dans RGB_to_vecteur_propre\n");
	  printf("flag incorect\n");
	}
    }
}

static void private_vecteur_propre_to_RGB(mat_passage P,
					  vecteur V,
					  vecteur lambda,
					  int ligne,
					  int flag)
{
  static mat_passage Pass;
  static vecteur somme_par_lambda;
  int i,j;
  double somme;

  if(flag == RETENIR)
    {
      for(i=0;i<DIM;i++)
	for(j=0;j<DIM;j++)
	  Pass[i][j] = P[i][j];
      
      somme = 0.0;
      for(i=0;i<DIM;i++)
	somme += lambda[i];
      for(i=0;i<DIM;i++)
	somme_par_lambda[i] = somme/lambda[i];
    }
  else
    {
      if(flag == RETOURNER)
			{
			  for(j=0;j<DIM;j++)
			    {
			      V[j] = Pass[ligne][j];
			      lambda[j] = somme_par_lambda[j];
			    }
			}
      else
	{
	  printf("Erreur dans_vecteur_propre to RGB\n");
	  printf("flag incorect\n");
	}
    }
}
static void private_Calcul_vecteur_propre(image img)
{
  
  matrice  val_initiale = 
    {
      {UN_TIER,UN_TIER,UN_TIER},
      { 1.0,0,-1.0},
      { -0.5,1.0,-0.5},
    };
  
  mat_covariance cov;
  mat_passage P;
  vecteur V,val_propre;
  
  
  private_calcul_mat_cov(img,cov);    
  matrice_affiche_matrice(cov);
  matrice_val_vect_propre(cov,val_propre,P,val_initiale);
  private_RGB_to_vecteur_propre(P,V,val_propre,1,RETENIR);
  private_vecteur_propre_to_RGB(P,V,val_propre,1,RETENIR);
}

/****************************************************************************/

/***************** Espace L a b ********************************************/

static int private_RGB_to_a(int R,int G,int B)
{
  static double coef = 7.787;
  static double seuil = 0.008856;
  float X,Y,a;
  int retour;
  X = private_RGB_to_Xyz(R,G,B);
  Y = private_RGB_to_xYz(R,G,B);
  
  float XparXw = X/Xw;
  float YparYw = Y/Yw;
  
  if((XparXw < seuil))
    {
      if(YparYw < seuil)
	a = 500*coef*(XparXw-YparYw);
      else
	a = 500*(coef*XparXw+16/116-cbrt(YparYw));
    }
  else
    {
      if(YparYw < seuil)
	a = 500*(cbrt(XparXw)-(coef*YparYw+16/116));
      else
	a = 500*(cbrt(XparXw)-cbrt(YparYw));
    }

  retour = FLOAT_TO_INT(a);

  return(retour);
}
static int private_RGB_to_b(int R,int G,int B)
{
  static double coef = 7.787;
  static double seuil = 0.008856;
  float Y,Z,b;
  int retour;
  Y = private_RGB_to_xYz(R,G,B);
  Z = private_RGB_to_xyZ(R,G,B);
  
  float YparYw = Y/Yw;
  float ZparZw = Z/Zw;
  
  if((YparYw < seuil))
    {
      if(ZparZw < seuil)
	b = 200*coef*(YparYw-ZparZw);
      else
	b = 200*(coef*YparYw+16/116-cbrt(ZparZw));
    }
  else
    {
      if(ZparZw < seuil)
	b = 200*(cbrt(YparYw)-(coef*ZparZw+16/116));
      else
	b = 200*(cbrt(YparYw)-cbrt(ZparZw));
    }

  retour = FLOAT_TO_INT(b);

  return(retour);
}   
static int private_Lab_to_R(int L,int a,int b)
{
  int retour;
  float X,Y,Z;
  
  private_Lab_to_XYZ(L,a,b,&X,&Y,&Z);
  retour = private_XYZ_to_R(X,Y,Z);
  
  return(retour);
}
static int private_Lab_to_G(int L,int a,int b)
{
  int retour;
  float X,Y,Z;
  
  private_Lab_to_XYZ(L,a,b,&X,&Y,&Z);
  retour = private_XYZ_to_G(X,Y,Z);
  
  return(retour);
}
static int private_Lab_to_B(int L,int a,int b)
{
  int retour;
  float X,Y,Z;
  
  private_Lab_to_XYZ(L,a,b,&X,&Y,&Z);
  retour = private_XYZ_to_B(X,Y,Z);
  
  return(retour);
}


/****************************************************************************/

/***************** Espace G L H S ******************************************/

static void private_RGB_to_GLHS(int R, int G, int B,
			       float wmin, float wmid, float wmax,
			       int *H, int *S, int *L)
{
  int k;
  float min, mid, max, f, lq;
  
  max = (float)(Max(R,Max(G,B)));
  min = (float)(Min(R,Min(G,B)));
  if((R<max) && (R>min))
    mid = (float)R;
  else
    {
      if((G<max) && (G>min))
	mid = (float)G;
      else 
	mid = (float)B;
    }
  
  if(max==min)
    {
      *L = (int)max;
      *H = 0;
      *S = 0;
    }
  else
    {
      float h,s,l;
      
      l = wmax*max+wmid*mid+wmin*min;
      
      if((R>G) && (G>=B)) k=0;
      if((G>=R) && (R>B)) k=1;
      if((G>B) && (B>=R)) k=2;
      if((B>=G) && (G>R)) k=3;
      if((B>R) && (R>=G)) k=4;
      if((R>=B) && (B>G)) k=5;
      
      if(k%2==0)
	f = (mid-min)/(max-min);
      else
	f = (max-mid)/(max-min);
      
      h = (k+f)*60;
      
      lq = (wmid*(mid-min)/(max-min)+wmax)*M_GLHS;

      if(l <= lq)
	s = (l-min)/l;
      else
	s = (max-l)/(M_GLHS-l);

      *H = (int)(h+0.5);
      *S = (int)(255*s+0.5);
      *L = (int)(l+0.5);
    }
}

static void private_GLHS_to_RGB(int H, int S, int L,
				float wmin, float wmid, float wmax,
				int *R , int *G, int *B)
{
  float stmp, f, f2, lq;
  float min, mid, max;
  int k;

  stmp = ((float)S/255.0);

  if(stmp==0)
    {
      *R = L;
      *G = L;
      *B = L;
    }
  else
    {
      k = H/60;
      f = (float)H/60.0-k;
      
      if(k%2==0)
	f2 = f;
      else
	f2 = 1-f;

      lq = (wmid*f2+wmax)*M_GLHS;
      
      if(L<=lq)
	{
	  min = (1-stmp)*L;
	  mid = (f2*L+min*((1-f2)*wmax-f2*wmin))/(wmax+f2*wmid);
	  max = (L-wmid*mid-wmin*min)/wmax;
;
	}
      else
	{
	  max = stmp*M_GLHS+(1-stmp)*L;
	  mid = ((1-f2)*L-max*((1-f2)*wmax -f2*wmin))/((1-f2)*wmid+wmin);
	  
	  if(wmin>0)
	    min = (L-wmax*max-wmid*mid)/wmin;
	  else
	    min = (mid-f2*max)/(1-f2);
	}

      switch(k)
	{
	case 0 :
	  *R = (int)(max+0.5);
	  *G = (int)(mid+0.5);
	  *B = (int)(min+0.5);
	  break;
	case 1 :
	  *R = (int)(mid+0.5);
	  *G = (int)(max+0.5);
	  *B = (int)(min+0.5);
	  break;
	case 2 :
	  *R = (int)(min+0.5);
	  *G = (int)(max+0.5);
	  *B = (int)(mid+0.5);
	  break;
	case 3 :
	  *R = (int)(min+0.5);
	  *G = (int)(mid+0.5);
	  *B = (int)(max+0.5);
	  break;
	case 4 :
	  *R = (int)(mid+0.5);
	  *G = (int)(min+0.5);
	  *B = (int)(max+0.5);
	  break;
	case 5 :
	  *R = (int)(max+0.5);
	  *G = (int)(min+0.5);
	  *B = (int)(mid+0.5);
	  break;
	}
    }
}



/****************************************************************************/

/***************** Espace H S L ********************************************/

static int private_RGB_to_Ht(int R,int G,int B)
{
  int H,S,L;
  
  private_RGB_to_GLHS(R,G,B,Wmin_t,Wmid_t,Wmax_t,&H,&S,&L);

  return H;
}
static int private_RGB_to_St(int R,int G,int B)
{
  int H,S,L;
  
  private_RGB_to_GLHS(R,G,B,Wmin_t,Wmid_t,Wmax_t,&H,&S,&L);

  return S;
}   
static int private_RGB_to_Lt(int R,int G,int B)
{
  int H,S,L;
  
  private_RGB_to_GLHS(R,G,B,Wmin_t,Wmid_t,Wmax_t,&H,&S,&L);

  return L;
}
static int private_HSL_to_R(int H,int S,int L)
{
  int R,G,B;

  private_GLHS_to_RGB(H,S,L,Wmin_t,Wmid_t,Wmax_t,&R,&G,&B);

  return R;
}
static int private_HSL_to_G(int H,int S,int L)
{
  int R,G,B;

  private_GLHS_to_RGB(H,S,L,Wmin_t,Wmid_t,Wmax_t,&R,&G,&B);

  return G;
}
static int private_HSL_to_B(int H,int S,int L)
{
  int R,G,B;

  private_GLHS_to_RGB(H,S,L,Wmin_t,Wmid_t,Wmax_t,&R,&G,&B);

  return B;
}



/****************************************************************************/

/***************** Espace H S V ********************************************/

static int private_RGB_to_Hh(int R,int G,int B)
{
  int H,S,V;
  
  private_RGB_to_GLHS(R,G,B,Wmin_h,Wmid_h,Wmax_h,&H,&S,&V);

  return H;
}
static int private_RGB_to_Sh(int R,int G,int B)
{
  int H,S,V;
  
  private_RGB_to_GLHS(R,G,B,Wmin_h,Wmid_h,Wmax_h,&H,&S,&V);

  return S;
}   
static int private_RGB_to_Vh(int R,int G,int B)
{
  int H,S,V;
  
  private_RGB_to_GLHS(R,G,B,Wmin_h,Wmid_h,Wmax_h,&H,&S,&V);

  return V;
}
static int private_HSV_to_R(int H,int S,int V)
{
  int R,G,B;

  private_GLHS_to_RGB(H,S,V,Wmin_h,Wmid_h,Wmax_h,&R,&G,&B);

  return R;
}
static int private_HSV_to_G(int H,int S,int V)
{
  int R,G,B;

  private_GLHS_to_RGB(H,S,V,Wmin_h,Wmid_h,Wmax_h,&R,&G,&B);

  return G;
}
static int private_HSV_to_B(int H,int S,int V)
{
  int R,G,B;

  private_GLHS_to_RGB(H,S,V,Wmin_h,Wmid_h,Wmax_h,&R,&G,&B);

  return B;
}



/****************************************************************************/

/***************** Espace H L S ********************************************/

static int private_RGB_to_Hd(int R,int G,int B)
{
  int H,L,S;
  
  private_RGB_to_GLHS(R,G,B,Wmin_d,Wmid_d,Wmax_d,&H,&S,&L);

  return H;
}
static int private_RGB_to_Ld(int R,int G,int B)
{
  int H,L,S;
  
  private_RGB_to_GLHS(R,G,B,Wmin_d,Wmid_d,Wmax_d,&H,&S,&L);

  return L;
}   
static int private_RGB_to_Sd(int R,int G,int B)
{
  int H,L,S;
  
  private_RGB_to_GLHS(R,G,B,Wmin_d,Wmid_d,Wmax_d,&H,&S,&L);

  return S;
}
static int private_HLS_to_R(int H,int L,int S)
{
  int R,G,B;

  private_GLHS_to_RGB(H,S,L,Wmin_d,Wmid_d,Wmax_d,&R,&G,&B);

  return R;
}
static int private_HLS_to_G(int H,int L,int S)
{
  int R,G,B;

  private_GLHS_to_RGB(H,S,L,Wmin_d,Wmid_d,Wmax_d,&R,&G,&B);

  return G;
}
static int private_HLS_to_B(int H,int L,int S)
{
  int R,G,B;

  private_GLHS_to_RGB(H,S,L,Wmin_d,Wmid_d,Wmax_d,&R,&G,&B);

  return B;
}














