/*  descartes
 *  Copyright (C) 2001 Luc Brun
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

static const char* sccsid = 
"@(#) dart tab, Luc Brun, last revision 30/08/95";

#include <assert.h>

#include "dart_tab.h"


#define Nb_ligne      SELF(nb_ligne)
#define Element_tab  SELF(element_tab)

#define ALLOC_LIGNE 2000
#define ALLOC_NB_LIGNE 50

#define NUM_LIGNE(n) ((n) / ALLOC_LIGNE)
#define NUM_COLUMN(n)((n) % ALLOC_LIGNE)

static void private_realloc_dart_tab(dart_tab);
static dart* private_alloc_ligne();

struct dart_tab
{
  int **element_tab;
  int nb_ligne;
};

dart_tab FAIRE_dart_tab()
{
  INSTANCIER(dart_tab);

  Element_tab = (int **) malloc(ALLOC_NB_LIGNE*sizeof(int*));
  *Element_tab= private_alloc_ligne();
  
  Nb_ligne = 1;
  return self;
}

/*---------------------------------------------------------------------------*/

void DEFAIRE_dart_tab(dart_tab self)
{  
  int **tmp;
  int i;

  assert(self);

  tmp  = Element_tab;
  for(i=0;i<Nb_ligne;i++)
    {
      free (*tmp);
      tmp++;
    }
  free(Element_tab);
  free(self);
}
/*---------------------------------------------------------------------------*/
/* renvoie la valeur contenue a l'indice ind */
/*---------------------------------------------------------------------------*/
dart  dart_tab_give_value(dart_tab self,dart ind)
{
  int ligne,column;

  assert(self);

  ligne  = NUM_LIGNE(ind);
  column = NUM_COLUMN(ind);

/* ligne non encore alloue */
  if ( (ligne >= Nb_ligne) )
    return FINITE_FACE;
 
  return Element_tab[ligne][column];
}
/*---------------------------------------------------------------------------*/
/* met value a l'indice ind du tableau virtuel */
/*---------------------------------------------------------------------------*/
void dart_tab_put_value(dart_tab self,dart ind,dart value)
{
  int ligne,column;

  assert(self);

  ligne =  NUM_LIGNE(ind);
  column = NUM_COLUMN(ind);

  /* on ajoute autant de lignes que necessaire */
  while(ligne >= Nb_ligne)
    private_realloc_dart_tab(self);
  
  Element_tab[ligne][column] = value;
}
      
/*---------------------------------------------------------------------------*/
static dart *private_alloc_ligne()
{
  dart* return_ligne = (dart *) malloc(ALLOC_LIGNE*sizeof(dart));
  int i;

  assert(return_ligne);
  
  for(i=0;i<ALLOC_LIGNE;i++)
    return_ligne[i] = FINITE_FACE;
  
  return return_ligne;
}

/*---------------------------------------------------------------------------*/


static void private_realloc_dart_tab(dart_tab self)
{
  assert(self);

  /* On regarde si on na pas depasse le nb max de ligne alloue */
  if(Nb_ligne%ALLOC_NB_LIGNE == 0)
    {
      Element_tab = (dart**)realloc(Element_tab,
				      (Nb_ligne+ALLOC_NB_LIGNE)*
				      sizeof(dart*));
      assert(Element_tab);
    }
  Element_tab[Nb_ligne] = private_alloc_ligne();
  Nb_ligne++;
}



