/*  descartes
 *  Copyright (C) 2001 Luc Brun
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <stdio.h>
#include <math.h>
#include "euclid_path_set.h"
#include "chemin_euclidien.h"
#include "tab_euclid.h"
#include "contour_set.h"
#include "output_segment.h"
#include "node_euclid_set.h"
#include "display.h"
#include "objet.h"
#include "macros.h"


#define Tab    SELF(tab)
#define Nodes  SELF(nodes)
#define Width  SELF(width)
#define Hight  SELF(hight)

static void init_from_face_priv   (euclid_path,
				   Chemin_Eucl,
				   contour_set,
				   dart);
static void init_from_segment_priv(euclid_path,
				   output_segment,
				   Chemin_Eucl,
				   dart,
				   int);

static void write_face_priv(euclid_path,
			    contour_set,
			    tab_mark_dart,
			    dart,
			    display,
			    int,
			    point,
			    unsigned char*);

static void write_segment_priv(euclid_path,
			       output_segment,
			       dart,
			       display,
			       int,
			       point,
			       unsigned char*);

static void draw_line_priv(euclid_path,
			   float x1,float y1,
			   float x2,float y2,
			   display,
			   int,
			   point,
			   unsigned char*);


struct euclid_path
{
  tab_euclid tab;
  node_euclid_set nodes;
  int width;
  int hight;
};

euclid_path FAIRE_euclid_path(contour_set contours)
{

  INSTANCIER(euclid_path);

  Width  = contour_set_give_image_largeur(contours);
  Hight = contour_set_give_image_hauteur(contours);
  
  Tab = FAIRE_tab_euclid(Width,Hight);
  Nodes = FAIRE_node_euclid_set();
  
  return self;
}
void DEFAIRE_euclid_path(euclid_path self)
{
  DEFAIRE_tab_euclid(Tab);
  DEFAIRE_node_euclid_set(Nodes);
  free(self);
}
void euclid_path_reinitialize(euclid_path self)
{
  tab_euclid_reinitialize(Tab);
  node_euclid_set_reinitialize(Nodes);
}

   /************************************************/
  /* Initialise l'ensemble des chemins euclidiens */
 /* a partir de l'ensemble des faces de l'image. */
/************************************************/
void euclid_path_init_from_all_faces(euclid_path self,
				     contour_set contours)
{
  dart_list face_set;
  Chemin_Eucl  euclid;
  dart f;

  face_set =  contour_set_give_image_faces(contours);
  euclid = FAIRE_Chemin_Eucl();
  
  do
    {
      f = dart_list_give_current(face_set);
      Chemin_Eucl_compute_from_face(euclid,contours,f);
      init_from_face_priv(self,euclid,contours,f);
      Chemin_Eucl_reinitialize(euclid);
    }
  while(dart_list_next(face_set));

  node_euclid_set_update_tab_euclid(Nodes,Tab);

  DEFAIRE_Chemin_Eucl(euclid);  
  DEFAIRE_dart_list(face_set);
}
   /*******************************************/
  /* Ecris l'ensemble des chemins euclidiens */
 /* dans la ximage                          */
/*******************************************/
void euclid_path_write_all_euclidean_paths(euclid_path self,
					   contour_set contours,
					   display gen_disp,
					   unsigned char* value)
{ 
  tab_mark_dart marked_seg;
  dart_list face_list;
  dart f;
  Point zero = {0,0};

  marked_seg = FAIRE_tab_mark_dart();
  
  face_list = contour_set_give_image_faces(contours);
  dart_list_first(face_list);
  do
    {
      f = dart_list_give_current(face_list);
      write_face_priv(self,contours,marked_seg,f,gen_disp,1,&zero,value);
    }
  while(dart_list_next(face_list));
}
   /************************************************/
  /* Initialise l'ensemble des chemins euclidiens */
 /* a partir d'une face de l'image.              */
/************************************************/
void euclid_path_init_from_face(euclid_path self,
				contour_set contours,
				dart f)
{
  Chemin_Eucl  euclid;
  

  euclid = FAIRE_Chemin_Eucl();

  Chemin_Eucl_compute_from_face(euclid,contours,f); 
  init_from_face_priv(self,euclid,contours,f);	
  node_euclid_set_update_tab_euclid(Nodes,Tab);
  
  DEFAIRE_Chemin_Eucl(euclid);  
}
void euclid_path_write_face(euclid_path self,
			    contour_set contours,
			    dart f,
			    display gen_disp,
			    int scale,
			    point p_orig,
			    unsigned char* value)
{ 
  dart_list seg_list;
  dart b,b_abs;
  output_segment seg;

  seg_list = FAIRE_dart_list();
  seg = FAIRE_output_segment();

  contour_set_give_segments_of_face( contours,f,seg_list);
  
  dart_list_first(seg_list);
  do
    {
      b = dart_list_give_current(seg_list);
      contour_set_give_segment(contours,b,seg);
      write_segment_priv(self,seg,b,gen_disp,scale,p_orig,value);
    }
  while(dart_list_next(seg_list));

  DEFAIRE_output_segment(seg);
  DEFAIRE_dart_list(seg_list);
}

static void init_from_face_priv(euclid_path self,
				Chemin_Eucl euclid,
				contour_set contours,
				dart f)
{
  dart_list seg_list;
  dart b;
  output_segment seg;
  int ind;
  
  seg_list = FAIRE_dart_list();
  seg = FAIRE_output_segment();
  ind = 0;
  contour_set_give_segments_of_face(contours,f,seg_list);
  dart_list_first(seg_list);
  do
    {
      b = dart_list_give_current(seg_list);
      contour_set_give_segment(contours,b,seg);
      init_from_segment_priv(self,seg,euclid,b,ind);
      ind += output_segment_give_lenght(seg); 
    }
  while(dart_list_next(seg_list));
  
  DEFAIRE_output_segment(seg);
  DEFAIRE_dart_list(seg_list);
}
static void init_from_segment_priv(euclid_path self,
				   output_segment seg,
				   Chemin_Eucl euclid,
				   dart b,
				   int ind)
{
  Point p;
  void (*first)(output_segment);
  bouleen (*next)(output_segment);
  x_value val_x;
  y_value val_y;
  move_type move;
  node_euclid_value val_node;


  if(b<0)
    {
      first = output_segment_last;
      next  = output_segment_previous;
    }
  else
    {
      first = output_segment_first;
      next  = output_segment_next;
    }

  first(seg);
  output_segment_give_current_point(seg,&p);

  val_node[0] = Chemin_Eucl_x_decal(euclid,ind);
  val_node[1] = Chemin_Eucl_y_decal(euclid,ind);
  node_euclid_set_store_update_node(Nodes,&p,val_node);
  
  tab_euclid_move_to(Tab,&p);
  move = output_segment_give_current_move(seg);
  if(b<0) move = REVERSE_SENSE(move);

       
  while(next(seg))
    {
      tab_euclid_move_in_direction(Tab,move);
      move = output_segment_give_current_move(seg);
      if(b<0) move = REVERSE_SENSE(move);
      ind++;
      
      val_x = Chemin_Eucl_x_decal(euclid,ind);
      val_y = Chemin_Eucl_y_decal(euclid,ind);
      tab_euclid_put_x(Tab,val_x);
      tab_euclid_put_y(Tab,val_y);
    }
}
static void write_face_priv(euclid_path self,
			    contour_set contours,
			    tab_mark_dart marked_seg,
			    dart f,
			    display gen_disp,
			    int scale,
			    point p_orig,
			    unsigned char* value)
{
  dart_list seg_list;
  dart b,b_abs;
  output_segment seg;

  seg_list = FAIRE_dart_list();
  seg = FAIRE_output_segment();

  contour_set_give_segments_of_face( contours,f,seg_list);
  
  dart_list_first(seg_list);
  do
    {
      b = dart_list_give_current(seg_list);
      b_abs = ABS(b);
      if(!tab_mark_dart_is_marked(marked_seg,b_abs))
	{
	  tab_mark_dart_mark_dart(marked_seg,b_abs);
	  contour_set_give_segment(contours,b,seg);
	  write_segment_priv(self, seg,b,gen_disp,scale,p_orig,value);
	}
    }
  while(dart_list_next(seg_list));
  DEFAIRE_output_segment(seg);
  DEFAIRE_dart_list(seg_list);
}
static void write_segment_priv(euclid_path self,
			       output_segment seg,
			       dart b,
			       display gen_disp,
			       int scale,
			       point p_orig,
			       unsigned char* value)
{
  Point p,current_point;
  float x1,y1,x2,y2;
  x_value x_val;
  y_value y_val;
  move_type move;
  bouleen  (*next)(output_segment);

  if(b>0)
    {
      next=output_segment_next;
      output_segment_give_first_point(seg,&p);
      move = output_segment_give_first_move(seg);
      output_segment_first(seg);
    }
  else
    {
      next=output_segment_previous;
      output_segment_give_last_point(seg,&p);
      move = REVERSE_SENSE(output_segment_give_last_move(seg));
      output_segment_last(seg);
    }
  tab_euclid_move_to(Tab,&p);
  x_val = tab_euclid_give_x(Tab);
  y_val = tab_euclid_give_y(Tab);
  x1 = COORDX(p) + Chemin_Eucl_real_value(x_val);/*+0.5*/
  y1 = COORDY(p) - Chemin_Eucl_real_value(y_val); /*+0.5*/



  while(next(seg))
    {
      tab_euclid_move_in_direction(Tab,move);
      output_segment_give_current_point(seg,&current_point);
      x_val = tab_euclid_give_x(Tab);
      y_val = tab_euclid_give_y(Tab);
      x2 = COORDX(current_point)  +  Chemin_Eucl_real_value(x_val);/*+0.5*/
      y2 = COORDY(current_point)  -  Chemin_Eucl_real_value(y_val);/*+0.5*/

     // if(display_draw_point != DRAW_POINT_NULL)

      if( !display_draw_point_is_set(gen_disp))
	draw_line_priv(self, x1,y1,x2,y2,gen_disp,scale,p_orig,value);
      else
	display_draw_line(gen_disp,x1,y1,x2,y2);

      x1 = x2;
      y1 = y2;

      move = output_segment_give_current_move(seg);
      if(b<0)
	move=REVERSE_SENSE(move);
    }
  tab_euclid_move_in_direction(Tab,move);
  output_segment_give_current_point(seg,&current_point);
  x_val = tab_euclid_give_x(Tab);
  y_val = tab_euclid_give_y(Tab);
  x2 = COORDX(current_point) +  Chemin_Eucl_real_value(x_val);/*+0.5*/
  y2 = COORDY(current_point) -  Chemin_Eucl_real_value(y_val);/*+0.5*/

  if( !display_draw_point_is_set(gen_disp))
    draw_line_priv(self, x1,y1,x2,y2,gen_disp,scale,p_orig,value);
  else
    display_draw_line(gen_disp,x1,y1,x2,y2);
}

static void draw_line_priv(euclid_path self,
			   float x1,float y1,
			   float x2,float y2,
			   display gen_disp,
			   int scale,
			   point p_orig,
			   unsigned char* value)
{
  Point p;
  float x_tmp, y_tmp;
  int xd,yd,xf,yf,x,y;
  double a;
  
  x1 = scale*(x1 - COORDX(*p_orig)-1)+(scale-1)/2;
  y1 = scale*(y1 - COORDY(*p_orig)-1)+(scale-1)/2;
  x2 = scale*(x2 - COORDX(*p_orig)-1)+(scale-1)/2;
  y2 = scale*(y2 - COORDY(*p_orig)-1)+(scale-1)/2;

  if(x1 > x2) { /* parcours en x du segment */
    x_tmp = x1; y_tmp = y1;
    x1 = x2; y1 = y2;
    x2 = x_tmp; y2 = y_tmp;
  }
  xd = ceil(x1);
  xf = floor(x2);

  if(x1 != x2)
    {
      a = (y2-y1) / (x2-x1);

      for(x=xd; x<=xf; x++) 
	{
	  COORDX(p) = x;
	  COORDY(p) = (int) (y1 + (x-x1)*a + 0.5);
	  display_draw_point(gen_disp,&p,value);
	}
    }

  if(y1 > y2) 
    { /* parcours en y du segment */
      x_tmp = x1; y_tmp = y1;
      x1 = x2; y1 = y2;
      x2 = x_tmp; y2 = y_tmp;
    }
  if(y1 != y2)
    {
      yd = ceil(y1);
      yf = floor(y2);
      a = (x2-x1) / (y2-y1);
      for(y=yd; y<=yf; y++)
	{
	  COORDY(p) = y;
	  COORDX(p) = (int) (x1 + (y-y1)*a + 0.5);
	  display_draw_point(gen_disp,&p,value);
	}
    }
}












