/*  descartes
 *  Copyright (C) 2001 Luc Brun
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <assert.h>
#include "type_obj.h"

#include "face.h"

#define  Computed SELF(computed)
#define  Face_nb  SELF(face_nb)
#define  Dim      SELF(dim)
#define  Moment0  SELF(moment0)
#define  Moment1  SELF(moment1)
#define  Moment2  SELF(moment2)

struct face
{
  bouleen computed;
  dart face_nb;
  int dim;
  double moment0;
  vector moment1;
  vector moment2;
};

face FAIRE_face(int dim)
{
  INSTANCIER(face);

  Computed = faux;
  Dim = dim;

  return self;
}

void DEFAIRE_face(face self)
{
  assert(self);
  free(self);
}

bouleen face_is_computed(face self)
{
  assert(self);
  return Computed;
}
double face_give_area(face self)
{
  assert(self);

  return Moment0;
}
double* face_give_mean_value(face self)
{
  static vector value;
  int i;

  assert(self);

  for(i=0;i<Dim;i++)
    value[i] = Moment1[i]/Moment0;

  return value;
}
double* face_give_moment1_value(face self)
{
  static vector value;
  int i;

  assert(self);

  for(i=0;i<Dim;i++)
    value[i] = Moment1[i];

  return value;
}
void face_give_moments(face self,double* m0,vector m1,vector m2)
{
  int i;

  assert(self);

  *m0 = Moment0;

  for(i=0;i<Dim;i++)
    {
      m1[i] = Moment1[i];
      m2[i] = Moment2[i];
    }
}

double face_give_mean_squared_error(face self)
{
  int i;
  double msqe;

  assert(self);

  msqe = 0.0;
  
  for(i=0;i<Dim;i++)
    msqe += (Moment2[i] - Moment1[i]*Moment1[i]/Moment0)/Moment0;

  return msqe;
}
double face_give_squared_error(face self)
{
  int i;
  double sqe;

  assert(self);

  sqe = 0.0;
  
  for(i=0;i<Dim;i++)
    sqe += (Moment2[i] - Moment1[i]*Moment1[i]/Moment0);

  return sqe;
}
void face_initialize(face self,
		     double m0,
		     vector m1,vector m2,
		     int dim,
		     dart face_number)
{
  int i;

  assert(self);

  Computed = vrai;
  Face_nb = face_number;
  Dim = dim;  
  Moment0 = m0;
  for(i=0;i<Dim;i++)
    {
      Moment1[i] = m1[i];
      Moment2[i] = m2[i];
    }
}

void face_uninitialize(face self)
{

  if(self == NULL)
    return;

  Computed = faux;
}

    /********************************************************************/
   /** Realise l'union de deux faces. Utilise lorsque l'on supprime un */
  /** segment entre 2 faces ou si l'on supprime une composante connexe*/
 /** fille d'une autre face                                          */
/********************************************************************/
void face_merge(face self,face f)
{
  int i;
  face_uninitialize(f);

  if(self == NULL)
    return;

  if(f != NULL)
    {
      Moment0 += f->moment0;
      for(i=0;i<Dim;i++)
	{
	  Moment1[i] += f->moment1[i];
	  Moment2[i] += f->moment2[i];
	}
    }
}
