/*  descartes
 *  Copyright (C) 2001 Luc Brun
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <assert.h>

#include "hash_table.h"


#define BASE_DIM  1000
#define Node_number   SELF ( node_number )
#define Element_table SELF ( element_table )
#define Modulo        SELF ( modulo )

#define ACCESS(obj_, var_) obj_->var_

static int       private_hashcoding(int,int,int);
static L_ELEMENT private_remove_from_list(L_ELEMENT,point);

struct hash_table
{
    L_ELEMENT*  element_table;
    int        modulo;
    int        node_number;
};

hash_table FAIRE_hash_table()
{
  int i;

  INSTANCIER(hash_table);

  Modulo        = BASE_DIM;
  Element_table = (L_ELEMENT *)calloc(Modulo,sizeof(L_ELEMENT));
  Node_number   = 0;

  return self;
}

void DEFAIRE_hash_table(hash_table self)
{
  L_ELEMENT *current_pointer = Element_table;
  L_ELEMENT list_pointer,free_pointer;
  int i;

  for(i=0;i<Modulo;i++)
    {
      list_pointer = *current_pointer;
      while(list_pointer != NULL)
	{
	  free_pointer =list_pointer;
	  list_pointer=list_pointer->next;
	  free(free_pointer);
	}
      current_pointer++;
    }
  free(Element_table);
  free(self);
}


/*---------------------------------------------------------------------------*/

bouleen hash_table_is_in(hash_table self, point p)
{
  L_ELEMENT current_pointer;
  node n;
  int entry = private_hashcoding(COORDX(*p),COORDY(*p),Modulo);
  Point pt_node;

  current_pointer = Element_table[entry];

  while(current_pointer != NULL)
    {
      n = ACCESS(current_pointer, element);
      node_give_point(n,&pt_node);

      if( EGAL_POINT(pt_node,*p) )
	return vrai;
      else
	current_pointer=current_pointer->next;

    }
  return faux;
  
}
/*---------------------------------------------------------------------------*/

node hash_table_give_node(hash_table self, point p)
{
  L_ELEMENT current_pointer;
  node n;
  int entry = private_hashcoding(COORDX(*p),COORDY(*p),Modulo);
  Point pt_node;

  current_pointer = Element_table[entry];

  while(current_pointer != NULL)
    {
      n = ACCESS(current_pointer, element);
      node_give_point(n,&pt_node);
      if( EGAL_POINT(pt_node,*p) )
	return current_pointer->element;
      else
	current_pointer=current_pointer->next;
    }
  return (node) NULL;
  
}
/*---------------------------------------------------------------------------*/
void hash_table_insert(hash_table self,node n)
{
  int entry;
  L_ELEMENT insert_element = (L_ELEMENT)malloc(sizeof(L_element));
  Point p;

  insert_element->element=n;

  if( Node_number>=Modulo )
    hash_table_realloc(self);

  node_give_point(n,&p);
  entry=private_hashcoding(COORDX(p),COORDY(p),Modulo);

  insert_element->next=Element_table[entry];
  Element_table[entry]=insert_element;
  
  Node_number++;
}

void hash_table_realloc(hash_table self)
{
  int old_dim=Modulo;
  int i;
  L_ELEMENT *current_pointer,*new_table,*old_table;
  L_ELEMENT list_pointer,free_pointer;

  current_pointer = old_table=Element_table;
  Modulo += BASE_DIM;
  Node_number=0;

  new_table = (L_ELEMENT *) calloc(Modulo,sizeof(L_ELEMENT));
  Element_table=new_table;

  for(i=0;i<old_dim;i++)
    {
      list_pointer = *current_pointer;
      while(list_pointer !=NULL)
	{
	  hash_table_insert(self,list_pointer->element);
	  free_pointer=list_pointer;
	  list_pointer=list_pointer->next;
	  free(free_pointer);
	}
      current_pointer++;
    }

  free(old_table);
}

void hash_table_remove_elt(hash_table self, point p)
{
  int entry;

  assert(p);
  entry = private_hashcoding(COORDX(*p),COORDY(*p),Modulo);

  if(Element_table[entry] != NULL)
    Element_table[entry]= private_remove_from_list(Element_table[entry],p);

  Node_number--;
}
  
int hash_table_give_node_number(hash_table self)
{
  return Node_number;
}

static int private_hashcoding(int x,int y,int modulo)
{
  int temp = ((x^10+y^9)%modulo);
  return (temp < 0 ? 0 :temp);
}

static L_ELEMENT private_remove_from_list(L_ELEMENT list,point p)
{
 bouleen trouve = faux; 
 node n;
 Point pt_node;
 L_ELEMENT prev,link;

 /* La liste est vide */
  if (list==(L_ELEMENT)NULL)
    return NULL;

/* L'element a suprimer est en tete de liste */
  n= ACCESS(list,element);
  node_give_point(n,&pt_node);
  if( EGAL_POINT(pt_node,*p) )
    {
      link=list->next;
      free(list);
      return link;
    }
/* L'element a suprimer est qq pqrt ds la liste */
 prev = list;
 link = list->next;
 while( (link != NULL)&& (!trouve) )
   {
     n= ACCESS(link,element);
     node_give_point(n,&pt_node);
     if( EGAL_POINT(pt_node,*p) )
       {
	 prev->next = link->next;
	 free(link);
	 trouve = vrai;
       }
     prev = link;
     link = link->next;
   }

 return list;
}








