/*  descartes
 *  Copyright (C) 2001 Luc Brun
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <stdio.h>
#include <assert.h>

#include "inclusions.h"

#include "map.h"
#include "dart_tab.h"
#include "macros.h"



#define Infinite_set   SELF(infinite_set)
#define Finite_set     SELF(finite_set)
#define Inclusion_map  SELF(inclusion_map)
#define Dim            SELF(dim)

#define ALLOC_SIZE     1000


struct inclusions
{
  dart_tab     infinite_set;
  dart_list   *finite_set;
  map          inclusion_map;
  int          dim;
};

inclusions FAIRE_inclusions(map incl_map)
{
  
  int i;

  INSTANCIER(inclusions);
  Infinite_set = FAIRE_dart_tab();
  Finite_set   = (dart_list *) malloc(ALLOC_SIZE * sizeof(dart_list));
  
  Dim = ALLOC_SIZE;

  for(i=0;i < Dim;i++)
    Finite_set[i]   = NULL;

  Inclusion_map     = incl_map;

  return self;
}

void DEFAIRE_inclusions(inclusions self)
{
  int i;
  
  
  for (i = 0; i< Dim ;i++)
    {
      if(Finite_set[i] != NULL)
	DEFAIRE_dart_list(Finite_set[i]);
    }
  
  DEFAIRE_dart_tab(Infinite_set);
  free(Finite_set);
}

/*---------------------------------------------------------------------------*/

void inclusions_realloc_inclusions(inclusions self, dart b)
{
  int       i;

  assert(self);
  assert(b>0);

  if(b<Dim)
    return;


  Finite_set   = (dart_list *)realloc(Finite_set,  
				      (Dim+ALLOC_SIZE)*sizeof(dart_list)
				      );

  for(i = Dim; i < Dim + ALLOC_SIZE; i++)
    Finite_set[i]   = NULL;
  
  Dim += ALLOC_SIZE;
  
}


bouleen inclusions_is_infinite(inclusions self, dart face)
{
  if( dart_tab_give_value(Infinite_set,face)== FINITE_FACE)
    return faux;

  return vrai;
}

dart inclusions_give_father(inclusions self, dart face)
{
  return dart_tab_give_value(Infinite_set,face);
}
dart_list inclusions_give_daughter(inclusions self,dart b)
{
  return Finite_set[b];
}
dart_list inclusions_give_sister(inclusions self,dart b)
{
  dart father = dart_tab_give_value(Infinite_set,b);
  
  assert(father != FINITE_FACE);

  return Finite_set[father];
}
/*************************************************************************/
/* Initialisation de newface ds la structure */
/* family_face : une soeur */
/*************************************************************************/
dart_list inclusions_make_descent(inclusions self,
				  dart family_face,
				  dart new_face)
{
  int new_face_mother,family_face_mother;

  inclusions_realloc_inclusions(self, new_face);
  
  new_face_mother    = dart_tab_give_value(Infinite_set,new_face);
  family_face_mother = dart_tab_give_value(Infinite_set,family_face);

/* on enleve new_face de la liste des filles de sa mere */
  if(new_face_mother != FINITE_FACE)
    {
      assert(Finite_set[new_face_mother]);
      
      dart_list_remove_elt(Finite_set[new_face_mother],new_face);
    }
  
  if( family_face_mother == FINITE_FACE)
    {
      /*mere(new_face) = family_face */
      dart_tab_put_value(Infinite_set,new_face,family_face);

      if(Finite_set[family_face] == NULL) 
	Finite_set[family_face] = FAIRE_dart_list();

      /* fille(family_face) = fille(familly_face) U new_face */
/*      if( !dart_list_is_in(Finite_set[family_face],new_face) )*/
	dart_list_insert(Finite_set[family_face],new_face);

      return Finite_set[family_face];
    }

  /* mere(new_face) = mere(family_face) */
  dart_tab_put_value(Infinite_set,new_face,family_face_mother);
  
  if(Finite_set[family_face_mother] == NULL) 
    Finite_set[family_face_mother] = FAIRE_dart_list();

  /* fille(mere(family_face)) = fille(mere(family_face)) U  new_face */  
/*  if(!dart_list_is_in(Finite_set[family_face_mother],new_face))*/
    dart_list_insert(Finite_set[family_face_mother],new_face);
  
  return Finite_set[family_face_mother];
  
}
/*************************************************************************/
/* merge d'une face fini et infinie. merge de deux infinies. */
/*************************************************************************/
void inclusions_fusion(inclusions self, dart face1, dart face2)
{
  dart father1,father2;
 
  father1 = dart_tab_give_value(Infinite_set,face1);
  father2 = dart_tab_give_value(Infinite_set,face2);

  /* faces soeurs, on suprime face2 */
  if(father1 == father2)
    {
      dart_list_remove_elt(Finite_set[father1],face2);
      dart_tab_put_value(Infinite_set,face2,FINITE_FACE);
      return ;
    }
  /* mere(face1) = face2, on suprime face1 */
  if(father1 == face2)
    {
      dart_tab_put_value(Infinite_set,face1,FINITE_FACE);
      if(Finite_set[face2] != NULL)
	dart_list_remove_elt(Finite_set[face2],face1);
    }
  else
    {  /* mere(face2) = face1, on suprime face2 */
      dart_tab_put_value(Infinite_set,face2,FINITE_FACE);
      if(Finite_set[face1] != NULL)
	dart_list_remove_elt(Finite_set[face1],face2);
    }
}

/*************************************************************************/
/* declare son fille de etiq. */
/*************************************************************************/
void inclusions_split_mother_son(inclusions self, dart son, dart etiq)
{
  inclusions_realloc_inclusions(self,son);

  if (Finite_set[etiq] == NULL)
    Finite_set[etiq] = FAIRE_dart_list();
  
/*  if (!dart_list_is_in(Finite_set[etiq], son))*/
    dart_list_insert(Finite_set[etiq], son);

  dart_tab_put_value(Infinite_set,son,etiq);
}

   /***************************************************************/
  /*  Tous les fils de f_fini sont reportes sur mere de f_infini */
 /* Pour tout f  fille de f_fini mere(f) = mere(f_infini)       */
/***************************************************************/
void inclusions_report_daughter(inclusions self,dart f_fini,dart
				f_infini)
{
  dart mother_face;

  assert(self);

  mother_face = dart_tab_give_value(Infinite_set,f_infini);
  inclusions_merge_finite_face(self,f_fini,mother_face);
}
  /***************************************************************/
 /* f_infini est enlevee de la liste des filles de sa mere      */
/***************************************************************/
void inclusions_remove_infinite_face(inclusions self,dart f_infini)
{
  dart mother_face;

  mother_face = dart_tab_give_value(Infinite_set,f_infini);

  if(Finite_set[mother_face] != NULL)
    dart_list_remove_elt(Finite_set[mother_face],f_infini);

  dart_tab_put_value(Infinite_set,f_infini,FINITE_FACE);
}
  /****************************************************/
 /* Transfere les filles de f1 dans les filles de f2 */
/****************************************************/
void inclusions_merge_finite_face(inclusions self,dart f1,dart f2)
{
  dart_list son_list = Finite_set[f1];
  dart b;

  if(son_list != NULL_DART_LIST)
    {
      if(!dart_list_is_empty(son_list))
	{
	  dart_list_first(son_list);
	  do
	    {
	      b = dart_list_give_current(son_list);
	      dart_tab_put_value(Infinite_set,b,f2);
	    }
	  while(dart_list_next(son_list));
	}
      if(Finite_set[f2] != NULL_DART_LIST)
	dart_list_merge_list(Finite_set[f2], son_list);
      else
	Finite_set[f2] = son_list;

      Finite_set[f1] = NULL_DART_LIST;
    }
}







 



