/*  descartes
 *  Copyright (C) 2001 Luc Brun
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <values.h>
#include "classe.h"
#include "contour_set.h"
#include "tab_mark.h"
#include "merge_faces.h"
#include "rule.h"
#include "tab_mark_dart.h"

#define  MAX_SEGMENT_LENGHT 2
#define  AREA_MERGE_THRESHOLD 0.2

struct own_seg
{
  Point pt;
  move_type move;
  int lengh;
};

CLASSE(own_seg);

static void add_set_of_segs_priv         (contour_set,
					  dart,
					  list_gen,
					  tab_mark);
static void break_face_priv              (contour_set,
					  dart,
					  dart_list);
static void insert_our_seg_priv          (contour_set,
					  own_seg,
					  tab_mark);
static void search_neighbour_priv        (contour_set,
					  tab_mark,
					  move_type,
					  move_type,
					  point,
					  move_type,
					  list_gen);

static own_seg init_seg_priv             (point,move_type,int);

static void    smooth_rule_priv          (contour_set,
					  dart,
					  va_list);
static void    mark_seg_priv             (output_segment,tab_mark);
static void    mark_face_priv            (contour_set,
					  dart_list seg,
					  tab_mark);

static void    remove_piece_of_face_priv(contour_set,
					 dart,
					 tab_mark_dart,
					 dart_list);

void smooth_face(contour_set contours,dart f)
{
  dart_list inserted_faces,seg_to_del;
  double f_area;
  dart piece_of_f;
  double piece_of_f_area;
  double threshold;
  tab_mark_dart broken_face;

  seg_to_del     = FAIRE_dart_list();
  inserted_faces = FAIRE_dart_list();
  if(contour_set_is_infinite(contours,f))
    f_area =  contour_set_give_face_area(contours,
					 contour_set_give_including_face(contours,
									 f)
					 );
  else
    f_area    = contour_set_give_face_area(contours,f);
  threshold = f_area*AREA_MERGE_THRESHOLD;

  break_face_priv(contours,f,inserted_faces);

  if (dart_list_is_empty(inserted_faces))
    {
      DEFAIRE_dart_list(seg_to_del);
      DEFAIRE_dart_list(inserted_faces);
      return;
    }

  broken_face = FAIRE_tab_mark_dart();
  dart_list_first(inserted_faces);
  do
    {
      piece_of_f = dart_list_give_current(inserted_faces);
      tab_mark_dart_mark_dart(broken_face,piece_of_f);
    }
  while(dart_list_next(inserted_faces));

  dart_list_first(inserted_faces);
  do
    {
      piece_of_f = dart_list_give_current(inserted_faces);
      piece_of_f_area = contour_set_give_face_area(contours,
						   piece_of_f);
      if(piece_of_f_area < threshold)
	remove_piece_of_face_priv(contours,
				  piece_of_f,
				  broken_face,
				  seg_to_del);
    }
  while(dart_list_next(inserted_faces));
  
  DEFAIRE_tab_mark_dart(broken_face);
  DEFAIRE_dart_list(inserted_faces);
  contour_set_remove_set_of_arc(contours,seg_to_del);
  DEFAIRE_dart_list(seg_to_del);
}
static void break_face_priv(contour_set contours,
			    dart f,
			    dart_list inserted_faces)
{
  list_gen seg_to_insert;
  own_seg tmp;
  int height;
  int width;
  tab_mark marked_pts;
  
  height = contour_set_give_image_hauteur(contours);
  width = contour_set_give_image_largeur(contours);
  marked_pts = FAIRE_tab_mark(height,width);
  seg_to_insert = FAIRE_list_gen();

  add_set_of_segs_priv(contours,
		       f,
		       seg_to_insert,
		       marked_pts);

  if (list_gen_is_empty(seg_to_insert))
    {
      DEFAIRE_list_gen(seg_to_insert,free);
      return;
    }
  contour_set_memorize_inserted_faces(contours);
  list_gen_first(seg_to_insert);
  do
    {
      tmp = list_gen_give_current(seg_to_insert);
      insert_our_seg_priv(contours,tmp,marked_pts);
    }
  while(list_gen_next(seg_to_insert));

  DEFAIRE_list_gen(seg_to_insert,free);
  DEFAIRE_tab_mark(marked_pts);
  contour_set_give_inserted_faces(contours,inserted_faces);
}
/* Parcour la face afin d'inserer des petits bout de segment */
static void add_set_of_segs_priv(contour_set contours,
				 dart f,
				 list_gen inserted_segs,
				 tab_mark marked_pts)
{
  dart_list seg_list,face_to_process;
  output_segment seg;
  dart     b;
  bouleen  (*next)(output_segment);
  move_type prev_move,next_move;
  Point p;


  /* On marque la face a parcourir */
  face_to_process = FAIRE_dart_list();
  dart_list_insert(face_to_process,f);

  DEFAIRE_dart_list(face_to_process);

  seg_list      = FAIRE_dart_list();
  seg           = FAIRE_output_segment();
  /* On parcour la face */
  contour_set_give_segments_of_face(contours,f,seg_list);
  mark_face_priv(contours,seg_list,marked_pts);
  dart_list_last(seg_list);
  b = dart_list_give_current(seg_list);
  contour_set_give_segment(contours,b,seg);

  if (b>0)
    prev_move = output_segment_give_last_move(seg);
  else
    prev_move = REVERSE_SENSE(output_segment_give_first_move(seg));

  dart_list_first(seg_list);
  do
    {
      b = dart_list_give_current(seg_list);
      contour_set_give_segment(contours,b,seg);
      if(b>0)
	{
	  output_segment_first(seg);
	  next = output_segment_next;
	}
      else
	{
	  output_segment_last(seg);
	  next = output_segment_previous;
	}

      do
	{
	  next_move = output_segment_give_current_move(seg);
	  if(b<0)
	    {
	      next_move = REVERSE_SENSE(next_move);
	    }
	  output_segment_give_current_point(seg,&p);
	  search_neighbour_priv(contours,
				marked_pts,
				prev_move,
				next_move,
				&p,
				RIGHT,
				inserted_segs);
	  search_neighbour_priv(contours,
				marked_pts,
				prev_move,
				next_move,
				&p,
				UP,
				inserted_segs);
	  prev_move = next_move;
	}
      while (next(seg));     
    }
  while(dart_list_next(seg_list));
  
  DEFAIRE_dart_list(seg_list);
  DEFAIRE_output_segment(seg);
} 
/* Teste si on peut splitter la face avec un bout de segment de */
/* longueur <= MAX_SEGMENT_LENGHT et de direction trying_move */
static void search_neighbour_priv(contour_set contours,
				  tab_mark marked_pts,
				  move_type prec_move,
				  move_type next_move,
				  point p,
				  move_type trying_move,
				  list_gen ins_seg)
{
  static unsigned char mask1[NB_MOVE] = {0,1,2,3};
  static unsigned char mask2[NB_MOVE] = {0,4,8,12};

  static bouleen is_valid[NB_MOVE*NB_MOVE][NB_MOVE] =
    {
      /* R    U    L   D  */
      {faux,faux,faux,vrai},/* R-R : 0 */
      {faux,faux,faux,faux},/* U-R : 1 */
      {faux,faux,faux,faux},/* L-R : 2 */
      {faux,faux,vrai,vrai},/* D-R : 3 */
      
      {vrai,faux,faux,vrai},/* R-U : 4 */
      {vrai,faux,faux,faux},/* U-U : 5 */
      {faux,faux,faux,faux},/* L-U : 6 */
      {faux,vrai,vrai,faux},/* D-U : 7 */
      
      {faux,faux,faux,faux},/* R-L : 8 */
      {vrai,vrai,faux,faux},/* U-L : 9 */
      {faux,vrai,faux,faux},/* L-L :10*/
      {faux,faux,faux,faux},/* D-L :11*/
      
      {faux,faux,faux,faux},/* R-D :12*/
      {faux,faux,faux,faux},/* U-D :13*/
      {faux,vrai,vrai,faux},/* L-D :14*/
      {faux,faux,vrai,faux},/* D-D :15*/
    };
  unsigned char mask_value;
  Point current_pt;
  own_seg seg;
  int i;
  bouleen same;

  mask_value  = mask1[prec_move];
  mask_value |= mask2[next_move];
  
  if (!is_valid[mask_value][trying_move])
    return;

  if(contour_set_direction_belong_segment(contours,p,trying_move))
    return;

  AFFECTE_POINT(current_pt,*p);
  i=0;
  while (i<MAX_SEGMENT_LENGHT)
    {
      i++;
      modify_point_position(&current_pt,trying_move);      
      if (tab_mark_is_marked(marked_pts,&current_pt))	
	break;
      if(contour_set_is_in_segment(contours,&current_pt))
	break;
    }

  if (tab_mark_is_marked(marked_pts,&current_pt))       
    {
      prec_move = REVERSE_SENSE(prec_move);
      same = contour_set_direction_belong_segment(contours,
						  &current_pt,
						  prec_move);
      same = same&&contour_set_direction_belong_segment(contours,
							&current_pt,
							next_move);
      if(!same)
	{
	  seg = init_seg_priv(p,trying_move,i);
	  list_gen_insert(ins_seg,seg);
	}
    }

}
static own_seg init_seg_priv(point p,
			     move_type dir,
			     int lengh)
{
  own_seg seg;
  
  seg = (own_seg) malloc(sizeof(struct own_seg));
  
  AFFECTE_POINT(seg->pt,*p);
  seg->move = dir;
  seg->lengh = lengh;

  return seg;
}
static void insert_our_seg_priv(contour_set contours,
				own_seg our_seg,
				tab_mark marked_pts)
{
  int i;
  Point p;
  move_type dir;
  seg_tmp seg;

  seg = FAIRE_seg_tmp();
  dir = our_seg->move;
  AFFECTE_POINT(p,our_seg->pt);
  i =0;
  while(i<our_seg->lengh)
    {
      seg_tmp_insert_segment1(seg,&p,dir);
      modify_point_position(&p,dir);
      if(tab_mark_is_marked_two(marked_pts,&p))
	{
	  contour_set_add_new_contour(contours,seg);
	  seg_tmp_reinitialize(seg);
	}
      i++;
      tab_mark_mark_point_two(marked_pts,&p);
    }
  if(!seg_tmp_is_empty(seg))
    contour_set_add_new_contour(contours,seg);
      
  DEFAIRE_seg_tmp(seg);
}
static void mark_face_priv(contour_set contours,
			   dart_list seg_list,
			   tab_mark pts_to_mark)
{
  dart b;
  output_segment seg;
  
  seg = FAIRE_output_segment();

  dart_list_first(seg_list);
  do
    {
      b = dart_list_give_current(seg_list);
      contour_set_give_segment(contours,b,seg);
      mark_seg_priv(seg,pts_to_mark);
    }
  while(dart_list_next(seg_list));
}
static void mark_seg_priv(output_segment seg,tab_mark pts_to_mark)

{
  Point p;
 
 output_segment_first(seg);
  do
    {
      output_segment_give_current_point(seg,&p);
      tab_mark_mark_point(pts_to_mark,&p);
    }
  while(output_segment_next(seg));

  output_segment_give_current_point(seg,&p);
  tab_mark_mark_point(pts_to_mark,&p);
}
static void remove_piece_of_face_priv(contour_set contours,
				      dart f,
				      tab_mark_dart broken_faces,
				      dart_list b_to_remove)
{
  dart_list b_list;
  dart seg;
  dart neighbour,selected_face;
  vector f_mean,neighbour_mean;
  double dist,min = DBL_MAX;
  int dim;

  contour_set_remove_double_arc_of_face(contours,f);

  dim = contour_set_give_image_dim(contours);
  contour_set_give_face_mean(contours,f,f_mean);
  b_list = FAIRE_dart_list();
  contour_set_give_segments_of_face(contours,f,b_list);
  
  dart_list_first(b_list);
  do
    {
      seg = dart_list_give_current(b_list);
      if(contour_set_face_has_neighbour(contours,f,seg))
	{
	  neighbour = contour_set_give_face_neighbour(contours,f,seg);
	  if(!tab_mark_dart_is_marked(broken_faces,neighbour))
	    {
	      contour_set_give_face_mean(contours,neighbour,neighbour_mean);
	      dist = vector_dist(f_mean,neighbour_mean,dim);
	      if(dist < min)
		{
		  min = dist;
		  selected_face = neighbour;
		}
	    }
	}
    }
  while (dart_list_next(b_list));
  
  dart_list_first(b_list);
  do
    {
      seg = dart_list_give_current(b_list);
      if(contour_set_face_has_neighbour(contours,f,seg))
	{
	  neighbour = contour_set_give_face_neighbour(contours,f,seg);
	  if(neighbour == selected_face)
	    dart_list_insert(b_to_remove,seg);
	}
    }  
  while (dart_list_next(b_list));
  
  DEFAIRE_dart_list(b_list);
}
static void smooth_rule_priv(contour_set contours,
			     dart f,
			     va_list list)
{

  smooth_face(contours,f);
}
void smooth_rule_set_algorithm(rule father)
{
  father->specified_rule = smooth_rule_priv;
}









