/*  descartes
 *  Copyright (C) 2001 Luc Brun
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <assert.h>
#include <glib.h>

#include "map.h"

#include "tab_mark_dart.h"
#include "dart_list.h"
#include "macros.h"

#define  Phi             SELF (phi)
#define  Lambda          SELF (lambda)
#define  Map_dim         SELF (map_dim)
#define  Mark            SELF (mark)
#define  Free_dart_list  SELF (free_dart_list)
#define  Label_list      SELF (label_list)
#define  Label_dim       SELF (label_dim)
#define  Label_to_dart   SELF (label_to_dart)
#define  Free_lambda     SELF (free_lambda)

#define  Sigma(b)        Phi[-b]

#define ALLOC_MAP 2000

static void realloc_priv(map);
static int  alloc_dart_priv (map);
static int  alloc_lambda_priv(map, dart);
static void free_lambda_priv(map, dart);
static void set_lambda_cycle_priv(map, int, dart);

struct map
{
  int           *phi;
  int           *lambda;
  tab_mark_dart  mark;
  int           *label_to_dart;
  int            map_dim;
  dart_list      free_dart_list;
  dart_list      label_list;
  tab_mark_dart  free_lambda;
  int            label_dim;
};

map  FAIRE_map()
{
  int i;
  int alloc_dim = ALLOC_MAP * 2 + 1;

  INSTANCIER(map);

  Phi     = (int *)  calloc (alloc_dim, sizeof (int));
  Lambda  = (int *)  calloc (alloc_dim, sizeof (int));
  Mark    = FAIRE_tab_mark_dart();

  Map_dim = ALLOC_MAP;


  Phi    += ALLOC_MAP;
  Lambda += ALLOC_MAP;


  Free_dart_list = FAIRE_dart_list();
  Label_list     = FAIRE_dart_list();
  Free_lambda      = FAIRE_tab_mark_dart();

  Label_dim      = ALLOC_MAP;

  Label_to_dart  = (int *) calloc (ALLOC_MAP + 2 , sizeof(int));
  Label_to_dart++;

  dart_list_insert(Free_dart_list, 1);
  dart_list_insert(Label_list, 1);
  
  for (i = ALLOC_MAP; i > 1; --i) 
    {
      /* penser a descendre cette initialisation ds dart_list */
      dart_list_insert(Free_dart_list, i);
      dart_list_insert(Label_list, i);
    }

  dart_list_remove_elt(Free_dart_list,1);
  dart_list_remove_elt(Label_list,1);
  
  dart_list_first(Free_dart_list);
  dart_list_first(Label_list);
  
  Phi [1]            =  1;
  Phi [-1]           = -1;
  Lambda [1]         =  1;
  Lambda [-1]        = -1;
  Label_to_dart [1]  =  1;
  Label_to_dart [-1] = -1;

  return self;
}

/*---------------------------------------------------------------------------*/

void DEFAIRE_map(map self)
{
  int *ptr;
  
  ptr = Phi    - Map_dim;  free (ptr);
  ptr = Lambda - Map_dim;  free (ptr);  
  ptr = Label_to_dart - 1; free (ptr);
  
  DEFAIRE_tab_mark_dart(Mark);
  DEFAIRE_dart_list(Free_dart_list);
  DEFAIRE_dart_list(Label_list);
  DEFAIRE_tab_mark_dart(Free_lambda);
  free(self);
}

/*---------------------------------------------------------------------------*/

void map_merge_lambda(map self, dart label, dart old_label)
{
  dart old_dart;

  old_dart = Label_to_dart[old_label];

  set_lambda_cycle_priv(self, label, old_dart);
  free_lambda_priv(self, old_label);
}

/*---------------------------------------------------------------------------*/

dart map_change_lambda_cycle(map self, dart b)
{
  dart new_label = alloc_lambda_priv(self,b);
  
  set_lambda_cycle_priv(self,new_label,b);

  return new_label;
}

/*---------------------------------------------------------------------------*/

void map_remove_label(map self, dart b)
{
  free_lambda_priv(self,b);
}


dart map_give_sigma(map self, dart num)     
{
  return Sigma(num);
}

/*---------------------------------------------------------------------------*/

dart map_give_sigma_1(map self, dart num)
{
  dart sigma_1 = Sigma(num);

  while(Sigma(sigma_1) != num) sigma_1 = Sigma(sigma_1);

  return sigma_1;
}

/*---------------------------------------------------------------------------*/

dart map_give_phi(map self, dart num)
{
  return Phi [num];
}

/*---------------------------------------------------------------------------*/

dart map_give_phi_1(map self, dart num)     
{
  dart sigma_1 = map_give_sigma_1(self,num);

  return -sigma_1; 
  
}

/*---------------------------------------------------------------------------*/

dart map_give_lambda(map self, dart num)
{
  return  Lambda [num];
}
/*---------------------------------------------------------------------------*/

int map_give_lambda_1(map self, dart label)
{
  return Label_to_dart[label];
}
/*---------------------------------------------------------------------------*/
void map_set_lambda_1(map self, dart label, dart b)
{
  Label_to_dart[label] = b;
}

/*---------------------------------------------------------------------------*/
/* Ajout d'un noeud Voir thse p 98*/
/*---------------------------------------------------------------------------*/
dart map_split(map self, dart num)
{
  int new_num;
  dart phi_1 = map_give_phi_1(self,-num);

  new_num = alloc_dart_priv(self);
  
  Phi [phi_1]        = -new_num;
  Phi [new_num]      = Phi [num];
  Phi [-new_num]     = -num;
  Phi [num]          = new_num;
  
  Lambda[new_num]  = Lambda [num];
  Lambda[-new_num] = Lambda [-num];
  
  return new_num;
}

/*---------------------------------------------------------------------------*/
/* Insere un arc dans la carte p 78 */
/*---------------------------------------------------------------------------*/
dart map_insert_segment(map self,dart negative_dart, dart positive_dart)
{
  int new_num;
  dart label1,label2;

  label1 = Lambda [-negative_dart];
  label2 = Lambda [-positive_dart];

  new_num = alloc_dart_priv(self);
  
  Phi [new_num]                = Phi[-positive_dart];
  Phi [-new_num]               = Phi[-negative_dart] ;
  Phi [-negative_dart]         = new_num;
  Phi [-positive_dart]         = -new_num;
  
  Lambda [new_num]  = label1;
  Lambda [-new_num] = label2;

  /* evite qu'un label to dart devienne faux quand on renumerote une */
  /* face */
  Label_to_dart[label1] = new_num;
  Label_to_dart[label2] = -new_num;

  return new_num;
}

/*---------------------------------------------------------------------------*/
/* Insertion d'une composante connexe tangeante a un segment en un */
/* noeud. */
dart map_insert_tangeant(map self,dart add_dart)
{
  dart new_num,b_temp;
  
  new_num  = alloc_dart_priv(self);

  b_temp = map_give_sigma_1(self,add_dart);
  
  Phi [-add_dart] = -new_num;
  Phi [new_num]   = new_num;
  Phi [-new_num]  = b_temp;
  
  
  Lambda [new_num]  = alloc_lambda_priv(self,new_num);
  Lambda [-new_num] = Lambda [-add_dart];
  
  return new_num;
}

/*---------------------------------------------------------------------------*/

dart_list map_give_cycle_phi(map self,dart b)
{
  dart      current_dart;
  dart_list list;
  
  list = FAIRE_dart_list();
  
  current_dart = b;

  do 
    {
      dart_list_insert(list,current_dart);
      current_dart = Phi [current_dart];
    }
  while (current_dart != b);
  
  return list;
}

/*---------------------------------------------------------------------------*/
dart_list map_give_cycle_sigma(map self, dart b)
{
  dart      current_dart;
  dart_list list;
  
  list = FAIRE_dart_list();
  current_dart = b;
  
  do 
    {
      dart_list_insert(list,current_dart);
      current_dart = Sigma(current_dart);
    }
  while (current_dart != b);
  return list;
}
/*---------------------------------------------------------------------------*/

int map_give_card_cycle_sigma(map self,dart b)
{
  dart current_dart;
  int  cardinal;
  
  cardinal =0;
  current_dart = b;

  do 
    {
      cardinal++;
      current_dart = Sigma(current_dart);
    }
  while (current_dart != b);
  
  return cardinal;
}

/*---------------------------------------------------------------------------*/
/* ajout d' une composante connexe */
/*---------------------------------------------------------------------------*/
dart map_insert_map(map self)
{
  dart b = alloc_dart_priv(self);

  Phi [b]     =  b;
  Phi [-b]    = -b;

  Lambda [b]  =  alloc_lambda_priv(self, b);
  Lambda [-b] =  alloc_lambda_priv(self, -b);
  
  return b;
}
/*---------------------------------------------------------------------------*/
void map_set_mark(map self, dart b)
{
  dart b_abs;
  int dim_max;

  assert(self);
  assert(b != 0);
  /* On verifie que l'on ne tape pas n'importe ou */
  /* A supprimer un jour */

  b_abs   = ABS(b);
  dim_max = MAX(Map_dim,Label_dim);

  assert(b_abs < dim_max);

  tab_mark_dart_mark_dart(Mark,b);
}
/*---------------------------------------------------------------------------*/
bouleen map_give_mark(map self,dart b)
{
  dart b_abs;
  int dim_max;

  assert(self);
  assert(b != 0);
  /* On verifie que l'on ne tape pas n'importe ou */
  /* A supprimer un jour */
  b_abs = ABS(b);
  dim_max = MAX(Map_dim,Label_dim);

  assert(b_abs < dim_max);
  
  return tab_mark_dart_is_marked(Mark,b);
}
/*---------------------------------------------------------------------------*/
void map_remove_mark(map self,dart b)
{
  dart b_abs;
  int dim_max;
  
  assert(self);
  assert(b != 0);
  /* On verifie que l'on ne tape pas n'importe ou */
  /* A supprimer un jour */

  b_abs = ABS(b);
  dim_max = MAX(Map_dim,Label_dim);

  assert(b_abs < dim_max);
  
  tab_mark_dart_unmark_dart(Mark,b);
}
/*---------------------------------------------------------------------------*/
void map_remove_all_mark(map self)
{
  tab_mark_dart_reinitialize(Mark);
}
/*---------------------------------------------------------------------------*/
/* Supression d'un arc p 88 */
/*---------------------------------------------------------------------------*/
void map_remove_edge(map self,dart edge)
{
  dart temp;
  dart label1 = Lambda[edge];
  dart label2 = Lambda[-edge];

  edge = edge > 0? edge: -edge ;
  
  Lambda[edge] = Lambda[-edge] = 0;
  dart_list_insert(Free_dart_list, edge);

  if( Label_to_dart[label1] == edge )
    Label_to_dart[label1] = Phi[edge];

  if( Label_to_dart[label2] == -edge )
    Label_to_dart[label2] = Phi[-edge];
  
  if(Phi[edge] != edge)
    {
      temp = map_give_phi_1(self,edge);
      Phi[temp] = Phi[-edge];
      
      temp      = map_give_phi_1(self,-edge);
      Phi[temp] = Phi[edge];
      
    }
  Phi[edge]   = Phi[-edge]    = 0;

}
void map_remove_map(map self,dart edge)
{
  dart temp;
  
  edge = edge > 0? edge: -edge ;
  
  free_lambda_priv(self, Lambda[edge]);
  free_lambda_priv(self, Lambda[-edge]);

  Lambda[edge] = Lambda[-edge] = 0;
  dart_list_insert(Free_dart_list, edge);
  
  Phi[edge]   = Phi[-edge]    = 0;
}
/* Suppression d'un noeud, Voir page 88 */
/* On utilise la propriete sigma^2(b)=b ou ce qui est equivalent */
/* phi(-phi(-b)) = b ==> phi-1(b) = -phi(-b) */
void map_remove_node(map self, dart b)
{
  dart brin_pos = ABS(b);
  dart phi_1_b = -Phi[-b];
  dart phi_1_moins_b = map_give_phi_1(self,-b);
  dart label1 = Lambda[b];
  dart label2 = Lambda[-b];

  if(Phi[phi_1_b] != b)
    {
      fprintf(stderr,"essai d'effacement d'un noeud d'arite superieure a 2\n");
      assert(Phi[phi_1_b] == b);
    }
  dart_list_insert(Free_dart_list,brin_pos);
  
  Phi[phi_1_moins_b]    = Phi[-b];
  Phi[phi_1_b] = Phi[b];

  if( Label_to_dart[label1] ==b )
    Label_to_dart[label1] = phi_1_b;

  if( Label_to_dart[label2] == -b )
    Label_to_dart[label2] = phi_1_moins_b;

}
void map_remove_tangeant(map self,dart b)
{
  dart b1,b2;
  dart f;
  dart brin_pos = ABS(b);

/* b1, b2 vont contenir les 2 autres brins */
  b1 = Phi[b];
  b2 = Phi[-b];

  assert((b1==b)||(b2==-b));
/* On a Phi[b] == b ou Phi[-b] == -b On en tient compte pour aller */
/* chercher le brin qui nous manque */
  if(b1 ==b)
    {
      b1 = Phi[-b2];
      f = Lambda[b];
      Label_to_dart[Lambda[-b]] = b2;
    }
  else
    {
      b2 = Phi[-b1];
      f = Lambda[-b];
      Label_to_dart[Lambda[b]] = b1;
    }

  Phi[-b2] = b1;
  Phi[-b1] = b2;

  dart_list_insert(Free_dart_list,brin_pos);
  free_lambda_priv(self,f);

  Phi[b]=Phi[-b] = 0;
  Lambda[b] = Lambda[-b] = 0;  
}
dart_list map_give_all_faces(map self)
{
  dart_list all_face;
  int i;

  assert(self);
  
  all_face = FAIRE_dart_list();

  for(i=1;i<Label_dim;i++)
    if(Label_to_dart[i] != 0)
      dart_list_insert(all_face,i);

  return all_face;
}
void map_mark_face(map self,dart f)
{
  dart b,b_first;

  b_first = Label_to_dart[f];
  b = b_first;
  do
    {
      tab_mark_dart_mark_dart(Mark,b);
      b = Phi[b];
    }
  while(b != b_first);
}
void map_unmark_face(map self,dart f)
{
  dart b,b_first;

  b_first = Label_to_dart[f];
  b = b_first;
  do
    {
      tab_mark_dart_unmark_dart(Mark,b);
      b = Phi[b];
    }
  while(b != b_first);
}
static void realloc_priv(map self)
{
  int *temp_1, *temp_2,new_size,  i;
  int dep;

  dep      =  Map_dim + ALLOC_MAP;
  new_size =  dep * 2;


  temp_1 = (int *) malloc ((new_size +1)*sizeof(int));
  temp_1 += dep;
  
  temp_2 = (int *) malloc ((new_size +1)*sizeof(int));
  temp_2 += dep;
  
  for (i = 1; i <= Map_dim; i++) 
    {
      temp_1 [i]  = Phi    [i];
      temp_1 [-i] = Phi    [-i];
      temp_2 [i]  = Lambda [i];
      temp_2 [-i] = Lambda [-i];
    }
  
  dart_list_insert(Free_dart_list,Map_dim+1);

  for (i = Map_dim + ALLOC_MAP; i > Map_dim + 1; --i)
    {
      temp_1[i]= temp_1[-i] =0;
      temp_2[i]= temp_2[-i] =0;
      dart_list_insert(Free_dart_list, i);
    }
  
  free (Phi    - Map_dim);
  free (Lambda - Map_dim);
  
  Phi      = temp_1;
  Lambda   = temp_2;
  
  Map_dim += ALLOC_MAP;
}

static int alloc_dart_priv (map self)
{
  int ret_val;

  if( dart_list_is_empty(Free_dart_list) ) 
    realloc_priv(self);

  dart_list_first(Free_dart_list);
  ret_val = dart_list_give_current(Free_dart_list);

  dart_list_remove_elt(Free_dart_list, ret_val);

  return ret_val;
}



static int alloc_lambda_priv(map self, dart d)
{
  int i, new_label;

  if( dart_list_is_empty(Label_list) )
    {
      Label_to_dart = (int *) realloc(Label_to_dart -1 , (Label_dim + ALLOC_MAP + 2) * sizeof(int));
      Label_to_dart++;
      dart_list_insert(Label_list,Label_dim+1);
      for(i = Label_dim + ALLOC_MAP; i > Label_dim +1 ; i--)
	{
	  dart_list_insert(Label_list,i);
	  Label_to_dart[i] = 0;
	}
      Label_dim += ALLOC_MAP;
    }
  dart_list_first(Label_list);
  new_label = dart_list_give_current(Label_list);

  tab_mark_dart_unmark_dart(Free_lambda,new_label);
  Label_to_dart[new_label] = d;
  dart_list_remove_elt(Label_list, new_label);

  return new_label;
}



static void free_lambda_priv(map self, dart label)
{

  Label_to_dart[label] = 0;

  if( !tab_mark_dart_is_marked(Free_lambda,label))
    {
      tab_mark_dart_mark_dart(Free_lambda,label);
      dart_list_insert(Label_list, label);
    }
}


static void set_lambda_cycle_priv(map self, int label, dart d)
{
  int current_dart;

  current_dart = d;
  
  do 
    {
      Lambda [current_dart] = label;
      current_dart = Phi [current_dart];
    }
  while (current_dart != d);
}















