/*  descartes
 *  Copyright (C) 2001 Luc Brun
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <float.h>
#include <assert.h>

#include "rule.h"
#include "tab_mark_dart.h"

#define VAR_BREAK_MERGE_THRESHOLD     1.5
#define AREA_CONTINUE_MERGE_THRESHOLD 0.1
static  void merge_face_by_gradiant_rule_priv(contour_set,
					      dart,
					      va_list);
static void merge_face_by_mean_rule_priv     (contour_set,
					      dart,
					      va_list);

static dart select_face_priv                (contour_set,
					     dart_list,
					     vector,
					     double*);
static dart select_face_of_min_SE_priv      (contour_set,
					     dart_list,
					     vector,
					     double,
					     double*);

static void give_border_priv                (contour_set,
					     dart_list,
					     dart,
					     dart_list);

static void inspect_included_faces_priv     (contour_set,
					     dart,
					     dart_list,
					     dart_list);

static void get_intersection_priv           (contour_set,
					     dart,
					     dart,
					     dart_list);

void merge_face_by_mean_give_segments_to_delete(contour_set contours,
						dart f,
						dart_list b_to_remove)
{
  dart_list b_list;
  dart seg;
  dart neighbour,selected_face;
  vector f_mean,neighbour_mean;
  double dist,min = DBL_MAX;
  int dim;

  contour_set_remove_double_arc_of_face(contours,f);

  dim = contour_set_give_image_dim(contours);
  contour_set_give_face_mean(contours,f,f_mean);
  b_list = FAIRE_dart_list();
  contour_set_give_segments_of_face(contours,f,b_list);
  
  dart_list_first(b_list);
  do
    {
      seg = dart_list_give_current(b_list);
      if(contour_set_face_has_neighbour(contours,f,seg))
	{
	  neighbour = contour_set_give_face_neighbour(contours,f,seg);
	  contour_set_give_face_mean(contours,neighbour,neighbour_mean);
	  dist = vector_dist(f_mean,neighbour_mean,dim);
	  if(dist < min)
	    {
	      min = dist;
	      selected_face = neighbour;
	    }
	}
    }
  while (dart_list_next(b_list));
  
  dart_list_first(b_list);
  do
    {
      seg = dart_list_give_current(b_list);
      if(contour_set_face_has_neighbour(contours,f,seg))
	{
	  neighbour = contour_set_give_face_neighbour(contours,f,seg);
	  if(neighbour == selected_face)
	    dart_list_insert(b_to_remove,seg);
	}
    }  
  while (dart_list_next(b_list));
  
  DEFAIRE_dart_list(b_list);
}
void merge_face_get_intersection(contour_set contours,
				 dart f1,
				 dart f2,
				 dart_list intersect)
{
  get_intersection_priv(contours,f1,f2,intersect);
  if(dart_list_is_empty(intersect))
    get_intersection_priv(contours,f2,f1,intersect);
}
dart merge_face(contour_set contours,
		dart f1,
		dart f2)
{
  
  dart_list seg_to_del;
  
  seg_to_del = FAIRE_dart_list();
  
  merge_face_get_intersection(contours,f1,f2,seg_to_del);
  
  contour_set_remove_set_of_arc(contours,seg_to_del);

  if(contour_set_is_face_defined(contours,f1))
    {
      if(contour_set_is_infinite(contours,f1))
	return f2;

      return f1;
    }
  return f2;
}
void merge_face_by_mean(contour_set contours,
			dart f,
			list_gen removed_seg)
{
  dart_list b_to_remove;
  
  b_to_remove = FAIRE_dart_list();

  merge_face_by_mean_give_segments_to_delete(contours,f,b_to_remove);

  contour_set_memorize_modified_segments(contours);
  contour_set_remove_set_of_arc(contours,b_to_remove);
  contour_set_give_modified_segments(contours,removed_seg);
  
  DEFAIRE_dart_list(b_to_remove);
}

void merge_face_by_gradiant(contour_set contours,dart f)
{
  dart_list seg_list;
  dart seg,seg_to_remove;
  double grad,min = DBL_MAX;
  
  assert(contours);
  
  seg_list = FAIRE_dart_list();
  contour_set_give_segments_of_face(contours,f,seg_list);
  dart_list_first(seg_list);
  do
    {
      seg = dart_list_give_current(seg_list);
      if(contour_set_face_has_neighbour(contours,f,seg))
	{
	  grad = contour_set_give_segment_gradiant(contours,seg);
	  if(grad < min)
	    {
	      min = grad;
	      seg_to_remove = seg;
	    }
	}
    }
  while(dart_list_next(seg_list));

  contour_set_remove_arc(contours,seg_to_remove);
  DEFAIRE_dart_list(seg_list);
}
void move_face_to_specified_area(contour_set contours,
				 dart f,
				 guint area,
				 double root_var,
				 tab_mark_dart merge_faces,
				 dart_list seg_to_del,
				 point meta_face_pt,
				 bouleen* new_meta_face)
{
  double face_list_area;
  vector face_list_mean,face_list_m1;
  double face_to_add_area;
  vector face_to_add_m1;
  dart_list face_list,added_faces;
  dart face_to_add;
  int dim;
  double dist;
  

  if(tab_mark_dart_is_marked(merge_faces,f))
    {
      *new_meta_face = faux;
      return;
    }

  *new_meta_face = vrai;
  contour_set_assoc_face_point(contours,f,meta_face_pt);

  dim = contour_set_give_image_dim(contours);
  
  face_list   = FAIRE_dart_list();
  added_faces = FAIRE_dart_list();
  dart_list_insert(face_list,f);
  tab_mark_dart_mark_dart(merge_faces,f);
  face_list_area = contour_set_give_face_area(contours,f);
  contour_set_give_face_mean(contours,f,face_list_mean);
  contour_set_give_face_moment1(contours,f,face_list_m1);
  
  while(face_list_area < area)
    {
      dart_list_reinitialize(added_faces);
      contour_set_complete_face_adjacence(contours,face_list,added_faces);
      
      face_to_add      = select_face_priv(contours,
					  added_faces,
					  face_list_mean,
					  &dist);

      if ((dist > VAR_BREAK_MERGE_THRESHOLD*root_var)&&
	  (face_list_area > AREA_CONTINUE_MERGE_THRESHOLD*area))
	  break;	  
      face_to_add_area = contour_set_give_face_area(contours,face_to_add);



      contour_set_give_face_moment1(contours,
				    face_to_add,
				    face_to_add_m1);
      dart_list_insert(face_list,face_to_add);
      vector_equal_plus(face_list_m1,face_to_add_m1,dim);
      face_list_area += face_to_add_area;
      vector_divide(face_list_mean,face_list_m1,face_list_area,dim);
      give_border_priv(contours,face_list,face_to_add,seg_to_del);
      inspect_included_faces_priv(contours,face_to_add,face_list,seg_to_del);

      if(tab_mark_dart_is_marked(merge_faces,face_to_add))
	{
	  *new_meta_face = faux;
	  break;	  
	}
      tab_mark_dart_mark_dart(merge_faces,face_to_add);
    }

  DEFAIRE_dart_list(added_faces);
  DEFAIRE_dart_list(face_list);
}
void move_face_to_specified_squared_error(contour_set contours,
					  dart f,
					  double threshold,
					  tab_mark_dart merge_faces,
					  dart_list seg_to_del)
{
  double face_list_area;
  vector face_list_mean,face_list_m1;
  double face_list_SE;
  double face_to_add_area;
  vector face_to_add_m1;
  double SE;
  dart_list face_list,added_faces;
  dart face_to_add;
  int dim;
  

  if(tab_mark_dart_is_marked(merge_faces,f))
    return;


  dim = contour_set_give_image_dim(contours);
  
  face_list   = FAIRE_dart_list();
  added_faces = FAIRE_dart_list();
  dart_list_insert(face_list,f);
  tab_mark_dart_mark_dart(merge_faces,f);

  face_list_area = contour_set_give_face_area(contours,f);
  face_list_SE = contour_set_give_face_squared_error(contours,f);
  
  contour_set_give_face_mean(contours,f,face_list_mean);
  contour_set_give_face_moment1(contours,f,face_list_m1);

  
  while(face_list_SE < threshold)
    {
      dart_list_reinitialize(added_faces);
      contour_set_complete_face_adjacence(contours,face_list,added_faces);
      
      face_to_add      = select_face_of_min_SE_priv(contours,
						    added_faces,
						    face_list_mean,
						    face_list_area,
						    &SE);
      
      if(tab_mark_dart_is_marked(merge_faces,face_to_add)&& 
	 (face_list_SE> threshold/100.0))
	break;
      if (face_list_SE + SE >= threshold)
	break;	  
      /* On met a jour les params */
      face_to_add_area = contour_set_give_face_area(contours,face_to_add);
      contour_set_give_face_moment1(contours,
				    face_to_add,
				    face_to_add_m1);
      vector_equal_plus(face_list_m1,face_to_add_m1,dim);      
      face_list_area += face_to_add_area;
      vector_divide(face_list_mean,face_list_m1,face_list_area,dim);
      face_list_SE += SE;
      /* On efface (symboliquement) les segments frontieres */
      dart_list_insert(face_list,face_to_add);
      give_border_priv(contours,face_list,face_to_add,seg_to_del);
      inspect_included_faces_priv(contours,face_to_add,face_list,seg_to_del);

      if(tab_mark_dart_is_marked(merge_faces,face_to_add))
	break;
      tab_mark_dart_mark_dart(merge_faces,face_to_add);
    }
    printf("En Sortie  face_list_SE et threshold %lf, %lf\n",face_list_SE,threshold);
  DEFAIRE_dart_list(added_faces);
  DEFAIRE_dart_list(face_list);
}
void merge_same_color_face(contour_set contours,
			   dart f,
			   double threshold,
			   dart_list merge_faces,
			   dart_list seg_to_del)
{
  dart_list added_faces;
  dart face_to_add;
  vector merge_faces_mean,merge_faces_m1;
  vector face_to_add_m1;
  double merge_faces_area,face_to_add_area;
  double dist;
  int dim;

  dim = contour_set_give_image_dim(contours);
  added_faces = FAIRE_dart_list();

  merge_faces_area = contour_set_give_face_area(contours,f);
  contour_set_give_face_mean   (contours,f,merge_faces_mean);
  contour_set_give_face_moment1(contours,f,merge_faces_m1);
  dart_list_insert(merge_faces,f);
  do
    {
      dart_list_reinitialize(added_faces);
      contour_set_complete_face_adjacence(contours,merge_faces,added_faces);
      
      face_to_add = select_face_priv(contours,
				     added_faces,
				     merge_faces_mean,
				     &dist);
      if(dist > threshold)
	break;

      face_to_add_area = contour_set_give_face_area(contours,face_to_add);

      contour_set_give_face_moment1(contours,
				    face_to_add,
				    face_to_add_m1);

      dart_list_insert(merge_faces,face_to_add);
      vector_equal_plus(merge_faces_m1,face_to_add_m1,dim);
      merge_faces_area += face_to_add_area;
      vector_divide(merge_faces_mean,merge_faces_m1,merge_faces_area,dim);
      give_border_priv(contours,merge_faces,face_to_add,seg_to_del);
      inspect_included_faces_priv(contours,face_to_add,merge_faces,seg_to_del);
    }
  while(dist < threshold);
}
static dart select_face_priv(contour_set contours,
			     dart_list set_of_face,
			     vector main_mean,
			     double * min_dist)
{
  dart f,selected_face;
  vector f_mean;
  double dist,min = DBL_MAX;
  int dim;

  assert(!dart_list_is_empty(set_of_face));

  dim = contour_set_give_image_dim(contours);
  dart_list_first(set_of_face);
  do
    {
      f = dart_list_give_current(set_of_face);
      contour_set_give_face_mean(contours,f,f_mean);
      dist = vector_dist(f_mean,main_mean,dim);
      if(dist < min)
	{
	  selected_face = f;
	  min = dist;
	}
    }
  while(dart_list_next(set_of_face));

  *min_dist = min;
  return selected_face;
}
static dart select_face_of_min_SE_priv(contour_set contours,
				       dart_list set_of_face,
				       vector main_mean,
				       double main_area,
				       double * SE)
{
  dart f,selected_face=-1;
  vector f_mean;
  double f_se,f_area;
  double alpha,dist,current_SE,min = DBL_MAX;
  int dim;
  
  assert(!dart_list_is_empty(set_of_face));

  dim = contour_set_give_image_dim(contours);
  dart_list_first(set_of_face);
  do
    {
      f = dart_list_give_current(set_of_face);
      f_area = contour_set_give_face_area(contours,f);
      f_se  = contour_set_give_face_squared_error(contours,f);      
      contour_set_give_face_mean(contours,f,f_mean);
      alpha = f_area*main_area/(f_area+main_area);
      dist = vector_dist(f_mean,main_mean,dim);
      current_SE =  f_se + alpha*dist;
      if(current_SE < min)
	{
	  selected_face = f;
	  min = current_SE;	  
	}
    }
  while(dart_list_next(set_of_face));

  *SE = min;
  return selected_face;
}
void  merge_face_give_border(contour_set contours,
			     tab_mark_dart meta_face,
			     dart f,
			     dart_list set_of_seg)
{
  dart_list seg_of_face;
  dart seg;
  dart neighbour;
  
  seg_of_face = FAIRE_dart_list();
  
  contour_set_give_segments_of_face(contours,f,seg_of_face);
  
  dart_list_first(seg_of_face);
  do
    {
      seg = dart_list_give_current(seg_of_face);
      if(contour_set_face_has_neighbour(contours,f,seg))
	{
	  neighbour = contour_set_give_face_neighbour(contours,f,seg);
	  if(tab_mark_dart_is_marked(meta_face,neighbour))
	    dart_list_insert(set_of_seg,seg);
	}
    }
  while(dart_list_next(seg_of_face));
  
  DEFAIRE_dart_list(seg_of_face);
} 
static void give_border_priv(contour_set contours,
			     dart_list face_list,
			     dart f,
			     dart_list seg_list)
{
  dart_list seg_of_face;
  dart seg;
  dart neighbour;

  seg_of_face = FAIRE_dart_list();
  
  contour_set_give_segments_of_face(contours,f,seg_of_face);
  
  dart_list_first(seg_of_face);
  do
    {
      seg = dart_list_give_current(seg_of_face);
      if(contour_set_face_has_neighbour(contours,f,seg))
	{
	  neighbour = contour_set_give_face_neighbour(contours,f,seg);
	  if(dart_list_is_in(face_list,neighbour))
	    dart_list_insert(seg_list,seg);
	}
    }
  while(dart_list_next(seg_of_face));
  
  DEFAIRE_dart_list(seg_of_face);
}
static void inspect_included_faces_priv(contour_set contours,
					dart f,
					dart_list face_list,
					dart_list seg_list)
{
  dart_list included_faces;
  dart ext_face;

  included_faces = FAIRE_dart_list();
  contour_set_give_included_faces(contours,f,included_faces);

  if(dart_list_is_empty(included_faces))
    {
      DEFAIRE_dart_list(included_faces);
      return;
    }

  dart_list_first(included_faces);
  do
    {
      ext_face = dart_list_give_current(included_faces);
      give_border_priv(contours,face_list,ext_face,seg_list);
    }
  while(dart_list_next(included_faces));

  DEFAIRE_dart_list(included_faces);
}
     /*********************************************************************/
    /* Donne les segments d'intersections entre f1 et f2 si f1 et f2 sont*/
   /* adjacents ou si f1 est inclus dans f2. Ne donne Pas l'intersection*/
  /* si f2 est inclus dans f1. Dans ce cas il faut reapliquer la       */
 /* fonction en inversant f1 et f2                                    */
/*********************************************************************/
static void get_intersection_priv(contour_set contours,
				  dart f1,
				  dart f2,
				  dart_list inter)
{
  dart_list f1_seg;
  dart seg;
  dart neighbour;
  
  f1_seg = FAIRE_dart_list();
  
  contour_set_give_segments_of_face(contours,f1,f1_seg);
  dart_list_first(f1_seg);
  do
    {
      seg = dart_list_give_current(f1_seg);
      if(contour_set_face_has_neighbour(contours,f1,seg))
	{
	  neighbour = contour_set_give_face_neighbour(contours,f1,seg);
	  if(neighbour == f2)
	    dart_list_insert(inter,seg);
	}
    }
  while(dart_list_next(f1_seg));
  
  DEFAIRE_dart_list(f1_seg);
}
				  
static void merge_face_by_mean_rule_priv(contour_set contours,
					 dart f,
					 va_list list)
{
  list_gen removed_seg;
  
  removed_seg = va_arg(list,list_gen);
  merge_face_by_mean(contours,f,removed_seg);
}
void merge_face_by_mean_rule_set_algorithm(rule father)
{
  father->specified_rule = merge_face_by_mean_rule_priv;
}

static  void merge_face_by_gradiant_rule_priv(contour_set contours,
					      dart f,
					      va_list list)
{
  merge_face_by_gradiant(contours,f);
}
void merge_face_by_gradiant_rule_set_algorithm(rule father)
{
  father->specified_rule = merge_face_by_gradiant_rule_priv;
}








