/*  descartes
 *  Copyright (C) 2001 Luc Brun
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <assert.h>
#include <stdarg.h>
#include <values.h>

#include "contour_set.h"
#include "cluster_set.h"
#include "rag.h"
#include "meta_seg_uniform.h"
#include "segmentation.h"
#include "type_obj.h"
#include "segmentation_link.h"
#include "macros.h"
#include "ximage.h"

static void   do_segmentation_priv              (contour_set,dart,double);
static void   move_faces_to_specified_ecart_priv(contour_set,rag,double);
static double merge_two_faces_priv              (contour_set,rag,double);
static void   mark_faces_priv                   (dart_list,tab_mark_dart);


void meta_seg_uniform_rule_set_algorithm(rule father)
{
  father->specified_rule = meta_seg_uniform;
}

void meta_seg_uniform(contour_set contours,
		      dart f,
		      va_list list)
{
  rag graph;
  double limit,ecart_max;
  dart_list face_list;
  ximage ximg;
  int couleur_normal;
  tab_mark_dart created_faces;

  ecart_max      = va_arg(list,double);
  ximg           = va_arg(list,ximage);
  couleur_normal = va_arg(list,int);

  contour_set_memorize_inserted_faces(contours);

  do_segmentation_priv(contours,f,ecart_max);

  face_list = FAIRE_dart_list();
  created_faces = FAIRE_tab_mark_dart();
  contour_set_give_inserted_faces(contours,face_list);
  dart_list_insert(face_list,f);
  mark_faces_priv(face_list,created_faces);
  
  ximage_draw_set_of_faces(ximg,contours,face_list,couleur_normal);
  ximage_display(ximg);
  ecart_max /= 100.0;
  ecart_max /= 30.0;
  
  graph = FAIRE_rag(contours,face_list,created_faces);
  DEFAIRE_dart_list(face_list);

  limit = rag_give_global_sqe(graph);
  limit *= ecart_max/4.0;
  move_faces_to_specified_ecart_priv(contours,graph,limit);
  
  DEFAIRE_rag(graph);
  DEFAIRE_tab_mark_dart(created_faces);
}
static void do_segmentation_priv(contour_set contours,dart f,double ecart_max)
{ 
  segmentation seg_alg;
  
  seg_alg = FAIRE_segmentation();
  segmentation_set_algorithm(seg_alg,algo_seg[uniform_segmentation]);
  segmentation_make_segments(seg_alg,contours,f,ecart_max);
  DEFAIRE_segmentation(seg_alg);
}

static void move_faces_to_specified_ecart_priv(contour_set contours,
					       rag graph,
					       double limit)
{
  double sqe_face;

  rag_remove_noise(graph,contours);

  if (rag_give_nb_current_faces(graph)==1)
    return;
  do
    sqe_face = merge_two_faces_priv(contours,graph,limit);
  while(sqe_face < limit && rag_give_nb_current_faces(graph) >1);
  printf("sqe_face :%f\n",sqe_face);
}
static double merge_two_faces_priv(contour_set contours,
				   rag graph,
				   double limit)
{
  dart f1,f2,f_merged;
  double f1_sqe,f2_sqe,sqe;

  rag_find_closest_node(graph,contours,&f1,&f2);
  f1_sqe = rag_give_sqe(graph,contours,f1);
  f2_sqe = rag_give_sqe(graph,contours,f2);
  sqe = MIN(f1_sqe,f2_sqe);

  if(sqe >= limit)
    return sqe;

  f_merged = rag_merge_face(graph,contours,f1,f2);
  
  return sqe;
}

static void   mark_faces_priv(dart_list face_list,
			      tab_mark_dart created_faces)
{
  dart f;

  if(dart_list_is_empty(face_list))
    return;
  
  dart_list_first(face_list);
  do
    {
      f = dart_list_give_current(face_list);
      tab_mark_dart_mark_dart(created_faces,f);
    }
  while(dart_list_next(face_list));
}


