/*  descartes
 *  Copyright (C) 2001 Luc Brun
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include "hash.h"
#include "tab_euclid.h"
#include "objet.h"
#include "classe.h"
#include "point.h"
#include "node_euclid_set.h"

typedef unsigned int xy_value;

/* tab_euclid que l'on desire mettre a jour */
static tab_euclid current_tab_euclid;

struct node_euclid
{
  Point n;
  xy_value x_val;
  xy_value y_val;
  unsigned char card;
};

CLASSE(node_euclid);

static void        update_tab_euclid_priv(hash_object);
static node_euclid alloc_node_priv       (point,node_euclid_value);

	/*  Fonctions pour le module de hashage */
static hash_anything alloc_priv(int);
static void          free_priv (hash_object);
static  int          equal_priv(hash_object,hash_object);
static int           key_priv  (hash_object);

#define Hash SELF(hash)

struct node_euclid_set
{
  hash_table hash;
};

node_euclid_set FAIRE_node_euclid_set()
{
  INSTANCIER(node_euclid_set);

  Hash = hash_create(alloc_priv,
		     free,
		     key_priv,
		     equal_priv,
		     free_priv);
  
  return self;
}
void DEFAIRE_node_euclid_set(node_euclid_set self)
{
  hash_destroy(Hash);
  free(self);
}
void node_euclid_set_reinitialize(node_euclid_set self)
{
  hash_reinitialize(Hash);
}

void node_euclid_set_store_update_node(node_euclid_set self,
				       point p,
				       node_euclid_value val) 
{ 
  struct node_euclid test_node;
  node_euclid n_euclid;
  
  assert(self); 
  assert(p);
  
  AFFECTE_POINT(test_node.n,*p);
  if(hash_position(Hash,&test_node))
    { 
      n_euclid = hash_get(Hash);
      n_euclid->x_val += val[0];
      n_euclid->y_val += val[1];
      n_euclid->card++;
      return; 
    }
  n_euclid = alloc_node_priv(p,val);
  hash_insert(Hash,n_euclid); 
}

void node_euclid_set_update_tab_euclid(node_euclid_set self, 
				       tab_euclid tab) 
{
  current_tab_euclid = tab;
  hash_map(Hash,update_tab_euclid_priv);
}
/**************************************************/
/* Fonctions privees */
/**************************************************/
static void update_tab_euclid_priv(hash_object obj)
{
  node_euclid n_euclid = (node_euclid)obj;
  x_value x;
  y_value y;
  
  tab_euclid_move_to(current_tab_euclid,&(n_euclid->n));
  x = n_euclid->x_val/(n_euclid->card);
  y = n_euclid->y_val/(n_euclid->card);
  
  tab_euclid_put_x(current_tab_euclid,x);
  tab_euclid_put_y(current_tab_euclid,y);
}


static node_euclid alloc_node_priv(point p, 
				   node_euclid_value val)
{
  node_euclid new_node;

  new_node = (node_euclid)malloc(sizeof(struct node_euclid));

  AFFECTE_POINT(new_node->n,*p);
  new_node->x_val = val[0];
  new_node->y_val = val[1];
  new_node->card = 1;
  
  return new_node;
}
static hash_anything alloc_priv(int size)
{
  hash_anything retour = malloc(size);
  return(retour);
}
static void free_priv(hash_object obj)
{
  node_euclid del_node= obj;
  free(del_node);
}
static  int equal_priv(hash_object any1,hash_object any2)
{
  node_euclid n1 = any1;
  node_euclid n2 = any2;
  
  return EGAL_POINT(n1->n,n2->n);
}
static int key_priv(hash_object obj)
{
 node_euclid key_node = (node_euclid)obj;

  return(COORDY(key_node->n)+1);
}
