/*  descartes
 *  Copyright (C) 2001 Luc Brun
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <stdio.h>
#include <string.h>
/*#include <device.h>*/
#include <math.h>
#include <sys/types.h>
#include <malloc.h>
#include <time.h>
#include <limits.h>
#include "segmente_global.h"
#include "distance.h"
#include "coul_freq.h"
#include "conversion.h"
#include "quantification.h"

/*#define DEBUG 1*/

static void  private_creer_table_couleur(image,
					 arbre_binaire,			  
					 int,
					 int);
/***********************************************************************/
/* Utilise la table de couleur pour affecter a chaque pixel sa couleur */
/* la plus proche */
/***********************************************************************/
void  quantification_map_image(arbre_binaire arbre,
			       image *table_rgb,
			       image img,
			       int nb_de_couleur,
			       int espace_choisi)
{
  int i;
  int R,G,B;
  int indice_couleur;
  int *pixel;
  int *table_cour;
  int *table_cour_rgb;
  int C[3];
  PFconversion *c1c2c3_to_rgb = revenir_espace(espace_choisi);
  PFconversion *rgb_to_c1c2c3 =changer_espace(espace_choisi);
  int* table_C1C2C3;
#ifdef IMAGE_DIST 
  cubeRGB classe,distance;
  delaunay del;
#endif
  
  table_C1C2C3 = (int*)malloc(3*nb_de_couleur*sizeof(int));
  *table_rgb  = FAIRE_image();
  image_initialize(*table_rgb,3,nb_de_couleur,1);
  
/*  *table_rgb = (Comp_rgb*)malloc(sizeof(Comp_rgb)*3*nb_de_couleur);*/
  arbre_binaire_donne_table(arbre,table_C1C2C3);   



  /* On passe la table de couleurs en RGB */
  table_cour = table_C1C2C3;
  
  image_debut(*table_rgb);
  for(i=0;i<nb_de_couleur;i++)
    {
      
      C[0] = *table_cour;
      C[1] = *(table_cour + 1);
      C[2] = *(table_cour + 2);
      
      /* On calcul et minore */
      
      R= c1c2c3_to_rgb[0](C[0],C[1],C[2]);
      G= c1c2c3_to_rgb[1](C[0],C[1],C[2]);
      B= c1c2c3_to_rgb[2](C[0],C[1],C[2]);
      
#ifdef DEBUG		    
      printf("(C1,C2,C3 = %d,%d,%d)-->(R,G,B = %d,%d,%d)\n",C[0],C[1],C[2],R,G,B);
#endif		    
      *table_cour       = R;
      *(table_cour + 1) = G;
      *(table_cour + 2) = B;
      image_ecrire_pixel(*table_rgb,table_cour);
      
      image_pixel_suivant(*table_rgb);
      table_cour +=3;
    }

#ifdef IMAGE_DIST
  printf("Je calcule les images de distance\n");
  image_dist(table_C1C2C3,nb_de_couleur,classe,distance,del);      
#endif 

  /* On passe l' image en n couleurs */
  image_debut(img);
  do
    {
      pixel = image_lire_pixel(img);
      R = *pixel;
      G = *(pixel+1);
      B = *(pixel+2);
      
#ifndef IMAGE_DIST
      C[0] = rgb_to_c1c2c3[0](R,G,B);
      C[1] = rgb_to_c1c2c3[1](R,G,B);
      C[2] = rgb_to_c1c2c3[2](R,G,B);
      arbre_binaire_chercher_couleur_ds_table(arbre,C,&indice_couleur);
#endif	
#ifdef IMAGE_DIST
      indice_couleur = classe[R>>K_REDUCTION][G>>K_REDUCTION][B>>K_REDUCTION];
#endif
      table_cour_rgb = table_C1C2C3 + 3*indice_couleur;
      
      image_ecrire_pixel(img,table_cour_rgb);
    }
  while(image_pixel_suivant(img));

  free(table_C1C2C3);


}
Comp_rgb*  quantification_creer_image_indice(image img,			     
					     arbre_binaire arbre,
					     unsigned long* cellules,
					     int espace_choisi)
{
  int i;
  int R,G,B;
  int indice_couleur;
  int *pixel;
  int C[3];
  PFconversion *rgb_to_c1c2c3 =changer_espace(espace_choisi);
  Comp_rgb*  tab,*indices;
  int largeur = image_give_largeur(img);
  int hauteur = image_give_hauteur(img);
  int dim     = image_give_dim(img);
  
  indices =(Comp_rgb*) malloc(largeur*hauteur*sizeof(Comp_rgb));
  
  /* On passe l' image en n couleurs */
  image_debut(img);
  tab = indices;
  do
    {
      pixel = image_lire_pixel(img);

      for(i=0;i<dim;i++)
	C[i] = rgb_to_c1c2c3[i](pixel[0],pixel[1],pixel[2]);

      arbre_binaire_chercher_couleur_ds_table(arbre,C,&indice_couleur);
      
      *tab = (Comp_rgb)cellules[indice_couleur];
      tab++;
    }
  while(image_pixel_suivant(img));

  return indices;
}
void quantification_creer_table(image img,
				arbre_binaire* arbre_result,
				image* table_couleur,
				int nb_couleur,
				int* nb_couleur_cree,
				int cluster_choisi,
				int axe_choisi,
				int decoupe_choisie,
				int espace_choisi,
				int init_choisi)
{
  arbre_binaire arbre;
  coul_freq c_freq;
  image   tab_coul = FAIRE_image();
  int nb_color_in_image;

  /*  Calcul_vecteur_propre(img,largeur,hauteur,espace_choisi);*/

  c_freq = FAIRE_coul_freq();
  coul_freq_initialize_from_image(c_freq,img,espace_choisi);
  nb_color_in_image = coul_freq_give_nb_of_color(c_freq);
  if(nb_color_in_image <= nb_couleur)
    nb_couleur = nb_color_in_image;

  arbre  = FAIRE_arbre_binaire(c_freq,
			       cluster_choisi,
			       axe_choisi,
			       init_choisi,
			       decoupe_choisie);
  
  arbre_binaire_creer_la_table(arbre,c_freq,nb_couleur,nb_couleur_cree);    

  arbre_binaire_etiqueter_feuilles(arbre);    
  

  private_creer_table_couleur(tab_coul,arbre,nb_couleur,espace_choisi);

  *arbre_result = arbre;
  *table_couleur = tab_coul;
  
  DEFAIRE_coul_freq(c_freq);
}
static void  private_creer_table_couleur(image table_rgb,
					 arbre_binaire arbre,
					 int nb_de_couleur,
					 int espace_choisi)
{
  int i,j;
  int C[3],RGB[3];
  PFconversion *c1c2c3_to_rgb = revenir_espace(espace_choisi);
  int* table_C1C2C3;
  int* table_cour;
  
  table_C1C2C3 = (int*)malloc(3*nb_de_couleur*sizeof(int));
  image_initialize(table_rgb,3,nb_de_couleur,1);

  arbre_binaire_donne_table(arbre,table_C1C2C3);   



  /* On passe la table de couleurs en RGB */
  table_cour = table_C1C2C3;
  
  image_debut(table_rgb);
  for(i=0;i<nb_de_couleur;i++)
    {
      for(j=0;j<3;j++)
	{
	  C[j] = *table_cour;
	  table_cour++;
	}
      
      /* On calcul et minore */
      
      RGB[0] = c1c2c3_to_rgb[0](C[0],C[1],C[2]);
      RGB[1] = c1c2c3_to_rgb[1](C[0],C[1],C[2]);
      RGB[2] = c1c2c3_to_rgb[2](C[0],C[1],C[2]);
      
#ifdef DEBUG		    
      printf("(C1,C2,C3 = %d,%d,%d)-->(R,G,B = %d,%d,%d)\n",C[0],C[1],C[2],R,G,B);
#endif		    
      image_ecrire_pixel(table_rgb,RGB);
      
      image_pixel_suivant(table_rgb);
    }
  free(table_C1C2C3);
}


