/*  descartes
 *  Copyright (C) 2001 Luc Brun
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <assert.h>
#include "seg_tmp.h"
#include "image.h"
#include "contour_set.h"
#include "move_type.h"
#include "point.h"
#include "tab_mark.h"
#include "segmente.h"
/* La majorite des fonctions utilisent le tableau de marque mark_seg. */
/* Celui ci contient les composantes connexes incluent dans la face */
/* marquees a 2. Les segments en cours d'ajouts sont quant a eux */
/* marques a un. */ 

typedef void (*PFinsert)(seg_tmp,point,move_type);

/* Definition de fonction locales au module */

static move_type next_move_priv               (contour_set,point,move_type);
static void      search_segment_on_line_priv  (point,contour_set,tab_mark);
static void      follow_contour_priv          (contour_set,
					       seg_tmp,
					       tab_mark,
					       point,
					       move_type,
					       bouleen);

static void      test_begining_of_segment_priv(contour_set,	
					       tab_mark,
					       point);

static tab_mark  alloc_tab_mark_priv          (contour_set);
static dart_list alloc_dart_list_priv         ();
static void      pass_hole_priv               (contour_set,
					    tab_mark,
					    point);
static void      mark_point_two_one_time_priv (tab_mark,
					       point);

static point_set alloc_point_set_priv(contour_set);

void segmente_do_segments(contour_set contours,dart f)
{
  int i,nb_points;
  point_set  begin_pts;
  point      begin_line;
  tab_mark   mark_seg;
  dart_list  list_of_faces;

  mark_seg = alloc_tab_mark_priv(contours);
  list_of_faces = alloc_dart_list_priv();
  begin_pts = alloc_point_set_priv(contours);

  contour_set_give_included_faces(contours,f,list_of_faces);
  contour_set_mark_set_of_face(contours,
			       list_of_faces,
			       mark_seg,
			       mark_point_two_one_time_priv
			       );

  contour_set_give_begin_points(contours,f,begin_pts);
  nb_points = point_set_give_cardinal(begin_pts);
  contour_set_mark_end_points_of_face(contours,f);

  for(i=0;i<nb_points;i++)
    {
      begin_line = point_set_give_point(begin_pts,i);

      search_segment_on_line_priv(begin_line,contours,mark_seg);
    }
}

static void search_segment_on_line_priv(point begin_line,
					contour_set contours,
					tab_mark mark_seg)
{
  Point current_pt;

  if (contour_set_end_of_line(contours,begin_line))
    return;      

  AFFECTE_POINT(current_pt,*begin_line);
  do
    {
      test_begining_of_segment_priv(contours,mark_seg,&current_pt);

      pass_hole_priv(contours,mark_seg,&current_pt);
    
      if( contour_set_end_of_line(contours,&current_pt) )
	return;      

      MOVE_RIGHT_POINT(current_pt);
    }
  while(  !contour_set_end_of_line(contours,&current_pt) );
}

static void pass_hole_priv(contour_set contours,
			   tab_mark mark_seg,
			   point p)
{

  /* On teste si on ne rencontre pas une composante connexe de la face */
  /* On diferencie ces composantes de celles que l'on vient d'ajouter */
  /* par le fait qu'elle sont marquees a 2 dans mark_seg */
  if (tab_mark_is_marked_two(mark_seg,p))
    {
      tab_mark_unmark_point_two(mark_seg,p);

      do
	{
	  MOVE_RIGHT_POINT(*p);
	}
      while (!tab_mark_is_marked_two(mark_seg,p));

      tab_mark_unmark_point_two(mark_seg,p);
    }

}  
static void  test_begining_of_segment_priv(contour_set contours,
					   tab_mark mark_seg,
					   point pt)
{
  bouleen contour_right,contour_up;
  seg_tmp seg;
  move_type move1,move2;
  bouleen part_one;

  contour_right = contour_set_direction_is_allowed(contours,pt,RIGHT);
  if(contour_right)
    contour_right = !contour_set_must_be_separated(contours,pt,RIGHT);
  contour_up = contour_set_direction_is_allowed(contours,pt,UP);
  if(contour_up)
    contour_up    = !contour_set_must_be_separated(contours,pt,UP);

  if(contour_right || contour_up)
    {
      seg  = FAIRE_seg_tmp();      	  
      if(contour_right&&contour_up)
	{
	  move1 = RIGHT;
	  move2 = UP;
	}
      else
	{
	  if(contour_right)
	    {
	      move1 = RIGHT;
	      if(!contour_set_direction_is_allowed(contours,pt,LEFT))
		move2 = DOWN;
	      else
		if(!contour_set_must_be_separated(contours,pt,LEFT) )
		  move2 = LEFT;
		else
		  move2 = DOWN;
	    }
	  else
	    /* contour UP est vrai */
	    {
	      move1 = UP;
	      if(!contour_set_direction_is_allowed(contours,pt,DOWN))
		move2 = LEFT;	      
	      else
		if(!contour_set_must_be_separated(contours,pt,DOWN) )
		  move2 = DOWN;
		else
		  move2 = LEFT;
	    }
	}
      part_one = faux;
      /* On teste si on essaye de partir sur un bout du contour */
      /* d'une face*/      
      if(contour_set_direction_is_allowed(contours,pt,move1))
	{
	  follow_contour_priv(contours,
				 seg,
				 mark_seg,
				 pt,
				 move1,
				 vrai);
	  /* Si on est parti d'un segment il faut inserer ce que l'on */
	  /* a trouver avant de chercher le reste */
	  if( contour_set_is_in_segment(contours,pt)&&
	     (seg_tmp_give_longueur(seg) != SEG_TMP_EMPTY))
	    {
	      contour_set_add_new_contour(contours,seg);
	      seg_tmp_unmark_seg(seg,tab_mark_unmark_point,mark_seg);
	      DEFAIRE_seg_tmp(seg);
	      seg = FAIRE_seg_tmp();
	      part_one = vrai;
	    }
	  /* si on a rien ajoute, il faut repartir en inserant dans 1 */
	  if (seg_tmp_give_longueur(seg) == SEG_TMP_EMPTY)
	    part_one = vrai;
	}
      else
	part_one = vrai;

      if(contour_set_direction_is_allowed(contours,pt,move2))
	follow_contour_priv(contours,
			       seg,
			       mark_seg,
			       pt,
			       move2,
			       part_one);	  

	  /* Le segment est non vide on l'insere */
      if( seg_tmp_give_longueur(seg) != SEG_TMP_EMPTY )
	{
	  contour_set_add_new_contour(contours,seg);
	  seg_tmp_unmark_seg(seg,tab_mark_unmark_point,mark_seg);
	}
      DEFAIRE_seg_tmp(seg);	  	            
    }

}

static void follow_contour_priv(contour_set contours,
				seg_tmp seg,
				tab_mark mark_seg,
				point p_init,
				move_type init_move,
				bouleen first_part)
{
      PFinsert insert;
      Point pt_aux;
      move_type move;
      seg_tmp seg2;

  if(first_part)
    insert = seg_tmp_insert_segment1;
  else
    insert = seg_tmp_insert_segment2;

  
  AFFECTE_POINT(pt_aux,*p_init);
  
  move = init_move;
  do
    {
      insert(seg,&pt_aux,move);
      tab_mark_mark_point(mark_seg,&pt_aux);

      modify_point_position(&pt_aux,move);

      /************* Conditions de sortie  de boucle ************/

      /* On a atteint un bout de segment deja insere dans le bitmap */
      if( contour_set_is_in_segment(contours,&pt_aux) )
	break;

      /* Le segment  crait  une boucle. */
      if( tab_mark_is_marked(mark_seg,&pt_aux) )
	{
	  /* On a cree une composante connexe tout est ok. */
	  if( seg_tmp_is_closed(seg) )
	    {
	      contour_set_add_new_contour(contours,seg);
	      seg_tmp_unmark_seg(seg,tab_mark_unmark_point,mark_seg);
	      seg_tmp_reinitialize(seg);
	      return;
	    }
	  /* Un bout de segment pendouille  on copie la boucle dans seg2. */
	  /* On modifie seg de facon a ce qu'il s'arrete au point de */
	  /* fermeture de la boucle*/
	  seg2 = contour_set_copy_closed_curve(contours,seg);
	  contour_set_add_new_contour(contours,seg2);
	  seg_tmp_unmark_seg(seg2,tab_mark_unmark_point,mark_seg);
	  return;
	}
      move = next_move_priv(contours,&pt_aux,move);
    }
  while(1);
  
}
static move_type next_move_priv(contour_set contours,
				point pt,
				move_type prec_move)
{
  move_type move;
  move_type demi_tour;

  demi_tour = REVERSE_SENSE(prec_move);
  move = demi_tour;
  do
    {
      move = NEXT_DIRECTION(move);
      assert(move != demi_tour);
    }
  while( contour_set_must_be_separated(contours,pt,move) );
  
  return move;
}
/* Si on relance la segmentation sur la meme image. Le tableau de */
/* marques utilise la fois precedente est reutilise. */
static tab_mark alloc_tab_mark_priv(contour_set contours)
{
  static tab_mark mark_seg = NULL_TAB_MARK;
  
  int  hauteur_mark_seg,largeur_mark_seg;
  int  hauteur,largeur; 

  hauteur = contour_set_give_image_hauteur(contours);
  largeur = contour_set_give_image_largeur(contours);
  
  if(mark_seg == NULL_TAB_MARK)
    {
      mark_seg = FAIRE_tab_mark(hauteur,largeur);
      return mark_seg;
    }

  hauteur_mark_seg = tab_mark_give_hauteur(mark_seg);
  largeur_mark_seg = tab_mark_give_largeur(mark_seg);
  
  if( (largeur != largeur_mark_seg) || (hauteur != hauteur_mark_seg) )
    {
      DEFAIRE_tab_mark(mark_seg);
      mark_seg = FAIRE_tab_mark(hauteur,largeur);
    }

  return mark_seg;
}
static dart_list alloc_dart_list_priv()
{
  static dart_list list_of_faces = NULL_DART_LIST;
  
  if(list_of_faces == NULL_DART_LIST)
    {
      list_of_faces = FAIRE_dart_list();
      return list_of_faces;
    }
  dart_list_reinitialize(list_of_faces);

  return list_of_faces;
}
       /**************************************************************/
      /* Marque les points une seule fois. Si on desire             */
     /* marque deux fois un point, cela signifie que               */
    /* l'on est passe deux fois dans le segment auquel            */
   /* il appartient. C'est donc un segment reliant deux faces.   */
  /* Il ne definit pas une region et n'a donc pas a etre marque */
 /* See also : pass_hole_priv.                                 */
/**************************************************************/
static void  mark_point_two_one_time_priv(tab_mark mark_point,point p)
{
  if(tab_mark_is_marked_two(mark_point,p))
    {
      tab_mark_unmark_point_two(mark_point,p);
      return;
    }
  tab_mark_mark_point_two(mark_point,p);
}

static point_set alloc_point_set_priv(contour_set contours)
{
  static point_set begin_pts = NULL_POINT_SET;
  int hauteur;

  if(begin_pts == NULL_POINT_SET)
    {
      hauteur = contour_set_give_image_hauteur(contours);
      begin_pts = FAIRE_point_set(hauteur);
      return begin_pts;
    }

  point_set_initialize(begin_pts);
  
  return begin_pts;
}
