/*  descartes
 *  Copyright (C) 2001 Luc Brun
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <stdio.h>
#include <malloc.h>
#include "segmente_global.h"
#include "distance.h"
#include "coul_freq.h"
#include "conversion.h"


static  void    init_params_priv                       (segmentation,
							contour_set,
							dart,
							double);
static  void    deallocate_params_priv                 ();
static  bouleen same_cluster_criterion_priv            (point,
							int*,
							point,
							int*);
static  bouleen same_cluster_criterion_n_equal_two_priv(int*,int*);
static  bouleen same_cluster_criterion_n_sup_two_priv  (int*,int*);
static  void    init_plane_coeff_priv                  (arbre_binaire);

static arbre_binaire arbre=NULL_ARBRE_BINAIRE;
static PFconversion *convert;
static int nb_couleur;
static double plane_coeff[DIM_MAX+1];

void segmente_global_init_segmente(segmentation seg)
{
  seg->compute_params = init_params_priv;
  seg->same_cluster   = same_cluster_criterion_priv;
  seg->close_params   = deallocate_params_priv;
}
  /*******************************************/
 /* Calcule l'histograme associe a l'image  */
/*******************************************/
static void init_params_priv(segmentation seg,
			     contour_set contours,
			     dart f,
			     double ecart_max)	
{
  coul_freq c_freq;
  int cluster_choisi =1;
  int axe_choisi= 0;
  int decoupe_choisie=0;
  int init_choisi =0;
  int espace_choisi;

  printf("Segmente global\n");

  if(ecart_max <=0.0)
    {
      printf("ecart max intra classe non valide\n");
      printf("entrez en un nouveau ");
      scanf("%lf",&ecart_max);
    }
  
  espace_choisi = segmentation_give_space(seg);

  convert= changer_espace(espace_choisi);  
  
  c_freq = FAIRE_coul_freq();
  coul_freq_initialize_from_face(c_freq,contours,f,espace_choisi);

  arbre  = FAIRE_arbre_binaire(c_freq,
			       cluster_choisi,
			       axe_choisi,
			       init_choisi,
			       decoupe_choisie);
  
  
  arbre_binaire_creer_clusters(arbre,c_freq,&nb_couleur,ecart_max);    
  printf("\n nb de couleurs %d\n",nb_couleur);
  if(nb_couleur ==2)
    init_plane_coeff_priv(arbre);
  else
    arbre_binaire_etiqueter_feuilles(arbre);

  DEFAIRE_coul_freq(c_freq);
}
static void deallocate_params_priv()
{
  DEFAIRE_arbre_binaire(arbre);
}
static void init_plane_coeff_priv(arbre_binaire arbre)
{
  int color[2*DIM_MAX],mean_value[DIM_MAX];
  int *color1,*color2;
  int i;
  color1 = color;
  color2 = color+3;

  arbre_binaire_donne_table(arbre,color);
  for(i=0;i<3;i++)
    {
      plane_coeff[i] = color2[i]-color1[i];
      mean_value[i]  = color2[i]+color1[i];
    }
  plane_coeff[3] =0.0;
  for(i=0;i<3;i++)
    plane_coeff[3] += mean_value[i]*plane_coeff[i];
  
  plane_coeff[3] *= 0.5;
}
  
static bouleen same_cluster_criterion_n_equal_two_priv(int* C1,int* C2)
{
  double plane_pos1,plane_pos2;
  int i;

  plane_pos1 = 0.0;
  for(i=0;i<3;i++)
    plane_pos1 += plane_coeff[i]*C1[i];
  plane_pos1 = plane_pos1-plane_coeff[3];
  
  plane_pos2 = 0.0;
  for(i=0;i<3;i++)
    plane_pos2 += plane_coeff[i]*C2[i];
  plane_pos2 = plane_pos2-plane_coeff[3];  

  if ((plane_pos1 >=0)&&(plane_pos2 >=0))
    return vrai;

  if ((plane_pos1 <0)&&(plane_pos2 <0))
    return vrai;
  
  return faux;
}
static bouleen same_cluster_criterion_n_sup_two_priv(int* C1,int* C2)
{
  int indice1,indice2;

  arbre_binaire_chercher_couleur_ds_table(arbre,C1,&indice1);
  
  arbre_binaire_chercher_couleur_ds_table(arbre,C2,&indice2);
  
  return (indice1 == indice2);
 
}

static bouleen same_cluster_criterion_priv(point p1,
				    int* pixel1,
				    point p2,
				    int* pixel2)
{
  int C1[3],C2[3];
  int i;

  for(i=0;i<3;i++)
    C1[i] = convert[i](pixel1[0],pixel1[1],pixel1[2]);
  
  for(i=0;i<3;i++)
    C2[i] = convert[i](pixel2[0],pixel2[1],pixel2[2]);
  
  if(nb_couleur == 2)
    return same_cluster_criterion_n_equal_two_priv(C1,C2);
  
  return same_cluster_criterion_n_sup_two_priv(C1,C2);
}





