/*  descartes
 *  Copyright (C) 2001 Luc Brun
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <stdio.h>
#include <malloc.h>
#include "segmente_uniform.h"
#include "conversion.h"
#include "partition.h"
#include "cluster_set.h"
#include "merge.h"

#define NB_CLUSTER 2048

static  void    init_params_priv                       (segmentation,
							contour_set,
							dart,
							double);
static  void    deallocate_params_priv                 ();
static  bouleen same_cluster_criterion_priv            (point,
							int*,
							point,
							int*);
static  bouleen same_cluster_criterion_n_equal_two_priv(int*,int*);
static  bouleen same_cluster_criterion_n_sup_two_priv  (int*,int*);
static  void    init_plane_coeff_priv                  (cluster_set,int);
static  void    deallocate_params_priv                 ();

static PFconversion *convert;
static int nb_colors;
static double plane_coeff[DIM_MAX+1];
static partition init_part;
static cluster_set clusters;

void segmente_uniform_init_segmente(segmentation seg)
{
  seg->compute_params = init_params_priv;
  seg->same_cluster   = same_cluster_criterion_priv;
  seg->close_params   = deallocate_params_priv;
}
  /*******************************************/
 /* Calcule l'histograme associe a l'image  */
/*******************************************/
static void init_params_priv(segmentation seg,
			     contour_set contours,
			     dart f,
			     double ecart_max)	
{
  int space;
  int nb_cluster ;

  printf("Segmente Uniforme\n");

  if(ecart_max <=0.0)
    {
      printf("ecart max intra classe non valide\n");
      printf("entrez en un nouveau ");
      scanf("%lf",&ecart_max);
    }

  nb_cluster = NB_CLUSTER;

  space = segmentation_give_space(seg);

  convert= changer_espace(space);  
  
  init_part = FAIRE_partition(&nb_cluster);
  clusters  = FAIRE_cluster_set();
  
  cluster_set_init_from_face(clusters,
			     contours,
			     f,
			     init_part,
			     nb_cluster,
			     space);

  merge_clusters(clusters,&nb_colors,ecart_max);
  cluster_set_set_canonic(clusters);
  printf("\n nb de couleurs %d\n",nb_colors);
  if(nb_colors ==2)
    init_plane_coeff_priv(clusters,space);
  
}
static void deallocate_params_priv()
{
  DEFAIRE_cluster_set(clusters);
  DEFAIRE_partition(init_part);
}
static void init_plane_coeff_priv(cluster_set clusters,int space)
{
  double color[2*DIM_MAX],mean_value[DIM_MAX];
  double *color1,*color2;
  int i;
  color1 = color;
  color2 = color+3;
  
  cluster_set_give_table(clusters,color);
  for(i=0;i<3;i++)
    {
      plane_coeff[i] = color2[i]-color1[i];
      mean_value[i]  = color2[i]+color1[i];
    }
  
  plane_coeff[3] =0.0;
  for(i=0;i<3;i++)
    plane_coeff[3] += mean_value[i]*plane_coeff[i];
  
  plane_coeff[3] *= 0.5;
}
  
static bouleen same_cluster_criterion_n_equal_two_priv(int* C1,int* C2)
{
  double plane_pos1,plane_pos2;
  int i;

  plane_pos1 = 0.0;
  for(i=0;i<3;i++)
    plane_pos1 += plane_coeff[i]*convert[i](C1[0],C1[1],C1[2]);
  plane_pos1 = plane_pos1-plane_coeff[3];
  
  plane_pos2 = 0.0;
  for(i=0;i<3;i++)
    plane_pos2 += plane_coeff[i]*convert[i](C2[0],C2[1],C2[2]);
  plane_pos2 = plane_pos2-plane_coeff[3];  

  if ((plane_pos1 >=0)&&(plane_pos2 >=0))
    return vrai;


  if ((plane_pos1 <0)&&(plane_pos2 <0))
    return vrai;
  
  return faux;
}
static bouleen same_cluster_criterion_n_sup_two_priv(int* C1,int* C2)
{
  int ind1,ind2;

  ind1 = partition_give_cluster_nb(init_part,C1);
  ind2 = partition_give_cluster_nb(init_part,C2);

  ind1 = cluster_set_give_canonic_cluster(clusters,ind1);
  ind2 = cluster_set_give_canonic_cluster(clusters,ind2);

  return (ind1 == ind2);
}

static bouleen same_cluster_criterion_priv(point p1,
					   int* pixel1,
					   point p2,
					   int* pixel2)
{
  if(nb_colors == 2)
    return same_cluster_criterion_n_equal_two_priv(pixel1,pixel2);
  
  return same_cluster_criterion_n_sup_two_priv(pixel1,pixel2);
}





