/*  descartes
 *  Copyright (C) 2001 Luc Brun
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <stdio.h>
#include <assert.h>
#include "tab_gen.h"





#define Main_tab SELF (main_tab)
#define Cardinal SELF (cardinal)
#define Nb_ligne SELF (nb_ligne)

#define ALLOC_LIGNE    50
#define ALLOC_NB_LIGNE 50

#define NUM_LIGNE(n) ((n)/(ALLOC_LIGNE))
#define NUM_COLONNE(n) ((n)%ALLOC_LIGNE)

static void private_realloc_tab_gen(tab_gen);
 
struct tab_gen
{    
  any_ptr **main_tab;
  int      cardinal;
  int      nb_ligne;
};

tab_gen FAIRE_tab_gen()
{
  INSTANCIER(tab_gen);

  Main_tab = (any_ptr **) calloc(ALLOC_NB_LIGNE,
				 sizeof(any_ptr*));
  *Main_tab = (any_ptr*) calloc(ALLOC_LIGNE,
				 sizeof(any_ptr));
  Cardinal = 0;
  Nb_ligne = 1;
  
  return self;
}

/*---------------------------------------------------------------------------*/
/* Ne supprime pas les segments stocke, voir si c'est souhaitable */
/*---------------------------------------------------------------------------*/
void DEFAIRE_tab_gen(tab_gen self)
{
  any_ptr **tmp = Main_tab;
  int i;

  for(i=0;i<Nb_ligne;i++)
    {
      free (*tmp);
      tmp++;
    }
  free(Main_tab);
  free(self);
}

/*---------------------------------------------------------------------------*/

void tab_gen_insert_object(tab_gen self, any_ptr obj, int num)
{
  int ligne,colonne;
  any_ptr* ligne_seg;

  if (num < 0) num = -num;

  ligne  = NUM_LIGNE(num);
  colonne = NUM_COLONNE(num);
  
  /* On tombe sur un ligne non alloue */
  while(ligne >= Nb_ligne)
    private_realloc_tab_gen(self);
  
  ligne_seg = *(Main_tab+ligne);  
  if ( ligne_seg[colonne])
      {
	/* this case never occur */
	fprintf (stderr,"tab_gen.c insert objet \n");
	fprintf (stderr,": brin (%d) deja attache a un any_ptr \n",num);
	exit(1);
      }
    else
      {
	ligne_seg[colonne] = obj;
	Cardinal ++;
      }
  }

/*---------------------------------------------------------------------------*/

any_ptr tab_gen_give_object(tab_gen self, int num)
{
  int ligne,colonne;
  any_ptr  object;
  bouleen  object_present;

  if (num < 0) num = -num;
  ligne  = NUM_LIGNE(num);
  colonne = NUM_COLONNE(num);
  
  object_present = (ligne < Nb_ligne) && (colonne < ALLOC_LIGNE);

  if(!object_present)
    return NULL;

  object = Main_tab[ligne][colonne];
  
  return object;
}

/*---------------------------------------------------------------------------*/

int tab_gen_give_dart(tab_gen self, any_ptr obj)
{
  bouleen trouve;
  int ligne = 0;
  int colonne = 0;
  any_ptr** liste_ligne = Main_tab;
  any_ptr* ligne_obj;

  ligne =0;
/* On parcourt les differentes lignes */
  while(ligne < Nb_ligne)
    {
      ligne_obj = *liste_ligne;
      colonne = 0;
      /* On parcourt les elements d'une ligne */
      while(colonne < ALLOC_LIGNE)
	{
	  trouve = (*ligne_obj == obj);
	  if(trouve)
	    return ligne*ALLOC_LIGNE+colonne;

	  colonne++;
	  ligne_obj++;
	}
      ligne++;
      liste_ligne++;
    }

  return 0;
}

/*---------------------------------------------------------------------------*/

void tab_gen_remove_object(tab_gen self, int num)
{
  int ligne,colonne;
  any_ptr *ligne_obj;
  bouleen object_present;

  if (num < 0) num = -num;
  ligne  = NUM_LIGNE(num);
  colonne = NUM_COLONNE(num);
  
  object_present = (ligne < Nb_ligne) && (colonne < ALLOC_LIGNE);

  if(!object_present)
    return;
  Main_tab[ligne][colonne] = NULL;
  Cardinal--;
}
int tab_gen_give_cardinal(tab_gen self)
{
  return Cardinal;
}
static void private_realloc_tab_gen(tab_gen self)
{
  /* On regarde si on na pas depasse le nb max de ligne alloue */
  if(Nb_ligne%ALLOC_NB_LIGNE == 0)
    {
      Main_tab = (any_ptr**)realloc(Main_tab,
				   (Nb_ligne+ALLOC_NB_LIGNE)*
				   sizeof(any_ptr*));
      assert(Main_tab);
    }
  Main_tab[Nb_ligne] = (any_ptr *)calloc(ALLOC_LIGNE,sizeof(any_ptr));
  Nb_ligne++;
}
