/*  descartes
 *  Copyright (C) 2001 Luc Brun
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <assert.h>
#include "tab_int.h"
#include "objet.h"
#include "garbage.h"

#define Nb_line      SELF(nb_line)
#define Element_tab  SELF(element_tab)
#define Nb_pointer   SELF(nb_pointer)

#define ALLOC_LINE 200
#define ALLOC_NB_LINE 50

#define NUM_LIGNE(n) ((n) / ALLOC_LINE)
#define NUM_COLUMN(n)((n) % ALLOC_LINE)

static garbage line_collector    = NULL_GARBAGE;
static garbage tab_int_collector = NULL_GARBAGE;

		/* Fonctions liees au garbage */
static void    init_line_priv(int*);
static int*    alloc_line_priv();
static void    dealloc_line_priv(int *);
static tab_int alloc_tab_int_priv();
static void    dealloc_tab_int_priv(tab_int);
static void    realloc_tab_int_priv(tab_int);
static void    free_tab_int_priv(void*);

struct tab_int
{
  int** element_tab;
  int nb_line;
  int nb_pointer;
};

void ALLOC_tab_int()
{
  if(line_collector == NULL_GARBAGE)
    line_collector = FAIRE_garbage();
  if(tab_int_collector == NULL_GARBAGE)
    tab_int_collector = FAIRE_garbage();
}
void DEALLOC_tab_int()
{
  if(line_collector != NULL_GARBAGE)
    {
      DEFAIRE_garbage(line_collector,free);
      line_collector = NULL_GARBAGE;
    }
  if(tab_int_collector != NULL_GARBAGE)
    {
      DEFAIRE_garbage(tab_int_collector,free_tab_int_priv);
      tab_int_collector = NULL_GARBAGE;
    }
}
tab_int FAIRE_tab_int()
{
  tab_int self;

  self = alloc_tab_int_priv();

  *Element_tab= alloc_line_priv();
  
  Nb_line = 1;

  return self;
}

/*---------------------------------------------------------------------------*/

void DEFAIRE_tab_int(tab_int self)
{  
  int i;

  assert(self);

  for(i=0;i<Nb_line;i++)
    dealloc_line_priv(Element_tab[i]);
  
  dealloc_tab_int_priv(self);
}

void tab_int_reinitialize(tab_int self)
{  
  int i;

  assert(self);

  for(i=1;i<Nb_line;i++)
    dealloc_line_priv(Element_tab[i]);

  init_line_priv(Element_tab[0]);
  Nb_line = 1;
}

/*---------------------------------------------------------------------------*/
/* renvoie la valeur contenue a l'indice ind */
/*---------------------------------------------------------------------------*/
int tab_int_give_value(tab_int self,int ind)
{
  int ligne,column;

  assert(self);

  ligne  = NUM_LIGNE(ind);
  column = NUM_COLUMN(ind);
  
/* ligne non encore allouee */
  if (ligne >= Nb_line) 
    return 0;
 
  return Element_tab[ligne][column];
}

/*---------------------------------------------------------------------------*/
/* met value a l'indice ind du tableau virtuel */
/*---------------------------------------------------------------------------*/
void tab_int_put_value(tab_int self,int ind,int value)
{
  int ligne,column;

  assert(self);

  ligne =  NUM_LIGNE(ind);
  column = NUM_COLUMN(ind);

  /* on ajoute autant de lignes que necessaire */
  while(ligne >= Nb_line)
    realloc_tab_int_priv(self);
  
  Element_tab[ligne][column] = value;
}
      
   /********************************************************************/
  /* Alloue une ligne si on a pas enclenche le garbage ou si celui ci */
 /* est nul                                                          */
/********************************************************************/
static int* alloc_line_priv()
{
  int* line;

  if(line_collector == NULL_GARBAGE)
    {
      line = (int*) calloc(ALLOC_LINE,sizeof(int));
      assert(line);
      return line;
    }
  if(garbage_is_empty(line_collector))
    {
      line = (int*) calloc(ALLOC_LINE,sizeof(int));
      assert(line);
      return line;
    }
  
  line =(int*) garbage_give_object(line_collector);
  assert(line);
  init_line_priv(line);

  return line;
}
   /*********************************************************************/
  /* Alloue un tab_int si on a pas enclenche le garbage ou si celui ci */
 /* est nul                                                           */
/*********************************************************************/
static tab_int alloc_tab_int_priv()
{
  tab_int self;
  
  if(tab_int_collector == NULL_GARBAGE)
    {
      self = (tab_int)malloc(sizeof(struct tab_int));
      Element_tab    = (int **) malloc(ALLOC_NB_LINE*sizeof(int*));
      Nb_pointer  = ALLOC_NB_LINE;
      
      return self;
    }
  if(garbage_is_empty(tab_int_collector))
    {
      self = (tab_int)malloc(sizeof(struct tab_int));
      Element_tab    = (int **) malloc(ALLOC_NB_LINE*sizeof(int*));
      Nb_pointer  = ALLOC_NB_LINE;

      return self;
    }

  self = garbage_give_object(tab_int_collector);

  return self;
}
  /****************************************************/
 /* Libere ou insere ds le garbage une ligne liberee */
/****************************************************/
static void dealloc_line_priv(int * line)
{
  assert(line);
  if(line_collector == NULL_GARBAGE)
    {
      free(line);
      return;
    }  
  garbage_insert_object(line_collector,line);
}
  /****************************************************/
 /* Libere ou insere ds le garbage un tab_int libere */
/****************************************************/
static void dealloc_tab_int_priv(tab_int self)
{
  assert(self);
  if(tab_int_collector == NULL_GARBAGE)
    {
      free(self);
      return;
    }
  garbage_insert_object(tab_int_collector,self);
}
  /****************************************************/
 /* Alloue dynamiquement de la place pour le tab_int */
/****************************************************/
static void realloc_tab_int_priv(tab_int self)
{
  assert(self);

  /* On regarde si on na pas depasse le nb max de LINE alloue */
  if(Nb_line == Nb_pointer)
    {
      Element_tab = (int**)realloc(Element_tab,
				   (Nb_pointer+ALLOC_NB_LINE)*
				   sizeof(int*));
      assert(Element_tab);
      Nb_pointer += ALLOC_NB_LINE;
    }
  Element_tab[Nb_line] = alloc_line_priv();
  Nb_line++;
}
   /*********************************************************************/
  /*  Fonction utilisee par le garbage pour liberer les tab_int encore */
 /*  en memoire.                                                      */
/*********************************************************************/
static void free_tab_int_priv(void* obj)
{
  tab_int self = obj;
  
  free(Element_tab);
  free(self);
}
static void init_line_priv(int* line)
{
  int i;
  int *ptr;
  
  ptr = line;
  for(i=0;i<ALLOC_LINE;i++)
    {
      *ptr = 0;
      ptr++;
    }
}




