/*  descartes
 *  Copyright (C) 2001 Luc Brun
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <stdio.h>

#include <tab_label.h>
#include "image.h"
#include "segmente.h"

void write_pixels( any_ptr obj, point p, int flag );
static bouleen criterion_priv( point p1, int *pixel1, point p2, int *pixel2 );

static image dst, imgnou;
static int cpt;
static int pixel[3];
static int pas=16777216; // 2^24 ie le nb de couleurs possibles
static int ind;
static int cri_dim;

int tab_label_save( contour_set contours, const char *name )
{
  dart_list dl;
  int flag;

  //--- module d'initialisation 
  dl = FAIRE_dart_list();
  dl = contour_set_give_image_faces(contours);

  dst = FAIRE_image();
  image_initialize(dst,
		   3,
		   contour_set_give_image_largeur(contours),
		   contour_set_give_image_hauteur(contours));
  
  pas /= dart_list_give_cardinal(dl);

  do
    {
      contour_set_apply_function(contours,dart_list_give_current(dl),write_pixels);
      cpt += pas;

      pixel[0] = cpt%255;
      ind = cpt/255;
      pixel[1] = ind%255;
      pixel[2] = ind/255;
      
    }
  while(dart_list_next(dl));

  flag = image_sauvegarder(dst,(char *)name);
   
  //--- module de de-init
  
  DEFAIRE_image(dst);
  DEFAIRE_dart_list(dl);

  return ( flag == -1 ? flag : 0 );
}

void write_pixels( any_ptr obj, point p, int flag )
{
  image_move_to(dst,p);
  image_ecrire_pixel(dst,pixel);
}

int tab_label_load( contour_set contours, const char *name )
{
  assert(contours);
  // module d'initialisation
  imgnou = FAIRE_image();
  
  if(image_charger(imgnou,(char *)name) != 0)
    return -1;

  if( image_give_hauteur(imgnou) != contour_set_give_image_hauteur(contours)
      || image_give_largeur(imgnou) != contour_set_give_image_largeur(contours))
    {
      DEFAIRE_image(imgnou);
      return -2;
    }
  
  cri_dim = image_give_dim(imgnou);
  
  contour_set_set_same_cluster_criterion(contours,criterion_priv);
  segmente_do_segments(contours,1);

  // module de de-init
  DEFAIRE_image(imgnou);


  return 0;
}    

static int cri_decal[3] = {0,8,16};

static int cri_compute_priv( int *cri_pix, int cri_dim )
{
  int cri_res =0, i;

  for( i=0; i < cri_dim; i++ )
    cri_res += (cri_pix[i] << cri_decal[i]);

  return cri_res;
}

static bouleen criterion_priv( point p1, int *pixel1, point p2, int *pixel2 )
{
  int ind1, ind2;
  int *pix;

  image_move_to(imgnou,p1);
  pix = image_lire_pixel(imgnou);
  ind1 = cri_compute_priv(pix,cri_dim);

  image_move_to(imgnou,p2);
  pix = image_lire_pixel(imgnou);
  ind2 = cri_compute_priv(pix,cri_dim);

  return ind1 == ind2;
}
