/*  descartes
 *  Copyright (C) 2001 Luc Brun
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <limits.h>
#include "point.h"
#include "contour_set.h"
#include "type_obj.h"
#include "output_segment.h"
static void compute_bound_box_segment_priv(output_segment,
					   dart,
					   int*,
					   int*,
					   int*,
					   int*);


void compute_bounding_box(contour_set contours, 
			  dart f, 
			  point pmin, point pmax){
  int xmin, ymin, xmax, ymax;
  dart_list seg_list;
  dart brin;
  output_segment seg;

  xmin = INT_MAX;
  ymin = INT_MAX;
  xmax = INT_MIN;
  ymax = INT_MIN;

  seg_list = FAIRE_dart_list();
  seg = FAIRE_output_segment();

  contour_set_give_segments_of_face(contours, f, seg_list);
  dart_list_first(seg_list);
  do{ 
    brin = dart_list_give_current(seg_list);
    contour_set_give_segment(contours, brin, seg);
    compute_bound_box_segment_priv(seg, brin, 
				   &xmin, &ymin, 
				   &xmax, &ymax);
    output_segment_reinitialize(seg);
  }
  while(dart_list_next(seg_list));
  DEFAIRE_output_segment(seg);
  DEFAIRE_dart_list(seg_list);

  COORDX(*pmin) = xmin;
  COORDY(*pmin) = ymin;
  COORDX(*pmax) = xmax;
  COORDY(*pmax) = ymax;

}


static void compute_bound_box_segment_priv(output_segment seg,
					   dart brin,
					   int* xmin, int* ymin,
					   int* xmax, int* ymax){

  void (*first)(output_segment);
  bouleen (*next)(output_segment);
  Point p;
  
  if (brin < 0){
    first = output_segment_last;
    next = output_segment_previous;
  }
  else{
    first = output_segment_first;
    next = output_segment_next;
  }

  first(seg);
  do{
    output_segment_give_current_point(seg, &p);
    if(*xmin > COORDX(p))
      *xmin = COORDX(p);
    if(*ymin > COORDY(p))
      *ymin = COORDY(p);
    if(*xmax < COORDX(p))
      *xmax = COORDX(p);
    if(*ymax < COORDY(p))
      *ymax = COORDY(p);
    }
  while(next(seg));
}


dart_list intersect_box( contour_set cs, point pmin, point pmax )
{
  tab_mark_dart tmd;
  Point parc;
  point p;
  dart id;
  dart_list dl;

  // declarations pour le traitement des segments de taille 1
  dart_list faces, segments;
  dart current_face;
  output_segment os;
  Point a,b;

  // si les points sont mal placs, on les swap
  if( COORDX(*pmin) > COORDX(*pmax) && COORDY(*pmin) > COORDY(*pmax) )
    {
      p = pmin; pmin = pmax; pmax = p;
    }

  tmd = FAIRE_tab_mark_dart();
  dl = FAIRE_dart_list();
  segments = FAIRE_dart_list();
  os = FAIRE_output_segment();

  for( COORDY(parc) = COORDY(*pmin); COORDY(parc) < COORDY(*pmax)+1; COORDY(parc)++ )
    for( COORDX(parc) = COORDX(*pmin); COORDX(parc) < COORDX(*pmax)+1; COORDX(parc)++ )
     	if( contour_set_is_in_segment(cs,&parc) )
	  {
	    id = contour_set_assoc_point_arc(cs,&parc);

	    if(!tab_mark_dart_is_marked(tmd,id))
	      {
		tab_mark_dart_mark_dart(tmd,id);
		tab_mark_dart_mark_dart(tmd,-id);
		dart_list_insert(dl,id);
	      }
	  }

  faces = contour_set_give_image_faces(cs);

  dart_list_first(faces);
  do
    {
      current_face = dart_list_give_current(faces);
      contour_set_give_segments_of_face(cs,current_face,segments);
      
      dart_list_first(segments);
      do
	{
	  id = dart_list_give_current(segments);
	  
	  if( contour_set_give_segment_lenght(cs,id) > 1 )
	    continue;

	  if( tab_mark_dart_is_marked(tmd,id) || tab_mark_dart_is_marked(tmd,-id) )
	    continue;

	  contour_set_give_segment(cs,id,os);
	  output_segment_give_first_point(os,&a);
	  output_segment_give_last_point(os,&b);

	  if( !(( COORDX(a) < COORDX(*pmin) || COORDX(a) > COORDX(*pmax) 
		|| COORDY(a) < COORDY(*pmin) || COORDY(a) > COORDY(*pmax ) )
	      && ( COORDX(b) < COORDX(*pmin) || COORDX(b) > COORDX(*pmax) 
		   || COORDY(b) < COORDY(*pmin) || COORDY(b) > COORDY(*pmax ))))
	    {
	      dart_list_insert(dl,id);
	      tab_mark_dart_mark_dart(tmd,id);
	      tab_mark_dart_mark_dart(tmd,-id);
	    }
	  
	  output_segment_reinitialize(os);
	  
	}
      while(dart_list_next(segments));
      
      dart_list_reinitialize(segments);
    
    }
  while(dart_list_next(faces));

  DEFAIRE_tab_mark_dart(tmd);
  DEFAIRE_dart_list(segments);
  DEFAIRE_output_segment(os);

  return dl;
}

