/*  descartes
 *  Copyright (C) 2001 Luc Brun
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include "my_gtk.h"
#include <gnome.h>

static void
destroy_toplevel                     (GtkButton       *button,
				      gpointer         user_data);

GtkWidget* create_file_selection(const char* message,
				 const gchar* pattern,
				 GtkSignalFunc ok_func,
				 GtkSignalFunc cancel_func
				 )
{
  GtkWidget * ImageSelection;
  
  ImageSelection =  gtk_file_selection_new(message);
  gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(ImageSelection));

  if(pattern != NULL)
    gtk_file_selection_complete(GTK_FILE_SELECTION(ImageSelection),
				pattern);
  
  gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(ImageSelection)->cancel_button),
			    "clicked",
			    cancel_func,
			    GTK_OBJECT(ImageSelection)
			    );

  gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(ImageSelection)->ok_button),
			    "clicked",
			    ok_func,
			    GTK_OBJECT(ImageSelection)
			    );

  gtk_widget_show(ImageSelection);
  return ImageSelection;
}

GtkWidget* create_confirm(const char* message,
			 GtkSignalFunc ok_func,
			 GtkSignalFunc cancel_func
			 )
{
  
  GtkWidget *Dialogue;
  GtkWidget *HBox;
  GtkWidget *BoutonAnuller;
  GtkWidget *BoutonQuitter;
  GtkWidget *Label;
  

  Dialogue = gtk_dialog_new();

  Label = gtk_label_new(message);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(Dialogue)->vbox),
		     Label,
		     TRUE,TRUE,20);
  gtk_widget_show(Label);


  HBox = gtk_hbox_new(TRUE,5);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(Dialogue)->action_area),
		     HBox,
		     TRUE,TRUE,0);
  gtk_widget_show(HBox);

  BoutonQuitter = gtk_button_new_with_label("Quitter");
  gtk_signal_connect_object(GTK_OBJECT(BoutonQuitter),
			    "clicked",
			    (GtkSignalFunc)ok_func,
			    NULL);
  gtk_box_pack_start(GTK_BOX(HBox),BoutonQuitter,TRUE,TRUE,0);
  gtk_widget_show(BoutonQuitter);
  
  BoutonAnuller = gtk_button_new_with_label("Annuler");
  gtk_signal_connect_object(GTK_OBJECT(BoutonAnuller),
			    "clicked",
			    (GtkSignalFunc)cancel_func,
			    (GtkObject*)Dialogue);
  
  gtk_box_pack_start(GTK_BOX(HBox),BoutonAnuller,TRUE,TRUE,0);
  gtk_widget_show(BoutonAnuller);

  gtk_widget_show(Dialogue);
  gtk_widget_set_uposition(Dialogue,
			   (gdk_screen_width()-(Dialogue->allocation.width))>>1,
			   (gdk_screen_height()-(Dialogue->allocation.height))>>1);

  return Dialogue;
}
void create_dialog(const char* message,
		   GtkSignalFunc ok_func
		   )
{
  
  GtkWidget *Dialogue;
  GtkWidget *HBox;
  GtkWidget *BoutonOK;
  GtkWidget *Label;
  

  Dialogue = gtk_dialog_new();

  Label = gtk_label_new(message);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(Dialogue)->vbox),
		     Label,
		     TRUE,TRUE,20);
  gtk_widget_show(Label);


  HBox = gtk_hbox_new(TRUE,5);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(Dialogue)->action_area),
		     HBox,
		     TRUE,TRUE,0);
  gtk_widget_show(HBox);

  BoutonOK = gtk_button_new_with_label("OK");
  gtk_signal_connect_object(GTK_OBJECT(BoutonOK),
			    "clicked",
			    (GtkSignalFunc)ok_func,
			    (GtkObject*)Dialogue);
  gtk_box_pack_start(GTK_BOX(HBox),BoutonOK,TRUE,TRUE,0);
  gtk_widget_show(BoutonOK);

  gtk_widget_show(Dialogue);
}
GtkWidget* create_entry (char* user_title,
			 char* user_label,
			 int value,
			 GtkSignalFunc ok_func
			 ) 
{
  GtkWidget *window1;
  GtkWidget *vbox1;
  GtkWidget *text;
  GtkWidget *entry;
  GtkWidget *hbuttonbox1;
  GtkWidget *OK;
  GtkWidget *Cancel;
  char* title,*default_title="window1";
  char* label,*default_label="zone de texte";
  char user_value[10];


  title= (user_title==NULL?default_title:user_title);
  label= (user_label==NULL?default_label:user_label);
  sprintf(user_value,"%d",value);

  window1 = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_object_set_data (GTK_OBJECT (window1), "window1", window1);
  gtk_window_set_title (GTK_WINDOW (window1), title);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox1);
  gtk_object_set_data_full (GTK_OBJECT (window1), "vbox1", vbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (window1), vbox1);

  text = gtk_label_new (label);
  gtk_widget_ref (text);
  gtk_object_set_data_full (GTK_OBJECT (window1), "text", text,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (text);
  gtk_box_pack_start (GTK_BOX (vbox1), text, FALSE, FALSE, 0);

  entry = gtk_entry_new_with_max_length (10);
  gtk_widget_ref (entry);
  gtk_object_set_data_full (GTK_OBJECT (window1), "entry", entry,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (entry);
  gtk_box_pack_start (GTK_BOX (vbox1), entry, FALSE, FALSE, 0);
  gtk_entry_set_text (GTK_ENTRY (entry),user_value);

  hbuttonbox1 = gtk_hbutton_box_new ();
  gtk_widget_ref (hbuttonbox1);
  gtk_object_set_data_full (GTK_OBJECT (window1), "hbuttonbox1", hbuttonbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbuttonbox1);
  gtk_box_pack_start (GTK_BOX (vbox1), hbuttonbox1, TRUE, TRUE, 0);

  OK = gtk_button_new_with_label ("OK");
  gtk_widget_ref (OK);
  gtk_object_set_data_full (GTK_OBJECT (window1), "OK", OK,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (OK), "clicked",
		      ok_func,
                      entry);

  gtk_widget_show (OK);
  gtk_container_add (GTK_CONTAINER (hbuttonbox1), OK);
  GTK_WIDGET_SET_FLAGS (OK, GTK_CAN_DEFAULT);

  Cancel = gtk_button_new_with_label ("Cancel");
  gtk_widget_ref (Cancel);
  gtk_object_set_data_full (GTK_OBJECT (window1), "Cancel", Cancel,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (Cancel), "clicked",
                      GTK_SIGNAL_FUNC (destroy_toplevel),
                      NULL);

  gtk_widget_show (Cancel);
  gtk_container_add (GTK_CONTAINER (hbuttonbox1), Cancel);
  GTK_WIDGET_SET_FLAGS (Cancel, GTK_CAN_DEFAULT);

  gtk_widget_show(window1);
  
  return window1;
}

GtkWidget* create_entries (char* title,
			   char* text,
			   int nb_entries,
			   char * entries_label[],
			   int *values,
			   GtkSignalFunc on_ok)

{
  GtkWidget *entries,*entry;
  GtkWidget *dialog_vbox1;
  GtkWidget *vbox2;
  GtkWidget *label1;
  GtkWidget *vbox3;
  GtkWidget *table1;
  GtkWidget *label_entry;
  GtkWidget *dialog_action_area1;
  GtkWidget *hbuttonbox1;
  GtkWidget *OK;
  GtkWidget *cancel;


  char number[10];
  char widget_name[15];
  int i;

  entries = gtk_dialog_new ();
  gtk_object_set_data (GTK_OBJECT (entries), "entries", entries);
  gtk_window_set_title (GTK_WINDOW (entries), title);
  gtk_window_set_policy (GTK_WINDOW (entries), TRUE, TRUE, FALSE);

  dialog_vbox1 = GTK_DIALOG (entries)->vbox;
  gtk_object_set_data (GTK_OBJECT (entries), "dialog_vbox1", dialog_vbox1);
  gtk_widget_show (dialog_vbox1);

  vbox2 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox2);
  gtk_object_set_data_full (GTK_OBJECT (entries), "vbox2", vbox2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox2);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox2, TRUE, TRUE, 0);

  label1 = gtk_label_new (text);
  gtk_widget_ref (label1);
  gtk_object_set_data_full (GTK_OBJECT (entries), "label1", label1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label1);
  gtk_box_pack_start (GTK_BOX (vbox2), label1, TRUE, FALSE, 0);
  gtk_label_set_line_wrap (GTK_LABEL (label1), TRUE);

  vbox3 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox3);
  gtk_object_set_data_full (GTK_OBJECT (entries), "vbox3", vbox3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox3);
  gtk_box_pack_start (GTK_BOX (vbox2), vbox3, FALSE, FALSE, 0);

  table1 = gtk_table_new (nb_entries, 2, FALSE);
  gtk_widget_ref (table1);
  gtk_object_set_data_full (GTK_OBJECT (entries), "table1", table1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (table1);
  gtk_box_pack_start (GTK_BOX (vbox3), table1, TRUE, TRUE, 0);

  for(i=0;i<nb_entries;i++)
    {
      /* Labels Creation */
      label_entry = gtk_label_new (entries_label[i]);
      gtk_widget_ref (label_entry);
      sprintf(widget_name,"label_%d",i);
      gtk_object_set_data_full (GTK_OBJECT (entries), widget_name, label_entry,
				(GtkDestroyNotify) gtk_widget_unref);
      gtk_widget_show (label_entry);
      gtk_table_attach (GTK_TABLE (table1), label_entry, 0, 1, i, i+1,
			(GtkAttachOptions) (0),
			(GtkAttachOptions) (0), 0, 0);
      gtk_misc_set_padding (GTK_MISC (label_entry), 20, 0);

      /* Entries Creation */
      entry = gtk_entry_new ();
      gtk_widget_ref (entry);
      sprintf(widget_name,"entry_%d",i);
      
      gtk_object_set_data_full (GTK_OBJECT (entries), 
				widget_name,
				entry,
				(GtkDestroyNotify) gtk_widget_unref
				);
      gtk_widget_show (entry);
      gtk_table_attach (GTK_TABLE (table1), entry, 1, 2, i, i+1,
			(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			(GtkAttachOptions) (0), 0, 0);
      sprintf(number,"%d",values[i]);
      gtk_entry_set_text (GTK_ENTRY (entry),number);
    }

  dialog_action_area1 = GTK_DIALOG (entries)->action_area;
  gtk_object_set_data (GTK_OBJECT (entries), "dialog_action_area1", dialog_action_area1);
  gtk_widget_show (dialog_action_area1);
  gtk_container_set_border_width (GTK_CONTAINER (dialog_action_area1), 10);

  hbuttonbox1 = gtk_hbutton_box_new ();
  gtk_widget_ref (hbuttonbox1);
  gtk_object_set_data_full (GTK_OBJECT (entries), "hbuttonbox1", hbuttonbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbuttonbox1);
  gtk_box_pack_start (GTK_BOX (dialog_action_area1), hbuttonbox1, TRUE, TRUE, 0);

  OK = gnome_stock_button (GNOME_STOCK_BUTTON_OK);
  gtk_widget_ref (OK);
  gtk_object_set_data_full (GTK_OBJECT (entries), "OK", OK,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (OK);
  gtk_container_add (GTK_CONTAINER (hbuttonbox1), OK);
  GTK_WIDGET_SET_FLAGS (OK, GTK_CAN_DEFAULT);

  if(on_ok != NULL)
    gtk_signal_connect (GTK_OBJECT (OK), "clicked",
			on_ok,
			entries);
  else
    gtk_signal_connect (GTK_OBJECT (OK), "clicked",
			destroy_toplevel,
			NULL);
    
  cancel = gnome_stock_button (GNOME_STOCK_BUTTON_CANCEL);
  gtk_widget_ref (cancel);
  gtk_object_set_data_full (GTK_OBJECT (entries), "cancel", cancel,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (cancel), "clicked",
		      destroy_toplevel,
                      NULL);

  gtk_widget_show (cancel);
  gtk_container_add (GTK_CONTAINER (hbuttonbox1), cancel);
  GTK_WIDGET_SET_FLAGS (cancel, GTK_CAN_DEFAULT);


  return entries;
}

static void
destroy_toplevel                     (GtkButton       *button,
				      gpointer         user_data)
{
  gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(button)));
}
