#include <stdio.h>
#include <assert.h>
#include "seg_tmp.h"

#include "node.h"
#include "node_set.h"
#include "point.h"
#include "bitmap.h"

#define Bitm           SELF (bitm)
#define Current_point  SELF (current_point)
#define Word           SELF (word)
#define Word_move      SELF (word_move)
#define DimX           SELF (dimX)
#define DimY           SELF (dimY)
#define X_max          SELF (x_max)
#define Y_max          SELF (y_max)
#define Limit          SELF (limit)
#define Freebit        SELF (freebit)

#define MASK       ((*Word>>(4*(1-Word_move)))&0x0f)

typedef bouleen (*PFhas_neighbour)(bitmap,angle);

#define HAS_RIGHT_NEIGHBOUR(val) ( ((val) & RIGHT_NEIGHBOUR) != 0)
#define HAS_LEFT_NEIGHBOUR(val)  ( ((val) & LEFT_NEIGHBOUR)  != 0)
#define HAS_UP_NEIGHBOUR(val)    ( ((val) & UP_NEIGHBOUR)    != 0)

static void      write_right_neighbour_priv(bitmap, bouleen);
static void      write_up_neighbour_priv   (bitmap, bouleen);
static void      write_left_neighbour_priv (bitmap, bouleen);
static void      write_down_neighbour_priv (bitmap, bouleen);
static void      init_border_priv          (bitmap);
static move_type give_direction_priv       (bitmap,angle, move_type);
/****************************************************************************/
/* Les pixels sont codes sur 4 bits. Donc un octet(un any) represente */
/* 2 pixels */
/* Word represente le nombre d'octets depuis l'adresse de depart. */
/* Wordmove indique si l'on s'interesse a la premiere moitie de */
/* l'octet ou a la seconde */
/* move to et give_current_point effectuent des translations de (1,1) */
/* et (-1,-1) de facon a placer l'origine en (-1,-1) pour l'exterieur */
/****************************************************************************/
struct bitmap 
{
  any    *freebit;
  any    *bitm;
  any    *word;
  any    *limit;
  Point  current_point;
  int    word_move;
  long   dimX;
  long   dimY;
  long   x_max;
  long   y_max;
};

bitmap FAIRE_bitmap(long dimy,long dimx)
{
  long  integer,rest;

  INSTANCIER(bitmap);
  
  DimX  = dimx+1;
  DimY  = dimy+1;
  
  X_max = DimX - 1;
  Y_max = DimY - 1;
  
  
  integer  = DimX * DimY;
  rest     = integer & 1;
  integer  >>= 1;
  
  Bitm  = (any *) malloc (integer + rest);
  
  Limit    = Bitm + integer + rest;
  bitmap_clear_bitmap(self);


  Word                  = Bitm;
  Word_move             =  0;
  COORDX(Current_point) = -1;
  COORDY(Current_point) = -1;

  init_border_priv(self);

  return self;
}

/* DimX and DimY code the coordinates */

/*---------------------------------------------------------------------------*/

void DEFAIRE_bitmap(bitmap self)
{

  free (Bitm);
  free(self);
}
/*---------------------------------------------------------------------------*/

void bitmap_clear_bitmap(bitmap self)
{
  any *pt1;
  
  pt1 = Bitm;
  while (pt1 < Limit) *pt1++ = UNLIGHT;
}

/*---------------------------------------------------------------------------*/

angle bitmap_move_to(bitmap self,point p)
{
  long  dep;
  Point p_bitmap;

  if( p == NULL_POINT)
    return INFINITE;

  if(INFINITE_POINT(*p))
    return INFINITE;

  COORDX(p_bitmap) = COORDX(*p)+1;
  COORDY(p_bitmap) = COORDY(*p)+1;

  if ( (COORDX(p_bitmap) > X_max) || (COORDY(p_bitmap) > Y_max) )
    return INFINITE;

  dep   = DimX*COORDY(p_bitmap)  + COORDX(p_bitmap);
  Word_move = dep&1;
  dep >>= 1;
  
  Word = Bitm + dep;
      
  AFFECTE_POINT(Current_point,p_bitmap);
      
  return MASK;
}

/*---------------------------------------------------------------------------*/

void bitmap_give_current(bitmap self,point p)
{
  if( p != NULL_POINT )
    {
      COORDX(*p) = COORDX(Current_point)-1;
      COORDY(*p) = COORDY(Current_point)-1;
    }
}
/*---------------------------------------------------------------------------*/
angle bitmap_give_mask(bitmap self)
{
  return MASK;
}

/*---------------------------------------------------------------------------*/

angle bitmap_right_point(bitmap self)
{

  if (COORDX(Current_point) >= X_max)
    return  INFINITE;
  
  MOVE_RIGHT_POINT(Current_point);
  if (Word_move == 0)
    Word_move++;
  else
    {
      Word++;
      Word_move = 0;
    }
  return MASK;
  
}

/*---------------------------------------------------------------------------*/

angle bitmap_left_point(bitmap self)
{
  
  if (COORDX(Current_point) == -1)
    return INFINITE;

  MOVE_LEFT_POINT(Current_point);
  if (Word_move > 0)
    --Word_move;
  else
    {
      --Word;
      Word_move = 1;
    }

  return MASK;
}

/*---------------------------------------------------------------------------*/

angle bitmap_down_point(bitmap self)
{

  if (COORDY(Current_point) >= Y_max)
    return INFINITE;

  MOVE_DOWN_POINT(Current_point);
  Word     += DimX >> 1;
  if (DimX & 1) Word += Word_move;
  Word_move ^= DimX & 1;

  return MASK;

}

/*---------------------------------------------------------------------------*/

angle  bitmap_up_point(bitmap self)
{

  if (COORDY(Current_point) == 0)
    return INFINITE;

  MOVE_UP_POINT(Current_point);
  Word     -= DimX >> 1;
  if (DimX & 1) Word -= Word_move ^ 1;
  Word_move ^= DimX & 1;

  return  MASK;

}

/*---------------------------------------------------------------------------*/

void bitmap_unlight_point(bitmap self)
{
  if(Word_move == 0)
    *Word &= 0x0f;
  else
    *Word &= 0xf0;
}

/*---------------------------------------------------------------------------*/

void bitmap_make_node(bitmap self,point p, bouleen on)
{
  angle val;
  
  bitmap_move_to(self,p);
  
  val = NODE_CONST;

  if (Word_move == 0) val <<= 4;

  if (on)
    *Word |= val;
  else
    *Word &= ~val;
}

/*---------------------------------------------------------------------------*/

void bitmap_light_point(bitmap self, move_type prev_move, move_type next_move, bouleen on)
  {
    switch (prev_move)
      {
        case RIGHT:
	  write_left_neighbour_priv (self, on);
	  break;
        case UP:
	  write_down_neighbour_priv (self, on);
	  break;
        case LEFT:
	  write_right_neighbour_priv (self, on);
	  break;
        case DOWN:
	  write_up_neighbour_priv (self, on);
	  break;
      }

    switch (next_move)
      {
        case RIGHT:
	  write_right_neighbour_priv (self, on);
	  break;
        case UP:
	  write_up_neighbour_priv (self, on);
	  break;
        case LEFT:
	  write_left_neighbour_priv (self, on);
	  break;
        case DOWN:
	  write_down_neighbour_priv (self, on);
	  break;
      }
  }

/*---------------------------------------------------------------------------*/

void bitmap_draw_segment(bitmap self,seg_tmp gen)
{
  move_type prev_move,
            next_move;
  Point     p;
  
  prev_move = INFINITE;
  
  seg_tmp_first(gen);
  seg_tmp_give_current_point(gen,&p);
  bitmap_move_to(self,&p);
  do
    {
      next_move = seg_tmp_give_current_move(gen);
      bitmap_light_point(self, prev_move, next_move, vrai);
      bitmap_move_in_bitmap(self, next_move);
      prev_move = next_move;
    }
  while(seg_tmp_next(gen));

  bitmap_light_point(self, prev_move, INFINITE, vrai);
}

/*---------------------------------------------------------------------------*/

void bitmap_contour_tracking(bitmap self, 
			     point p, 
			     move_type *dep,
			     point return_point)
{
  angle      val,valp;
  move_type  new_dep;
  static move_type first_dep[8] =
    {
      INFINITE,/*0*/
      UP,/*1*/
      RIGHT,/*2*/
      RIGHT,/*3*/
      UP,/*4*/
      LEFT,/*5*/
      RIGHT,/*6*/
      RIGHT/*7*/
    };
      
  assert(return_point);

  val = bitmap_move_to(self,p);
  
  if (IS_UNLIGHT (val) || IS_INFINITE (val))
    {
      COORDX(*return_point) = INFINITE;
      COORDY(*return_point) = INFINITE;
      *dep = INFINITE;
  
      return;
    }

  valp = val&(~NODE_CONST);
  *dep = first_dep[valp];
  if(val == 13)
    if ( bitmap_has_down_neighbour(self, val) )
      *dep = UP;

  while (!IS_INFINITE (val) && !IS_NODE (val))
    {
      new_dep = bitmap_give_direction(self, val, *dep);
      val = bitmap_move_in_bitmap(self, new_dep);
      *dep = new_dep;
    }
  
  if (IS_INFINITE (val)) 
    { 
      COORDX(*return_point) = INFINITE;
      COORDY(*return_point) = INFINITE; 
      *dep = INFINITE; 
      return;
    }

  *dep = REVERSE_SENSE(*dep); 
  bitmap_give_current(self,return_point);

}

/*---------------------------------------------------------------------------*/

void bitmap_sounding(bitmap self,point p_dep,point return_point)

{
  angle  val;

  assert(return_point);
    
  bitmap_move_to(self,p_dep);
  val = bitmap_give_mask(self);
  while (IS_UNLIGHT (val))
    {
      val = bitmap_right_point(self);
    }
  
  if (!IS_INFINITE (val))
    bitmap_give_current(self,return_point);
  else
      {
	COORDX(*return_point) = INFINITE;
	COORDY(*return_point) = INFINITE;
      }

}

/*---------------------------------------------------------------------------*/

angle bitmap_move_in_bitmap(bitmap self, move_type move_value)
{
  angle ret_val;
  
  switch (move_value)
    {
    case RIGHT:
      ret_val = bitmap_right_point(self);
      break;
      
    case UP:
      ret_val = bitmap_up_point(self);
      break;
      
    case LEFT:
      ret_val = bitmap_left_point(self);
      break;
      
    case DOWN:
      ret_val = bitmap_down_point(self);
      break;

    default:
      ret_val = INFINITE;
      break;
    }

  return ret_val;
}	      

/*---------------------------------------------------------------------------*/

bouleen bitmap_has_right_neighbour(bitmap self, angle val)
{
  return HAS_RIGHT_NEIGHBOUR (val);
}

/*---------------------------------------------------------------------------*/

bouleen bitmap_has_up_neighbour(bitmap self, angle val)
{
  return HAS_UP_NEIGHBOUR (val);
}	      

/*---------------------------------------------------------------------------*/

bouleen bitmap_has_left_neighbour(bitmap self, angle val)
{
  return HAS_LEFT_NEIGHBOUR (val);
}	      

/*---------------------------------------------------------------------------*/

bouleen bitmap_has_down_neighbour(bitmap self, angle val)
{
  move_type  v;

  v = bitmap_down_point(self);

  if (!IS_INFINITE (v)) bitmap_up_point(self);

  return HAS_UP_NEIGHBOUR (v);
}	      
/*---------------------------------------------------------------------------*/
/* Retourne si le point un demis envisage a un voisin dans la */
/* direction move */
/*---------------------------------------------------------------------------*/
bouleen bitmap_has_neighbour_in_direction(bitmap self,
					  angle val,
					  move_type move)
{
  static PFhas_neighbour has_neighbour[4] =
    {
      bitmap_has_right_neighbour,
      bitmap_has_up_neighbour,
      bitmap_has_left_neighbour,
      bitmap_has_down_neighbour
    };
   
  return has_neighbour[move](self,val);
}
/*---------------------------------------------------------------------------*/

void bitmap_remove_seg(bitmap self, segment old_seg)
{
  move_type  prev_move,
             next_move;
  Point      p;
  angle val;

  segment_first(old_seg);
  segment_give_current_point(old_seg,&p);
  next_move = segment_get_first_move(old_seg);
  /* On s'occupe du noeud correspondant a first_point */
  bitmap_move_to(self,&p);
  bitmap_light_point(self,INFINITE,next_move, faux);
  val = bitmap_move_in_bitmap(self,next_move);
  prev_move = next_move;

  while (!IS_NODE(val))
    {
      next_move = give_direction_priv(self,val,prev_move);
      bitmap_light_point(self, prev_move, next_move, faux);
      val = bitmap_move_in_bitmap(self, next_move);
      prev_move = next_move;
      assert(val != INFINITE);
    }

/* On s'occupe du noeud correspondant a Last_point*/
  bitmap_light_point(self, prev_move, INFINITE, faux);
}

/*---------------------------------------------------------------------------*/
move_type bitmap_give_direction(bitmap self,
				angle val,
				move_type dep)
{
  static move_type next_direction[11][NB_MOVE] =
    { /* R        U        L        D */
      {INFINITE, INFINITE, INFINITE, INFINITE},/* 0 */
      {INFINITE, RIGHT   , DOWN    , INFINITE},/* 1 */
      {DOWN    , LEFT    , INFINITE, DOWN},    /* 2 */
      {RIGHT   , INFINITE, LEFT    , INFINITE},/* 3 */
      {INFINITE, UP      , INFINITE, DOWN},    /* 4 */
      {INFINITE, INFINITE, UP      , RIGHT},   /* 5 */
      {UP      , INFINITE, INFINITE, LEFT},    /* 6 */
      {INFINITE, INFINITE, INFINITE, INFINITE},/* 7 */
      {INFINITE, INFINITE, INFINITE, INFINITE},/* 8 */
      {INFINITE, RIGHT   , DOWN    , INFINITE},/* 9 */
      {DOWN    , LEFT    , INFINITE, DOWN}     /* 10 */
    };
  
  assert(self);

  if(val <= 10)
    return next_direction[val][dep];

  return give_direction_priv(self,val,dep);
}
static move_type give_direction_priv(bitmap self, angle val, move_type dep)
{
  bouleen       continu;
  move_type  direction,
             last_direction;

  direction = dep;

  last_direction = NEXT_DIRECTION(dep);
  
  continu = vrai;
  
  while (continu)
    {
      direction = NEXT_DIRECTION(direction);
      
      switch (direction)
	{
	case RIGHT:
	  if( bitmap_has_right_neighbour(self, val) ) continu = faux;
	  break;
	case UP:
	  if( bitmap_has_up_neighbour(self,val) ) continu = faux;
	  break;
	case LEFT:
	  if( bitmap_has_left_neighbour(self,val) ) continu = faux;
	  break;
	case DOWN:
	  if( bitmap_has_down_neighbour(self,val) ) continu = faux;
	  break;
	}
      
      if (continu)
	{
	  direction = REVERSE_SENSE(direction);
	  
	  if (direction == last_direction)
	    {
	      continu = faux;
	      direction = INFINITE;
	    }
	}
    }
  
  return direction;
}

void bitmap_save(bitmap self,char* name)
{
  any* ptr,*ptr_save;
  Point p;
  FILE *id;
  int x,y;
  
  ptr = (any*)malloc(DimX*DimY*sizeof(any));
  ptr_save = ptr;

  COORDX(p) = -1;
  COORDY(p) = -1;
  bitmap_move_to(self,&p);

  for(y=0;y<DimY;y++)
    {
      for(x=0;x<DimX;x++)
	{
	  *ptr = MASK;
	  ptr++;
	  bitmap_right_point(self);
	}
      COORDY(p)++;
      bitmap_move_to(self,&p);
    }
  id = fopen(name,"w");
  fprintf(id,"P5\n");
  fprintf(id,"%d %d\n",(int)DimX,(int)DimY);
  fprintf(id,"%d\n",15);
  fwrite(ptr_save,sizeof(any),DimX*DimY,id);
  fclose(id);
  free(ptr_save);
}
static void write_right_neighbour_priv (bitmap self, bouleen on)
{
  angle val;
  
  val = RIGHT_NEIGHBOUR;
  
  if (Word_move == 0) val <<= 4;
  
  if (on)
    *Word |= val;
  else
    *Word &= ~val;
}

/*---------------------------------------------------------------------------*/

static void write_up_neighbour_priv (bitmap self, bouleen on)
{
  angle val;
  
  val = UP_NEIGHBOUR;
  
  if (Word_move == 0) val <<= 4;
  
  if (on)
    *Word |= val;
  else
    *Word &= ~val;
}

/*---------------------------------------------------------------------------*/

static void write_left_neighbour_priv (bitmap self, bouleen on)

{
  angle val;
  
  val = LEFT_NEIGHBOUR;
  
  if (Word_move == 0) val <<= 4;
  
  if (on)
    *Word |= val;
  else
    *Word &= ~val;
}

/*---------------------------------------------------------------------------*/

static void write_down_neighbour_priv (bitmap self, bouleen on)
{
  angle val;
  
  val = bitmap_down_point(self);
  
  if (!IS_INFINITE (val))
    {
      val = UP_NEIGHBOUR;
      
      if (Word_move == 0) val <<= 4;
      
      if (on)
	*Word |= val;
      else
	*Word &= ~val;
      
      bitmap_up_point(self);
    }
}

static void init_border_priv(bitmap self)
{
  int i;
  Point p;

  COORDX(p) =-1;
  COORDY(p) =-1;
  bitmap_move_to(self,&p);
  bitmap_make_node(self,&p,vrai);

  /* Ligne du haut */
  bitmap_light_point(self,INFINITE,RIGHT,vrai);
  for(i=1;i<X_max;i++)
    {
      bitmap_move_in_bitmap(self,RIGHT);
      bitmap_light_point(self,RIGHT,RIGHT,vrai);
    }

/* Colonne de droite */
  bitmap_move_in_bitmap(self,RIGHT);
  bitmap_light_point(self,RIGHT,DOWN,vrai);
  for(i=1;i<Y_max;i++)
    {
      bitmap_move_in_bitmap(self,DOWN);
      bitmap_light_point(self,DOWN,DOWN,vrai);
    }

/* Ligne du bas */
  bitmap_move_in_bitmap(self,DOWN);
  bitmap_light_point(self,DOWN,LEFT,vrai);
  for(i=1;i<X_max;i++)
    {
      bitmap_move_in_bitmap(self,LEFT);
      bitmap_light_point(self,LEFT,LEFT,vrai);
    }
/* Colonne de gauche */
  bitmap_move_in_bitmap(self,LEFT);
  bitmap_light_point(self,LEFT,UP,vrai);
  for(i=1;i<Y_max;i++)
    {
      bitmap_move_in_bitmap(self,UP);
      bitmap_light_point(self,UP,UP,vrai);

    }
  bitmap_move_in_bitmap(self,UP);
  bitmap_light_point(self,UP,INFINITE,vrai);
}


