#include <math.h>
#include <glib.h>

#include "macros.h"
#include "type_obj.h"
#include "contour_set.h"
#include "objet.h"
#include "tab_int.h"

#include "droite.h"
#include "chemin_discret.h"
#include "chemin_euclidien.h"

#define PIXEL_ECH 256

/* TYPE CHEMIN EUCLIDIEN : suite de coordonnees x et y*/
#define X_coord    SELF (x_coord)
#define Y_coord    SELF (y_coord)
#define Nb_pts     SELF (nb_pts)
#define Chem_discr SELF(chem_discr)   

struct Chemin_Eucl
{
  Chemin_Discret chem_discr;
  tab_int        x_coord;
  tab_int        y_coord;
  int            nb_pts;
};

static void calcul_point_priv(Chemin_Eucl, int, Droite_Discrete);
static void calcul_decalages_priv(Chemin_Eucl, Chemin_Discret);
static float real_decal_x(Chemin_Eucl, int);
static float real_decal_y(Chemin_Eucl, int);
static float distance_euclidienne(float, float, float, float);


/* CREATION -  LIBERATION */

Chemin_Eucl FAIRE_Chemin_Eucl(){

  INSTANCIER(Chemin_Eucl);

  Chem_discr = FAIRE_Chemin_Discret();
  X_coord = FAIRE_tab_int();
  Y_coord = FAIRE_tab_int();
  Nb_pts = 0;
  return self;
}

void DEFAIRE_Chemin_Eucl(Chemin_Eucl self){
  DEFAIRE_Chemin_Discret(Chem_discr);
  DEFAIRE_tab_int(X_coord);
  DEFAIRE_tab_int(Y_coord);
  free(self);
}

void Chemin_Eucl_reinitialize(Chemin_Eucl self){
  tab_int_reinitialize(X_coord);
  tab_int_reinitialize(Y_coord);
  Nb_pts = 0;
}


/* CALCUL D'UN CHEMIN EUCLIDIEN A PARTIR D'UNE FACE */

void Chemin_Eucl_compute_from_face(Chemin_Eucl self, 
				   contour_set contours, 
				   dart f){

  Chemin_Discret_compute_from_face(Chem_discr, contours, f);
  Nb_pts = Chemin_Discret_nb_codes(Chem_discr);
  calcul_decalages_priv(self, Chem_discr);
}

/* CALCUL D'UN CHEMIN EUCLIDIEN A PARTIR D'UN SEGMENT */

void Chemin_Eucl_compute_from_segment(Chemin_Eucl self, 
				      contour_set contours, 
				      dart brin){
  Chemin_Discret_compute_from_segment(Chem_discr, contours, brin);
  Nb_pts = Chemin_Discret_nb_codes(Chem_discr) + 1;
  calcul_decalages_priv(self, Chem_discr);
}


/* ACCES AU TABLEAU DE COORDONNEES */

int Chemin_Eucl_x_decal(Chemin_Eucl self, int ind_pt){
  return (tab_int_give_value(X_coord, ind_pt));
}

int Chemin_Eucl_y_decal(Chemin_Eucl self, int ind_pt){
  return (tab_int_give_value(Y_coord, ind_pt));
}

/* CALCUL DU DECALAGE REEL EN FONCTION DU DECALAGE ENTIER */

float Chemin_Eucl_real_value(int delta_entier){
  float delta_reel;
  delta_reel = (float)delta_entier/(float)PIXEL_ECH - 0.5;
  return delta_reel;
}

/* CALCUL DE LA PROJECTION D'UN POINT : remplissage de X et Y*/

static void calcul_point_priv(Chemin_Eucl self, int ind_pt, 
			     Droite_Discrete d){
  double alpha;
  int a, b, mu, quadrant;
  int sym4[4][2] = {{1,1},{-1,1},{-1,-1},{1,-1}};
  int delt_X, delt_Y;

  a = Droite_Discrete_give_a(d);
  b = Droite_Discrete_give_b(d);
  mu = Droite_Discrete_give_mu(d);
  quadrant = Droite_Discrete_give_quadrant(d);
    
  alpha = (double)(2*mu + b - 1)/(double)(2*b);
  delt_X = (int)((sym4[quadrant][0]*alpha+0.5)*PIXEL_ECH);
  delt_Y = (int)((-sym4[quadrant][1]*alpha+0.5)*PIXEL_ECH);
  tab_int_put_value(X_coord, ind_pt, delt_X);
  tab_int_put_value(Y_coord, ind_pt, delt_Y);
}


/* CALCUL DU CHEMIN EUCLIDIEN CORRESPONDANT A 1 CHEMIN DISCRET */

static void calcul_decalages_priv(Chemin_Eucl self,
				  Chemin_Discret chem_discr){
  int i;
  Droite_Discrete tangente_locale;

  tangente_locale = FAIRE_Droite_Discrete();
  for(i = 0; i < Nb_pts; i++){
    Droite_Discrete_calcul_tangente(tangente_locale, chem_discr, i);
    calcul_point_priv(self, i, tangente_locale);
    Droite_Discrete_reinit(tangente_locale);
  }
  DEFAIRE_Droite_Discrete(tangente_locale);
}


/* CALCUL DE PARAMETRES SUR UN CHEMIN EUCLIDIEN*/

float Chemin_Eucl_surface(Chemin_Eucl self){

  int i, nb_pts = Chemin_Discret_nb_codes(Chem_discr);
  int Xi, Xi_plus, Xi_moins, Yi, Yi_plus, Yi_moins;
  float xi, xi_plus, xi_moins, yi, yi_plus, yi_moins, x0, y0, y1;
  float surf = 0;

  Xi_moins = 1000; /* pt d'ind 0. pour avoir des coordonnees positives */
  Yi_moins = 1000;
  x0 = Xi_moins + real_decal_x(self, 0);
  yi_moins = Yi_moins - real_decal_y(self, 0);
  y0 = yi_moins;

  Xi = Xi_moins + Chemin_Discret_delta_x(Chem_discr, 0); /* pt d'ind 1*/
  Yi = Yi_moins + Chemin_Discret_delta_y(Chem_discr, 0);
  xi = Xi + real_decal_x(self, 1);
  yi = Yi - real_decal_y(self, 1);
  y1 = yi;

  for(i = 1; i < nb_pts-1; i++){
    Xi_plus = Xi + Chemin_Discret_delta_x(Chem_discr, i);
    Yi_plus = Yi + Chemin_Discret_delta_y(Chem_discr, i);
    xi_plus = Xi_plus + real_decal_x(self, i+1);
    yi_plus = Yi_plus - real_decal_y(self, i+1);
    surf += xi*(yi_plus - yi_moins);

    yi_moins = yi;
    xi = xi_plus;
    yi = yi_plus;
    Xi = Xi_plus;
    Yi = Yi_plus;
  }
  surf += xi*(y0-yi_moins);/*dernier terme de la somme */
  surf += x0*(y1-yi); /*premier terme de la somme */

  surf = ABS(surf);
  return surf/2.0;
}

/* CALCUL DU PERIMETRE DU DOMAINE EUCLIDIEN */


float Chemin_Eucl_perimetre(Chemin_Eucl self){

  int i, nb_pts = Chemin_Discret_nb_codes(Chem_discr);
  float x1, y1, x2, y2;
  int X1, Y1, X2, Y2;
  float peri = 0;
  
  x1 = real_decal_x(self, 0);
  y1 = -real_decal_y(self, 0);
  X1 = 0;
  Y1 = 0;

  for(i = 0; i < nb_pts - 1; i++){
    X2 = X1 + Chemin_Discret_delta_x(Chem_discr, i);
    Y2 = Y1 + Chemin_Discret_delta_y(Chem_discr, i);
    x2 = X2 + real_decal_x(self, i+1);
    y2 = Y2 - real_decal_y(self, i+1);

    peri += distance_euclidienne(x1, y1, x2, y2); 
    X1 = X2;
    Y1 = Y2;
    x1 = x2;
    y1 = y2;
  }

  x2 = real_decal_x(self, 0);
  y2 = -real_decal_y(self, 0);

  peri += distance_euclidienne(x1, y1, x2, y2); 

  return peri;
}

static float real_decal_x(Chemin_Eucl self, int ind_pt){
 return Chemin_Eucl_real_value(Chemin_Eucl_x_decal(self, ind_pt));
}

static float real_decal_y(Chemin_Eucl self, int ind_pt){
 return Chemin_Eucl_real_value(Chemin_Eucl_y_decal(self, ind_pt));
}

static float distance_euclidienne(float x1, float y1, float x2, float y2){
  return sqrt((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2));
}

