#include <values.h>
#include "cluster_set.h"
#include "cluster.h"
#include "types.h"
#include "type_obj.h"
#include "objet.h"
#include "conversion.h"
#include "contour_set.h"
#include "partition.h"
#include "vector.h"
#include "point.h"

#define Nb_cluster     SELF(nb_cluster)
#define Clusters       SELF(clusters)
#define Equiv          SELF(equiv)
#define Global_sqe     SELF(global_sqe)
#define Sqe_sum        SELF(sqe_sum)

#define INIT_PARTITION       1
#define INIT_CLUSTER         2
#define INIT_CONVERT         3
#define RETURN_NB_CLUSTER 4


static void make_clusters_priv(cluster_set,
				 cluster,
				 int);
static void store_colors_priv(any_ptr,
			      point,
			      int);
struct cluster_set
{
  int     nb_cluster;
  int*    equiv;
  double  sqe_sum;
  double  global_sqe;
  cluster clusters;
};

cluster_set FAIRE_cluster_set(contour_set contours)
{
  INSTANCIER(cluster_set);

  return self;
}
  
void DEFAIRE_cluster_set(cluster_set self)
{
  free(Clusters);
  free(self);
}
void cluster_set_init_from_face(cluster_set self,  
				contour_set contours,
				dart f,
				partition init_part,
				int nb_cluster,
				int space)
{
  int i,j;
  cluster ptr,ptr_aux;
  PFconversion *rgb_to_c1c2c3 = changer_espace(space);

  ptr = (cluster)malloc(nb_cluster*sizeof(struct cluster));
  Nb_cluster = 0;
  
  ptr_aux = ptr;
  for(i=0;i<nb_cluster;i++)
    {
      ptr_aux->mom0 = 0.0;      
      for(j=0;j<DIM_MAX;j++)
	{	
	  ptr_aux->mom1[j] = 0.0;
	  ptr_aux->mom2[j] = 0.0;
	}
      ptr_aux++;
    }
  store_colors_priv(ptr,NULL_POINT,INIT_CLUSTER);
  store_colors_priv(init_part,NULL_POINT,INIT_PARTITION);
  store_colors_priv(rgb_to_c1c2c3,NULL_POINT,INIT_CONVERT);
  contour_set_apply_function(contours,f,store_colors_priv);
  store_colors_priv(&Nb_cluster,NULL_POINT,RETURN_NB_CLUSTER);

  if(Nb_cluster == nb_cluster)
    {
      Clusters = ptr;
      make_clusters_priv(self,ptr,nb_cluster);
      return;
    }

  Clusters = (cluster)malloc(Nb_cluster*sizeof(struct cluster));
  make_clusters_priv(self,ptr,nb_cluster);
  free(ptr);
  
  return;
}

void cluster_set_init_from_set_of_face(cluster_set self,
				       contour_set contours,
				       dart_list face_list)
{
  double Mom0,m0;
  vector m1,m2;
  vector Mom1 = {0.0,0.0,0.0};
  vector Mom2 = {0.0,0.0,0.0};
  int i;
  dart f;

  Nb_cluster = dart_list_give_cardinal(face_list);
  Clusters   = (cluster)malloc(Nb_cluster*sizeof(struct cluster));
  Equiv      = (int*)malloc(Nb_cluster*sizeof(int));

  Mom0 = 0.0;
  Sqe_sum = 0.0;
  i=0;
  dart_list_first(face_list);
  do
    {
      Equiv[i] = i;
      f = dart_list_give_current(face_list);
      contour_set_give_face_moments(contours,f,&m0,m1,m2);
      cluster_set_moments(Clusters+i,m0,m1,m2);
      Sqe_sum += cluster_give_sqe(Clusters+i);
      Mom0 +=m0;
      vector_equal_plus(Mom1,m1,DIM_MAX);
      vector_equal_plus(Mom2,m2,DIM_MAX);
      i++;
    }
  while(dart_list_next(face_list));
  Global_sqe = 0.0;
  for(i=0;i<DIM_MAX;i++)
    Global_sqe += (Mom2[i]-Mom1[i]*Mom1[i]/Mom0);
}
double cluster_set_give_global_sqe(cluster_set self)
{
  return Global_sqe;
}
double cluster_set_give_sqe_sum(cluster_set self)
{
  return Sqe_sum;
}

int cluster_set_give_cardinal(cluster_set self)
{
  return Nb_cluster;
}
double cluster_set_give_sqe(cluster_set self, int clus)
{  
  return Clusters[clus].sqe;
}
double cluster_set_give_color_variance(cluster_set self)
{
  int i,j;
  cluster ptr;
  double m0;
  vector m1,m2;
  double var;
  
  m0=0.0;
  for(i=0;i<DIM_MAX;i++)
    {
      m1[i] = 0.0;
      m2[i] = 0.0;
    }
  ptr = Clusters;
  for(i=0;i<Nb_cluster;i++)
    {
      if(ptr->exist)
	{
	  m0++;
	  for(j=0;j<DIM_MAX;j++)
	    {
	      m1[j] += ptr->mean[j];
	      m2[j] += (ptr->mean[j])*(ptr->mean[j]);
	    }
	}
      ptr++;
    }
  
  var = 0.0;
  for(i=0;i<DIM_MAX;i++)
    var += (m2[i]-m1[i]*m1[i]/m0);
  var /= m0;

  return var;
}
/* On ne traite pas les intervalles qui n'ont apres le merges a priori */
/* plus de signification */
void cluster_set_merge_cluster(cluster_set self,int clus1,int clus2)
{
  double alpha,mom0c1,mom0c2;
  cluster c1,c2;
  int i;

  c1 = Clusters+clus1;
  c2 = Clusters+clus2;

  c2->exist = faux;
  c2->canonic = clus1;

  mom0c1 = c1->mom0;
  mom0c2 = c2->mom0;
  alpha  = (mom0c1*mom0c2)/(mom0c1+mom0c2);
  c1->sqe += c2->sqe + alpha*vector_dist(c1->mean,c2->mean,DIM_MAX);

  c1->mom0 += mom0c2;

  for(i=0;i<DIM_MAX;i++)
    {
      c1->mom1[i] += c2->mom1[i];
      c1->mom2[i] += c2->mom2[i];
      c1->mean[i] = c1->mom1[i]/c1->mom0;
    }
}
double cluster_set_get_dist(cluster_set self,int clus1,int clus2)
{
  cluster c1,c2;
  double alpha,mom0c1,mom0c2;
  double dist;

  c1 = Clusters+ clus1;
  c2 = Clusters+ clus2;

  if(!c1->exist || !c2->exist)
    return DBL_MAX;

  mom0c1 = c1->mom0;
  mom0c2 = c2->mom0;
  alpha  = (mom0c1*mom0c2)/(mom0c1+mom0c2);
  dist   = c1->sqe + c2->sqe + alpha*vector_dist(c1->mean,c2->mean,DIM_MAX);

  return dist;
}
double cluster_set_get_dist_mean(cluster_set self,int clus1,int clus2)
{
  cluster c1,c2;
  double alpha,mom0c1,mom0c2;
  double dist;

  c1 = Clusters+ clus1;
  c2 = Clusters+ clus2;

  if(!c1->exist || !c2->exist)
    return DBL_MAX;

  mom0c1 = c1->mom0;
  mom0c2 = c2->mom0;
  alpha  = (mom0c1*mom0c2)/(mom0c1+mom0c2);
  dist   = alpha*vector_dist(c1->mean,c2->mean,DIM_MAX);

  return dist;
}
double cluster_set_get_color_dist(cluster_set self,int clus1,int clus2)
{
  cluster c1,c2;
  double alpha,mom0c1,mom0c2;
  double dist;

  c1 = Clusters+ clus1;
  c2 = Clusters+ clus2;

  if(!c1->exist || !c2->exist)
    return DBL_MAX;

  return vector_dist(c1->mean,c2->mean,DIM_MAX);
}

int cluster_set_give_canonic_cluster(cluster_set self,int clus)
{
  return Clusters[Equiv[clus]].canonic;
}
void cluster_set_set_canonic(cluster_set self)
{
  int i,ind;
  
  for(i=Nb_cluster-1;i>=0;i--)
    {
      if(Clusters[i].exist)
	Clusters[i].canonic = i;
      else
	{
	  ind = Clusters[i].canonic;
	  Clusters[i].canonic = Clusters[ind].canonic;
	}
    }
}
void cluster_set_give_canonics(cluster_set self,
			       dart_tab canonic_faces,
			       int* nb_canonic)
{
  int i,ind;
  
  ind =0;
  for(i=Nb_cluster-1;i>=0;i--)
    {
      if(Clusters[i].exist)
	{
	  dart_tab_put_value(canonic_faces,i,ind);
	  ind++;
	}
    } 
  *nb_canonic = ind;
}
void cluster_set_give_table(cluster_set self,double* means)
{
  int i,j;
  cluster ptr;
  
  ptr = Clusters;
  for(i=0;i<Nb_cluster;i++)
    {
      if(ptr->exist)
	{
	  for(j=0;j<DIM_MAX;j++)
	    {
	      *means = ptr->mean[j];
	      means++;
	    }
	}
      ptr++;
    }
}
static void make_clusters_priv(cluster_set self,
                               cluster ptr,
                               int nb_set)
{
  int ind;
  int i;
  double m0 = 0.0;
  vecteur m1 = {0.0,0.0,0.0};
  vecteur m2 = {0.0,0.0,0.0};
  

  Equiv    = (int*)  malloc(nb_set*sizeof(int));
  Sqe_sum = 0.0;

  ind =0;
  for(i=0;i<nb_set;i++)
    {
      if(!cluster_is_empty(ptr))
        {
          cluster_copy(ptr,Clusters+ind);
	  Sqe_sum += cluster_give_sqe(Clusters+ind);
	  cluster_give_moments(Clusters+ind,&m0,m1,m2);
          Equiv[i] = ind;
          ind++;
        }
      ptr++;
    }
  Global_sqe = 0.0;
  for(i=0;i<DIM_MAX;i++)
    Global_sqe += m2[i] - m1[i]*(m1[i]/m0);
}

void store_colors_priv(any_ptr obj,point p,int flag)
{
  static nb_cluster = 0;
  static partition init_part;
  static cluster ptr;
  static bouleen wrong_flag_in_case=faux;
  static PFconversion *rgb_to_c1c2c3;
  int* pixel;
  int clus;
  int i;
  int C[DIM_MAX];

  switch(flag)
    {
    case INIT_CLUSTER:
      nb_cluster = 0;
      ptr = (cluster) obj;
      break;      
    case INIT_PARTITION:
      nb_cluster = 0;
      init_part = (partition) obj;
      break;
    case INIT_CONVERT:
      rgb_to_c1c2c3 = (PFconversion*)obj;
      break;
    case RETURN_NB_CLUSTER:
      *((int*)obj) = nb_cluster;
      break;
    case STORE:
      pixel = (int*)obj;
      clus = partition_give_cluster_nb(init_part,pixel);
      for(i=0;i<DIM_MAX;i++)
	C[i] = rgb_to_c1c2c3[i](pixel[0],pixel[1],pixel[2]);
      
      if(cluster_is_empty(ptr+clus))
	nb_cluster++;
      cluster_add_color_to_cluster(ptr+clus,C);
      break;
    default:
      assert(wrong_flag_in_case);
      break;
    }
}








