#include <stdio.h>
#include <assert.h>
#include <stdlib.h>
#include <glib.h>

#include "tab_mark.h"
#include "face.h"
#include "contour_set.h"
#include "node_set.h"
#include "node.h"
#include "tab_gen.h"
#include "inclusions.h"
#include "bitmap.h"
#include "bitmap_gen.h"
#include "map.h"
#include "segment.h"
#include "macros.h"
#include "point_set.h"
#include "list_gen.h"
#include "vector.h"
#include "objet.h"
#include "output_segment.h"

#define MIN_PERIMETER_TO_INCLUDE 12

/* structure utilise pour trier des tableaux de faces en fonction de */
/* leurs points le + a droite */
struct face_pt
{
  dart face;
  Point p;
};
/* Tableaux utilises pour marker des points en vue du parcourt d'une */
/* face */
static bouleen  begin_mark_needed[4][4] = 
{          /* R    U    L    D   */
/*R*/      {faux,vrai,faux,faux},
/*U*/      {faux,vrai,faux,faux},
/*L*/      {faux,vrai,faux,faux},
/*D*/      {faux,faux,faux,faux}
    };
  
static bouleen  end_mark_needed[4][4] = 
    {      /* R    U    L    D   */
/*R*/      {faux,faux,faux,vrai},
/*U*/      {faux,faux,faux,faux},
/*L*/      {faux,faux,faux,vrai},
/*D*/      {faux,faux,faux,vrai }
    };

#define Segment_set_contour    SELF(segment_set_contour)
#define Node_set_contour       SELF(node_set_contour)
#define Map_contour            SELF(map_contour)
#define Bitmap_contour         SELF(bitmap_contour)
#define Inclusions_contour     SELF(inclusions_contour)
#define Image_contour          SELF(image_contour)
#define Face_set_contour       SELF(face_set_contour)
#define Marked_points          SELF(marked_points)
#define Belong_to_same_cluster SELF(belong_to_same_cluster)
#define Modified_segments      SELF(modified_segments)
#define Inserted_faces         SELF(inserted_faces)
#define Memorize_segment_flag  SELF(memorize_segment_flag)
#define Memorize_face_flag     SELF(memorize_face_flag)
#define Image_var              SELF(image_var)

static move_type  private_is_infinite             (contour_set, dart);
static int        private_comp_face_pt            (const void*,const void*);
static void       private_make_component          (contour_set,seg_tmp);
static dart       private_compute_dart            (contour_set,
						   node, 
						   move_type);
static dart       private_give_dart               (contour_set,
						   node, 
						   move_type);
static node       private_make_node               (contour_set,point);
static void       private_right_point_face        (contour_set, dart,point);

static void       private_remove_one_arc_in_set   (contour_set,dart,dart_list);
static void       private_remove_set_of_arc       (contour_set,dart_list);
static void       private_remove_arc              (contour_set,dart);
static void       private_remove_tangent          (contour_set,dart,dart,dart);
static void       private_remove_component        (contour_set,dart,dart,dart);
static void       private_remove_node             (contour_set,point,dart);
static bouleen    private_extend_arc              (contour_set,dart,dart_list);
static void       private_insert_segment_border   (contour_set,int,int);
static void       private_update_faces            (contour_set,dart,dart);
static void       private_uninitialize_face       (contour_set,dart);
static void       private_traverse_face           (contour_set,
						   dart,
						   double*,double*,double*,
						   PFapply_to_pixels,bouleen);
static void       private_traverse_line           (contour_set,
						   point,
						   double*,double*,double*,
						   PFapply_to_pixels,bouleen);
static void       private_apply_function          (contour_set,
						   dart,PFapply_to_pixels);
static void       private_apply_function_on_line  (contour_set,
						   point,PFapply_to_pixels);
static void       private_mark_face               (contour_set,
						   dart,
						   void
						   (*mark_fct)(tab_mark,
							       point),
						   tab_mark);
static void       private_mark_segment             (segment,
						    void
						    (*mark_point)(tab_mark,
								  point),
						    tab_mark);
static void       private_unmark_face              (contour_set,
						    dart,
						    void 
						    (*unmark_fct)(
								  tab_mark,
								  point),
						    tab_mark);
static void       private_mark_interior_of_face    (contour_set,dart);
static void       private_unmark_interior_of_face  (contour_set,dart);
static void       private_mark_point_if_needed     (contour_set,
						    point,
						    move_type,
						    move_type);
static void       private_mark_point_one_time      (tab_mark,point);
static void       private_give_unmarked_dart        (contour_set,
						     dart,
						     dart*,
						     int*);
static void       private_compute_face_class        (contour_set,
						     dart,
						     dart_list);
static void       private_update_inclusions         (contour_set,
						     dart_list);
static void       private_memorize_segment          (contour_set, 
						     segment);	
static void       private_memorize_face             (contour_set,
						     dart);
static void       private_give_face_adjacence       (contour_set,
						     dart,
						     dart_list);
static move_type  private_give_first_move_of_dart   (contour_set,dart);

struct contour_set
{
  node_set       node_set_contour;
  tab_gen        segment_set_contour;
  tab_gen        face_set_contour;
  map            map_contour;
  bitmap         bitmap_contour;
  inclusions     inclusions_contour;
  image          image_contour;
  tab_mark       marked_points;
  bouleen        memorize_segment_flag;
  list_gen       modified_segments;
  bouleen        memorize_face_flag;
  dart_list      inserted_faces;
  PFsame_cluster belong_to_same_cluster;
  double         image_var;
};

contour_set FAIRE_contour_set(guchar* data,
			      int dim,
			      int width,
			      int height)
{
  int hauteur,largeur;
  int k,pixel[3];
  
  INSTANCIER(contour_set);

  Node_set_contour    = FAIRE_node_set();
  Segment_set_contour = FAIRE_tab_gen();
  Face_set_contour    = FAIRE_tab_gen();
  Map_contour         = FAIRE_map();
  Inclusions_contour  = FAIRE_inclusions();
  Modified_segments   = FAIRE_list_gen();
  Inserted_faces      = FAIRE_dart_list();
  Memorize_segment_flag = faux;
  Memorize_face_flag    = faux;
  Image_contour = FAIRE_image();
  hauteur = height;
  largeur = width;
  image_initialize(Image_contour,
		   dim,
		   width,
		   height);
  if (data != NULL)
    {
      /* Initialisation de l'image */
      image_debut(Image_contour);
      do
	{
	  for(k=0;k<dim;k++)
	    {
	      pixel[k]=*data;
	      data++;
	    }
	  image_ecrire_pixel(Image_contour,pixel);
	}
      while(image_pixel_suivant(Image_contour));
    }
  Bitmap_contour      = FAIRE_bitmap(hauteur,largeur);
  Marked_points       = FAIRE_tab_mark(hauteur,largeur);

  Belong_to_same_cluster = (PFsame_cluster)NULL;
  
  private_insert_segment_border(self,hauteur,largeur);
  
  Image_var = contour_set_give_face_variance(self,1);
  
  return(self);
}

/*---------------------------------------------------------------------------*/
void DEFAIRE_contour_set(contour_set self)
{
  assert(self);

  if(Node_set_contour == NULL_NODE_SET)
    return;

  DEFAIRE_node_set(Node_set_contour);
  DEFAIRE_tab_gen(Segment_set_contour);
  DEFAIRE_map(Map_contour);
  DEFAIRE_image(Image_contour);
  DEFAIRE_bitmap(Bitmap_contour);
  DEFAIRE_inclusions(Inclusions_contour);  
  free(self);
}
/*---------------------------------------------------------------------------*/
void contour_set_save(contour_set self,char* name)
{
  assert(self);
  assert(name);

  bitmap_save(Bitmap_contour,name);
}
/*---------------------------------------------------------------------------*/
bouleen contour_set_is_infinite(contour_set self, dart f)
{
  assert(self);
  return inclusions_is_infinite(Inclusions_contour,f);
}
dart contour_set_give_including_face(contour_set self,dart f)
{
  assert(self);

  if(!inclusions_is_infinite(Inclusions_contour,f))
    return f;
  
  return inclusions_give_father(Inclusions_contour,f);
}
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Ajoute un nouveau segment dans la carte et  le bitmap. Met */
/* egalement a jour les relations d'inclusions */
/*---------------------------------------------------------------------------*/
bouleen contour_set_add_new_contour(contour_set self,seg_tmp seg)
{
  segment    ins_segment;
  Point      first_point,last_point;
  int        new_num, label1, label2;
  dart       start_dart,end_dart,father1, father2;
  dart_list  face_to_inspect = NULL_DART_LIST;
  angle      val_first,val_last;
  node       first_node,last_node;
  move_type  start_move,end_move;
  move_type  prev_move;
  dart       f,father;


  seg_tmp_give_first_point(seg,&first_point);
  seg_tmp_give_last_point(seg,&last_point);

  start_move = seg_tmp_give_first_move(seg);
  end_move   = seg_tmp_give_last_move(seg);
  end_move = REVERSE_SENSE(end_move);

  /* On verifie que l'on est partie d'un segment et que l'on arrive a */
  /* un segment */
  val_first = bitmap_move_to(Bitmap_contour,&first_point);
  val_last  = bitmap_move_to(Bitmap_contour,&last_point);
  
  /* Le segment se ferme sur lui meme */
  if(seg_tmp_is_closed(seg)&&IS_UNLIGHT(val_first))
    {
      private_make_component(self,seg);
      return vrai;
    }
  if(IS_UNLIGHT(val_first) || IS_UNLIGHT(val_last))
    {
      printf(" INSERSION d'un segment pendant ERREUR\n");
      assert(!(IS_UNLIGHT(val_first) || IS_UNLIGHT(val_last)));
    }
  assert(!contour_set_direction_belong_segment(self,&first_point,start_move));
  assert(!contour_set_direction_belong_segment(self,&last_point,end_move));
  /* On crait les noeuds */
  first_node = private_make_node(self,&first_point);
  last_node  = private_make_node(self,&last_point);

  /* On recupere les brins antecedents par sigma-1 */
  start_dart = private_compute_dart (self, first_node, start_move);
  end_dart   = private_compute_dart (self, last_node, end_move);

 /* On recupere les faces de ces 2 brins */ 
  label1 = map_give_lambda(Map_contour, -start_dart);
  label2 = map_give_lambda(Map_contour, -end_dart);

  /* On ecrit le segment ds le bitmap */

  bitmap_draw_segment(Bitmap_contour,seg);

/* On crait la structure segment s'appuyant sur le bitmap */
  ins_segment = FAIRE_segment(Bitmap_contour);
  segment_init_from_seg_tmp(ins_segment,seg);


/* On s'occupe d'abord de Map et on retient les faces a reexaminer */
  if(label1 == label2)
    {     
      if (start_dart == end_dart)
	{ 
	  /* On veux creer une composante tangente a une autre */
	  /* composante en un point */
	  prev_move = private_give_first_move_of_dart(self,start_dart);
	  if(NEXT_DIRECTION(prev_move) != end_move)
	    segment_reverse(ins_segment);
	  new_num = map_insert_tangeant(Map_contour,start_dart);

	  tab_gen_insert_object(Segment_set_contour,ins_segment, new_num);
	  segment_set_dart_nb(ins_segment, new_num);	  
	  if(segment_get_longueur(ins_segment) >=
	     MIN_PERIMETER_TO_INCLUDE)
	    {
	      if( inclusions_is_infinite(Inclusions_contour,label1) )
		face_to_inspect = inclusions_give_sister(Inclusions_contour,
							 label1);
	      else
		face_to_inspect = inclusions_give_daughter(Inclusions_contour,
							   label1);
	    }
	  if(segment_give_clock(ins_segment) <0)
	    {
	      f      = map_give_lambda(Map_contour,-new_num);
	      father = inclusions_give_father(Inclusions_contour,f);
	      inclusions_remove_infinite_face(Inclusions_contour,f);
	      f = map_give_lambda(Map_contour,new_num);
	      inclusions_split_mother_son(Inclusions_contour,f,father);
	    }
	}
      else
	{
	  /* Split d'une face finie ou infinie */
	  new_num = map_insert_segment(Map_contour,start_dart,end_dart);
	  tab_gen_insert_object(Segment_set_contour,ins_segment, new_num);
	  segment_set_dart_nb(ins_segment, new_num);

	  if( !inclusions_is_infinite(Inclusions_contour,label1) )
	    {
	      /* La face splite est finie */
	      face_to_inspect =
		inclusions_give_daughter(Inclusions_contour,label1);
	      map_change_lambda_cycle(Map_contour,new_num);
	    }
	  else
	    {
	      /* La face splite est infinie */
	      face_to_inspect=
		inclusions_give_sister(Inclusions_contour,label1);
	      
	      if( private_is_infinite(self,new_num) )
		{
		  map_set_lambda_1(Map_contour,label1,new_num);
		  map_change_lambda_cycle(Map_contour,-new_num);
		}
	      else
		{
		  map_set_lambda_1(Map_contour,label1,-new_num);
		  map_change_lambda_cycle(Map_contour,new_num);
		}
	      
	    }
	}
      private_memorize_face(self,new_num);
      /* On s'occupe de la relation d'inclusion */
      private_uninitialize_face(self,label1);
    }
  else
    {  /*         label1 != label2 */
      /* Union de 2 soeurs ou d'une mere et de sa fille */

      father1 = inclusions_give_father(Inclusions_contour, label1);
      father2 = inclusions_give_father(Inclusions_contour, label2);
      /* On met a jour les relations mere filles */
      inclusions_fusion(Inclusions_contour,label1,label2);

      /* On met a jour les lambda */
      /* Union de 2 soeurs */
      if (father1 == father2)
	map_merge_lambda(Map_contour, label1, label2);
      /* Union mere fille : mere(label1) = label2 */
      if (father1 == label2)
	map_merge_lambda(Map_contour,label2, label1);
      /* Union mere fille : mere(label2) = label1 */
      if (father2 == label1)
	map_merge_lambda(Map_contour, label1, label2);
      
      assert(
	     map_give_lambda(Map_contour,-start_dart)== 
	     map_give_lambda(Map_contour,-end_dart)
	     );
      new_num = map_insert_segment(Map_contour, start_dart, end_dart);
      tab_gen_insert_object(Segment_set_contour,ins_segment, new_num);
      segment_set_dart_nb(ins_segment, new_num);
    }

  inclusions_realloc_inclusions(Inclusions_contour,
				map_give_lambda(Map_contour,new_num));
  inclusions_realloc_inclusions(Inclusions_contour,
				map_give_lambda(Map_contour,-new_num));

  private_update_inclusions(self,face_to_inspect);
  
  /* Utile pour recuperer la liste des segments crees par une etape de */
  /* segmentation */
  private_memorize_segment(self,ins_segment);
  return vrai;
}

    /**************************************************/
   /* Associe a un point la face finie ou infinie    */
  /* dont la frontiere est la plus proche du point  */
 /*    en se deplacant vers la droite              */
/**************************************************/
dart contour_set_assoc_point_face(contour_set self, point p)
{
  dart       b,face_owner;
  move_type  move;
  node       n;
  Point      p_seg,p_node;

  bitmap_sounding(Bitmap_contour,p,&p_seg);
  bitmap_contour_tracking(Bitmap_contour,&p_seg,&move,&p_node);
  
  if (INFINITE_POINT(p_node))
    return 1;

  n = node_set_give_node(Node_set_contour,&p_node);
      
  b = private_give_dart (self,n, move);
  face_owner = map_give_lambda(Map_contour,b);

  if(face_owner == -1)
    face_owner = map_give_lambda(Map_contour,-b);

  return face_owner;

}
void contour_set_assoc_face_point(contour_set self,dart f,point p_f)
{
  dart b;
  Point p;
  segment seg;
  move_type move;

  assert(self);
  assert(p_f);

  b = map_give_lambda_1(Map_contour,f);
  seg = tab_gen_give_object(Segment_set_contour,b);
  if(b >0)
    {
      segment_get_first(seg,&p);
      move = segment_get_first_move(seg);
    }
  else
    {
      segment_get_last(seg,&p);
      move = REVERSE_SENSE(segment_get_last_move(seg));
    }
  point_give_right_point(&p,move,p_f);
}

  
  
  /******************************************************************/
 /*        Indique a qu'el segment appartient un point              */
/******************************************************************/
dart contour_set_assoc_point_arc(contour_set self, point p)
{
  dart       signed_dart;
  move_type  move;
  node       n;
  Point      p_node;
  Point      p_seg;

  assert(p);
  assert(self);
  
  bitmap_sounding(Bitmap_contour,p,&p_seg);
  bitmap_contour_tracking(Bitmap_contour,&p_seg,&move,&p_node);
  
  assert( !INFINITE_POINT(p_node) );
  
  n = node_set_give_node(Node_set_contour,&p_node);
  
  signed_dart = private_give_dart (self,n, move);
  
  return signed_dart;
}
  /*************************************/
 /* Donne le brin canonique d'une face*/
/*************************************/
dart contour_set_assoc_face_seg(contour_set self,dart f)
{
  assert(self);
  assert(f>0);

  return map_give_lambda_1(Map_contour,f);
}
    /******************************************************/
   /*Donne la face possedant le brin passe en parametre. */
  /* Peut se voir comme la fonction reciproque de       */
 /* contour_set_assoc_face_seg                         */
/******************************************************/
dart contour_set_assoc_seg_face(contour_set self,dart b)
{
  assert(self);

  return map_give_lambda(Map_contour,b);
}
    /*****************************************************/
   /* Supprime les arcs d'une face.                     */
  /* Enleve eventuellement d'autres arcs afin d'eviter */ 
 /* la creation de bout pendant.                      */
/*****************************************************/
void contour_set_remove_face(contour_set self,dart f)
{
  dart_list arc_list;
  dart b;

  assert(self);
  assert(f);
  
  b = map_give_lambda_1(Map_contour,f);
  arc_list = map_give_cycle_phi(Map_contour,b);
  
  private_remove_set_of_arc(self,arc_list);
  
  DEFAIRE_dart_list(arc_list);
}
  /**************************************************/
 /* Enleve un ensemble d'arcs du contour           */
/**************************************************/
void contour_set_remove_set_of_arc(contour_set self,
				   dart_list seg_list)
{
  dart_list b_list;
  dart b;

  assert(self);
  assert(seg_list);

  if(dart_list_is_empty(seg_list))
    return;

  b_list = FAIRE_dart_list();
  /* On verifie que chaque brin est present une seule fois */
  dart_list_first(seg_list);
  do
    {
      b = dart_list_give_current(seg_list);
      if(!map_give_mark(Map_contour,b))
	{
	  map_set_mark(Map_contour,b);
	  dart_list_insert(b_list,b);
	}
    }
  while(dart_list_next(seg_list));
  /* On demarque les brins marques */
  dart_list_first(b_list);
  do
    {
      b = dart_list_give_current(b_list);
      map_remove_mark(Map_contour,b);
    }
  while(dart_list_next(b_list));
  /* On efface */
  private_remove_set_of_arc(self,b_list);
  DEFAIRE_dart_list(b_list);
}
   /**********************************************************/
  /* Idem que private_remove_one_arc_in_set mais ici le set */
 /* se reduit a un seul element.                           */
/**********************************************************/
void contour_set_remove_arc(contour_set self, dart b)
{
  dart_list list_seg;

  assert(self);
  
  list_seg = FAIRE_dart_list();
  dart_list_insert(list_seg,b);

  private_remove_set_of_arc(self,list_seg);

  DEFAIRE_dart_list(list_seg);
}
  
    /******************************************************************/
   /* Recopie les filles d'une face dans la liste passe en parametre */
  /* Celle ci doit etre vide ou les faces se melangeront a l'ancien */
 /* contenu de la dart_list.                                       */
/******************************************************************/
void contour_set_give_included_faces(contour_set self,
				     dart f,
				     dart_list list_of_faces)
{
  dart_list daughter;
  
  assert(self);
  assert(list_of_faces);
  assert(dart_list_is_empty(list_of_faces));

  daughter = inclusions_give_daughter(Inclusions_contour,f);

  if(daughter != NULL_DART_LIST)
    dart_list_insert_list(list_of_faces,daughter);
}

    /*********************************************************************/
   /* Marque les points 1/2 de list_of_faces a 1 ou 2 celon la fonction */
  /* de marquage passe en parametre.                                   */
 /* Les marques sont ecrites dans tab_mark                            */
/*********************************************************************/
void contour_set_mark_set_of_face(contour_set self,
				  dart_list list_of_faces,
				  tab_mark mark_faces,
				  void (*mark_point)(tab_mark,point)
				  )
{

  dart current_face;

  assert(self);
  assert(mark_faces);

  if(list_of_faces == NULL_DART_LIST)
    return;

  if( dart_list_is_empty(list_of_faces) )
    return;
  

  dart_list_first(list_of_faces);
  do
    {
      current_face = dart_list_give_current(list_of_faces);
      private_mark_face(self,current_face,mark_point,mark_faces);
    }
  while( dart_list_next(list_of_faces) );
}

  /***************************************************/
 /*  Efface les marques des faces de list_of_faces  */
/***************************************************/

void contour_set_unmark_set_of_face(contour_set self,
				    dart_list list_of_faces,		    
				    tab_mark mark_faces)
{

  dart current_face;

  assert(self);
  assert(mark_faces);

  if(list_of_faces == NULL_DART_LIST)
    return;
  
  if( dart_list_is_empty(list_of_faces) )
    return;
  

  dart_list_first(list_of_faces);
  do
    {
      current_face = dart_list_give_current(list_of_faces);
      private_unmark_face(self,current_face,tab_mark_unmark_point,mark_faces);
    }
  while( dart_list_next(list_of_faces) );
}

bouleen contour_set_is_face_defined(contour_set self,dart f)
{
  assert(self);
  assert(f>0);

  return (map_give_lambda_1(Map_contour,f) != 0);
}
  /******************************************************/
 /* Donne ou calcule des parametres de segments        */
/******************************************************/
int contour_set_give_segment_lenght(contour_set self,dart b)
{
  segment seg;

  assert(self);
  assert(b !=0);
  
  seg = (segment)tab_gen_give_object(Segment_set_contour,b);

  return segment_get_longueur(seg);
}
double contour_set_give_segment_gradiant(contour_set self,dart b)
{
  segment seg;
  double grad;
  
  assert(self);
  assert(b !=0);

  seg = (segment)tab_gen_give_object(Segment_set_contour,b);
  
  grad = segment_compute_gradiant(seg,Image_contour);
  
  return grad;
}
  /***************************************************/
 /* Calcule les differents moments d'un face        */
/***************************************************/
void contour_set_compute_face_params(contour_set self,dart f)
{
  int dim;
  double moment0;
  vector moment1,moment2;
  face computed_face;

  assert(self);
  assert(f);

  dim = image_give_dim(Image_contour);
  /* On crait la face si elle n'existe pas */
  computed_face = (face)tab_gen_give_object(Face_set_contour,f);
  if(computed_face == NULL)
    {
      computed_face = FAIRE_face(dim);
      tab_gen_insert_object(Face_set_contour,computed_face,f);
    }

  if( face_is_computed(computed_face) )
    return;

  private_traverse_face(self,f,&moment0,moment1,moment2,NULL,faux);
  face_initialize(computed_face,moment0,moment1,moment2,dim,f);
}
double  contour_set_give_face_area(contour_set self,dart f)
{
  face computed_face;

  contour_set_compute_face_params(self,f);
  
  computed_face = (face)tab_gen_give_object(Face_set_contour,f);

  return face_give_area(computed_face);
}
void contour_set_give_face_mean(contour_set self,dart f,vector mean)
{
  face computed_face;
  double* m;
  int i,dim;

  dim = image_give_dim(Image_contour);
  contour_set_compute_face_params(self,f);
  
  computed_face = (face)tab_gen_give_object(Face_set_contour,f);
  m = face_give_mean_value(computed_face);
  for(i=0;i<dim;i++)
    mean[i] = m[i];
}
void contour_set_give_face_moment1(contour_set self,dart f,vector m1)
{
  face computed_face;
  double* m;
  int i,dim;

  dim = image_give_dim(Image_contour);
  contour_set_compute_face_params(self,f);
  
  computed_face = (face)tab_gen_give_object(Face_set_contour,f);
  m = face_give_moment1_value(computed_face);
  for(i=0;i<dim;i++)
    m1[i] = m[i];
}
void contour_set_give_face_moments(contour_set self,
				   dart f,
				   double *m0,
				   vector m1,
				   vector m2)
{
  face computed_face;

  contour_set_compute_face_params(self,f);
  
  computed_face = (face)tab_gen_give_object(Face_set_contour,f);
  face_give_moments(computed_face,m0,m1,m2);
}

double contour_set_give_face_variance(contour_set self,dart f)
{
  face computed_face;
  double msq;

  contour_set_compute_face_params(self,f);
  
  computed_face = (face)tab_gen_give_object(Face_set_contour,f);
  msq = face_give_mean_squared_error(computed_face);

  return msq;
}
double contour_set_give_face_squared_error(contour_set self,dart f)
{
  face computed_face;
  double sq;
  
  assert(contour_set_is_face_defined(self,f));
  contour_set_compute_face_params(self,f);
  
  computed_face = (face)tab_gen_give_object(Face_set_contour,f);
  sq = face_give_squared_error(computed_face);

  return sq;
}  
  /*********************************************************/
 /* Applique une fonctions sur tous les pixels d'une face */
/*********************************************************/
void contour_set_apply_function(contour_set self,
				dart f,
				PFapply_to_pixels apply)
{
  int dim = image_give_dim(Image_contour);
  double moment0;
  vector moment1,moment2;
  face computed_face;
  
  /* On crait la face si elle n'existe pas */
  computed_face = (face)tab_gen_give_object(Face_set_contour,f);
  if(computed_face == NULL)
    {
      computed_face = FAIRE_face(dim);
      tab_gen_insert_object(Face_set_contour,computed_face,f);
    }
  
  if( face_is_computed(computed_face) )
    private_apply_function(self,f,apply);
  else
    {/* On profite du parcours pour calculer les parametres de la face */
      private_traverse_face(self,f,&moment0,moment1,moment2,apply,vrai);
      face_initialize(computed_face,moment0,moment1,moment2,dim,f);
    }
}
   /********************************************************************/
  /* Donne les begins points d'une face.                              */
 /* Utilise par les logiciel de segmentation pour parcourir celle ci */
/********************************************************************/
void contour_set_give_begin_points(contour_set self,
				   dart f,
				   point_set begin_points)
{
  dart b,b_first,b_last;
  Point p_demis_entier;
  segment seg,seg_last;
  move_type prec_move,next_move;


  b = b_first  = map_give_lambda_1(Map_contour,f);
  seg = (segment)tab_gen_give_object(Segment_set_contour,b_first);

  b_last = map_give_phi_1(Map_contour,b_first);
  seg_last = (segment)tab_gen_give_object(Segment_set_contour,b_last);
  if(b_last >0)
    prec_move = segment_get_last_move(seg_last);
  else
    prec_move = REVERSE_SENSE(segment_get_first_move(seg_last));

  do
    {
      if(b >0)
	{
	  segment_first(seg);
	  do
	    {
	      next_move = segment_give_current_move(seg);
	      
	      if( begin_mark_needed[prec_move][next_move] )
		{
		  segment_give_current_point(seg,&p_demis_entier);
		  point_set_insert_point(begin_points,&p_demis_entier);
		}
	      prec_move = next_move;
	    }
	  while( segment_next(seg) );
	}
      else
	{
	  segment_last(seg);
	  do
	    {
	      next_move = REVERSE_SENSE(segment_give_current_move(seg));
	      
	      if( begin_mark_needed[prec_move][next_move] )
		{
		  segment_give_current_point(seg,&p_demis_entier);
		  point_set_insert_point(begin_points,&p_demis_entier);
		}
	      prec_move = next_move;
	    }
	  while( segment_previous(seg) );
	}

      b = map_give_phi(Map_contour,b);
      seg = (segment)tab_gen_give_object(Segment_set_contour,b);
    }
  while(b != b_first);

} 

/* Marque une face en vue de son parcourt */
void contour_set_mark_end_points_of_face(contour_set self,dart f)
{
  dart b,b_first,b_last;
  segment seg,seg_last;
  move_type prec_move,next_move;
  Point p_entier,p_demis_entier;

  b = b_first  = map_give_lambda_1(Map_contour,f);
  seg = (segment)tab_gen_give_object(Segment_set_contour,b_first);

  b_last = map_give_phi_1(Map_contour,b_first);
  seg_last = (segment)tab_gen_give_object(Segment_set_contour,b_last);

  if(b_last >0)
    prec_move = segment_get_last_move(seg_last);
  else
    prec_move = REVERSE_SENSE(segment_get_first_move(seg_last));

  /* On commence par parcourir le segment pour etablir les marques */
  /* necessaires pour parcourir la face */
  do
    {
      if(b >0)
	{
	  segment_first(seg);
	  do
	    {
	      next_move = segment_give_current_move(seg);
	      if(end_mark_needed[prec_move][next_move])
		{
		  segment_give_current_point(seg,&p_demis_entier);
		  point_translate(&p_demis_entier,
				  &p_entier,
				  prec_move,
				  next_move);
		  tab_mark_mark_point_two(Marked_points,&p_entier);
		}
	      prec_move = next_move;
	    }
	  while( segment_next(seg) );
	}
      else
	{
	  segment_last(seg);
	  do
	    {
	      next_move = REVERSE_SENSE(segment_give_current_move(seg));
	      
	      if(end_mark_needed[prec_move][next_move])
		{
		  segment_give_current_point(seg,&p_demis_entier);
		  point_translate(&p_demis_entier,
				  &p_entier,
				  prec_move,
				  next_move);
		  tab_mark_mark_point_two(Marked_points,&p_entier);
		}
	      prec_move = next_move;
	    }
	  while( segment_previous(seg) );
	}
      b = map_give_phi(Map_contour,b);
      seg = (segment)tab_gen_give_object(Segment_set_contour,b);
    }
  while(b != b_first);
}

bouleen contour_set_end_of_line(contour_set self,
				point pt_to_test)
{
  Point p;
  
  assert(self);
  assert(pt_to_test);
  
  
  AFFECTE_POINT(p,*pt_to_test);
  
  if (tab_mark_is_marked_two(Marked_points,&p) )
    {
      tab_mark_unmark_point_two(Marked_points,&p);
      return vrai;
    }
  
  return faux;
}
/**************************************************/
bouleen contour_set_direction_belong_segment(contour_set self,
					     point p_1_2,
					     move_type move)
{
  angle val;
  
  assert(self);
  assert(p_1_2);
  
  val = bitmap_move_to(Bitmap_contour,p_1_2);
  return bitmap_has_neighbour_in_direction(Bitmap_contour,val,move);
}
/**************************************************/
bouleen  contour_set_is_in_segment(contour_set self,point p_1_2)
{
  angle val;
  
  assert(self);
  assert(p_1_2);
  
  val = bitmap_move_to(Bitmap_contour,p_1_2);

  return ( !IS_UNLIGHT(val) );
}
/**************************************************/
void contour_set_set_same_cluster_criterion(contour_set self,
					    PFsame_cluster criterion)
{
  assert(self);
  
  Belong_to_same_cluster = criterion;
}
  /**************************************************/
 /* Teste si 2 pixels appartiennes au meme cluster */
/**************************************************/
bouleen contour_set_must_be_separated(contour_set self,
				      point p_1_2,
				      move_type move)
{
  Point p_entier[2];
  int pixel1[3];
  int pixel2[3];
  int* pixel;
  int dim = image_give_dim(Image_contour);
  int i;

  assert(self);
  assert(p_1_2);
  assert(Belong_to_same_cluster);


  /* On recupere les deux pixels adjacents par le point et le move */
  point_give_adjacents(p_1_2,move,p_entier);

  /* On recupere le premier pixel */
  image_move_to(Image_contour,p_entier);
  pixel = image_lire_pixel(Image_contour);
  for(i=0;i<dim;i++)
    pixel1[i] = pixel[i];

  /* On recupere le second */
  image_move_to(Image_contour,(p_entier+1));
  pixel = image_lire_pixel(Image_contour);
  for(i=0;i<dim;i++)
    pixel2[i] = pixel[i];  
      
  return Belong_to_same_cluster(p_entier,
				pixel1,
				p_entier+1,
				pixel2);
}
/**************************************************/
seg_tmp contour_set_copy_closed_curve(contour_set self,seg_tmp seg)
{
  return seg_tmp_copy_closed_curve(seg,Image_contour);
}
/**************************************************/
int contour_set_give_image_dim(contour_set self)
{
  return image_give_dim(Image_contour);
}
/**************************************************/
int  contour_set_give_image_hauteur(contour_set self)
{
  return image_give_hauteur(Image_contour);
}
/**************************************************/
int  contour_set_give_image_largeur(contour_set self)
{
  return image_give_largeur(Image_contour);
}
double contour_set_give_image_variance(contour_set self)
{
  return Image_var;
}
/**************************************************/
double contour_set_distance_in_image(contour_set self,
				     point p,
				     move_type move)
{
  return image_distance(Image_contour,p,move);
}
void  contour_set_give_modified_segments(contour_set self,
					 list_gen seg_list)
{
  assert(self);
  assert(seg_list);
  assert(list_gen_is_empty(seg_list));
  
  list_gen_copy(Modified_segments,seg_list);
  Memorize_segment_flag = faux;
}
void contour_set_memorize_modified_segments(contour_set self)
{
  assert(self);

  Memorize_segment_flag = vrai;

  if(list_gen_is_empty(Modified_segments))
    return;
  
  list_gen_reinitialize(Modified_segments);
}
   /************************************************************/
  /* Ces deux fonctions permettent de retrouver               */
 /* l'ensemble des faces crees par une etape de segmentation */
/************************************************************/
void contour_set_memorize_inserted_faces(contour_set self)
{
  assert(self);

  dart_list_reinitialize(Inserted_faces);
  Memorize_face_flag = vrai;
}
void contour_set_give_inserted_faces(contour_set self,dart_list face_list)
{
  assert(self);
  assert(face_list);
  assert(dart_list_is_empty(face_list));

  dart_list_insert_list(face_list,Inserted_faces);
  Memorize_face_flag = faux;
}
void contour_set_give_segments_of_face(contour_set self,
				       dart f,
				       dart_list seg_list)
{
  dart b,b_first;

  assert(self);
  assert(f >=1);
  assert(seg_list);
  assert(dart_list_is_empty(seg_list));

  b_first = map_give_lambda_1(Map_contour,f);

  assert(b_first);

  b = b_first;

  do
    {
      dart_list_insert(seg_list,b);
      b = map_give_phi(Map_contour,b);
    }
  while(b != b_first);
}
  /*******************************************************/
 /* Recopie de segment identifie a partir de son numero */
/*******************************************************/
void contour_set_give_segment(contour_set self,
			      dart b,
			      output_segment ext_seg)
{
  segment seg;

  assert(self);
  assert(ext_seg);

  seg = (segment)tab_gen_give_object(Segment_set_contour,b);
  assert(seg);
  segment_init_output_segment(seg,ext_seg);
}
  /***************************************************************/
 /* Donne l'ensemble des faces adjacentes a un ensemble de face */
/***************************************************************/
void contour_set_complete_face_adjacence(contour_set self,
					 dart_list face_list,
					 dart_list added_faces)
{
  dart f;

  assert(self);
  assert(face_list);
  assert(added_faces);
  
  if(dart_list_is_empty(face_list))
    return;

  dart_list_first(face_list);
  do
    {
      f = dart_list_give_current(face_list);
      map_set_mark(Map_contour,f);
    }
  while(dart_list_next(face_list));

  dart_list_first(face_list);
  do
    {
      f = dart_list_give_current(face_list);
      private_give_face_adjacence(self,f,added_faces);
    }
  while(dart_list_next(face_list));

  dart_list_first(face_list);
  do
    {
      f = dart_list_give_current(face_list);
      map_remove_mark(Map_contour,f);
    }
  while(dart_list_next(face_list));

  dart_list_first(added_faces);
  do
    {
      f = dart_list_give_current(added_faces);
      map_remove_mark(Map_contour,f);      
    }
  while(dart_list_next(added_faces));
}
void contour_set_mark_adjacent_faces(contour_set self,
				     dart f,
				     tab_mark_dart adjacent)
{
  dart b,b_first;
  dart neighbour;

  assert(self);
  assert(f>0);
  assert(adjacent);
  
  b_first = map_give_lambda_1(Map_contour,f);
  b = b_first;
  do
    {
      neighbour = map_give_lambda(Map_contour,-b);
      if(neighbour != -1)
	{
	  if(inclusions_is_infinite(Inclusions_contour,neighbour))
	    neighbour = inclusions_give_father(Inclusions_contour,neighbour);

	  tab_mark_dart_mark_dart(adjacent,neighbour);
	}
      b = map_give_phi(Map_contour,b);
    }
  while(b != b_first);
}
    /*****************************************************/
   /* Donne l'ensemble des faces finis equivalentes a f */
  /* Si f est equivalent a une face infinie, on  prend */ 
 /*la face mere correspondante                        */
/*****************************************************/
static void private_give_face_adjacence(contour_set self,
					dart f,
					dart_list added_faces)
{
  dart b,b_first;
  dart neighbour;
  
  b_first = map_give_lambda_1(Map_contour,f);
  b = b_first;
  do
    {
      neighbour = map_give_lambda(Map_contour,-b);
      if(neighbour != -1)
	{
	  if(inclusions_is_infinite(Inclusions_contour,neighbour))
	    neighbour = inclusions_give_father(Inclusions_contour,neighbour);
	  if(!map_give_mark(Map_contour,neighbour))
	    {
	      map_set_mark(Map_contour,neighbour);
	      dart_list_insert(added_faces,neighbour);
	    }
	}
      b = map_give_phi(Map_contour,b);
    }
  while(b != b_first);
}
	  
  
   /**************************************************/
  /* Donne la liste des faces finies qui composent  */
 /* une face infinie                               */
/**************************************************/
void contour_set_give_finite_faces(contour_set self,
				   dart f_infini,
				   dart_list face_list)
{
  dart_list marked_face;
  dart f;

  assert(self);
  assert(inclusions_is_infinite(Inclusions_contour,f_infini));
  
  marked_face = FAIRE_dart_list();
  /* On calcule la classe d'equivalence de f_infini */
  private_compute_face_class(self,f_infini,marked_face);

  assert(!dart_list_is_empty(marked_face));

  /* On l'enleve de l'ensemble des elements de la classe */
  map_remove_mark(Map_contour,f_infini);
  dart_list_remove_elt(marked_face,f_infini);
  /* On inclue cet ensemble de face dans la liste passe en parametre */
  dart_list_first(marked_face);
  do
    {
      f = dart_list_give_current(marked_face);
      map_remove_mark(Map_contour,f);
      dart_list_insert(face_list,f);
    }
  while(dart_list_next(marked_face));
  
  DEFAIRE_dart_list(marked_face);
}
   /**************************************************/      
  /* Donne la liste des faces finies incluses       */
 /*   dans une face finie                          */
/**************************************************/
dart_list contour_set_give_all_finite_faces(contour_set self,
					    dart f_fini)
{
  dart_list daughter_list;
  dart_list included_faces;
  dart f_infini;

  assert(self);
  assert(f_fini >0);

  daughter_list = inclusions_give_daughter(Inclusions_contour,f_fini);


    
  included_faces = FAIRE_dart_list();

  if(daughter_list == NULL_DART_LIST)
    return included_faces;

  if(dart_list_is_empty(daughter_list))
    return included_faces;

  dart_list_first(daughter_list);
  do
    {
      f_infini = dart_list_give_current(daughter_list);
      contour_set_give_finite_faces(self,f_infini,included_faces);
    }
  while(dart_list_next(daughter_list));

  return included_faces;
}
  /***********************************************************/
 /* Renvoie la liste des faces finies qui composent l'image */
/***********************************************************/
dart_list contour_set_give_image_faces(contour_set self)
{
  dart_list all_faces;
  dart_list all_finite_faces;
  dart f;

  assert(self);

  all_faces = map_give_all_faces(Map_contour);
  all_finite_faces = FAIRE_dart_list();
  dart_list_first(all_faces);
  do
    {
      f = dart_list_give_current(all_faces);
      if(!inclusions_is_infinite(Inclusions_contour,f))
	dart_list_insert(all_finite_faces,f);
    }
  while(dart_list_next(all_faces));
  DEFAIRE_dart_list(all_faces);

  return all_finite_faces;
}
void contour_set_remove_double_arc_of_face(contour_set self,
					   dart f)
{
  dart b,b_first;
  dart_list double_arc;

  assert(self);
  assert(f>0);

  double_arc = FAIRE_dart_list();
  b_first = map_give_lambda_1(Map_contour,f);
  b = b_first;
  do
    {
      if ((b>0)&&(map_give_lambda(Map_contour,-b) == f))
	dart_list_insert(double_arc,b);
      b = map_give_phi(Map_contour,b);
    }
  while(b != b_first);
  
  private_remove_set_of_arc(self,double_arc);
  
  DEFAIRE_dart_list(double_arc);
}
bouleen contour_set_face_has_neighbour(contour_set self,
				       dart f,
				       dart b)
{
  dart neighbour;
  
  assert(self);
  assert(f>0);
  
  
  neighbour = map_give_lambda(Map_contour,b);
  if(neighbour == f)
    neighbour = map_give_lambda(Map_contour,-b);
  
  return (neighbour != -1);
}
  /**********************************************/
 /* Donne la face voisine de f le long de seg  */
/**********************************************/
dart contour_set_give_face_neighbour(contour_set self,
				     dart f,
				     dart b)
{
  dart neighbour;
  
  assert(self);
  assert(f>0);
  
  
  neighbour = map_give_lambda(Map_contour,b);
  if(neighbour == f)
    neighbour = map_give_lambda(Map_contour,-b);
  
  assert(neighbour != -1);
  
  if(inclusions_is_infinite(Inclusions_contour,neighbour))
    neighbour  = inclusions_give_father(Inclusions_contour,neighbour);
  
  return neighbour;
}
bouleen contour_set_direction_is_allowed(contour_set self,
					 point p,
					 move_type move)
{
 
  angle val;
  static bouleen is_allowed[16][4]=
    {/* D    H    G     B   */
      {vrai,vrai,vrai,vrai},/*0 : */
      {faux,vrai,vrai,faux},/*1 : */
      {vrai,vrai,faux,faux},/*2 : */
      {faux,vrai,faux,faux},/*3 : */
      {faux,faux,faux,faux},/*4 : */
      {faux,faux,faux,faux},/*5 :== */
      {faux,faux,faux,faux},/*6 : */
      {faux,faux,faux,faux},/*7 : */
      {faux,faux,faux,faux},/*8 : */
      {faux,vrai,vrai,faux},/*9 : */
      {vrai,vrai,faux,faux},/*10 : */
      {faux,vrai,faux,faux},/*11 : */
      {faux,faux,faux,faux},/*12 : */
      {faux,faux,faux,faux},/*13 :== */
      {faux,faux,faux,faux},/*14 :== */
      {faux,faux,faux,faux} /*15 : */
    };

  val = bitmap_move_to(Bitmap_contour,p);

  return is_allowed[val][move];
 
}
/* Etend un arc Jusqu'a des noeuds minimaux */
void contour_set_extend_seg(contour_set self,
			    dart b)
{
  dart_list empty;
  
  empty = FAIRE_dart_list();
  
  while(private_extend_arc(self,b,empty));
 
  DEFAIRE_dart_list(empty);
}
  /*----------------------------------------------------------------*/
 /* affecte la liste de faces contenue dans b_list aux bonnes mere */
/*----------------------------------------------------------------*/
static void private_update_inclusions(contour_set self, dart_list b_list)
{
  dart             label,father;
  Point            right_point;
  int              card, i;
  static struct face_pt default_tab_face[100];
  struct face_pt   *tabface;
  angle val;

  if(b_list == NULL_DART_LIST)
    return;

  card =dart_list_give_cardinal(b_list);
  
  if (card == 0) return;

  if(card <=100)
    tabface = default_tab_face;
  else
    tabface   = (struct face_pt*) malloc(card*sizeof(struct face_pt));
  
/* On commence par ranger les faces ds un tableaux */
  i = 0;
  dart_list_first(b_list);
  do
    {
      label = dart_list_give_current(b_list);
      private_right_point_face (self, label,&(tabface[i].p));
      
      tabface[i].face = label;
      i++;
    }
  while(dart_list_next(b_list));
  /* On les trie en fonction de leurs pt le + a droite */

  if(card >1)
    qsort(tabface,card,sizeof(struct face_pt),private_comp_face_pt);

  /* On parcourt le tableau trie pour corriger les relations */
  /* d'inclusion de ces faces */
  for (i = 0 ; i < card; i++)
      {
	/* On prend un point entier situe a droite de la face */
	AFFECTE_POINT(right_point,tabface[i].p);
	val = bitmap_move_to(Bitmap_contour,&right_point);
	val = val&(~NODE_CONST);
	assert(
	       (val ==2)||
	       (val ==4)||
	       (val==6)
	       );
	MOVE_RIGHT_POINT(right_point);
	if(val == 2)
	  {
	    MOVE_DOWN_POINT(right_point);
	  }

	/* On prend la face englobante */
	father = contour_set_assoc_point_face(self,&right_point);
	inclusions_make_descent(Inclusions_contour,father,tabface[i].face);
      }

  if(card > 100)
    free (tabface);
}

static void private_remove_set_of_arc(contour_set self,
				      dart_list arc_list)
{
  dart b;

  if(dart_list_is_empty(arc_list))
    return;

  do
    {
      dart_list_first(arc_list);
      b = dart_list_give_current(arc_list);
      private_remove_one_arc_in_set(self,b,arc_list);
    }
  while(!dart_list_is_empty(arc_list));
}
     /**************************************************/
    /* Calcule la classe d'equivalence d'une face en  */
   /* fonction de la relation :                      */
  /* F1 ~ F2 <=> \exist b \in F1 / -b \in F2        */
 /* Entree un represantant de la classe            */
/**************************************************/
static void private_compute_face_class(contour_set self,
				       dart f1,
				       dart_list face_list)
{
  dart b,b_first;
  dart f2;

  assert(self);
  assert(f1>0);
  assert(face_list);

  map_set_mark(Map_contour,f1);
  dart_list_insert(face_list,f1);

  b_first = map_give_lambda_1(Map_contour,f1);
  b = b_first;
  do
    {
      f2 = map_give_lambda(Map_contour,-b); 
      if(!map_give_mark(Map_contour,f2))
	private_compute_face_class(self,f2,face_list);

      b = map_give_phi(Map_contour,b);
    }
  while(b != b_first);
}
  /**************************************************/
 /* Parcour la face pour calculer son orientation  */
/**************************************************/
static move_type private_is_infinite(contour_set self, dart f)
{
  dart      current_dart;
  int       increment;
  move_type direction = 0;
  move_type first_move,
            last_move,
            init;
  segment   seg;
  
  seg = (segment)tab_gen_give_object(Segment_set_contour,f);
  increment = segment_give_clock(seg);


  /* definition du Last move : dernier deplacement */
  /*   ie deplacement entre l' avant dernier et le dernier ?*/
  if (f > 0) 
    {
      init       = segment_get_first_move(seg);
      first_move = segment_get_last_move(seg);
      direction  = increment;
    }
  else
    {
      init       = REVERSE_SENSE( segment_get_last_move(seg) );
      first_move = REVERSE_SENSE( segment_get_first_move(seg) );
      direction  = -increment;
    }
    
  current_dart = map_give_phi(Map_contour, f);

  /* On elimine le cas de la composante connexe */
  if (current_dart == f)
    {
      assert(direction == 4 || direction ==-4);
      return (direction < 0);
    }

  if (segment_is_closed(seg))
    direction -= move_type_compute_step_clock(first_move,
					      init);
  while (current_dart != f)
    {
      seg = (segment)tab_gen_give_object(Segment_set_contour,current_dart);
      increment = segment_give_clock(seg);
      if (current_dart > 0)
	{
	  last_move = segment_get_first_move(seg);
	  direction += move_type_compute_step_clock(first_move,last_move);
	  first_move = segment_get_last_move(seg);
	  direction += increment;
	}
      else
	{
	  last_move  = REVERSE_SENSE( segment_get_last_move(seg) );
	  direction += move_type_compute_step_clock(first_move,last_move);
	  first_move = REVERSE_SENSE( segment_get_first_move(seg) );
	  direction -= increment;
	}
      /* Si le segment est ferme on arriv et repart sur un coin, il ne */
      /* faut donc pas tenir compte de celui-ci */
      if (segment_is_closed(seg))
	direction -= move_type_compute_step_clock(first_move,
						  last_move);
	  
						    
      current_dart = map_give_phi(Map_contour,current_dart);
    }
   
  direction += move_type_compute_step_clock(first_move,init);

  assert(direction == 4 || direction ==-4);

  return (direction<0); 
}

   /*********************************************************************/
  /* Calcul de sigma-1 d'un brin alors que celui ci n'a pas encore ete */
 /* insere dans la carte                                              */
/*********************************************************************/
static dart private_compute_dart(contour_set self,node n,move_type move)
{
  dart      dart_node,return_dart,current_dart;
  segment   seg;
  move_type current_move,
            distance = 4,
            temp;
    
  current_dart = dart_node = node_give_dart(n);
  do
    {
      seg = (segment)tab_gen_give_object(Segment_set_contour,current_dart);
      
      if(current_dart > 0)
	current_move = segment_get_first_move(seg);
      else
	current_move = REVERSE_SENSE( segment_get_last_move(seg) );
      
      temp = move_type_compute_dist_sigma_1(move,current_move);
      
      if(temp < distance) 
	{
	  distance   = temp;
	  return_dart = current_dart;
	}
      
      current_dart = map_give_sigma(Map_contour,current_dart);
    }
  while (current_dart!= dart_node);
  
  return return_dart;
}
   /***********************************************************************/
  /* Crait une composante connexe dans le bitmap, la carte + met a jour  */
 /* les inclusions                                                      */
/***********************************************************************/
static void private_make_component (contour_set self, seg_tmp seg)
{
  dart_list face_to_inspect = NULL_DART_LIST;
  segment    new_seg;
  dart       father_face,
             new_dart,
             son_face;
  node       first_node;
  Point      current_point;
  face       face_to_update;
  
  seg_tmp_give_first_point(seg,&current_point);
  
  new_dart      = map_insert_map(Map_contour);
  first_node    = FAIRE_node(&current_point, new_dart);
  
  inclusions_realloc_inclusions(Inclusions_contour,
				map_give_lambda(Map_contour, new_dart));
  inclusions_realloc_inclusions(Inclusions_contour,
				map_give_lambda(Map_contour,-new_dart));
  
  bitmap_draw_segment(Bitmap_contour, seg);

  bitmap_make_node(Bitmap_contour,&current_point,vrai);

  node_set_insert_node(Node_set_contour, first_node);

  new_seg = FAIRE_segment(Bitmap_contour);
  segment_init_from_seg_tmp(new_seg,seg);
  segment_set_dart_nb(new_seg, new_dart);  


  tab_gen_insert_object(Segment_set_contour, new_seg, new_dart);
  segment_give_right_point(new_seg,&current_point);
  
  MOVE_RIGHT_POINT(current_point);
  
  father_face = contour_set_assoc_point_face(self, &current_point);
/* Par definition le brin negatif correspond a la face infinie */
  if(segment_give_clock(new_seg)<0)
    son_face    = map_give_lambda(Map_contour, new_dart);
  else
    son_face    = map_give_lambda(Map_contour, -new_dart);
  inclusions_make_descent(Inclusions_contour, father_face, son_face);

  face_to_update =(face)tab_gen_give_object(Face_set_contour,father_face);
  face_uninitialize(face_to_update);

  if(segment_get_longueur(new_seg)>=MIN_PERIMETER_TO_INCLUDE)
    {
      face_to_inspect =inclusions_give_daughter(Inclusions_contour,
						father_face);
    }
  private_update_inclusions(self,face_to_inspect);
  /* Utile pour recuperer la liste des segments crees par un etape de */
  /* segmentation */
  private_memorize_segment(self,new_seg);
  private_memorize_face(self,new_dart);
}


static dart private_give_dart (contour_set self, node n, move_type move)
{

  dart    b,current_dart;
  Point   p;
  segment seg;
  move_type current_move;
  

  current_dart = b = node_give_dart(n);
  node_give_point(n,&p);

  do
    {

      seg  = (segment)tab_gen_give_object(Segment_set_contour,current_dart);

      if(current_dart > 0)
	current_move = segment_get_first_move(seg);
      else 
	current_move = REVERSE_SENSE( segment_get_last_move(seg) );

      if(current_move == move) 
	break;
      current_dart = map_give_phi(Map_contour,-current_dart);  
    } 
  while(current_dart != b);
  
  if(move == current_move)
    return current_dart;
  else
    {
      fprintf (stderr,"ERREUR DANS LA LOCALISATION D'UN BRIN\n");
      return INFINITE;
    }

}


static node private_make_node (contour_set self,point p)
{
  dart       dart_number,new_dart,b;
  node       n,
             node_detected;
  Point      point_prev_node;
  segment    split_segment,
             new_segment;
  move_type  move;
  Point      last_point;
  /* Si c'est deja un noeud on le renvois sans traitement */
  if( node_set_is_node(Node_set_contour, p) )
    return  node_set_give_node(Node_set_contour, p);

  /* Dans le cas contraire on le crait */
  bitmap_contour_tracking(Bitmap_contour, p, &move,&point_prev_node);
  node_detected = node_set_give_node(Node_set_contour,&point_prev_node);
  
  dart_number = private_give_dart(self, node_detected, move);
  
  dart_number = dart_number > 0 ? dart_number : -dart_number;
      
  split_segment = (segment)tab_gen_give_object(Segment_set_contour,
					       dart_number);

/* On splite le segment */
  new_dart    =  map_split(Map_contour,dart_number);
  new_segment =  segment_split(split_segment,p,new_dart);      
  
  /* On regarde si le noeud situe au last_point de new_seg ne doit pas */
  /* etre mis a jour */
  segment_get_last(new_segment,&last_point);
  n = node_set_give_node(Node_set_contour,&last_point);
  b = node_give_dart(n);
  if (b == -dart_number)
    node_set_dart(n,-new_dart);

  /* On ajoute le noeud */
  n = FAIRE_node(p, new_dart);
  node_set_insert_node(Node_set_contour, n);
  bitmap_make_node(Bitmap_contour, p, vrai);

  /* On insere le nouveau segment */
  tab_gen_insert_object(Segment_set_contour, new_segment, new_dart);

  return (n);
}



static void private_right_point_face (contour_set self,
				      dart f,
				      point right_point)
{
  dart      current_dart, b;
  Point     current_point;
  segment   seg;
  
  current_dart = b  = map_give_lambda_1(Map_contour, f);
  
  COORDX(*right_point) = INFINITE;
  COORDY(*right_point) = INFINITE;
  
  do
    {
      seg = (segment)tab_gen_give_object(Segment_set_contour, current_dart);
      segment_give_right_point(seg,&current_point);
      
      if( IS_RIGHTER_POINT(current_point,*right_point) )
	{
	  AFFECTE_POINT(*right_point,current_point);
	}
      current_dart = map_give_phi(Map_contour, current_dart);
    }
    while (current_dart != b);
}
/* Fonction servant a trier les faces incluses dans une face finie en */
/* fonction de leur point le plus a droite. Le resultat est la liste */
/* de ces face dans l'ordre des x decroissant. */
/* Voir private_update_inclusions.  */
static int private_comp_face_pt(const void* f1,const void* f2)
{
  struct face_pt* face1 = (struct face_pt*)f1;
  struct face_pt* face2 = (struct face_pt*)f2;

  if( COORDX(face1->p) > COORDX(face2->p) )
    return -1;
  
  if( COORDX(face1->p) == COORDX(face2->p) )
    return 0;
  
  return 1;
} 
  
    /***********************************************/
   /* Etend un arc jusqu'a ce que ces extermitees */ 
  /* ne soit plus d'arite 2 ou qu'il definisse   */
 /* une composante connexe.                     */
/***********************************************/
static bouleen  private_extend_arc(contour_set self,
				   dart b,
				   dart_list set_of_arc)
{
  dart sigma_b,sigma_m_b;
  dart sigma_2_b,sigma_2_m_b;
  Point first_point,last_point;
  segment seg;
  bouleen done_something;

  assert(self);
  assert(b);
  assert(set_of_arc);

  /* Le first_point correspond au noeud possedant b, le last_point -b */
  done_something = faux;
  seg = (segment)tab_gen_give_object(Segment_set_contour,b);
  assert(seg);
  if(b<0)
    {
      segment_get_last (seg,&first_point);
      segment_get_first(seg,&last_point);
    }
  else
    {
      segment_get_first(seg,&first_point);
      segment_get_last (seg,&last_point);
    }
  
  /* Si le segment a une de ses extremites d'arite 2, on le prolonge */
  sigma_b   = map_give_sigma(Map_contour,b);
  sigma_2_b = map_give_sigma(Map_contour,sigma_b);
  if ( (sigma_2_b == b)&&(sigma_b != -b) )
    {
      private_remove_node(self,&first_point,sigma_b);
      dart_list_remove_elt(set_of_arc,sigma_b);
      dart_list_remove_elt(set_of_arc,-sigma_b);
      done_something =vrai;
    }

  sigma_m_b   = map_give_sigma(Map_contour,-b);
  sigma_2_m_b = map_give_sigma(Map_contour,sigma_m_b);
  if ( (sigma_2_m_b == -b)&&(sigma_m_b != b) )
    {
      private_remove_node(self,&last_point,sigma_m_b);
      dart_list_remove_elt(set_of_arc,sigma_m_b);
      dart_list_remove_elt(set_of_arc,-sigma_m_b);
      done_something = vrai;
    }
  return done_something;
}


  
   /*****************************************************/
  /* Enleve un arc parmis un ensemble d'arc. En enleve */
 /* eventuellement plus pour eviter les bout pendant. */
/*****************************************************/
static void private_remove_one_arc_in_set(contour_set self,
					  dart b,
					  dart_list set_of_arc)
{
  dart sigma_b,sigma_m_b;
  dart sigma_2_b,sigma_2_m_b;
  
  assert(self);
  assert(b);
  assert(set_of_arc);


  dart_list_remove_elt(set_of_arc,b);
  dart_list_remove_elt(set_of_arc,-b);


  while (private_extend_arc(self,b,set_of_arc));

  sigma_b     = map_give_sigma(Map_contour,b);
  sigma_2_b   = map_give_sigma(Map_contour,sigma_b);
  
  sigma_m_b   = map_give_sigma(Map_contour,-b);
  sigma_2_m_b = map_give_sigma(Map_contour,sigma_m_b);

  /* On elimine le cas ou le segment definit une composante connexe */
  if ( (sigma_b ==-b)&&(sigma_m_b == b) )
    {
      private_remove_arc(self,b);
      return;
    }
  /* Si la supression du segment crait un bout pendant, donc si le */
  /* segment a un seul noeud d'arite 3 on supprime le bout pendant avant */
  if (sigma_2_b == -b)
    private_remove_one_arc_in_set(self,sigma_b,set_of_arc);
  
  if (sigma_2_m_b == b)
    private_remove_one_arc_in_set(self,sigma_m_b,set_of_arc);
  
  /* On suprime enfin le segment */
  private_remove_arc(self,b);
}
  /*******************************************************************/
 /* Enleve un arc de la structure                                   */
/*******************************************************************/
static void private_remove_arc(contour_set self, dart old_edge)
{
  
  segment    old_seg;
  dart       first_sigma, last_sigma; 
  dart       label1,label2;
  dart       f_infini,f_fini;
  dart       new_face,father;
  Point      first_point, last_point;
  bouleen    label1_is_infinite,label2_is_infinite;
  node       node_to_inspect;

  if (old_edge < 0) old_edge = -old_edge;

  label1 = map_give_lambda(Map_contour,old_edge);
  label2 = map_give_lambda(Map_contour,-old_edge);

/* On efface pas les segments du bord de l'ecran */
  if(label1 == -1 || label2 == -1)
    return;

  /* On recupere le segment que l'on va efface dans un segment */
  /* temporaire */
  old_seg  = (segment)tab_gen_give_object(Segment_set_contour,old_edge);
  private_memorize_segment(self,old_seg);

  label1_is_infinite = inclusions_is_infinite(Inclusions_contour,label1);
  label2_is_infinite = inclusions_is_infinite(Inclusions_contour,label2);

  if( label1_is_infinite || label2_is_infinite)
    {
      /* Si une des 2 faces est infinie on cherche laquelle */
      f_infini = (label1_is_infinite?label1:label2);
      f_fini   = (label1_is_infinite?label2:label1);
    }

  first_sigma = map_give_sigma(Map_contour, old_edge);
  last_sigma  = map_give_sigma(Map_contour, -old_edge);

  if( (first_sigma == -old_edge )||(last_sigma == old_edge) )
    {
      if( (first_sigma == -old_edge )&&(last_sigma == old_edge) )
	{/* On enleve une composante connexe */
	  assert(label1_is_infinite||label2_is_infinite);
	  private_remove_component(self,old_edge,f_fini,f_infini);
	  return;
	}
      else
	{/* On enleve un segment tangent */
	  private_remove_tangent(self,old_edge,label1,label2);
	  return;
	} 
    }


  if(label1 == label2)
    { /* mere-fille ou soeurs */
      map_remove_edge(Map_contour,old_edge);
      if(label1_is_infinite)
	{
	  /*soeurs*/	
	  new_face = map_change_lambda_cycle(Map_contour,first_sigma);
	  map_set_lambda_1(Map_contour,label1,last_sigma);
	  father = inclusions_give_father(Inclusions_contour,label1);
	}
      else
	{
	  /*mere-fille */
	  /* On cherche la face a l'interieur et on la renumerote */
	  if( private_is_infinite(self,first_sigma) )
	    {
	      new_face = map_change_lambda_cycle(Map_contour,first_sigma);
	      map_set_lambda_1(Map_contour,label1,last_sigma);
	    }
	  else
	    {
	      new_face = map_change_lambda_cycle(Map_contour,last_sigma);
	      map_set_lambda_1(Map_contour,label1,first_sigma);
	    }
	  father = label1;
	  
	}
      inclusions_make_descent(Inclusions_contour,father,new_face);
    }
  else
    { /* on detruit la face finie */
      if( label1_is_infinite || label2_is_infinite)
	{
	  map_merge_lambda(Map_contour,f_infini,f_fini);
	  inclusions_report_daughter(Inclusions_contour,f_fini,f_infini);
	  private_update_faces(self,f_infini,f_fini);
	}
      else
	{
	  /* On reunie 2 faces finies */
	  map_merge_lambda(Map_contour,label1,label2);
	  inclusions_merge_finite_face(Inclusions_contour,label2,label1);
	  private_update_faces(self,label1,label2);
	}
      map_remove_edge(Map_contour,old_edge);
    }
  /* On supprime le segment */
  bitmap_remove_seg(Bitmap_contour, old_seg);

  segment_get_first(old_seg,&first_point);  
  segment_get_last(old_seg,&last_point);
  DEFAIRE_segment(old_seg);
  tab_gen_remove_object(Segment_set_contour,old_edge);

  /* On modifie les noeuds extremites pour effacer d'eventuelles */
  /* traces de old_edge */
  node_to_inspect  = node_set_give_node(Node_set_contour,&first_point);
  node_set_dart(node_to_inspect,first_sigma);

  node_to_inspect  = node_set_give_node(Node_set_contour,&last_point);
  node_set_dart(node_to_inspect,last_sigma);

  /* On supprime les noeuds extremite s'ils sont d'arites 2 */
  /*
  arite_noeud = map_give_card_cycle_sigma(Map_contour,first_sigma);
  if(arite_noeud == 2)
       private_remove_node(self, &first_point, first_sigma);
  
  arite_noeud = map_give_card_cycle_sigma(Map_contour,last_sigma);
  if(arite_noeud==2)
       private_remove_node(self, &last_point, last_sigma);
    */ 
  
}
                  /*********************/
                 /* Supprime un noeud */
                /*********************/
static void private_remove_node(contour_set self, 
				point p, 
				dart dart_to_supress)
{
  node      node_to_inspect,node_to_supress;
  dart      dart_nb,b;
  segment   seg_to_supress,seg_to_keep,new_seg;
  Point     pt_to_inspect;
  
  node_to_supress  = node_set_give_node(Node_set_contour, p);
  b                = map_give_sigma(Map_contour, dart_to_supress);
  

  
  /* On ne supprime pas le seul noeud d'une boucle */
  if( (map_give_phi(Map_contour, b) == b) )
    return;

  node_set_dart(node_to_supress, b);

  seg_to_keep    = (segment)tab_gen_give_object(Segment_set_contour, b);
  seg_to_supress = (segment)tab_gen_give_object(Segment_set_contour
						, dart_to_supress);
  
  if (b > 0)
    {
      if (dart_to_supress > 0)
	{/* Les 2 segments partent du noeud */
	  segment_reverse(seg_to_supress);
	}
      /* Les deux segments sont forcement consecutifs */
      segment_concat(seg_to_supress,seg_to_keep);
	
      new_seg = seg_to_supress;
      seg_to_supress = seg_to_keep;	  
      segment_get_first(new_seg,&pt_to_inspect);
    }
  else
    {
      if(dart_to_supress < 0)
	{ /* Les deux segments arrivent sur le noeud */
	  segment_reverse(seg_to_supress);
	}
      
      segment_concat(seg_to_keep,seg_to_supress);

      new_seg = seg_to_keep;
      segment_get_last(new_seg,&pt_to_inspect);
    }
  DEFAIRE_segment(seg_to_supress);
  tab_gen_remove_object(Segment_set_contour, b);
  tab_gen_remove_object(Segment_set_contour, dart_to_supress);
  
  tab_gen_insert_object(Segment_set_contour, new_seg, b);
  /* On supprime dart_to_supress aux differents endroits ou il pourrait */
  /* apparaitre */
  dart_nb = ABS(b);
  segment_set_dart_nb(new_seg,dart_nb);
  node_to_inspect = node_set_give_node(Node_set_contour,&pt_to_inspect);
  if(node_give_dart(node_to_inspect) == -dart_to_supress)
    node_set_dart(node_to_inspect,b);
  
  bitmap_make_node(Bitmap_contour, p, faux);
  map_remove_node(Map_contour,dart_to_supress);
  node_set_remove_node(Node_set_contour, p);
}
     /***************************************************************/
    /* On suprime un segment tangent.                              */
   /* old_edge le segment en question                             */
  /* f_fini : La face finie definie par old_edge ou -old_edge    */
 /* f_infini : La face infinie definie par old_edge ou -old_edge*/
/***************************************************************/
static void private_remove_tangent(contour_set self, 
				   dart old_edge,
				   dart label1, 
				   dart label2)
{  
  dart b;
  dart first_sigma,last_sigma;
  dart father;
  segment old_seg;
  Point node_point;
  dart f1,f2;
  bouleen f1_is_infinite,f2_is_infinite;
  node n;
  
  assert(self);
  assert(old_edge);
  assert(label1>0);
  assert(label2>0);
  
  first_sigma = map_give_sigma(Map_contour, old_edge);
  last_sigma  = map_give_sigma(Map_contour, -old_edge);
  
  assert( (first_sigma == -old_edge) || (last_sigma == old_edge));

  old_seg  = (segment)tab_gen_give_object(Segment_set_contour,old_edge);
  segment_get_first(old_seg,&node_point);
  
  map_remove_tangeant(Map_contour,old_edge);

  bitmap_remove_seg(Bitmap_contour, old_seg);

/* On verifie que  old_edge ou -old_edge n'est pas brin canonique du */
/* noeud */ 
  n = node_set_give_node(Node_set_contour,&node_point);
  b = (first_sigma == -old_edge?last_sigma:first_sigma);
  node_set_dart(n,b);

  tab_gen_remove_object(Segment_set_contour,old_edge);
/* f1 est defini comme la face possedant un seul brin */
  f1 = (first_sigma == -old_edge?label2:label1);
  f2 = (first_sigma == -old_edge?label1:label2);
  
  f1_is_infinite = inclusions_is_infinite(Inclusions_contour,f1);
  f2_is_infinite = inclusions_is_infinite(Inclusions_contour,f2);

  if (f2_is_infinite)
    {
      inclusions_report_daughter(Inclusions_contour,f1,f2);
      private_update_faces(self,f2,f1);
    }
  if (f1_is_infinite)
    {
      inclusions_report_daughter(Inclusions_contour,f2,f1);
      father = inclusions_give_father(Inclusions_contour,f1);
      inclusions_split_mother_son(Inclusions_contour,f2,father);
      inclusions_remove_infinite_face(Inclusions_contour,f1);
      private_update_faces(self,f1,f2);
    }
  if ( (!f1_is_infinite)&&(!f2_is_infinite))
    {
      inclusions_merge_finite_face(Inclusions_contour,f1,f2);
      private_update_faces(self,f2,f1);
    }
  DEFAIRE_segment(old_seg);
}
     /***************************************************************/
    /* On suprime une composante conexe                            */
   /* old_edge le segment definissant la composante               */
  /* f_fini : La face finie definie par old_edge ou -old_edge    */
 /* f_infini : La face infinie definie par old_edge ou -old_edge*/
/***************************************************************/
static void private_remove_component(contour_set self,dart
				     old_edge,dart f_fini,dart
				     f_infini)
{
  segment old_seg;
  Point node_point;
  dart father;
  face face1,face2;

  father = inclusions_give_father(Inclusions_contour,f_infini);

  face1 = (face)tab_gen_give_object(Face_set_contour,father);
  face2 = (face)tab_gen_give_object(Face_set_contour,f_fini);
  face_merge(face1,face2);

  old_seg  = (segment)tab_gen_give_object(Segment_set_contour,old_edge);
  segment_get_first(old_seg,&node_point);
  
  map_remove_map(Map_contour,old_edge);

  node_set_remove_node(Node_set_contour, &node_point);
  
  bitmap_remove_seg(Bitmap_contour, old_seg);
  bitmap_make_node(Bitmap_contour , &node_point, faux);
  
  inclusions_report_daughter(Inclusions_contour,f_fini,f_infini);
  inclusions_remove_infinite_face(Inclusions_contour,f_infini);
  
  tab_gen_remove_object(Segment_set_contour,old_edge);
}
static void private_insert_segment_border(contour_set self,int
					  hauteur,int largeur)
{
  segment seg = FAIRE_segment(Bitmap_contour);
  Point p;
  int longueur;

  segment_set_clockSense(seg,4);
  segment_set_dart_nb(seg,1);

  COORDX(p) = -1;COORDY(p) = -1;
  segment_set_first_point(seg,&p);
  segment_set_last_point(seg,&p);
  
  segment_set_first_move(seg,RIGHT);
  segment_set_last_move(seg,UP);
  
  COORDX(p) = largeur-1;
  COORDY(p) = -1;
  segment_set_right_point(seg,&p);
  longueur = 2*(hauteur+largeur);
  segment_set_longueur(seg,longueur);

  tab_gen_insert_object(Segment_set_contour,seg,1);

}

static void private_update_faces(contour_set self,dart label1,dart label2)
{
  bouleen label1_is_infinite,label2_is_infinite;
  face face1,face2;
  dart father1,father2;

  label1_is_infinite =
    inclusions_is_infinite(Inclusions_contour,label1);
  label2_is_infinite =
    inclusions_is_infinite(Inclusions_contour,label2);

  father1 = inclusions_give_father(Inclusions_contour,label1);
  father2 = inclusions_give_father(Inclusions_contour,label2);

  if(label1_is_infinite || label2_is_infinite)
    {
      if(label1_is_infinite)
	{
	  face1 = (face)tab_gen_give_object(Face_set_contour,father1);
	  face2 = (face)tab_gen_give_object(Face_set_contour,label2);
	}
      else
	{
	  face1 = (face)tab_gen_give_object(Face_set_contour,father2);
	  face2 = (face)tab_gen_give_object(Face_set_contour,label1);
	}
    }
  else
    {
      face1 = (face)tab_gen_give_object(Face_set_contour,label1);
      face2 = (face)tab_gen_give_object(Face_set_contour,label2);
    }
  face_merge(face1,face2);      
}
      
static void private_uninitialize_face(contour_set self,dart label1)
{
  bouleen label1_is_infinite;
  face face_to_update;
  dart father1;
  
  label1_is_infinite =
    inclusions_is_infinite(Inclusions_contour,label1);
  
  father1 = inclusions_give_father(Inclusions_contour,label1);
  
  if(label1_is_infinite)
    face_to_update = (face)tab_gen_give_object(Face_set_contour,father1);
  else
    face_to_update = (face)tab_gen_give_object(Face_set_contour,label1);
  
  face_uninitialize(face_to_update);

}

/****************************************************************************/
/* On utilise les marques que l'on a etabli dans mark face pour */
/* parcourir toutes les lignes de la face */
/****************************************************************************/
static void private_traverse_face(contour_set self, 
				  dart f,
				  double* moment0,
				  vector moment1,
				  vector moment2,
				  PFapply_to_pixels apply,
				  bouleen apply_function)
{
  dart b,b_first,b_last;
  segment seg,seg_last;
  move_type prec_move,next_move;
  double m0;
  vector m1,m2;
  Point p_entier,p_demis_entier;
  int i;
  int dim = image_give_dim(Image_contour);
  dart_list daughter_list;

  assert(self);
  assert(f>0);
  assert(!inclusions_is_infinite(Inclusions_contour,f));

  /* On fait les marquages pour le parcourt */
  contour_set_mark_end_points_of_face(self,f);
  daughter_list = inclusions_give_daughter(Inclusions_contour,f);
  contour_set_mark_set_of_face(self,
			       daughter_list,
			       Marked_points,
			       private_mark_point_one_time
			       );

  *moment0 = 0.0;
  for(i=0;i<dim;i++)
    {
      moment1[i] = 0.0;
      moment2[i] = 0.0;
    }

  b = b_first = map_give_lambda_1(Map_contour,f);
  seg = (segment)tab_gen_give_object(Segment_set_contour,b_first);

  b_last = map_give_phi_1(Map_contour,b_first);
  seg_last = (segment)tab_gen_give_object(Segment_set_contour,b_last);
  if(b_last >0)
    prec_move = segment_get_last_move(seg_last);
  else
    prec_move = REVERSE_SENSE( segment_get_first_move(seg_last) );

  do
    {
      if(b >0)
	{
	  segment_first(seg);
	  do
	    {      
	      next_move = segment_give_current_move(seg);

	      if(begin_mark_needed[prec_move][next_move])
		{
		  segment_give_current_point(seg,&p_demis_entier);
		  point_translate(&p_demis_entier,&p_entier,prec_move,next_move);
		  private_traverse_line(self,
					&p_entier,
					&m0,
					m1,
					m2,
					apply,
					apply_function);
		  (*moment0) += m0;
		  for(i=0;i<dim;i++)
		    {
		      moment1[i] += m1[i];
		      moment2[i] += m2[i];
		    }
		}
	      prec_move = next_move;	    
	    }
	  while (segment_next(seg));
	}
      else
	{
	  segment_last(seg);
	  do
	    {
	      next_move = REVERSE_SENSE(segment_give_current_move(seg));

	      if(begin_mark_needed[prec_move][next_move])
		{
		  segment_give_current_point(seg,&p_demis_entier);
		  point_translate(&p_demis_entier,&p_entier,
				  prec_move,next_move);

		  private_traverse_line(self,
					&p_entier,
					&m0,
					m1,
					m2,
					apply,
					apply_function);
		  (*moment0) += m0;
		  for(i=0;i<dim;i++)
		    {
		      moment1[i] += m1[i];
		      moment2[i] += m2[i];
		    }
		}
	      prec_move = next_move;
	    }
	  while (segment_previous(seg));
	}      
      b = map_give_phi(Map_contour,b);
      seg = (segment)tab_gen_give_object(Segment_set_contour,b);
    }
  while(b != b_first);	      
}
    /*****************************************************/  
   /* Calcule les moments sur l'ensemble des points     */
  /* d'une face en sautant les trous. L'exterieur de   */
 /* ceux ci est marque a 1 dans le tableau de marques */
/*****************************************************/
static void private_traverse_line(contour_set self,
				  point p,
				  double *moment0,
				  vector moment1,
				  vector moment2,
				  PFapply_to_pixels apply,
				  bouleen apply_function)
{
  angle val;
  int* pixel;
  int i;
  int dim = image_give_dim(Image_contour);
  double m0;
  vector m1,m2;

  m0 = 0.0;
  for(i=0;i<dim;i++)
    {
      m1[i]=0.0;
      m2[i]=0.0;
    }
  val = bitmap_move_to(Bitmap_contour,p);
  if( !image_move_to(Image_contour,p) )
    {
      printf("Point : (%d,%d) non valide\n",COORDX(*p),COORDY(*p));
      printf("contour_set.c private_traverse_line\n");
      exit(1);
    }

  while( !tab_mark_is_marked_two(Marked_points,p) )
    {
      pixel = image_lire_pixel(Image_contour);

      if(apply_function)
	apply(pixel,p,STORE);

      m0++;
      for(i=0;i<dim;i++)
	{
	  m1[i] += (double)pixel[i];
	  m2[i] += (double)(pixel[i]*pixel[i]);
	} 
      if( tab_mark_is_marked(Marked_points,p) )
	{
	  tab_mark_unmark_point(Marked_points,p);

	  do
	    {
	      MOVE_RIGHT_POINT(*p);
	      val = bitmap_right_point(Bitmap_contour);
	      assert(image_pixel_droite(Image_contour));
	    }
	  while(!tab_mark_is_marked(Marked_points,p));

	  tab_mark_unmark_point(Marked_points,p);
	  
	}
      MOVE_RIGHT_POINT(*p);
      val = bitmap_right_point(Bitmap_contour);
      assert(image_pixel_droite(Image_contour));
    }
  
  /* On demarque le dernier point */
  tab_mark_unmark_point_two(Marked_points,p);

  /* On s'occupe du dernier pixel tout en transvasant dans les */
  /* variables de sortie*/
  pixel = image_lire_pixel(Image_contour);

  if(apply_function)
    apply(pixel,p,STORE);

  *moment0 = m0+1;
  for(i=0;i<dim;i++)
    {
      moment1[i] = m1[i]+ pixel[i];
      moment2[i] = m2[i]+ pixel[i]*pixel[i];
    }
}
static void private_apply_function(contour_set self,
				   dart f,
				   PFapply_to_pixels apply)
{
  dart b,b_first,b_last;
  segment seg,seg_last;
  move_type prec_move,next_move;
  Point p_entier,p_demis_entier;
  dart_list daughter_list;

  /* On fait les marquages pour le parcourt */
  contour_set_mark_end_points_of_face(self,f);
  daughter_list = inclusions_give_daughter(Inclusions_contour,f);
  contour_set_mark_set_of_face(self,
			       daughter_list,
			       Marked_points,
			       private_mark_point_one_time
			       );
  
  b = b_first = map_give_lambda_1(Map_contour,f);
  seg = (segment)tab_gen_give_object(Segment_set_contour,b_first);
  
  b_last = map_give_phi_1(Map_contour,b_first);
  seg_last = (segment)tab_gen_give_object(Segment_set_contour,b_last);
  if(b_last >0)
    prec_move = segment_get_last_move(seg_last);
  else
    prec_move = REVERSE_SENSE( segment_get_first_move(seg_last) );

  do
    {
      if(b >0)
	{
	  segment_first(seg);
	  do
	    {      
	      segment_give_current_point(seg,&p_demis_entier);
	      next_move = segment_give_current_move(seg);
	      
	      if( begin_mark_needed[prec_move][next_move] )
		{
		  point_translate(&p_demis_entier,
				  &p_entier,
				  prec_move,
				  next_move);
		  private_apply_function_on_line(self,&p_entier,apply);
		  
		}
	      prec_move = next_move;
	    }
	  while (segment_next(seg));
	}
      else
	{
	  segment_last(seg);
	  do
	    {
	      segment_give_current_point(seg,&p_demis_entier);
	      next_move = REVERSE_SENSE(segment_give_current_move(seg));

	      if( begin_mark_needed[prec_move][next_move] )
		{
		  point_translate(&p_demis_entier,
				  &p_entier,
				  prec_move,
				  next_move);

		  private_apply_function_on_line(self,&p_entier,apply);
		  
		}
	      prec_move = next_move;
	    }
	  while( segment_previous(seg) );
	}
      
      b = map_give_phi(Map_contour,b);
      seg = (segment)tab_gen_give_object(Segment_set_contour,b);
    }
  while(b != b_first);	      
  
}
    /*****************************************************/  
   /* Applique une fonction sur l'ensemble des points   */
  /* d'une face en sautant les trous. L'exterieur de   */
 /* ceux ci est marque a 1 dans le tableau de marques */
/*****************************************************/
static void private_apply_function_on_line(contour_set self,
					   point p,
					   PFapply_to_pixels apply)
					   
{
  angle val;
  int* pixel;

  val = bitmap_move_to(Bitmap_contour,p);
  if( !image_move_to(Image_contour,p) )
    {
      printf("Point : (%d,%d) non valide\n",COORDX(*p),COORDY(*p));
      printf("contour_set.c private_traverse_line\n");
      exit(1);
    }

  while( !tab_mark_is_marked_two(Marked_points,p) )
    {
      pixel = image_lire_pixel(Image_contour);
      apply(pixel,p,STORE);

      if( tab_mark_is_marked(Marked_points,p) )
	{
	  tab_mark_unmark_point(Marked_points,p);
	  do
	    {
	      MOVE_RIGHT_POINT(*p);
	      val = bitmap_right_point(Bitmap_contour);
	      assert(image_pixel_droite(Image_contour));
	    }
	  while( !tab_mark_is_marked(Marked_points,p) );

	  tab_mark_unmark_point(Marked_points,p);
	}
      MOVE_RIGHT_POINT(*p);
      val = bitmap_right_point(Bitmap_contour);
      assert(image_pixel_droite(Image_contour));
    }
  
/* On demarque le dernier point */
  tab_mark_unmark_point_two(Marked_points,p);

  /* On Applique la fonction sur le dernier point */
  pixel = image_lire_pixel(Image_contour);
  apply(pixel,p,STORE);
}
     /*******************************************************/
    /* Marque les points d'une face. Ne pas confondre avec */
   /* private_mark_interior_of_face. Dans le cas present  */
  /* ce sont les points 1/2 constituant la face qui sont */
 /*marques                                              */
/*******************************************************/

static void private_mark_face(contour_set self,
			      dart f,
			      void (*mark_point)(tab_mark,point),
			      tab_mark mark_face)

{
  segment seg;
  dart b_first,b;

  b = b_first = map_give_lambda_1(Map_contour,f);
  seg = (segment)tab_gen_give_object(Segment_set_contour,b_first);

  
  do
    {
      private_mark_segment(seg,mark_point,mark_face);

      b = map_give_phi(Map_contour,b);
      seg = (segment)tab_gen_give_object(Segment_set_contour,b);
    }
  while(b != b_first);

}
  /**************************************************/
 /* Marque les points du segment a la base d'un UP */
/**************************************************/
static void private_mark_segment(segment seg,
				 void (*mark_point)(tab_mark,point),
				 tab_mark mark_face)
{
  move_type prec_move,next_move;
  Point p;

  segment_first(seg);
  
  prec_move = segment_give_current_move(seg);
  if(prec_move == UP)
    {
      segment_give_current_point(seg,&p);
      mark_point(mark_face,&p);
    }
  
  while( segment_next(seg) )
    {
      segment_give_current_point(seg,&p);
      
      if(prec_move == DOWN)
	mark_point(mark_face,&p);
      
      next_move = segment_give_current_move(seg);

      if(next_move == UP)
	mark_point(mark_face,&p);

      prec_move = next_move;
    }

  if(prec_move == DOWN)
    {
      segment_get_last(seg,&p);
      mark_point(mark_face,&p);
    }
}
       /**************************************************************/
      /* Marque les points une seule fois. Si on desire             */
     /* marque deux fois un point, cela signifie que               */
    /* l'on est passe deux fois dans le segment auquel            */
   /* il appartient. C'est donc un segment reliant deux faces.   */
  /* Il ne definit pas une region et n'a donc pas a etre marque */
 /* See also : contour_set_traverse_face.                      */
/**************************************************************/
static void private_mark_point_one_time(tab_mark mark_point,point p)
{
  if(tab_mark_is_marked(mark_point,p))
    {
      tab_mark_unmark_point(mark_point,p);
      return;
    }
  tab_mark_mark_point(mark_point,p);
}
     /*********************************************************/
    /* Demarque les points d'une face. Ne pas confondre avec */
   /* private_unmark_interior_of_face. Dans le cas present  */
  /* ce sont les points 1/2 constituant la face qui sont   */
 /* demarques.                                            */
/*********************************************************/
static void private_unmark_face(contour_set self,
				dart f,
				void (*unmark_point)(tab_mark,
						     point),
				tab_mark mark_face)

{
  segment seg;
  dart b_first,b;

  b = b_first = map_give_lambda_1(Map_contour,f);
  seg = (segment)tab_gen_give_object(Segment_set_contour,b_first);

  
  do
    {
      segment_unmark_segment(seg,unmark_point,mark_face);

      b = map_give_phi(Map_contour,b);
      seg = (segment)tab_gen_give_object(Segment_set_contour,b);
    }
  while(b != b_first);

}

/* Marque une face en vue de son parcourt */
static void private_mark_interior_of_face(contour_set self,dart f)
{
  dart b,b_first,b_last;
  segment seg,seg_last;
  move_type init_move,prec_move,next_move;
  Point p_demis_entier;

  b = b_first  = map_give_lambda_1(Map_contour,f);
  seg = (segment)tab_gen_give_object(Segment_set_contour,b_first);
  
  b_last = map_give_phi_1(Map_contour,b_first);
  seg_last = (segment)tab_gen_give_object(Segment_set_contour,b_last);
  if(b_last >0)
    prec_move = segment_get_last_move(seg_last);
  else
    prec_move = REVERSE_SENSE( segment_get_first_move(seg_last) );

  /* On commence par parcourir le segment pour etablir les marques */
  /* necessaires pour parcourir la face */
  do
    {
      if(b >0)
	{
	  segment_first(seg);
	  do
	    {
	      segment_give_current_point(seg,&p_demis_entier);
	      next_move = segment_give_current_move(seg);
	      private_mark_point_if_needed(self,
					   &p_demis_entier,
					   prec_move,
					   next_move);
	      prec_move = next_move;
	    }
	  while (segment_next(seg));
	}
      else
	{
	  segment_last(seg);
	  do
	    {
	      segment_give_current_point(seg,&p_demis_entier);
	      next_move = REVERSE_SENSE(segment_give_current_move(seg));
	      private_mark_point_if_needed(self,
					   &p_demis_entier,
					   prec_move,
					   next_move);
	      prec_move = next_move;
	    }
	  while( segment_previous(seg) );
	}
      b = map_give_phi(Map_contour,b);
      seg = (segment)tab_gen_give_object(Segment_set_contour,b);
    }
  while(b != b_first);
  /* On s'occupe de l'angle entre le premier et le dernier segment */
  segment_give_current_point(seg,&p_demis_entier);
  private_mark_point_if_needed(self,&p_demis_entier,prec_move,init_move);
}

  /******************************************************************/
 /*    Demarque une face. Indispensable si l'on a fait un marquage */
/******************************************************************/
static void private_unmark_interior_of_face(contour_set self,dart f)
{
  dart b,b_first,b_last;
  Point p_entier,p_demis_entier;
  segment seg,seg_last;
  move_type prec_move,next_move;
  bouleen mark_needed;

  b = b_first  = map_give_lambda_1(Map_contour,f);
  seg = (segment)tab_gen_give_object(Segment_set_contour,b_first);

  b_last = map_give_phi_1(Map_contour,b_first);
  seg_last = (segment)tab_gen_give_object(Segment_set_contour,b_last);
  if(b_last >0)
    prec_move = segment_get_last_move(seg_last);
  else
    prec_move = REVERSE_SENSE( segment_get_first_move(seg_last) );

  /* On commence par parcourir le segment pour etablir les marques */
  /* necessaires pour parcourir la face */
  do
    {
      if(b >0)
	{
	  segment_first(seg);
	  do
	    {
	      segment_give_current_point(seg,&p_demis_entier);
	      next_move = segment_give_current_move(seg);
	      mark_needed = begin_mark_needed[prec_move][next_move];
	      mark_needed = mark_needed || end_mark_needed[prec_move][next_move];
	      if(mark_needed)
		{
		  point_translate(&p_demis_entier,&p_entier,prec_move,next_move);
		  tab_mark_unmark_point_all(Marked_points,&p_entier);
		}	      
	      prec_move = next_move;
	    }
	  while (segment_next(seg) );	  
	}
      else
	{
	  segment_last(seg);
	  do
	    {
	      segment_give_current_point(seg,&p_demis_entier);
	      next_move = REVERSE_SENSE(segment_give_current_move(seg));
	      mark_needed = begin_mark_needed[prec_move][next_move];
	      mark_needed = mark_needed || end_mark_needed[prec_move][next_move];
	      if(mark_needed)
		{
		  point_translate(&p_demis_entier,&p_entier,prec_move,next_move);
		  tab_mark_unmark_point_all(Marked_points,&p_entier);
		}
	      prec_move = next_move;
	    }
	  while( segment_previous(seg) );
	}
      
      b = map_give_phi(Map_contour,b);
      seg = (segment)tab_gen_give_object(Segment_set_contour,b);
    }
  while(b != b_first);
}

static void private_mark_point_if_needed(contour_set self,
					 point p_1_2,
					 move_type prec_move,
					 move_type next_move)
{
  
  Point p_entier;

  if(begin_mark_needed[prec_move][next_move])
    {
      point_translate(p_1_2,
		      &p_entier,
		      prec_move,
		      next_move);
      tab_mark_mark_point(Marked_points,&p_entier);
    }
  
  if(end_mark_needed[prec_move][next_move])
    {
      point_translate(p_1_2,
		      &p_entier,
		      prec_move,
		      next_move);
      tab_mark_mark_point_two(Marked_points,&p_entier);
    }
}
     /********************************************************/
    /* Donne le dernier brin non marque du cycle sigma de b */
   /* ainsi que le nombre de brin non marque.              */
  /* Si tous les brin sont marques le nb de points        */
 /* non marque est nul et le brin renvoye est egal a -1  */
/********************************************************/   
static void private_give_unmarked_dart(contour_set self,
				       dart b,
				       dart *unmarked_b,
				       int * nb_of_unmarked)
{
  dart b_aux;
  int unmarked_nb;

  assert(self);

  b_aux = b;
  unmarked_nb = 0;
  *unmarked_b = -1;
  do
    {
      if (!map_give_mark(Map_contour,b_aux))
	{
	  *unmarked_b = b_aux;
	  unmarked_nb++;
	}
      b_aux = map_give_sigma(Map_contour,b_aux);
    }
  while(b_aux != b);

  *nb_of_unmarked = unmarked_nb;
}
     /*******************************************************************/
    /* Cette fonction retient la face finie, definie par le segment    */
   /* (new_num,-new_num). Si f1 et f2 sont finis elle retient la face */ 
  /* definie par new_num. Cela correspond aux conventions de         */
 /* add_new_contour.                                                */
/*******************************************************************/
static void private_memorize_face(contour_set self,dart new_num)
{
  dart f1,f2;

  assert(self);
  
  if(!Memorize_face_flag)
    return;
  
  f1 = map_give_lambda(Map_contour,new_num);
  f2 = map_give_lambda(Map_contour,-new_num);
  if(inclusions_is_infinite(Inclusions_contour,f1))
    {
      assert(!inclusions_is_infinite(Inclusions_contour,f2));
      dart_list_insert(Inserted_faces,f2);
    }
  else
    dart_list_insert(Inserted_faces,f1);
}
static void private_memorize_segment(contour_set self, segment seg)
{
  output_segment ext_seg;

  assert(self);

  if(!Memorize_segment_flag)
    return;

  assert(seg);

  
  ext_seg = FAIRE_output_segment();
  segment_init_output_segment(seg,ext_seg);
  list_gen_insert(Modified_segments,ext_seg);
}
static move_type private_give_first_move_of_dart(contour_set self,
						 dart b)
{
  move_type move;
  segment seg;
  
  seg = (segment)tab_gen_give_object(Segment_set_contour,b);
  if(b>0)
    return segment_get_first_move(seg);
  
  move = segment_get_last_move(seg);
  
  return REVERSE_SENSE(move);
}
