#include "cag.h"
#include "rag.h"
#include "cluster_set.h"

static void   move_to_specified_ecart_priv(cluster_set,cag,int*,double);
static double merge_two_nodes_priv        (cluster_set,cag);
static double give_min_dist_priv          (cluster_set,
					   cag,
					   int*,
					   int*);
static void  merge_nodes_priv             (cluster_set,
					   cag,
					   int,
					   int);
static void  check_clusters_validity_priv (cluster_set,
					   cag,
					   int*,
					   double);

void merge_clusters(cluster_set clusters,int* nb_color,double ecart_max)
{
  cag graph;
  
  graph = FAIRE_cag(clusters);

  ecart_max /= 100.0;
  move_to_specified_ecart_priv(clusters,graph,nb_color,ecart_max);

  DEFAIRE_cag(graph);
}

static void move_to_specified_ecart_priv(cluster_set clusters,
					 cag graph,
					 int* nb_color,
					 double ecart_max)
{
  double sqe_sum;
  int nb_colors;
  double limit;

  limit = cluster_set_give_global_sqe(clusters);
  printf("Global sqe : %f\n",limit);
  limit = ecart_max*limit;  

  sqe_sum = cluster_set_give_sqe_sum(clusters);
  nb_colors = cluster_set_give_cardinal(clusters);
  /*tq n_color <2 si dist > eps return*/
  while(nb_colors>2)
    {
      if(sqe_sum > limit)
	{
	  check_clusters_validity_priv(clusters,graph,&nb_colors,ecart_max);
	  *nb_color = nb_colors;

	  printf("rapport final:%lf\n",limit/sqe_sum);
	  return;
	}
      sqe_sum += merge_two_nodes_priv(clusters,graph);
      nb_colors--;
    }
  *nb_color = nb_colors;
  printf("sqe_sum : %lf ,limit: %lf, rapport %lf:\n",sqe_sum,limit,limit/sqe_sum);
}
static double merge_two_nodes_priv(cluster_set clusters,cag graph)
{
  int clus1,clus2;
  double increase = 0.0;

  cag_find_closest_node(graph,&clus1,&clus2);
  increase -= cluster_set_give_sqe(clusters,clus1);
  increase -= cluster_set_give_sqe(clusters,clus1);
  merge_nodes_priv(clusters,graph,clus1,clus2);
  increase += cluster_set_give_sqe(clusters,clus1);

  return increase;
}
static double give_min_dist_priv(cluster_set clusters,
				 cag graph,
				 int *n1,
				 int* n2)
{
  cag_find_closest_node(graph,n1,n2);
  return cluster_set_get_dist(clusters,*n1,*n2);
}
static void  merge_nodes_priv(cluster_set clusters,
			      cag graph,
			      int n1,
			      int n2)
{
  cluster_set_merge_cluster(clusters,n1,n2);
  cag_merge_node(graph,clusters,n1,n2);
}
/*************************************/
/* A ameliorer */
/*************************************/
static void  check_clusters_validity_priv(cluster_set clusters,
					  cag graph,
					  int* nb_colors,
					  double ecart_max)
{
  double dist,var;
  double limit;
  int c1,c2;
  int nb_color;
  
  nb_color = *nb_colors;

  if(nb_color == 2)
    return;

  var = cluster_set_give_color_variance(clusters);
  var *= 2.0*nb_color/(nb_color-1);
  limit = var*ecart_max;
  do
    {
      cag_give_min_color_dist(graph,clusters,&c1,&c2);
      dist = cluster_set_get_color_dist(clusters,c1,c2);
      if(dist > limit)
	{
	  *nb_colors = nb_color;
	  return;
	}
      cluster_set_merge_cluster(clusters,c1,c2);
      cag_merge_node(graph,clusters,c1,c2);
      nb_color--;
/*      var = cluster_set_give_color_variance(clusters);
      var *= 2.0*nb_color/(nb_color-1);
      limit = ecart_max*var; */

   }
  while(nb_color > 2);

  *nb_colors = nb_color;
}


