#include <assert.h>
#include <stdarg.h>
#include <values.h>

#include "contour_set.h"
#include "cluster_set.h"
#include "rag.h"
#include "meta_seg_uniform_rec.h"
#include "segmentation.h"
#include "type_obj.h"
#include "segmentation_link.h"
#include "tab_mark_dart.h"
#include "macros.h"
#include "ximage.h"

/*************************************************************************/
                    /*  Fonctions privees */

static void    do_segmentation_priv              (contour_set,dart,double);
static bouleen move_faces_to_specified_ecart_priv(contour_set,rag,double);
static double  merge_two_faces_priv              (contour_set,rag,double);
static dart    find_face_of_max_sqe_priv         (contour_set,dart_list);
static void    meta_seg_uniform_priv             (contour_set,
						  dart,
						  dart_list,
						  tab_mark_dart,
						  double);
static void  check_list_of_faces_priv            (contour_set,
						  dart_list,
						  tab_mark_dart);
/*************************************************************************/

void meta_seg_uniform_rec_rule_set_algorithm(rule father)
{
  father->specified_rule = meta_seg_uniform_rec;
}

void meta_seg_uniform_rec(contour_set contours,
			  dart f,
			  va_list list)
{
  dart_list final_face_list;
  dart f_max;
  double f_sqe,sqe_max;
  double limit,ecart_max;
  ximage ximg;
  int couleur_normal;
  tab_mark_dart generated_faces;

  ecart_max      = va_arg(list,double);
  ximg           = va_arg(list,ximage);
  couleur_normal = va_arg(list,int);

  f_sqe = contour_set_give_face_squared_error(contours,f);

  limit = ecart_max*f_sqe/300.0;
  f_max = f;
  final_face_list = FAIRE_dart_list();
  generated_faces = FAIRE_tab_mark_dart();

  dart_list_insert(final_face_list,f);
  tab_mark_dart_mark_dart(generated_faces,f);
  do
    {
      meta_seg_uniform_priv(contours,
			    f_max,
			    final_face_list,
			    generated_faces,
			    ecart_max);
      
      ximage_draw_set_of_faces(ximg,contours,final_face_list,couleur_normal);
      ximage_display(ximg);
      /* Si On a plus de face a segmenter on sort */
      if(dart_list_is_empty(final_face_list))
	{
	  DEFAIRE_dart_list(final_face_list);
	  return;
	}
      f_max = find_face_of_max_sqe_priv(contours,final_face_list);
      ximage_draw_face(ximg,contours,f_max,251);
      ximage_display(ximg);
      sqe_max = contour_set_give_face_squared_error(contours,f_max);
      printf("f_max = %d,sqe_max = %f\n",f_max,sqe_max);
    }
  while(sqe_max > limit);

  DEFAIRE_dart_list(final_face_list);
}
  /*******************************************************************/
 /*                 Definition des fonctions privees.               */
/*******************************************************************/
static void meta_seg_uniform_priv(contour_set contours,
				  dart f,
				  dart_list final_face_list,
				  tab_mark_dart generated_faces,
				  double ecart_max)
{
  rag graph;
  double limit_merge;
  dart_list face_list;
  
  assert(contour_set_is_face_defined(contours,f));

  contour_set_memorize_inserted_faces(contours);
  
  do_segmentation_priv(contours,f,ecart_max);
  
  face_list = FAIRE_dart_list();
  contour_set_give_inserted_faces(contours,face_list);
  dart_list_insert(face_list,f);
  

  
  graph = FAIRE_rag(contours,face_list,generated_faces);


  limit_merge = rag_give_global_sqe(graph);
  limit_merge *= (ecart_max/3000.0);

  move_faces_to_specified_ecart_priv(contours,graph,limit_merge);

  dart_list_reinitialize(face_list);
  rag_give_faces(graph,contours,face_list);
  if(dart_list_give_cardinal(face_list) > 1)
    dart_list_merge_list(final_face_list,face_list);
  else
    {
      /* Si cette face entraine une boucle on ne la remet */
      /* pas dans l'ensemble des faces a traiter*/
      dart_list_remove_list(final_face_list,face_list);
      DEFAIRE_dart_list(face_list);
    }
  
  check_list_of_faces_priv(contours,final_face_list,generated_faces); 
  DEFAIRE_rag(graph);

}
static void do_segmentation_priv(contour_set contours,dart f,double ecart_max)
{ 
  segmentation seg_alg;
  
  seg_alg = FAIRE_segmentation();
  segmentation_set_algorithm(seg_alg,algo_seg[uniform_segmentation]);
  segmentation_make_segments(seg_alg,contours,f,ecart_max);
  DEFAIRE_segmentation(seg_alg);
}

static bouleen move_faces_to_specified_ecart_priv(contour_set contours,
						  rag graph,
						  double limit)
{
  double sqe_face;
  int nb_eff=0;					/* debug */

  rag_remove_noise(graph,contours);

  do
    {
      sqe_face = merge_two_faces_priv(contours,graph,limit);
      nb_eff++;
    }
  while(sqe_face < limit);
  printf("nb_grosse face efface :%d\n",nb_eff);
  return (sqe_face == DBL_MAX);
}
static double merge_two_faces_priv(contour_set contours,
				   rag graph,
				   double limit)
{
  dart f1,f2,f_merged;
  double f1_sqe,f2_sqe,sqe;

  rag_find_closest_node(graph,contours,&f1,&f2);
  f1_sqe = rag_give_sqe(graph,contours,f1);
  f2_sqe = rag_give_sqe(graph,contours,f2);
  sqe = MIN(f1_sqe,f2_sqe);

  if(sqe >= limit)
    return sqe;

  f_merged = rag_merge_face(graph,contours,f1,f2);
  
  return sqe;
}
static dart find_face_of_max_sqe_priv(contour_set contours,
				      dart_list face_list)
{
  dart f,f_max;
  double f_sqe,max_sqe;
  
  max_sqe = DBL_MIN;
  
  dart_list_first(face_list);
  do
    {
      f = dart_list_give_current(face_list);
      f_sqe = contour_set_give_face_squared_error(contours,f);
      if(f_sqe>max_sqe)
	{
	  f_max = f;
	  max_sqe = f_sqe;
	}
    }
  while(dart_list_next(face_list));

  return f_max;
}
  /****************************************************************/
 /*  Verifie la liste des faces. Supprime les faces non definies */
/****************************************************************/
static void check_list_of_faces_priv(contour_set contours,
				     dart_list list_of_face,
				     tab_mark_dart generated_faces)
{
  dart_list faces_to_remove;
  dart f;

  faces_to_remove = FAIRE_dart_list();

  dart_list_first(list_of_face);
  do
    {
      f = dart_list_give_current(list_of_face);
      if(!contour_set_is_face_defined(contours,f)||
	 contour_set_is_infinite(contours,f))
	{
	  dart_list_insert(faces_to_remove,f);
	  tab_mark_dart_unmark_dart(generated_faces,f);
	}
      else
	tab_mark_dart_mark_dart(generated_faces,f);
    }
  while(dart_list_next(list_of_face));
  
  printf("nb de faces devenues invalides : %d\n",
	 dart_list_give_cardinal(faces_to_remove));

  dart_list_remove_list(list_of_face,faces_to_remove);
  DEFAIRE_dart_list(faces_to_remove);
}
