#include <malloc.h>
#include <assert.h>
#include "output_segment.h"
#include "tab_move.h"
#include "point.h"
#include "move_type.h"

#include "objet.h"
#include "garbage.h"

#define Move_set          SELF (move_set)
#define Ind_last          SELF (ind_last)
#define Ind               SELF (ind)
#define First_point       SELF (first_point)
#define Last_point        SELF (last_point)
#define Right_point       SELF (right_point)
#define Current_point     SELF (current_point)
#define Clock_sense       SELF (clock_sense)
#define Lenght            SELF (lenght)
#define Dart_nb           SELF (dart_nb)

/* Fonctions utilisees pour le garbage collecting */
static output_segment init_output_segment_priv();
static void           free_output_segment_priv(output_segment);
static void           dealloc_output_segment_priv(void*);

struct output_segment
{
  tab_move move_set;
  int      ind_last;
  int      ind;
  Point    first_point;
  Point    last_point;
  Point    current_point;
  Point    right_point;
  int      lenght;
  long     clock_sense;
  dart     dart_nb;
};

static garbage output_segment_collector = NULL_GARBAGE;

/*****************************************************************************/

void ALLOC_output_segment()
{
  output_segment_collector = FAIRE_garbage();
}
void DEALLOC_output_segment()
{
  DEFAIRE_garbage(output_segment_collector,dealloc_output_segment_priv);
}
/*****************************************************************************/
output_segment FAIRE_output_segment()
{
  Point p;
  output_segment self;

  COORDX(p) = -1;
  COORDY(p) = -1;

  self =  init_output_segment_priv();
  
  Move_set = FAIRE_tab_move();
  AFFECTE_POINT(Current_point,p);
  AFFECTE_POINT(Right_point,p);
  Ind_last    = 0;
  Lenght      = 0;
  Clock_sense = 0;

  return self;
}
void DEFAIRE_output_segment(output_segment self)
{
  assert(self);

  DEFAIRE_tab_move(Move_set);
  free_output_segment_priv(self);
}
/*****************************************************************************/

void free_output_segment(void* obj)
{
  output_segment self = obj;

  assert(self);

  DEFAIRE_tab_move(Move_set);
  free_output_segment_priv(self);
}
void output_segment_reinitialize(output_segment self)
{
  Point p;
  
  assert(self);

  COORDX(p) = -1;
  COORDY(p) = -1;

  AFFECTE_POINT(Current_point,p);
  AFFECTE_POINT(Right_point,p);

  Ind_last    = 0;
  Lenght      = 0;
  Clock_sense = 0;
}
/****************************************************************************/
                      /* Partie reserve au coeur */
/****************************************************************************/

   /********************************************************/
  /* Stocke le segment, current est le point ou l'on est, */
 /*  move la direction ou l'on va                        */
/********************************************************/
void output_segment_insert(output_segment self,
			   move_type move)
			   
{
  tab_move_put_value(Move_set,Ind_last,move);
  Ind_last++;
}  
/*****************************************************************************/
         
        /****************************************************************/
       /* Partie reserve aux algo demandant des infos a output_segment */
      /****************************************************************/

/*****************************************************************************/
void output_segment_give_first_point(output_segment self,point p)
{
  assert(self);

  AFFECTE_POINT(*p,First_point);
}
/*****************************************************************************/
void output_segment_set_first_point(output_segment self,point p)
{
  assert(self);

  AFFECTE_POINT(First_point,*p);
}

/*****************************************************************************/
void output_segment_give_last_point(output_segment self,point p)
{
  assert(self);

  AFFECTE_POINT(*p,Last_point);
}
/*****************************************************************************/
void output_segment_set_last_point(output_segment self,point p)
{
  assert(self);

  AFFECTE_POINT(Last_point,*p);
}
/*****************************************************************************/
void output_segment_give_right_point(output_segment self,point p)
{
  assert(self);

  AFFECTE_POINT(*p,Right_point);
}
/*****************************************************************************/
void output_segment_set_right_point(output_segment self,point p)
{
  assert(self);

  AFFECTE_POINT(Right_point,*p);
}
/*****************************************************************************/
move_type output_segment_give_first_move(output_segment self)
{
  assert(self);


  return tab_move_give_value(Move_set,0);
}
/*****************************************************************************/
move_type output_segment_give_last_move(output_segment self)
{
  assert(self);

  return tab_move_give_value(Move_set,Ind_last-1);
}
/*****************************************************************************/
int output_segment_give_lenght(output_segment self)
{
  assert(self);

  return Lenght;
}
/*****************************************************************************/
void output_segment_set_lenght(output_segment self,int length)
{
  assert(self);

  Lenght = length;
}
/*****************************************************************************/
long output_segment_give_clock(output_segment self)
{
  assert(self);
  
  return Clock_sense;
}
/*****************************************************************************/
void  output_segment_set_clock(output_segment self,long clock)
{
  assert(self);
  
  Clock_sense = clock;
}
/*****************************************************************************/
dart output_segment_give_dart_nb(output_segment self)
{
  assert(self);
  
  return Dart_nb;
}
/*****************************************************************************/
void output_segment_set_dart_nb(output_segment self,dart nb)
{
  assert(self);
  
  Dart_nb = nb;
}

/*****************************************************************************/
void output_segment_first(output_segment self)
{
  assert(self);
  
  AFFECTE_POINT(Current_point,First_point);

  Ind = 0;
}
/*****************************************************************************/
/* Current point : point ou l'on est .*/
/* Ind : Indice du move suivant*/
/*****************************************************************************/
bouleen output_segment_next(output_segment self)
{
  move_type move;

  assert(self);

  move = tab_move_give_value(Move_set,Ind);
  modify_point_position(&Current_point,move);
  Ind++;

  if(Ind == Ind_last)
    return faux;

  return vrai;
}
/*****************************************************************************/
void output_segment_last(output_segment self)
{
  assert(self);

  AFFECTE_POINT(Current_point,Last_point);
  Ind = Ind_last-1;
}
/*****************************************************************************/
/* Current point : point ou l'on est .*/
/* Ind : Indice du move suivant*/
/*****************************************************************************/
bouleen output_segment_previous(output_segment self)
{
  move_type move;

  assert(self);

  move = tab_move_give_value(Move_set,Ind);
  move = REVERSE_SENSE(move);
  modify_point_position(&Current_point,move);


  if(Ind == 0)
    return faux;

  Ind--;  
  return vrai;
}
/*****************************************************************************/
move_type  output_segment_give_current_move(output_segment self)
{

  assert(self);  


  if (Ind == Ind_last)
    return INFINITE;

  return tab_move_give_value(Move_set,Ind);
}
/*****************************************************************************/
void output_segment_give_current_point(output_segment self,point p)
{
  assert(self);  

  AFFECTE_POINT(*p,Current_point);
}
/*****************************************************************************/
bouleen output_segment_is_closed(output_segment self)
{
  assert(self);  

  return EGAL_POINT(First_point,Last_point);
}
/*****************************************************************************/

    /***********************************************************/
   /* Initialize ou reinitialize un segment.                  */
  /***********************************************************/
static output_segment init_output_segment_priv()
{  
  output_segment self;

  if(output_segment_collector == NULL_GARBAGE)
    {
      self = (output_segment)malloc(sizeof(struct output_segment));
      Move_set = FAIRE_tab_move();

      return self;
    }
  if(garbage_is_empty(output_segment_collector))
    {
      self = (output_segment)malloc(sizeof(struct output_segment));
      Move_set = FAIRE_tab_move();
      return self;
    }

  return (output_segment)garbage_give_object(output_segment_collector);
}  
static void free_output_segment_priv(output_segment self)
{
  assert(self);

  if(output_segment_collector == NULL_GARBAGE)
    {
      dealloc_output_segment_priv(self);
      return;
    }

  garbage_insert_object(output_segment_collector,self);
}
static void dealloc_output_segment_priv(void* seg)
{
  output_segment self = (output_segment)seg;

  free(self);
}






