#include <assert.h>
#include <values.h>
#include <glib.h>

#include "objet.h"
#include "merge_faces.h"
#include "rag.h"
#include"tab_mark_dart.h"
#include "dart_tab.h"
#include "dart_list.h"
#include "tab_gen.h"
#include "macros.h"

#define NOISE_SIZE   4.0
#define UNKNOWN_FACE -1

static double give_min_dist_priv   (rag,
				    contour_set,
				    dart,
				    dart*);
static double face_dist_priv       (contour_set,
				    dart,
				    dart);


#define Nb_face         SELF(nb_face)
#define Nb_face_remains SELF(nb_face_remains)
#define Tab_face        SELF(tab_face)
#define Global_sqe      SELF(global_sqe)
#define Initial_face    SELF(initial_face)
#define Allowed_merge   SELF(allowed_merge)

struct rag
{
  int nb_face;
  int nb_face_remains;
  double global_sqe;
  dart_tab tab_face;
  tab_mark_dart initial_face;
  tab_mark_dart allowed_merge;
};

rag FAIRE_rag(contour_set contours,
	      dart_list created_faces,
	      tab_mark_dart allowed_merge)
{
  int i,j;
  dart f;
  double m0;
  vector m1,m2;
  vector M1 = {0.0,0.0,0.0};
  vector M2 = {0.0,0.0,0.0};
  double M0;

  INSTANCIER(rag);
  
  Nb_face = Nb_face_remains=dart_list_give_cardinal(created_faces);
  Tab_face = FAIRE_dart_tab();
  Initial_face = FAIRE_tab_mark_dart();
  Global_sqe = 0.0;
  Allowed_merge = allowed_merge;

  if(dart_list_is_empty(created_faces))
    return self;

  dart_list_first(created_faces);
  M0 = 0.0;
  i=0;
  do
    {
      f = dart_list_give_current(created_faces);
      contour_set_give_face_moments(contours,f,&m0,m1,m2);
      M0 += m0;
      for(j=0;j<DIM_MAX;j++)
	{
	  M1[j] += m1[j];
	  M2[j] += m2[j];
	}
      dart_tab_put_value(Tab_face,i,f);
      tab_mark_dart_mark_dart(Initial_face,f);
      i++;
    }
  while(dart_list_next(created_faces));
  
  printf("Nb de faces crees : %d\n",i);
  for(j=0;j<DIM_MAX;j++)
    Global_sqe += (M2[j]-M1[j]*(M1[j]/M0));

  return self;
}
void DEFAIRE_rag(rag self)
{
  DEFAIRE_tab_mark_dart(Initial_face);
  DEFAIRE_dart_tab(Tab_face);

  free(self);
}
void rag_remove_noise(rag self,contour_set contours)
{
  int i,nb_eff=0; /* debug */
  dart f,fp;
  double dist,f_area;
  
  for(i=0;i<Nb_face;i++)
    {
      f = dart_tab_give_value(Tab_face,i);
      if(!tab_mark_dart_is_marked(Initial_face,f))
	continue;
      
      f_area = contour_set_give_face_area(contours,f);
      while(f_area < NOISE_SIZE)
	{
	  nb_eff++;
	  dist = give_min_dist_priv(self,contours,f,&fp);
	  f = rag_merge_face(self,contours,f,fp);
	  f_area = contour_set_give_face_area(contours,f);
	}
    }
  Nb_face_remains -= nb_eff;
  printf("Nb de petites faces effaces : %d\n",nb_eff);
}      
void rag_give_faces(rag self,
		    contour_set contours,
		    dart_list face_list)
{
  int i;
  dart f;

  for(i=0;i<Nb_face;i++)
    {
      f = dart_tab_give_value(Tab_face,i);
      if(!tab_mark_dart_is_marked(Initial_face,f))
	continue;
      dart_list_insert(face_list,f);
    }
}
int    rag_give_nb_current_faces (rag self)
{
  return Nb_face_remains;
}
void rag_find_closest_node(rag self,contour_set contours,dart* f1,dart* f2)
{
  double min_se;
  double sqe;
  int i;
  dart f,fp;
  
  min_se = DBL_MAX; 
  
  for(i=0;i<Nb_face;i++) 
    {
      f =  dart_tab_give_value(Tab_face,i);
      
      if(!tab_mark_dart_is_marked(Initial_face,f))
	continue;
      
      sqe = give_min_dist_priv(self,contours,f,&fp);
      if(sqe < min_se)
	{
	  min_se = sqe;
	  *f1 = f;
	  *f2 = fp;
	}
    }
  if(min_se == DBL_MAX)
    {
      *f1 = UNKNOWN_FACE;
      *f2 = UNKNOWN_FACE;
    }
}
dart rag_merge_face(rag self,contour_set contours,dart f1,dart f2)
{
  dart f_merged,removed_face;

  f_merged = merge_face(contours,f1,f2);  
  removed_face = (f_merged==f1?f2:f1);

  tab_mark_dart_unmark_dart(Initial_face,removed_face);
  tab_mark_dart_mark_dart(Allowed_merge,f_merged);
  Nb_face_remains--;

  return f_merged;
}
double rag_give_sqe(rag self,contour_set contours,dart f)
{
  if(f == UNKNOWN_FACE)
    return DBL_MAX;

  return contour_set_give_face_squared_error(contours,f);
}
double rag_give_global_sqe(rag self)
{
  return Global_sqe;
}
  /*********************************************/
 /*              Fonctions prives             */
/*********************************************/

/* Calcule la distance entre la face de brin canonique b et toute les */
/* faces voisines. La fonction de distance etant face_dist_priv.*/
static double give_min_dist_priv(rag self,
				 contour_set contours,
				 dart f,
				 dart *fp)
{
  dart f_min;
  double sqe,min_sqe;
  dart_list f_list,neighbour_list,included;
  dart neighbour;
  
  f_list         = FAIRE_dart_list();
  neighbour_list = FAIRE_dart_list();
  included       = FAIRE_dart_list();
  dart_list_insert(f_list,f);
  
  contour_set_complete_face_adjacence(contours,f_list,neighbour_list);
  contour_set_give_included_faces(contours,f,included);
  contour_set_complete_face_adjacence(contours,included,neighbour_list);

  DEFAIRE_dart_list(f_list);
  DEFAIRE_dart_list(included);

  assert(!dart_list_is_empty(neighbour_list));

  min_sqe = DBL_MAX;
  dart_list_first(neighbour_list);
  do
    {
      neighbour = dart_list_give_current(neighbour_list);

      if(!tab_mark_dart_is_marked(Allowed_merge,neighbour)&&
	 !tab_mark_dart_is_marked(Initial_face,neighbour))
	continue;

      sqe = face_dist_priv(contours,f,neighbour);
      if(sqe<min_sqe)
	{
	  f_min = neighbour;
	  min_sqe = sqe;
	}
    }
  while(dart_list_next(neighbour_list));
  
  *fp  = f_min;
  DEFAIRE_dart_list(neighbour_list);
  
  return  min_sqe;
}


static double face_dist_priv(contour_set contours,dart f1,dart f2)
{
  vector f1_mean,f2_mean;
  double f1_area,f2_area;
  double f1_se,f2_se;
  double alpha,dist;
  double dist1,dist2,dist12;

  f1_area = contour_set_give_face_area(contours,f1);
  f2_area = contour_set_give_face_area(contours,f2);

  f1_se = contour_set_give_face_squared_error(contours,f1);
  f2_se = contour_set_give_face_squared_error(contours,f2);

  contour_set_give_face_mean(contours,f1,f1_mean);
  contour_set_give_face_mean(contours,f2,f2_mean);
  
  alpha  = f1_area*f2_area/(f1_area +f2_area);
  dist12 = alpha*vector_dist(f1_mean,f2_mean,DIM_MAX);
  
  dist1 = f2_se + dist12;
  dist2 = f1_se + dist12;
  
  dist = MIN(dist1,dist2);
  
  return dist;
}
  


