#include <malloc.h>
#include <assert.h>
#include "seg_tmp.h"
#include "tab_move.h"
#include "point.h"
#include "objet.h"
#include "garbage.h"

#define Seg1              SELF (seg1)
#define Seg2              SELF (seg2)
#define Ind1              SELF (ind1)
#define Ind2              SELF (ind2)
#define Ind               SELF (ind)
#define Start_seg1        SELF (start_seg1)
#define Part_one          SELF (part_one)
#define First_point       SELF (first_point)
#define Last_point        SELF (last_point)
#define Right_point       SELF (right_point)
#define Current_point     SELF (current_point)
#define Clock_sense       SELF (clock_sense)
#define Longueur          SELF (longueur)
#define Dart_nb           SELF (dart_nb)


/* Fonctions utilisees pour le garbage collecting */
static seg_tmp init_seg_tmp_priv();
static void    free_seg_tmp_priv(seg_tmp);
static void    dealloc_seg_tmp_priv(void*);
/* Fonctions internes */
static void    update_seg_tmp_priv(seg_tmp,seg_tmp);


struct seg_tmp
{
  tab_move seg1;
  int ind1;
  int start_seg1;
  tab_move seg2;
  int ind2;
  int ind;
  bouleen part_one;
  Point      first_point;
  Point      last_point;
  Point      current_point;
  Point      right_point;
  int        longueur;
  long       clock_sense;
  dart       dart_nb;
};

static garbage seg_tmp_collector = NULL_GARBAGE;


void ALLOC_seg_tmp()
{
  seg_tmp_collector = FAIRE_garbage();
}
void DEALLOC_seg_tmp()
{
  DEFAIRE_garbage(seg_tmp_collector,dealloc_seg_tmp_priv);
}
seg_tmp FAIRE_seg_tmp()
{
  Point p;
  seg_tmp self;

  COORDX(p) = -1;
  COORDY(p) = -1;

  self =  init_seg_tmp_priv();

  Ind1=Ind2=0;
  AFFECTE_POINT(Current_point,p);
  AFFECTE_POINT(Right_point,p);
  Longueur    =0;
  Clock_sense = 0;
  Start_seg1  = 0;

  return self;
}
void DEFAIRE_seg_tmp(seg_tmp self)
{
  assert(self);
  free_seg_tmp_priv(self);
}
void free_seg_tmp(void* obj)
{
  seg_tmp self = obj;

  assert(self);
  free_seg_tmp_priv(self);
}
void seg_tmp_reinitialize(seg_tmp self)
{
  Point p;
  
  assert(self);

  COORDX(p) = -1;
  COORDY(p) = -1;


  Ind1=Ind2=0;
  AFFECTE_POINT(Current_point,p);
  AFFECTE_POINT(Right_point,p);
  Longueur =0;
  Clock_sense = 0;
  Start_seg1 = 0;
}
/****************************************************************************/
              /* Partie reserve aux modules de segmentation */
/****************************************************************************/

/****************************************************************************/
/* Stocke le premier segment, current est le point ou l'on est move la direction ou l'on va */
/****************************************************************************/
void seg_tmp_insert_segment1(seg_tmp self,
			     point current,
			     move_type move)
{
  move_type prev_move;
  Point p;

  Part_one = vrai;

  if(Ind1==Start_seg1)
    {
      /* Utile si on a lance la segmentation a partir d'un segment. Dans */
      /* ce cas la on ne lancera pas d'insert sur segment2 */
      AFFECTE_POINT(First_point,*current);
    }
  

  AFFECTE_POINT(p,*current);
  modify_point_position(&p,move);
  
  
  if(IS_RIGHTER_POINT(p,Right_point))
    {
      AFFECTE_POINT(Right_point,p);
    }

  AFFECTE_POINT(Last_point,p);

  if(Ind1 !=Start_seg1)
    {
      prev_move = tab_move_give_value(Seg1,Ind1-1);
      Clock_sense += move_type_compute_step_clock(prev_move,move);
    }

  tab_move_put_value(Seg1,Ind1,move);
  Ind1++;

  Longueur++;
}  
/**************************************************/
void seg_tmp_insert_segment2(seg_tmp self,
			     point current,
			     move_type move)
{ 
  move_type prev_move; 
  Point p;
  
  Part_one = faux;
  
  AFFECTE_POINT(p,*current);
  modify_point_position(&p,move);
  
  
  if(IS_RIGHTER_POINT(p,Right_point))
    {
      AFFECTE_POINT(Right_point,p);
    }

  AFFECTE_POINT(First_point,p);

  move = REVERSE_SENSE(move);

  if(Ind2 !=0)
    {
      prev_move = tab_move_give_value(Seg2,Ind2-1);
      Clock_sense += move_type_compute_step_clock(move,prev_move);
    }
  tab_move_put_value(Seg2,Ind2,move);
  Ind2++;

  Longueur++;
}   
  /********************************************/
 /* Enleve un segment d'un tableau de marque */
/********************************************/
void seg_tmp_unmark_seg(seg_tmp self,
			void (*unmark_point)(tab_mark,
					     point),
			tab_mark marked_point)
{
  Point p;
  
  seg_tmp_first(self);
  
  do
    {
      seg_tmp_give_current_point(self,&p);
      unmark_point(marked_point,&p);
    }
  while(seg_tmp_next(self));

  seg_tmp_give_current_point(self,&p);
  unmark_point(marked_point,&p);
  
}
  /****************************************************************/
 /* Copie la boucle d'un segment dans un autre segment.          */
/****************************************************************/
seg_tmp seg_tmp_copy_closed_curve(seg_tmp self,
				  image img)
{
  int dep_x = 0;
  int dep_y = 0;
  seg_tmp seg2;
  Point p;
  move_type move;
  bouleen (*dep_in_seg)(seg_tmp);

  assert(self);
  assert(!seg_tmp_is_closed(self));
  
  seg2 = FAIRE_seg_tmp();
  
  
    if(Ind2 ==0)
    {
      seg_tmp_last(self);
      dep_in_seg = seg_tmp_previous;
    }
  else
    {
      seg_tmp_first(self);
      dep_in_seg = seg_tmp_next;
    }
  

  do
    {
      seg_tmp_give_current_point(self,&p);
      move = seg_tmp_give_current_move(self);

      if(Ind2 ==0)
	move = REVERSE_SENSE(move);
      
      seg_tmp_insert_segment1(seg2,&p,move);

      dep_x += dep_move_x[move];
      dep_y += dep_move_y[move];    
      
      if( (dep_x ==0)&&(dep_y ==0) )
	break;

      assert(dep_in_seg(self));
    }
  while(1);
  
  update_seg_tmp_priv(self,seg2);
  
  return seg2;
}
/*****************************************************************************/
         
        /*********************************************************/
       /* Partie reserve aux algo demandant des infos a seg_tmp */
      /*********************************************************/

/*****************************************************************************/
void seg_tmp_give_first_point(seg_tmp self,point p)
{
  assert(self);

  AFFECTE_POINT(*p,First_point);
}
/*****************************************************************************/
void seg_tmp_give_last_point(seg_tmp self,point p)
{
  assert(self);

  AFFECTE_POINT(*p,Last_point);
}
/*****************************************************************************/
void seg_tmp_give_right_point(seg_tmp self,point p)
{
  assert(self);

  AFFECTE_POINT(*p,Right_point);
}
/*****************************************************************************/
move_type seg_tmp_give_first_move(seg_tmp self)
{
  assert(self);

  if(Ind2 ==0)
    return tab_move_give_value(Seg1,Start_seg1);


  return tab_move_give_value(Seg2,Ind2-1);
}
/*****************************************************************************/
move_type seg_tmp_give_last_move(seg_tmp self)
{
  assert(self);

  return tab_move_give_value(Seg1,Ind1-1);
}
/*****************************************************************************/
int seg_tmp_give_longueur(seg_tmp self)
{
  assert(self);

  return Longueur;
}
/*****************************************************************************/
long seg_tmp_give_clock(seg_tmp self)
{
  move_type in_node,out_node;
  long  increment = 0;

  assert(self);
  
  if(Ind2 !=0)
    {
      in_node = tab_move_give_value(Seg2,0);
      out_node = tab_move_give_value(Seg1,Start_seg1);
      increment = move_type_compute_step_clock(in_node,out_node);
    }
  if(EGAL_POINT(First_point,Last_point))
    {/* Le segment forme une composante conexe */
      in_node  = seg_tmp_give_last_move(self);
      out_node = seg_tmp_give_first_move(self);
      increment += move_type_compute_step_clock(in_node,out_node);
    }
  return Clock_sense+increment;
}
/*****************************************************************************/
void seg_tmp_first(seg_tmp self)
{
  assert(self);
  
  AFFECTE_POINT(Current_point,First_point);
  if(Ind2 == 0)
    {
      Ind = Start_seg1;
      Part_one = vrai;
    }
  else
    {
      Ind = Ind2-1;
      Part_one = faux;
    }
}
/*****************************************************************************/
/* Current point : point ou l'on est .*/
/* Ind : Indice du move suivant*/
/*****************************************************************************/
bouleen seg_tmp_next(seg_tmp self)
{
  move_type move;
  tab_move part_seg;

  assert(self);

  if(Part_one&&(Ind == Ind1))
    return faux;

  part_seg = (Part_one?Seg1:Seg2);
/* On bouge le point */
  move = tab_move_give_value(part_seg,Ind);
  modify_point_position(&Current_point,move);
/* On cherche le futur deplacement */
  if(Part_one)
    Ind++;
  else
    {
      if(Ind == 0)
	{
	  Part_one = vrai;
	  Ind = Start_seg1;
	}
      else
	Ind--;
    }

  
  return vrai;
}
/*****************************************************************************/
void seg_tmp_last(seg_tmp self)
{
  assert(self);

  AFFECTE_POINT(Current_point,Last_point);
  Part_one = vrai;
  Ind = Ind1-1;
}
/*****************************************************************************/
/* Current point : point ou l'on est .*/
/* Ind : Indice du move suivant*/
/*****************************************************************************/
bouleen seg_tmp_previous(seg_tmp self)
{
  move_type move;
  tab_move part_seg;
  int prev_ind;

  assert(self);

  prev_ind = Ind;

  if(Part_one)
    {
      if(Ind == Start_seg1)
	{
	  Part_one = faux;
	  Ind =0;
	}
      else
	Ind--;
      part_seg = Seg1;
    }
  else
    {
      if(Ind == Ind2)
	return faux;
      Ind++;
      part_seg = Seg2;
    }

  move = tab_move_give_value(part_seg,prev_ind);
  move = REVERSE_SENSE(move);
  modify_point_position(&Current_point,move);

  return vrai;
}
/*****************************************************************************/
move_type  seg_tmp_give_current_move(seg_tmp self)
{
  tab_move part_seg;

  assert(self);  

  part_seg = (Part_one?Seg1:Seg2);

  if( Part_one &&(Ind == Ind1))
    return INFINITE;

  if(!Part_one && (Ind == Ind2))
    return INFINITE;

  return tab_move_give_value(part_seg,Ind);
}
/*****************************************************************************/
void seg_tmp_give_current_point(seg_tmp self,point p)
{
  assert(self);  

  AFFECTE_POINT(*p,Current_point);
}
/*****************************************************************************/
bouleen seg_tmp_is_closed(seg_tmp self)
{
  assert(self);  

  return EGAL_POINT(First_point,Last_point);
}
/*****************************************************************************/
bouleen seg_tmp_is_empty(seg_tmp self)
{
  assert(self);
  
  return Longueur == 0;
}
dart seg_tmp_give_dart_nb(seg_tmp self)
{
  assert(self);
  
  return Dart_nb;
}
void seg_tmp_set_dart_nb(seg_tmp self,dart nb)
{
  assert(self);
  
  Dart_nb = nb;
}
    /***********************************************************/
   /* Initialize ou reinitialize un segment.                  */
  /* Notez que l'on ne se pose pas la question de savoir     */
 /* si un segment est utilise ou pas.                       */
/***********************************************************/
static seg_tmp init_seg_tmp_priv()
{  
  seg_tmp self;

  if(seg_tmp_collector == NULL_GARBAGE)
    {
      self = (seg_tmp)malloc(sizeof(struct seg_tmp));
      Seg1 = FAIRE_tab_move();
      Seg2 = FAIRE_tab_move();
      return self;
    }
  if(garbage_is_empty(seg_tmp_collector))
    {
      self = (seg_tmp)malloc(sizeof(struct seg_tmp));
      Seg1 = FAIRE_tab_move();
      Seg2 = FAIRE_tab_move();
      return self;
    }

  return (seg_tmp)garbage_give_object(seg_tmp_collector);
}  
static void free_seg_tmp_priv(seg_tmp self)
{
  assert(self);

  if(seg_tmp_collector == NULL_GARBAGE)
    {
      dealloc_seg_tmp_priv(self);
      return;
    }

  garbage_insert_object(seg_tmp_collector,self);
}
static void dealloc_seg_tmp_priv(void* seg)
{
  seg_tmp self = (seg_tmp)seg;

  DEFAIRE_tab_move(Seg1);
  DEFAIRE_tab_move(Seg2);
  free(self);
}
    /***********************************************************/
   /* Met a jour un segment apres la supression d'une boucle  */
  /***********************************************************/    
static void update_seg_tmp_priv(seg_tmp self,seg_tmp seg2)
{
  move_type move_self,move_seg2;
  angle link;
  bouleen same_orientation;

/* link contient l'angle entre le segment et la composante connexe qui */
/* doit etre retranche du calcul de coubure du segment */

  same_orientation = (Ind2==0);


  if(Ind2 ==0)
    Ind1 = Ind;
  else
    {
      Ind2 = Ind;
      if(Part_one)
	{	  
	  Start_seg1 = Ind+1;
	  Ind2 =0;
	}
    }
  /* On s'occupe du clock */
  
  if(same_orientation)
    {
      Clock_sense += seg2->clock_sense;
      move_self = seg_tmp_give_last_move(self);
      move_seg2 = REVERSE_SENSE( seg_tmp_give_last_move(seg2) );
      link = move_type_compute_step_clock(move_self,move_seg2);
      Clock_sense -= link;
    }
  else
    {
      Clock_sense -= seg2->clock_sense;
      move_self = REVERSE_SENSE( seg_tmp_give_first_move(self) );
      move_seg2 = REVERSE_SENSE( seg_tmp_give_last_move(seg2)  );
      link = move_type_compute_step_clock(move_self,move_seg2);
      Clock_sense += link;
      if(Part_one)
	{
	  /* On recupere l'angle entre S2(0) et S1(0) */
	  move_self = tab_move_give_value(Seg2,0);
	  move_seg2 = tab_move_give_value(Seg1,0);
	  link = move_type_compute_step_clock(move_self,move_seg2);
	  Clock_sense += link;
	}
    }
  Longueur -= seg2->longueur;
  
  if( !IS_RIGHTER_POINT(Right_point,seg2->right_point) )
    {

      /* On reparcourt le segment pour trouver le point le plus a droite */
      AFFECTE_POINT(Right_point,Last_point);
      seg_tmp_first(self);
      do
	{
	  if(IS_RIGHTER_POINT(Current_point,Right_point))
	    {
	      AFFECTE_POINT(Right_point,Current_point);
	    }
	}
      while(seg_tmp_next(self));
    }
}




