#include <assert.h>

#include "segment.h"
#include "point.h"
#include "bitmap.h"
#include "image.h"
#include "seg_tmp.h"

#include "objet.h"

#define First_point       SELF (first_point)
#define Last_point        SELF (last_point)
#define Right_point       SELF (right_point)
#define Current_point     SELF (current_point)
#define First_move        SELF (first_move)
#define Last_move         SELF (last_move)
#define Current_move      SELF (current_move)
#define Dart_nb           SELF (dart_nb)
#define ClockSense        SELF (clockSense)
#define Bitmap_seg        SELF (bitmap_seg)
#define Longueur          SELF (longueur)
#define Right_point_computed SELF(right_point_computed)

static segment cut_segment_priv(segment, point);
static  void   calc_right_priv(segment,point);

struct segment
{

  bitmap     bitmap_seg;
  Point      first_point;
  Point      last_point;
  move_type  first_move;
  move_type  last_move;
  move_type  current_move;
  Point      current_point;
  Point      right_point;
  bouleen    right_point_computed;
  int        longueur;
  dart       dart_nb;
  long       clockSense;
};


segment FAIRE_segment(bitmap bitmap_seg)
{
  INSTANCIER(segment);

  Right_point_computed = faux;
  Bitmap_seg = bitmap_seg;
  
  return self;
}

/*---------------------------------------------------------------------------*/

void DEFAIRE_segment(segment self)
{
  assert(self);
  free(self);
}

/*---------------------------------------------------------------------------*/
void segment_init_from_seg_tmp(segment self,seg_tmp init)
{
  assert(self);

  seg_tmp_give_first_point(init,&First_point);
  seg_tmp_give_last_point(init,&Last_point);
  seg_tmp_give_right_point(init,&Right_point);
  Right_point_computed = vrai;
  ClockSense = seg_tmp_give_clock(init);  
  First_move = seg_tmp_give_first_move(init);
  Last_move  = seg_tmp_give_last_move(init);
  Longueur   = seg_tmp_give_longueur(init);
}
void segment_init_output_segment(segment self, output_segment ext_seg)
{
  assert(self);
  assert(ext_seg);
  
  output_segment_reinitialize(ext_seg);
  segment_first(self);
  do
    output_segment_insert(ext_seg,Current_move);
  while (segment_next(self));
  
  output_segment_set_first_point(ext_seg,&First_point);
  output_segment_set_last_point (ext_seg,&Last_point);
/*  output_segment_set_right_point(ext_seg,&Right_point);*/
  output_segment_set_lenght     (ext_seg,Longueur);
  output_segment_set_clock      (ext_seg,ClockSense);
  output_segment_set_dart_nb    (ext_seg,Dart_nb);
}
/*---------------------------------------------------------------------------*/
double segment_compute_gradiant(segment self,image img)
{
  double grad;
  
  assert(self);
  assert(img);
  
  grad = 0.0;
  segment_first(self);
  do
    {
      grad += image_distance(img,&Current_point,Current_move);
    }
  while(segment_next(self));
  
  return grad;
}
/*---------------------------------------------------------------------------*/
void  segment_first(segment self)
{
  assert(self);
  
  AFFECTE_POINT(Current_point,First_point);
  Current_move  = First_move;
  
}

/*---------------------------------------------------------------------------*/

void segment_last(segment self)
{
  assert(self);
  
  AFFECTE_POINT(Current_point,Last_point);
  Current_move  = Last_move;
  
}
/*---------------------------------------------------------------------------*/

bouleen segment_is_first(segment self,point p)
{
  assert(self);

  if(p==NULL)
    return faux;

  return EGAL_POINT(*p,First_point); 
    
}

/*---------------------------------------------------------------------------*/

bouleen segment_is_last(segment self, point p)
{
  assert(self);

  if(p == NULL)
    return faux;
  
  return EGAL_POINT(*p,Last_point); 
}
/*---------------------------------------------------------------------------*/
bouleen segment_is_closed(segment self)
{
  assert(self);

  return EGAL_POINT(First_point,Last_point);
}
/*---------------------------------------------------------------------------*/

void segment_give_current_point(segment self,point p)
{
  assert(self);

  if(p != NULL)
    {
      AFFECTE_POINT(*p,Current_point);
    }
}
/*---------------------------------------------------------------------------*/
move_type segment_give_current_move(segment self)
{
  assert(self);
  return Current_move;
}

/*---------------------------------------------------------------------------*/

bouleen segment_next(segment self)
{
  angle     mask_value;

  assert(self);

  modify_point_position(&Current_point,Current_move);

  if( EGAL_POINT(Current_point,Last_point) )
    return faux;


  mask_value = bitmap_move_to(Bitmap_seg,&Current_point);
  Current_move = bitmap_give_direction(Bitmap_seg,
				       mask_value,
				       Current_move); 

  assert(Current_move != INFINITE);
  return vrai;
}

/*---------------------------------------------------------------------------*/
    
bouleen  segment_previous(segment self)
{
  angle mask_value;
  move_type inv_Current_move,move;

  assert(self);

  
  move = REVERSE_SENSE(Current_move);
  modify_point_position(&Current_point,move);

  if( EGAL_POINT(Current_point,First_point) )
    return faux;

  mask_value = bitmap_move_to(Bitmap_seg,&Current_point);
  inv_Current_move = bitmap_give_direction(Bitmap_seg,
						      mask_value,
						      move);

  Current_move = REVERSE_SENSE(inv_Current_move);
  
  return vrai; 
}

/*---------------------------------------------------------------------------*/

segment  segment_split(segment self,point cut_point, dart newdart)
{
  segment  return_val;
  
  assert(self);
  assert(cut_point);


  if (EGAL_POINT(First_point,*cut_point) || EGAL_POINT(Last_point,*cut_point))
    return  self;

  /* Le point de coupure existe est n'est egal ni au premier */
  /* ni au dernier point */
  return_val = cut_segment_priv(self,cut_point);
  segment_set_dart_nb(return_val,newdart);
  
  return return_val; 
}

/*---------------------------------------------------------------------------*/

bouleen segment_is_in(segment self,point cut_point)
{
  Point p;
  move_type move;
  angle mask;

  assert(self);

  if(cut_point == NULL)
    return faux;

  AFFECTE_POINT(p,First_point);
  move = First_move;
  mask = bitmap_move_to(Bitmap_seg,&p);
  
  while ( !( EGAL_POINT(p,*cut_point)||EGAL_POINT(p,Last_point) ) )
    {
      modify_point_position(&p,move);
      mask = bitmap_move_in_bitmap(Bitmap_seg,move);
      move = bitmap_give_direction(Bitmap_seg,mask,move); 
    }
  
  if (EGAL_POINT(p,*cut_point))
    return vrai;
  
  return faux; 
}

/*---------------------------------------------------------------------------*/

void segment_get_first(segment self,point p)
{
  assert(self);

  if(p != NULL)
    {
      AFFECTE_POINT(*p,First_point);
    }
}
/*---------------------------------------------------------------------------*/

move_type segment_get_first_move(segment self)
{
  assert(self);
  return First_move;
}
/*---------------------------------------------------------------------------*/

void  segment_get_last(segment self,point p)
{
  assert(self);
  AFFECTE_POINT(*p,Last_point);
}

/*---------------------------------------------------------------------------*/

move_type segment_get_last_move(segment self)
{
  assert(self);
  return Last_move;
}

/*---------------------------------------------------------------------------*/

int  segment_get_longueur(segment self)
{
  assert(self);
  return Longueur;
}
/*---------------------------------------------------------------------------*/

int segment_give_clock(segment self)
{
  assert(self);
  return ClockSense;
}

/*---------------------------------------------------------------------------*/
void segment_concat(segment self,segment seg)
{
  Point     p1,
            p2,
            p;

  move_type move;
  
  long      ecart;

  assert(self);


  segment_get_first(seg,&p1);
  
  assert(EGAL_POINT(Last_point,p1));

  
/* Mise a jour de point le plus a droite */
  Right_point_computed = Right_point_computed&&(seg->right_point_computed);

  if(Right_point_computed)
    {
      segment_give_right_point(seg,&p2);
      if( IS_RIGHTER_POINT(p2, Right_point) )
	{
	  AFFECTE_POINT(Right_point,p2);
	}
    }
  Longueur += segment_get_longueur(seg); /*mise a jour de longueur */

/* Etude du cas ou la concatenation crait un segment ferme */

  /* mise a jour de clocksense */
  ClockSense += segment_give_clock(seg);
  move = segment_get_first_move(seg);
  ecart = move_type_compute_step_clock(Last_move,move);
  ClockSense += ecart;

  segment_get_last(seg,&p);
  if ( EGAL_POINT(p,First_point) )
    {
      move = segment_get_last_move(seg);
      ecart = move_type_compute_step_clock(move,First_move);
      ClockSense += ecart;
    }

  segment_get_last(seg,&Last_point);        /* mise a jour du last point */
  Last_move =  segment_get_last_move(seg); /* mise a jour du last move */
}
/*---------------------------------------------------------------------------*/
void  segment_reverse(segment self)
{

  Point     tmp_pt;
  move_type tmp_move;

  assert(self);
  
  AFFECTE_POINT(tmp_pt,Last_point);
  AFFECTE_POINT(Last_point,First_point);
  AFFECTE_POINT(First_point,tmp_pt);
  
  tmp_move = REVERSE_SENSE(Last_move);
  Last_move = REVERSE_SENSE(First_move);
  First_move = tmp_move;
  
  ClockSense = -ClockSense; 
}

/*---------------------------------------------------------------------------*/

void segment_set_dart_nb(segment self,dart b)
{
  assert(self);

  Dart_nb = b > 0 ? b : -b;
}

/*---------------------------------------------------------------------------*/

dart segment_give_dart_nb(segment self)
{
  assert(self);

  return Dart_nb;
}

/*---------------------------------------------------------------------------*/

void segment_give_right_point(segment self,point p)
{
  assert(self);
  assert(p);

  if(!Right_point_computed)
    {
      calc_right_priv(self,p);
      return;
    }
  
  AFFECTE_POINT(*p,Right_point);
}

void segment_set_first_point(segment self,point first_point)
{
  assert(self);

  AFFECTE_POINT(First_point,*first_point);
}

void segment_set_last_point(segment self, point last_point)
{
  assert(self);

  AFFECTE_POINT(Last_point,*last_point);
}
void segment_set_first_move(segment self,move_type first_move)
{
  assert(self);
  First_move = first_move;
}
void segment_set_last_move(segment self,move_type last_move)
{
  assert(self);
  Last_move = last_move;
}
void segment_set_right_point(segment self, point right_point)
{
  assert(self);

  Right_point_computed = vrai;
  AFFECTE_POINT(Right_point,*right_point);
}

void segment_set_clockSense(segment self, long clockSense)
{
  assert(self);
  ClockSense = clockSense;
}

void segment_set_longueur(segment self, int longueur)
{
  assert(self);
  Longueur = longueur;
}
  /********************************************************/
 /* Marque le segment dans le tableau passe en parametre */
/********************************************************/

void segment_mark_segment(segment self,
			  void (*mark_point)(tab_mark,
					     point),
			  tab_mark mark_seg)
{
  Point p,saved_current_point;
  move_type saved_current_move;
  
  AFFECTE_POINT(saved_current_point,Current_point);
  saved_current_move = Current_move;
  
  segment_first(self);
  do
    {
      segment_give_current_point(self,&p);
      mark_point(mark_seg,&p);
    }
  while( segment_next(self) );
  mark_point(mark_seg,&Last_point);
  
  AFFECTE_POINT(Current_point,saved_current_point);
  Current_move = saved_current_move;
}

  /**********************************************************/
 /* Demarque le segment dans le tableau passe en parametre */
/**********************************************************/

void segment_unmark_segment(segment self,
			    void (*unmark_point)(tab_mark,
						 point),
			    tab_mark mark_seg)
{
  Point p,saved_current_point;
  move_type saved_current_move;
  
  AFFECTE_POINT(saved_current_point,Current_point);
  saved_current_move = Current_move;
  
  segment_first(self);
  do
    {
      segment_give_current_point(self,&p);
      unmark_point(mark_seg,&p);
    }
  while( segment_next(self) );
  unmark_point(mark_seg,&Last_point);
  
  AFFECTE_POINT(Current_point,saved_current_point);
  Current_move = saved_current_move;
}

static segment cut_segment_priv(segment self, point cut_point)
{
  segment newseg = FAIRE_segment(Bitmap_seg);
  Point      tmp_point;
  
  long       new_clock_sense = 0;
  long       close_clock     = 0;
  long       cut_clock      = 0;
  long       tmp_clock;
  Point      old_right_point,new_right_point;
  move_type  prec_move, real_move;
  angle      mask;
  int longueur=0;
  
  assert(self);
  
  if(EGAL_POINT(First_point,Last_point))
    close_clock =  move_type_compute_step_clock(Last_move,First_move);
  
    AFFECTE_POINT(tmp_point,First_point);
    AFFECTE_POINT(new_right_point,First_point);
    AFFECTE_POINT(old_right_point,Right_point);
    mask            = bitmap_move_to(Bitmap_seg,&First_point);
    real_move       = First_move;
    longueur = 1;
    
    do
      {
	prec_move = real_move;
	modify_point_position(&tmp_point,real_move);
	mask      = bitmap_move_in_bitmap(Bitmap_seg,real_move);
	real_move = bitmap_give_direction(Bitmap_seg,mask,real_move); 
	
	if (EGAL_POINT(*cut_point,tmp_point))
	  break;
	
	/* On calcule les parametres du segment : First-point -> cut_point*/
	new_clock_sense += move_type_compute_step_clock(prec_move,real_move);
	longueur++;
	
	if( IS_RIGHTER_POINT(tmp_point,new_right_point) )
	  {
	    AFFECTE_POINT(new_right_point,tmp_point);
	  }
	
      }
    while ( !EGAL_POINT(Last_point,tmp_point) );    
    
    assert(EGAL_POINT(*cut_point,tmp_point));
    
    cut_clock        = move_type_compute_step_clock(prec_move,real_move);
    
    /* On met a jour les deux bouts de segment */
    tmp_clock = ClockSense -(close_clock + 
			     new_clock_sense + 
			     cut_clock); 
    
    segment_set_first_point(newseg,cut_point);
    segment_set_last_point(newseg,&Last_point);
    segment_set_first_move(newseg,real_move);
    segment_set_last_move(newseg,Last_move);
    
    segment_set_last_point(self,cut_point);
    segment_set_last_move(self,prec_move);
    
    segment_set_clockSense(self,new_clock_sense);
    segment_set_clockSense(newseg,tmp_clock);
    
    segment_set_longueur(newseg,Longueur-longueur);
    segment_set_longueur(self,longueur);
    
    
    if(!Right_point_computed)
      {
	segment_set_right_point(self,&new_right_point);    
	newseg->right_point_computed = faux;
	return newseg;
      }
    
    segment_set_right_point(self,&new_right_point);    
    if( COORDX(new_right_point)== COORDX(old_right_point) )
      newseg->right_point_computed = faux;
    else
      segment_set_right_point(newseg,&old_right_point);                      
    
    
    return newseg;
}

/*---------------------------------------------------------------------------*/

void calc_right_priv(segment self, point retour)
{
  Point     p;
  move_type move;
  angle     mask;

  assert(self);
  
  Right_point_computed = vrai;
  
  AFFECTE_POINT(Right_point,Last_point);
  AFFECTE_POINT(p,First_point);
  bitmap_move_to(Bitmap_seg,&p);
  move = First_move;
  do
    {
      if( IS_RIGHTER_POINT(p,Right_point) )
	{
	  AFFECTE_POINT(Right_point,p);
	}
      modify_point_position(&p,move);
      mask = bitmap_move_in_bitmap(Bitmap_seg,move);
      move = bitmap_give_direction(Bitmap_seg,mask,move); 
    }
  while ( !EGAL_POINT(p,Last_point) );

  AFFECTE_POINT(*retour,Right_point); 
}




