#include <stdio.h>
#include <assert.h>

#include "tab_euclid.h"
#include "point.h"
#include "type_obj.h"
#include "objet.h"

#define Begin_x        SELF (begin_x)
#define Begin_y        SELF (begin_y)
#define DimX           SELF (dimX)
#define DimY           SELF (dimY)
#define X_max          SELF (x_max)
#define Y_max          SELF (y_max)
#define Current_x      SELF (current_x)
#define Current_y      SELF (current_y)
#define Size           SELF (size)

typedef unsigned char xy_value;

/****************************************************************************/
struct tab_euclid 
{
  xy_value    *begin_x;
  xy_value    *begin_y;
  xy_value    *current_x;
  xy_value    *current_y;
  long   dimX;
  long   dimY;
  long   x_max;
  long   y_max;
  long   size;
};

tab_euclid FAIRE_tab_euclid(long dimx,long dimy)
{
  INSTANCIER(tab_euclid);
  
  /* On ajoute 1 a dimx et dimy pour pouvoir eventuellement travailler */
  /* dans P1/2 */
  DimX  = dimx+1;
  DimY  = dimy+1;
  
  X_max = dimx;
  Y_max = dimy;
  
  
  Size  = DimX * DimY;
  
  Current_x = Begin_x  = (xy_value *) calloc(Size,1);
  Current_y = Begin_y  = (xy_value *) calloc(Size,1);

  return self;
}



/*---------------------------------------------------------------------------*/

void DEFAIRE_tab_euclid(tab_euclid self)
{
  free (Begin_x);
  free (Begin_y);
  free (self);
}
/*---------------------------------------------------------------------------*/

  /*************************/
 /* Remet les valeurs a 0 */
/*************************/
void tab_euclid_reinitialize(tab_euclid self)
{
  xy_value *pt1,*pt2;
  long size;
  int i;

  size = DimX*DimY;

  pt1 = Begin_x;
  pt2 = Begin_y;
  
  for(i=0;i<size;i++)
    {
      *pt1 = 0;pt1++;
      *pt2 = 0;pt2++;
    }
}
/*---------------------------------------------------------------------------*/
int tab_euclid_give_hauteur(tab_euclid self)
{
  return (DimY-1);
}
/*---------------------------------------------------------------------------*/
int tab_euclid_give_largeur(tab_euclid self)
{
  return (DimX-1);
}

  /****************************/
 /* On se positionne point p */
/****************************/
bouleen  tab_euclid_move_to(tab_euclid self,point p)
{
  long dep;
  int i,j;

  assert(self);
  assert(p);
  
  i = COORDY(*p) + 1;
  j = COORDX(*p) + 1;
  
  assert((i>=0)&&(j>=0)); 
  assert((i<DimY)&&(j<DimX)); 

  dep = i*DimX+j;

  if(dep >= Size)
    return faux;

  Current_x = Begin_x + dep;
  Current_y = Begin_y + dep;

  return vrai;
}

  /****************************/
 /* On va au pixel de droite */
/****************************/
bouleen tab_euclid_move_right(tab_euclid self)
{
  int nb_pixel;

  assert(self);

  nb_pixel = (int)(Current_x-Begin_x);

  if((nb_pixel+1)%DimX == 0)
    return faux;

  Current_x++;
  Current_y++;

  return vrai;
}
  /****************************/
 /* On va au pixel de gauche */
/****************************/
bouleen tab_euclid_move_left(tab_euclid self)
{
  int nb_pixel;

  assert(self);

  nb_pixel = (int)(Current_x-Begin_x);

  if(nb_pixel%DimX == 0)
    return faux;

  Current_x--;
  Current_y--;

  return vrai;
}
  /****************************/
 /* On va au pixel du dessus */
/****************************/
bouleen tab_euclid_move_up(tab_euclid self)
{
  int nb_pixel;

  assert(self);

  nb_pixel = (int)(Current_x-Begin_x);

  if(nb_pixel <= DimX)
    return faux;

  Current_x -= DimX;
  Current_y -= DimX;

  return vrai;
}
  /*****************************/
 /* On va au pixel en dessous */
/*****************************/
bouleen tab_euclid_move_down(tab_euclid self)
{
  int nb_pixel;

  assert(self);

  nb_pixel = (int)(Current_x-Begin_x);

  if(Size - nb_pixel <= DimX)
    return faux;

  Current_x += DimX;
  Current_y += DimX;

  return vrai;
}
  /****************************************/
 /* On se deplace dans la direction move */
/****************************************/
bouleen tab_euclid_move_in_direction(tab_euclid self,move_type move)
{
  static bouleen (*move_tab[4])(tab_euclid) = 
    {
      tab_euclid_move_right,
      tab_euclid_move_up,
      tab_euclid_move_left,
      tab_euclid_move_down
    };
  return move_tab[move](self);
}

  /**********************************************************/
 /* Pixel suivant, ordre classique haut-bas, gauche-droite */
/**********************************************************/
bouleen tab_euclid_next(tab_euclid self)
{
  int nb_pixel;
  
  assert(self);

  nb_pixel = Current_x-Begin_x+1;
  if(nb_pixel == Size)
    return faux;

  Current_x++;
  Current_y++;

  return vrai;
}
  /************************************************************/
 /* Pixel precedent, ordre classique haut-bas, gauche-droite */
/************************************************************/
bouleen tab_euclid_previous(tab_euclid self)
{
  
  assert(self);

  if( Current_x == Begin_x)
    return faux;

  Current_x--;
  Current_y--;

  return vrai;
}

  /**************************************/
 /* On lit la valeur x dans le tableau */
/**************************************/
x_value tab_euclid_give_x(tab_euclid self)
{
  assert(self);
  return (*Current_x);
}
  /**************************************/
 /* On lit la valeur y dans le tableau */
/**************************************/
y_value tab_euclid_give_y(tab_euclid self)
{
  assert(self);
  return (*Current_y);
}
  /*********************************************/
 /* On positionne la valeur x dans le tableau */
/*********************************************/
void tab_euclid_put_x(tab_euclid self, x_value x)
{
  unsigned int update;

  assert(self);

  if(*Current_x == 0)
    {
      *Current_x = x;
	
      return;
    }

  update     = (unsigned int)*Current_x;
  update    += (unsigned int)x;

  *Current_x = (x_value)(update/2);
  
}
  /*********************************************/
 /* On positionne la valeur y dans le tableau */
/*********************************************/
void tab_euclid_put_y(tab_euclid self, y_value y)
{
  unsigned int update;
  assert(self);
  
  if(*Current_y == 0)
    {
      *Current_y = y;
      
      return;
    }      

  update     = (unsigned int)*Current_y;
  update    += (unsigned int)y;

  *Current_y = (y_value)(update/2);
}





