static const char* sccsid = "@(#) tab_move, Luc Brun, last revision 18/02/96";

#include <assert.h>
#include <stdlib.h>
#include "tab_move.h"
#include "garbage.h"

#include "objet.h"

#define Nb_line       SELF(nb_line)
#define Nb_of_pointer SELF(nb_of_pointer)
#define Element_tab   SELF(element_tab)
#define Cardinal      SELF(cardinal)

#define ALLOC_LINE 20
#define NB_MOVE_PER_BYTE 4
#define NB_MOVE_PER_LINE 80 /* NB_MOVE_PER_BYTE*ALLOC_LINE*/
#define ALLOC_NB_LINE 10

#define NUM_LINE(n)   ((n) / NB_MOVE_PER_LINE)
#define NUM_COLUMN(n) (((n) % NB_MOVE_PER_LINE)/NB_MOVE_PER_BYTE)
#define NUM_MASK(n)   (((n) % NB_MOVE_PER_LINE)%NB_MOVE_PER_BYTE)

typedef unsigned char four_move; 

static void       realloc_tab_move_priv(tab_move);
/* Fonctions liees au garbage */
static four_move* alloc_line_priv      ();
static tab_move   alloc_tab_move_priv  ();
static void       dealloc_line_priv      (four_move*);
static void       dealloc_tab_move_priv  (tab_move);
static void       free_tab_move_priv     (void*);
static void       init_line_priv         (four_move*);

static garbage line_collector     = NULL_GARBAGE;
static garbage tab_move_collector = NULL_GARBAGE;

static give_mask[NB_MOVE_PER_BYTE] = {0x03,0x0c,0x30,0xc0};
static init_mask[NB_MOVE_PER_BYTE] = {0xfc,0xf3,0xcf,0x3f};
static put_mask[NB_MOVE_PER_BYTE][NB_MOVE] = 
{
  {0x00,0x01,0x02,0x03},
  {0x00,0x04,0x08,0x0c},
  {0x00,0x10,0x20,0x30},
  {0x00,0x40,0x80,0xc0}
};

struct tab_move
{
  four_move **element_tab;
  int nb_line;
  int nb_of_pointer;
  int cardinal;
};

void ALLOC_tab_move()
{
  if(line_collector == NULL_GARBAGE)
    line_collector = FAIRE_garbage();
  if(tab_move_collector == NULL_GARBAGE)
    tab_move_collector = FAIRE_garbage();
}
void DEALLOC_tab_move()
{
  if(line_collector != NULL_GARBAGE)
    {
      DEFAIRE_garbage(line_collector,free);
      line_collector = NULL_GARBAGE;
    }
  if(tab_move_collector != NULL_GARBAGE)
    {
      DEFAIRE_garbage(tab_move_collector,free_tab_move_priv);
      tab_move_collector = NULL_GARBAGE;
    }
}
tab_move FAIRE_tab_move()
{
  tab_move self;
  
  self           = alloc_tab_move_priv();
  *Element_tab   = alloc_line_priv();
  Nb_line        = 1;
  Cardinal       = 0;

  return self;
}

/*---------------------------------------------------------------------------*/

void DEFAIRE_tab_move(tab_move self)
{  
  int i;

  assert(self);

  for(i=0;i<Nb_line;i++)
    dealloc_line_priv(Element_tab[i]);

  dealloc_tab_move_priv(self);
}
void tab_move_reinitialize(tab_move self)
{  
  int i;

  assert(self);
  
  for(i=1;i<Nb_line;i++)
    dealloc_line_priv(Element_tab[i]);
  
  init_line_priv(Element_tab[0]);
}
  
/*---------------------------------------------------------------------------*/
/* renvoie le cardinal du tableau */
/*---------------------------------------------------------------------------*/
int  tab_move_give_cardinal(tab_move self)
{
  return Cardinal;
}

/*---------------------------------------------------------------------------*/
/* renvoie la valeur contenue a l'indice ind */
/*---------------------------------------------------------------------------*/
move_type  tab_move_give_value(tab_move self,int ind)
{
  int line,column,mask;
  move_type move;

  assert(self);
  assert(ind >=0);

  line  =  NUM_LINE(ind);
  column = NUM_COLUMN(ind);
  mask   = NUM_MASK(ind); 
/* ligne non encore alloue */
  assert(line < Nb_line);
    
  /* On recupere le move */
  move = Element_tab[line][column]&give_mask[mask];
  /* On le ramene sur les 2 premiers bits de l'octet */
  move = move >> (mask<<1);
  
  return move;
}
/*---------------------------------------------------------------------------*/
/* met value a l'indice ind du tableau virtuel */
/*---------------------------------------------------------------------------*/
void tab_move_put_value(tab_move self,int ind,move_type value)
{
  int line,column,mask;
  four_move move;

  assert(self);

  Cardinal++;

  line   = NUM_LINE(ind);
  column = NUM_COLUMN(ind);
  mask   = NUM_MASK(ind);
  /* on ajoute autant de lignes que necessaire */
  while(line >= Nb_line)
    realloc_tab_move_priv(self);

  /* On positionne les bits qu'il faut dans l'octet */
  move = Element_tab[line][column] & init_mask[mask];
  move = move | put_mask[mask][value];

  Element_tab[line][column] = move;
}
      
/*---------------------------------------------------------------------------*/
static four_move* alloc_line_priv()
{
  four_move* line;

  if(line_collector == NULL_GARBAGE)
    {
      line = (four_move *) calloc(ALLOC_LINE,sizeof(four_move));
      assert(line);
      return line;
    }
  if(garbage_is_empty(line_collector))
    {
      line = (four_move *) calloc(ALLOC_LINE,sizeof(four_move));
      assert(line);
      return line;
    }
  
  line =(four_move*) garbage_give_object(line_collector);
  assert(line);
  init_line_priv(line);
  return line;
}
/*---------------------------------------------------------------------------*/
static tab_move alloc_tab_move_priv()
{
  tab_move self;
  
  if(tab_move_collector == NULL_GARBAGE)
    {
      self = (tab_move)malloc(sizeof(struct tab_move));
      Element_tab    = (four_move **) malloc(ALLOC_NB_LINE*sizeof(four_move*));
      Nb_of_pointer  = ALLOC_NB_LINE;
      
      return self;
    }
  if(garbage_is_empty(tab_move_collector))
    {
      self = (tab_move)malloc(sizeof(struct tab_move));
      Element_tab    = (four_move **) malloc(ALLOC_NB_LINE*sizeof(four_move*));
      Nb_of_pointer  = ALLOC_NB_LINE;

      return self;
    }
  self = garbage_give_object(tab_move_collector);
  return self;
}
/*---------------------------------------------------------------------------*/
static void dealloc_line_priv(four_move * line)
{
  if(line_collector == NULL_GARBAGE)
    {
      free(line);
      return;
    }
  garbage_insert_object(line_collector,line);
}
static void dealloc_tab_move_priv(tab_move self)
{
  if(tab_move_collector == NULL_GARBAGE)
    {
      free(self);
      return;
    }
  garbage_insert_object(tab_move_collector,self);
}
static void realloc_tab_move_priv(tab_move self)
{
  assert(self);

  /* On regarde si on na pas depasse le nb max de LINE alloue */
  if(Nb_line == Nb_of_pointer)
    {
      Element_tab = (four_move**)realloc(Element_tab,
					 (Nb_of_pointer+ALLOC_NB_LINE)*
					 sizeof(four_move*));
      assert(Element_tab);
      Nb_of_pointer += ALLOC_NB_LINE;
    }
  Element_tab[Nb_line] = alloc_line_priv();
  Nb_line++;
}
static void free_tab_move_priv(void* obj)
{
  tab_move self = obj;
  
  free(Element_tab);
  free(self);
}
  
static void init_line_priv(four_move* line)
{
  int i;
 
  for(i=0;i<ALLOC_LINE;i++)
    line[i] = 0;
}

