#include <glib.h>

#include "coul_freq.h"
#include "hash.h"
#include "conversion.h"
#include "macros.h"
#include "point.h"


/****************************************/
/* Type auxiliere */
typedef struct {
    int I1;
    int I2;
    int I3;
    long freq;
} f_coul;
/****************************************/

  /****************************************/
 /* Donnees utilisees pour le trie       */
/****************************************/
static f_coul* donnee;	
static int flag_trie;

#define TRIE_C1C2 0
#define TRIE_C2C1 1
#define TRIE_C3C1 2

   /*************************************************/
  /* Variables utilisees pour le parcourt de faces */
 /* Parametres pour initialiser la fonction       */
/*************************************************/

#define INITIALIZE_H_TABLE    1
#define INITIALIZE_CONVERSION 2
#define INITIALIZE_NB_OF_DATA 3
#define RETURN_NB_OF_DATA     4

                       /***************************/
                      /*Definition des fonctions */
                     /***************************/

   /**********************************************************/
  /* Fonctions utiles pour exploiter la structure de donnee.*/
 /* Appelees par coul_freq_projette                        */
/**********************************************************/
static double private_projette_I1(coul_freq,
				  int,
				  intervalle*,
				  intervalle*,
				  double*,
				  double*);
static double private_projette_I2(coul_freq,
				  int,
				  intervalle*,
				  intervalle*,
				  double*,
				  double*);
static double private_projette_I3(coul_freq,
				  int,
				  intervalle*,
				  intervalle*,
				  double*,
				  double*);

static int private_chercher(int,f_coul*,
			    int*,
			    int,
			    int,
			    int (*comp)());

     /**********************************************************/
    /* Fonctions utilisees pour creer la structure de donnee  */
   /* crait les tableaux d'entree permettant un acces direct */
  /* au couleurs dont une composante est superieure a       */
 /* une valeur k donnee                                    */
/**********************************************************/
static void private_creer_entre(coul_freq,
				int,
				int,
				int*,
				int*,
				int (*comp)(),
				int (*acce)(f_coul*,int));

static int private_acce_I1(f_coul*,int);
static int private_acce_I2(f_coul*,int);
static int private_acce_I3(f_coul*,int);

static int private_compare_a_val_I1(f_coul*,int,int,int  (*comp)());
static int private_compare_a_val_I2(f_coul*,int,int,int  (*comp)());
static int private_compare_a_val_I3(f_coul*,int,int,int  (*comp)());

  /****************************************************/
 /* Fonctions utilisees par les fonctions de hashage */
/****************************************************/

static int           private_egal(hash_object,hash_object);
static int           private_cle(hash_object);
static void          private_transfer(hash_object);
static hash_anything private_alloue(int);
static void          private_desalloue(hash_object);
   /**********************************************************/
  /* Fonction utilises par qsort dans private_trie.         */
 /* Utilisees pour trier les Tab_Ij dans l'ordre croissant */
/**********************************************************/
static int   private_compare_I1(const void *,const void* );
static int   private_compare_I2(const void *,const void* );
static int   private_compare_I3(const void *,const void* );

static void  private_trie(coul_freq,int);

/* Fonctions permetant d'initialiser la table de hashage a partir d'un */
/* face de l'image ou de l'image entiere */
static hash_table private_remplir_hash_table_from_face(coul_freq,
						       contour_set,
						       dart,
						       int,
						       int*);
static hash_table private_remplir_hash_table_from_image(coul_freq,
							image,
							int,
							int*);
static void      private_store_colors(any_ptr,point,int);

static void private_initialize_from_h_table(coul_freq,hash_table,int);

#define Tab_freq  SELF(tab_freq)
#define Tab_I1    SELF(tab_I1)
#define Tab_I2    SELF(tab_I2)
#define Tab_I3    SELF(tab_I3)
#define Entre_I1  SELF(entre_I1)
#define Entre_I2  SELF(entre_I2)
#define Entre_I3  SELF(entre_I3)
#define Taille    SELF(taille)

struct coul_freq 
{
  f_coul* tab_freq;
  int*    tab_I1;
  int*    tab_I2;
  int*    tab_I3;
  int*    entre_I1;
  int*    entre_I2;
  int*    entre_I3;
  int     taille;
};
coul_freq FAIRE_coul_freq()
{
  INSTANCIER(coul_freq);

  return self;
}

void DEFAIRE_coul_freq(coul_freq self)
{
  int min_I1,min_I2,min_I3;
  
  min_I1 = Tab_freq[Tab_I1[0]].I1;
  min_I2 = Tab_freq[Tab_I2[0]].I2;
  min_I3 = Tab_freq[Tab_I3[0]].I3;

  free(Tab_freq);
  free(Tab_I1);
  free(Tab_I2);
  free(Tab_I3);
  free(Entre_I1+min_I1);
  free(Entre_I2+min_I2);
  free(Entre_I3+min_I3);
    
  free(self);
}

void  coul_freq_initialize_from_face(coul_freq self,
				     contour_set contours,
				     dart f,
				     int espace_choisi)
{
  
  hash_table h_table;
  int taille;


  h_table = private_remplir_hash_table_from_face(self,
						 contours,
						 f,
						 espace_choisi,
						 &taille);

 private_initialize_from_h_table(self,h_table,taille);

}

void coul_freq_initialize_from_image(coul_freq self,
				     image img,
				     int espace_choisi)
{
  hash_table h_table;
  int taille;
            
  h_table = private_remplir_hash_table_from_image(self,
						  img,
						  espace_choisi,
						  &taille);

 private_initialize_from_h_table(self,h_table,taille);
}
double coul_freq_projection(coul_freq self,
			    int axe_proj,
			    int val,
			    intervalle* inter,
			    double* moment1,
			    double* moment2)
{
  float retour;
  switch(axe_proj)
    {
    case 0:
      retour = private_projette_I1(self,val,inter+1,inter+2,moment1,moment2);
      break;
    case 1:
      retour = private_projette_I2(self,val,inter,inter+2,moment1,moment2);
      break;
    case 2:
      retour = private_projette_I3(self,val,inter,inter+1,moment1,moment2);
      break;
    }
  return retour;
}
/**************************************************/
/**************************************************/
int coul_freq_give_nb_of_color(coul_freq self)
{
  return Taille;
}
/**************************************************/
int coul_freq_give_min_C1(coul_freq self)
{
  return Tab_freq[Tab_I1[0]].I1;
}

int coul_freq_give_max_C1(coul_freq self)
{
  return Tab_freq[Tab_I1[Taille-1]].I1;
}
/**************************************************/
int coul_freq_give_min_C2(coul_freq self)
{
  return Tab_freq[Tab_I2[0]].I2;
}

int coul_freq_give_max_C2(coul_freq self)
{
  return Tab_freq[Tab_I2[Taille-1]].I2;
}
/**************************************************/
int coul_freq_give_min_C3(coul_freq self)
{
  return Tab_freq[Tab_I3[0]].I3;
}
/**************************************************/
int coul_freq_give_max_C3(coul_freq self)
{
  return Tab_freq[Tab_I3[Taille-1]].I3;
}
/**************************************************/
int* coul_freq_give_median(coul_freq self,int* min,int* max)
{
  static int median[3];
  int ind;
  
  ind = (Entre_I1[min[0]]+Entre_I1[max[0]])/2;
  median[0] = Tab_freq[Tab_I1[ind]].I1;
  
  ind = (Entre_I2[min[1]]+Entre_I2[max[1]])/2;
  median[1] = Tab_freq[Tab_I2[ind]].I2;
  
  ind = (Entre_I3[min[2]]+Entre_I2[max[2]])/2;
  median[2] = Tab_freq[Tab_I3[ind]].I3;
  
  return median;
}
/*****************************************************************/
static hash_table private_remplir_hash_table_from_face(coul_freq self,
						       contour_set contours,
						       dart f,
						       int espace_choisi,
						       int* taille)
{
  hash_table h_table;
  PFconversion *convert= changer_espace(espace_choisi);

  h_table = hash_create(private_alloue,
			free,
			private_cle,
			private_egal,
			private_desalloue);
  
  private_store_colors(h_table,NULL,INITIALIZE_H_TABLE);
  private_store_colors(convert,NULL,INITIALIZE_CONVERSION);
  private_store_colors(NULL,NULL,INITIALIZE_NB_OF_DATA);
  contour_set_apply_function(contours,f,private_store_colors);
  private_store_colors(taille,NULL,RETURN_NB_OF_DATA);
  
  return h_table;
}
static void   private_store_colors(any_ptr obj,point p,int flag)
{
  static hash_table h_table;
  static int taille;
  static PFconversion *convert;
  int* pixel;
  int R,G,B;
  f_coul cel,*pcel;
  float un_par_reduction = 1.0/REDUCTION;
  
  switch(flag)
    {
    case INITIALIZE_H_TABLE:
      h_table = (hash_table)obj;
      break;
    case INITIALIZE_CONVERSION:
      convert = (PFconversion*)obj;
      break;
    case INITIALIZE_NB_OF_DATA:
      taille=0;
      break;
    case STORE:
      pixel = (int*)obj;
      R = *pixel;
      G = *(pixel+1);
      B = *(pixel+2);
      
      cel.I1 = (int)convert[0](R,G,B)*un_par_reduction;
      cel.I2 = (int)convert[1](R,G,B)*un_par_reduction;
      cel.I3 = (int)convert[2](R,G,B)*un_par_reduction;
      cel.freq = 1;
      
      if(hash_position(h_table,&cel))
	{
	  pcel = hash_get(h_table);
	  pcel->freq++;
	}
      else
	{
	  pcel =(f_coul*)malloc(sizeof(f_coul));
	  pcel->I1 = cel.I1;
	  pcel->I2 = cel.I2;
	  pcel->I3 = cel.I3;
	  pcel->freq = cel.freq;
	  taille++;
	  hash_insert(h_table,pcel);
	}
      break;
    case RETURN_NB_OF_DATA:
      *(int*)obj = taille;
    default:
      break;
    }
}

static hash_table private_remplir_hash_table_from_image(coul_freq self,
							image img,
							int espace_choisi,
							int* nb_coul)
{
  int *pixel;
  int taille=0;
  f_coul cel,*pcel;
  int R,G,B;
  PFconversion *convert= changer_espace(espace_choisi);
  double un_par_reduction = 1.0/REDUCTION;
  hash_table h_table;

  h_table = hash_create(private_alloue,
			free,
			private_cle,
			private_egal,
			private_desalloue);

  image_debut(img);
  do
    {
      pixel = image_lire_pixel(img);

      R = *pixel;
      G = *(pixel+1);
      B = *(pixel+2);
      
      cel.I1 = (int)convert[0](R,G,B)*un_par_reduction;
      cel.I2 = (int)convert[1](R,G,B)*un_par_reduction;
      cel.I3 = (int)convert[2](R,G,B)*un_par_reduction;
      cel.freq = 1;
  
      if(hash_position(h_table,&cel))
	{
	  pcel = hash_get(h_table);
	  pcel->freq++;
	}
      else
	{
	  pcel =(f_coul*)malloc(sizeof(f_coul));
	  pcel->I1 = cel.I1;
	  pcel->I2 = cel.I2;
	  pcel->I3 = cel.I3;
	  pcel->freq = cel.freq;
	  taille++;
	  hash_insert(h_table,pcel);
	}
    }
  while( image_pixel_suivant(img) );
  
  *nb_coul = taille;

  return h_table;
}
static double private_projette_I1(coul_freq self,
				  int val,
				  intervalle* inter2,
				  intervalle* inter3,
				  double* moment1,
				  double *moment2)
{
  double freq = 0.0;				/* variable de retour */
  f_coul* pcel;					/* variables */
  int j;					/* auxilieres*/        
  int binf2 = inter2->borne_inf;    
  int bsup2 = inter2->borne_sup;
  int binf3 = inter3->borne_inf;
  int bsup3 = inter3->borne_sup;		/* donnees*/
  int* tab = Tab_I1;
  int min = Entre_I1[val];
  int max;
  int max_C1 = Tab_freq[Tab_I1[Taille-1]].I1;
  int min2,max2;
  double valI2,valI3;
  int demi_int = REDUCTION >>1;
  

  moment1[0] = 0.0;
  moment1[1] = 0.0;
  moment2[0] = 0.0;
  moment2[1] = 0.0;
  

  if( val != max_C1)
    {
      max = Entre_I1[(val+1)];
      if(max ==-1)
	max = Taille;
    }
  else
    max = Taille;
  
  if((max>Taille)||(max<0)||(min<0)||(min>Taille))
    printf("DANGER : min = %d,max = %d",min,max);
  if((min != -1)&&(min !=max))			/* sinon pas de valeurs superieures ou */
    /* egale a val et strictement */
    /* inferieures a val+1*/
    {
      if(max==min+1)				/* une seule valeur verifie la propriete */
	/* ci dessus*/
	{
	  pcel = Tab_freq+tab[min];
	  if((pcel->I2 >= binf2)&&(pcel->I2<bsup2)&&(pcel->I3 >= binf3)&&(pcel->I3<bsup3))
	    {
	      freq += pcel->freq;
	      valI2 =	(pcel->I2)*REDUCTION;
	      if(pcel->I2 <0)
		valI2 -= demi_int;
	      else
		valI2 += demi_int;
	      
	      valI3 =	(pcel->I3)*REDUCTION;
	      if(pcel->I3 <0)
		valI3 -= demi_int;
	      else
		valI3 += demi_int;
	      
	      moment1[0] = (pcel->freq)*valI2;
	      moment2[0] = (pcel->freq)*valI3;
	      moment1[1] = (pcel->freq)*valI2*valI2;
	      moment2[1] = (pcel->freq)*valI3*valI3;
	    }
	}
      else
	{
	  
	  min2 =private_chercher(binf2,
				 Tab_freq,
				 tab,
				 min,
				 max-1,
				 private_compare_a_val_I2);
	  if(min2 != -1)
	    {
	      max2 =private_chercher(bsup2,
				     Tab_freq,
				     tab,
				     min,
				     max-1,
				     private_compare_a_val_I2);
	      max2 = (max2==-1?max:max2);
	      for(j=min2;j<max2;j++)
		{
		  pcel = Tab_freq+tab[j];
		  if((pcel->I3>=binf3)&&(pcel->I3 < bsup3))
		    {
		      freq += pcel->freq;
		      valI2=(pcel->I2)*REDUCTION;
		      
		      if(pcel->I2 <0)
			valI2 -= demi_int;
		      else
			valI2 += demi_int;
		      valI3= (pcel->I3)*REDUCTION;
		      if(pcel->I3 <0)
			valI3 -= demi_int;
		      else
			valI3 += demi_int;
		      
		      moment1[0] += (pcel->freq)*valI2;
		      moment2[0] += (pcel->freq)*valI3;
		      moment1[1] += (pcel->freq)*valI2*valI2;
		      moment2[1] += (pcel->freq)*valI3*valI3;
		      
		    }
		}
	    }
	}
    }
  return freq;
}    

static double private_projette_I2(coul_freq self,
				  int val,
				  intervalle* inter1,
				  intervalle* inter3,
				  double* moment1,
				  double* moment2)
{
  double freq = 0.0;				/* variable de retour */
  f_coul* pcel;					/* variables */
  int j;					/* auxilieres*/        
  int binf1 = inter1->borne_inf;    
  int bsup1 = inter1->borne_sup;
  int binf3 = inter3->borne_inf;
  int bsup3 = inter3->borne_sup;		/* donnees*/

  int* tab = Tab_I2;

  int min = Entre_I2[val];
  int max;
  int max_C2 = Tab_freq[tab[Taille-1]].I2;
  int min1,max1;
  double valI1,valI3;
  int demi_int = REDUCTION >>1;
  double v1,v3,f,f_val_I1,f_val_I3;

  moment1[0] =moment1[1] = 0.0;
  moment2[0] =moment2[1] = 0.0;

  if( val != max_C2)
    {
      max = Entre_I2[val+1];
      if(max ==-1)
	max = Taille;
    }
  else
    max = Taille;
  
  if((max>Taille)||(max<0)||(min<0)||(min>Taille))
    printf("DANGER : min = %d,max = %d",min,max);
  if((min != -1)&&(min!=max))
    {
      if(max==min+1)
	{
	  pcel = Tab_freq+tab[min];
	  if((pcel->I1>=binf1)&&(pcel->I1<bsup1)&&
	     (pcel->I3>=binf3)&&(pcel->I3<bsup3))
	    {
	      freq +=  pcel->freq;
	      valI1= pcel->I1*REDUCTION + (pcel->I1 >=0?demi_int:-demi_int);
	      valI3= pcel->I3*REDUCTION + (pcel->I3 >=0?demi_int:-demi_int);
	      
	      moment1[0] = (pcel->freq)*valI3;
	      moment2[0] = (pcel->freq)*valI1;
	      moment1[1] = (pcel->freq)*valI3*valI3;
	      moment2[1] = (pcel->freq)*valI1*valI1;
	    }
	}
      
      else
	{
	  min1 =private_chercher(binf1,
				 Tab_freq,
				 tab,
				 min,
				 max-1,
				 private_compare_a_val_I1);
	  if(min1 !=-1)
	    
	    {
	      max1 =private_chercher(bsup1,
				     Tab_freq,
				     tab,
				     min,
				     max-1,
				     private_compare_a_val_I1);
	      max1 = (max1==-1?max:max1);
	      for(j=min1;j<max1;j++)
		{
		  pcel = Tab_freq+tab[j];
		  v3 = pcel->I3;
		  if((v3>=binf3)&&(v3 <  bsup3))
		    {
		      v1 = pcel->I1;
		      f  = pcel->freq;


		      valI1=v1*REDUCTION + (v1 >=0?demi_int:-demi_int);
		      valI3=v3*REDUCTION + (v3 >=0?demi_int:-demi_int);	

		      f_val_I1 = f*valI1;
		      f_val_I3 = f*valI3;

		      freq       += f;
		      moment1[0] += f_val_I3;
		      moment2[0] += f_val_I1;
		      moment1[1] += f_val_I3*valI3;
		      moment2[1] += f_val_I1*valI1;
		    }					
		}
	    }
	}
    }

  return freq;
}
static double private_projette_I3(coul_freq   self,
				  int         val,
				  intervalle* inter1,
				  intervalle* inter2,
				  double*     moment1,
				  double*     moment2)
{
  double freq = 0.0;				/* variable de retour */
  f_coul* pcel;					/* variables */
  int j;					/* auxilieres*/        
  int binf1 = inter1->borne_inf;    
  int bsup1 = inter1->borne_sup;
  int binf2 = inter2->borne_inf;
  int bsup2 = inter2->borne_sup;		/* donnees*/
 
  int* tab = Tab_I3;

  int min = Entre_I3[val];
  int max;
  int max_C3 = Tab_freq[tab[Taille-1]].I3;
  int min1,max1;
  double valI1,valI2;
  int demi_int= REDUCTION >>1;

  moment1[0] =moment1[1] = 0.0;
  moment2[0] =moment2[1] = 0.0;
  
  
  if( val != max_C3)
    {
      max = Entre_I3[val+1];
      if(max ==-1)
	max = Taille;
    }
  else
    max = Taille;
  if((max>Taille)||(max<0)||(min<0)||(min>Taille))
    printf("DANGER : min = %d,max = %d",min,max);
  if((min != -1)&&(min!=max))
    {
      if(max==min+1)
	{
	  pcel = Tab_freq+tab[min];
	  if((pcel->I1 >=binf1)&&(pcel->I1<bsup1)&&(pcel->I2 >= binf2)&&(pcel->I2<bsup2)) 
	    {
	      freq += pcel->freq;
	      valI1 = pcel->I1*REDUCTION + (pcel->I1 >=0?demi_int:-demi_int);
	      valI2 = pcel->I2*REDUCTION + (pcel->I2  >=0?demi_int:-demi_int);
	      
	      moment1[0] += (pcel->freq)*valI1;
	      moment2[0] += (pcel->freq)*valI2;
	      moment1[1] += (pcel->freq)*valI1*valI1;
	      moment2[1] += (pcel->freq)*valI2*valI2;
	    }
	}
      else
	{
	  min1 =private_chercher(binf1,
				 Tab_freq,
				 tab,
				 min,
				 max-1,
				 private_compare_a_val_I1);
	  if(min1 != -1)
	    {
	      max1=private_chercher(bsup1,
				    Tab_freq,
				    tab,
				    min,
				    max-1,
				    private_compare_a_val_I1);
	      max1= (max1==-1?max:max1);
	      for(j=min1;j<max1;j++)
		{
		  pcel = Tab_freq+tab[j];
		  if((pcel->I2>=binf2)&&(pcel->I2 < bsup2))
		    {
		      freq +=	pcel->freq;
		      valI1 = pcel->I1*REDUCTION + (pcel->I1 >=0?demi_int:-demi_int);
		      valI2 = pcel->I2*REDUCTION + (pcel->I2 >=0?demi_int:-demi_int);

		      moment1[0] += (pcel->freq)*valI1;
		      moment2[0] += (pcel->freq)*valI2;
		      moment1[1] += (pcel->freq)*valI1*valI1;
		      moment2[1] += (pcel->freq)*valI2*valI2;
		    }
		}
	    }
	}
    }
  
  return freq;
}


/* Recherche dichomatique de val dans le tableau V a l'aide de la */
/* fonction comp */

static int private_chercher(int val,
		     f_coul* V,
		     int * tab,
		     int min,
		     int max,
		     int (*comp)())
{
  int milieu = (min+max)/2;
  
  if(comp(V,tab[min],val,NULL) >=0)
    return min;
  if(comp(V,tab[max],val,NULL)<0)
    return -1;

  if(max-min ==1)
    {
      if(comp(V,tab[min],val,NULL)<0)
	return max;
      else
	return min;
    }

  if(comp(V,tab[milieu],val,NULL)<0)
    return private_chercher(val,V,tab,milieu,max,comp);
  else
    return private_chercher(val,V,tab,min,milieu,comp);
}
/****************************************************************************/
static void private_creer_entre(coul_freq self,
				int min,
				int max,
				int *tab,
				int* entre,
				int (*comp)(),
				int (*acce)(f_coul*,int))
{
  int j,i=0;
  int ind = min;
  int val;

  while(i<Taille)
    {
      while((comp(Tab_freq,tab[i],ind,NULL)<0)&&(i<Taille-1)) i++;
      val = acce(Tab_freq,tab[i]);
      for(j=ind;j<= val;j++) entre[j-min] =i; 
      ind = val+1;
      i++;
    }
  for(j=ind;j<=max;j++) entre[j-min] = -1;
}
/****************************************************************************/
static int private_acce_I1(f_coul* tab,int i)
{
  return (tab[i].I1);
}
/****************************************************************************/
static int private_acce_I2(f_coul* tab,int i)
{
  return (tab[i].I2);
}
/****************************************************************************/
static int private_acce_I3(f_coul* tab,int i)
{
  return (tab[i].I3);
}
/****************************************************************************/
static int private_compare_a_val_I1(f_coul * tab,int i,int val,int  (*comp)())
{
  int valp = tab[i].I1;
    
  if(valp < val)
    return -1;
  if(valp ==val)
    {
      if(comp != NULL)
	return comp(tab,i,val,NULL);
      else
	return 0;
    }
  return 1;
}
/****************************************************************************/
static int private_compare_a_val_I2(f_coul * tab,int i,int val,int  (*comp)())
{
  int valp = tab[i].I2;

  if(valp < val)
    return -1;
  if(valp==val)
    {
      if(comp != NULL)
	return comp(tab,i,val,NULL);
      else
	return 0;
    }
  return 1;
}
/****************************************************************************/
static int private_compare_a_val_I3(f_coul * tab,int i,int val,int  (*comp)())
{
  int valp = tab[i].I3;

  if(valp < val)
    return -1;
  if(valp==val)
    {
      if(comp != NULL)
	return comp(tab,i,val,NULL);
      else
	return 0;
    }
  return 1;
}

/****************************************************************************/
/* Fonctions de comparaisons utilisees pour l'algorithme de trie:  */
/* private_trie */
/****************************************************************************/
static int private_compare_I3(const void *ivoid,const void* jvoid)
{
  int i = *(int*)ivoid;
  int j = *(int*)jvoid;

  int val1 = donnee[i].I3;
  int val2 = donnee[j].I3;
  
  
  if(val1 < val2)
    return -1;
  if(val1==val2)
    {
      if(flag_trie == TRIE_C3C1)
	return private_compare_I1(ivoid,jvoid);
      else
	return 0;
    }
  return 1;
}

static int private_compare_I2(const void* ivoid,const void* jvoid)
 {
   int i = *(int*)ivoid;
   int j = *(int*)jvoid;

   int val1 = donnee[i].I2;
   int val2 = donnee[j].I2;

    if(val1 < val2)
	    return -1;
    if(val1==val2)
	    {
		if(flag_trie == TRIE_C2C1)
			return private_compare_I1(ivoid,jvoid);
		else
			return 0;
	    }
    return 1;
}

static int private_compare_I1(const void* ivoid,const void* jvoid)
{
  int i = *(int*)ivoid;
  int j = *(int*)jvoid;
  
  int val1 = donnee[i].I1;
  int val2 = donnee[j].I1;

    if(val1 < val2)
	    return -1;
    if(val1==val2)
	    {
		if(flag_trie == TRIE_C1C2)
			return private_compare_I2(ivoid,jvoid);
		else
			return 0;
	    }
    return 1;
}

static void private_trie(coul_freq self,int coupe)
{
  flag_trie = coupe;
  donnee = Tab_freq;
  
  switch(coupe)
    {
    case TRIE_C1C2:
      qsort(Tab_I1,Taille,sizeof(int),private_compare_I1);
      break;
    case TRIE_C2C1:
      qsort(Tab_I2,Taille,sizeof(int),private_compare_I2);
      break;
    case TRIE_C3C1:
      qsort(Tab_I3,Taille,sizeof(int),private_compare_I3);
      break;
    default:
      printf("Mauvais parametre dans le trie\n");
    }
}


static hash_anything private_alloue(int size)
{
  hash_anything retour = malloc(size);
  return(retour);
}
static void private_desalloue(hash_object pcel)
{
  free(pcel);
}

/****************************************************************************/
static  int private_egal(hash_object any1,hash_object any2)
{
  f_coul* pcel1 = (f_coul*)any1;
  f_coul* pcel2 = (f_coul*)any2;
  bouleen egal_I1 = (pcel1->I1 == pcel2->I1);
  bouleen egal_I2 = (pcel1->I2 == pcel2->I2);
  bouleen egal_I3 = (pcel1->I3 == pcel2->I3);

  return(egal_I1&&egal_I2&&egal_I3);
}
/****************************************************************************/
static int private_cle(hash_object coul)
{
  f_coul* pcel = (f_coul*)coul;
  unsigned int aI1 = ABS(pcel->I1);
  unsigned int aI2 = ABS(pcel->I2);
  unsigned int aI3 = ABS(pcel->I3);

  int retour = (aI1 << 16)|(aI2 <<8)|aI3;

  return(retour);
}


/****************************************************************************/
static void private_transfer(hash_object coul)
{
  static int compteur;
  static f_coul* tab_freq;
  f_coul* pcel;

  pcel = (f_coul*)coul;  

  if(pcel->I1 ==INT_MIN)
    {
      compteur =0;
      tab_freq = (f_coul*)pcel->freq;
    }
  else
    {
      tab_freq[compteur].I1 = pcel->I1;
      tab_freq[compteur].I2 = pcel->I2;
      tab_freq[compteur].I3 = pcel->I3;
      tab_freq[compteur].freq = pcel->freq;
      compteur++;
    }
}




static void private_initialize_from_h_table(coul_freq self,
					    hash_table h_table,
					    int taille)
{
  int i;
  int min_I1,min_I2,min_I3;
  int max_I1,max_I2,max_I3;
  f_coul cel;  

  Taille = taille;

  Tab_I1 = (int*)malloc(Taille*sizeof(int));
  Tab_I2 = (int*)malloc(Taille*sizeof(int));
  Tab_I3 = (int*)malloc(Taille*sizeof(int));
  Tab_freq = (f_coul*)malloc(Taille*sizeof(f_coul));

  for(i=0;i<Taille;i++)
    {
      Tab_I1[i] = i;
      Tab_I2[i] = i;
      Tab_I3[i] = i;
    }
  /* on positionne tab_freq dans tranfer */  
  cel.I1 =INT_MIN;
  cel.freq= (long)Tab_freq;
  private_transfer(&cel);
  /* On transvase les donnees de h_table dans Tab_freq */
  hash_map(h_table,private_transfer);
  
  /* on detruit la table */
  hash_destroy(h_table);                  
  
  /* On trie Tab_freq suivant I1,I2,I3 le resultat etant place dans les */
  /* tableaux Tab_Ij */
  private_trie(self,TRIE_C1C2);
  private_trie(self,TRIE_C2C1);
  private_trie(self,TRIE_C3C1);
  
  min_I1 = Tab_freq[Tab_I1[0]].I1;
  min_I2 = Tab_freq[Tab_I2[0]].I2;
  min_I3 = Tab_freq[Tab_I3[0]].I3;

  max_I1 = Tab_freq[Tab_I1[Taille-1]].I1;
  max_I2 = Tab_freq[Tab_I2[Taille-1]].I2;
  max_I3 = Tab_freq[Tab_I3[Taille-1]].I3;

  Entre_I1 = (int*)malloc((max_I1-min_I1+1)*sizeof(int));
  Entre_I2 = (int*)malloc((max_I2-min_I2+1)*sizeof(int));
  Entre_I3 = (int*)malloc((max_I3-min_I3+1)*sizeof(int));



/*On crait des tables permettant */
/* d' acceder directement a une valeur*/
/* de C1, C2 ou C3*/
  private_creer_entre(self,
		      min_I1,max_I1,
		      Tab_I1,Entre_I1,
		      private_compare_a_val_I1,private_acce_I1); 
  private_creer_entre(self,
		      min_I2,max_I2,
		      Tab_I2,Entre_I2,
		      private_compare_a_val_I2,private_acce_I2); 
  private_creer_entre(self,
		      min_I3,max_I3,
		      Tab_I3,Entre_I3,
		      private_compare_a_val_I3,private_acce_I3); 

#ifdef DEBUG
  printf("j' ai cree les entrees\n");
#endif

  Entre_I1 = Entre_I1-min_I1;
  Entre_I2 = Entre_I2-min_I2;
  Entre_I3 = Entre_I3-min_I3;
}





